package Install;
use strict;
use Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(which);

sub which;

# Create the fastx user if it doesn't already exist, and return it.
# If we can't create it, return undef
sub fastx_user
{
	my $args = shift;
	# If the user is named differently, it will still have this home directory
	my $homedir = ::abs_path(::dirname($0) . '/..');
	open PASSWD, '<', '/etc/passwd' or die "/etc/passwd: $!\n";
	my $name;
	while (<PASSWD>) {
		my ($login, $dir);
		($login, undef, undef, undef, undef, $dir) = split /:/;
		return $login if $dir eq $homedir;
		$name = $login if $login eq 'fastx';
	}
	return $name if $name;
	# Didn't find it, so create it.
	return undef if $args && $args->{nomake};
	$name = 'fastx';
	print "Creating $name user... ";
	my $rc = system ('useradd', '-r', '-s', '/sbin/nologin', '-d', $homedir, $name);
	print "ok\n",  return $name if $rc == 0;
	$rc = $! if $rc == -1;
	print "failed ($rc).\n";
	return undef;
}

sub add_service
{
	my $args = shift;
	my $name = $args->{name} || $args->{service};
	my $nostart = $args->{nostart};
	my $service = $args->{service};
	my $node = $args->{node};
	my $fastx_user = fastx_user();
	return undef unless $fastx_user;
	my $system = 'redhat' if -e '/etc/redhat-release';
	$system = 'debian' if -e '/etc/debian_version';
	$system = 'suse' if -e '/etc/SuSE-release';
	my $installdir = ::dirname($0);
	my $homedir = ::abs_path($installdir . '/..');
	print "Installing $name... ";

	my $systemd;
	if (open my $fd, '-|', 'systemctl') {
		while (<$fd>) {
			$systemd = 1, last if /\.mount/;
		}
	}
	if ($systemd && open my $input, '<', "$installdir/init.d/${service}.service") {
		open my $output, '>', "/etc/systemd/system/${service}.service" or
			return (print "failed.\n/etc/systemd/system/${service}.service: $!\n"), undef;
		while (<$input>) {
			s/^User=\w*/User=$fastx_user/;
			s#[^\s\=]*/?node#$node# if $node;
			s#/usr/lib/fastx2#$homedir# if $homedir;
			print $output $_;
		}
		close $output;
		chmod 0644, "/etc/systemd/system/${service}.service";
		if (-e "/etc/init.d/$service") {
			system 'service', $service, 'stop';
			if ($system eq 'redhat' or $system eq 'suse') {
				system 'chkconfig', '--del', $service;
				system '/sbin/chkconfig', '--del', $service;
			}
			if ($system eq 'debian') {
				system 'update-rc.d', $service, 'remove' and
				system '/usr/sbin/update-rc.d', $service, 'remove';
			}
			unlink "/etc/init.d/$service";
		}
		system 'systemctl', 'daemon-reload';
		system 'systemctl', 'enable', "${service}.service";
		system 'systemctl', 'restart', "${service}.service" unless $nostart;
		return (print "failed.\n"), undef if $?;
		return (print "done.\n"), 1;
	}

	return (print "failed.\nCan't determine system type.\n"), undef unless $system;
	my $input;
	open $input, '<', "$installdir/init.d/${service}_${system}" or
		open $input, '<', "$installdir/init.d/${service}" or
		return (print "failed.\nCan't find template for $service.\n"), undef;
	open my $output, '>', "/etc/init.d/$service" or
		return (print "failed.\n/etc/init.d/$service: $!\n"), undef;
	while (<$input>) {
		s/^rlmuser=\w*/rlmuser=$fastx_user/;
		s#/usr/lib/fastx2#$homedir# if $homedir;
		s#^licensedir=\S*#licensedir=\"$homedir/var/license\"# if $homedir;
		s/^FASTX_USER=\w*/FASTX_USER=$fastx_user/;
		s/^NODE=\S*/NODE=\"$node\"/ if $node;
		s/^DAEMON=\S*/DAEMON=\"$node\"/ if $node;
		print $output $_;
	}
	close $output;
	chmod 0755, "/etc/init.d/$service";
	if ($system eq 'redhat' or $system eq 'suse') {
		unless (system 'chkconfig', '--add', $service and
		    system '/sbin/chkconfig', '--add', $service) {
			system 'service', $service, 'restart' unless $nostart;
			print "done.\n";
			return 1;
		}
		print "chkconfig failed.\n";
		return undef;
	}
	if ($system eq 'debian') {
		my $pid = fork;
		return undef unless defined $pid;
		unless ($pid) {
			open STDOUT, '>', '/dev/null';
			open STDERR, '>', '/dev/null';
			exec 'update-rc.d', $service, 'defaults';
			exec '/usr/sbin/update-rc.d', $service, 'defaults';
			exit 127;
		}
		waitpid $pid, 0;
		unless ($?) {
			system 'service', $service, 'restart' unless $nostart;
			print "done.\n";
			return 1;
		}
		print "update-rc.d failed.\n";
		return undef;
	}
	print "failed.\nCan't handle $system.\n";
	return undef;
}

sub start_service
{
	my $args = shift;
	my $service = $args->{service};

	system "systemctl restart '${service}.service' >/dev/null 2>&1";
	return 1 unless $?;
	system 'service', $service, 'start';
	return 1 unless $?;
	return undef;
}

sub remove_service
{
	my $args = shift;
	my $service = $args->{service};
	my $installdir = ::dirname($0);
	my $homedir = ::abs_path($installdir . '/..');
	my $systemd;
	my $system = 'redhat' if -e '/etc/redhat-release';
	$system = 'debian' if -e '/etc/debian_version';
	$system = 'suse' if -e '/etc/SuSE-release';

	if (open my $fd, '-|', 'systemctl') {
		while (<$fd>) {
			$systemd = 1, last if /\.mount/;
		}
	}
	if ($systemd) {
		if (open my $fd, '<', "/etc/systemd/system/${service}.service") {
			my $work_dir;
			my $server_file;
			while (<$fd>) {
				chomp;
				$work_dir = $1 if /^WorkingDirectory\s*=\s*"([^"]+)"/;
				$work_dir = $1 if /^WorkingDirectory\s*=\s*'([^']+)'/;
				$work_dir = $1 if /^WorkingDirectory\s*=\s*([\w\/\.\-]+)/;
				$server_file = $1 if /^ExecStart\s*=.*node\S*\s+(\S+)/;
			}
			if ($work_dir) {
				return undef if substr($work_dir, 0, length $homedir) ne $homedir;
			} elsif ($server_file) {
				return undef if substr($server_file, 0, length $homedir) ne $homedir;
			} else {
				return undef;
			}
			system 'systemctl', 'stop', "${service}.service";
			system 'systemctl', 'disable', "${service}.service";
			unlink "/etc/systemd/system/${service}.service";
			system 'systemctl', 'daemon-reload';
		}
	}
	# Do the init.d stuff even if $systemd, because it can't hurt.
	if (open my $fd, '<', "/etc/init.d/$service") {
		my $rlmdir;
		my $fastxdir;
		while (<$fd>) {
			chomp;
			$rlmdir = $1 if /^rlmdir="([^"]+)"/;
			$rlmdir = $1 if /^rlmdir='([^']+)'/;
			$rlmdir = $1 if /^rlmdir=([\w\/\.\-]+)/;
			$fastxdir = $1 if /^BASEDIR="([^"]+)"/;
			$fastxdir = $1 if /^BASEDIR='([^']+)'/;
			$fastxdir = $1 if /^BASEDIR=([\w\/\.\-]+)/;
		}
		if ($fastxdir) {
			return undef if $fastxdir ne $homedir;
		} elsif ($rlmdir) {
			return undef if $rlmdir ne "$homedir/rlm";
		} else {
			return undef;	# Can't parse init.d file, give up.
		}
		system 'service', $service, 'stop';
		if ($system eq 'redhat' or $system eq 'suse') {
			system 'chkconfig', '--del', $service;
			system '/sbin/chkconfig', '--del', $service;
		}
		if ($system eq 'debian') {
			system 'update-rc.d', $service, 'remove' and
			system '/usr/sbin/update-rc.d', $service, 'remove';
		}
		unlink "/etc/init.d/$service";
	}
}

sub run_with_log
{
	my $log = shift;
	my $pid = fork;
	die 'fork' unless defined $pid;
	unless ($pid) {
		open STDERR, ">&", $log;
		open STDOUT, ">&", $log;
		open STDIN, '<', '/dev/null';
		print STDERR "@_\n";
		exec @_;
		die "$_[0]: $!\n";
	}
	waitpid $pid, 0;
	my $code = $?;
	print $log "Exit code: $code\n" if $code;
	return $code;
}

sub system_info
{
	my $info = { };
	my %os_pm = (centos => 'yum', rhel => 'yum', debian => 'apt-get', suse => 'zypper', sles => 'zypper');
	my %pm_os = (yum => 'rhel', dnf => 'fedora', 'apt-get' => 'debian', zypper => 'suse');

	my $fd;
	if (open $fd, '<', '/etc/os-release' or open $fd, '<', '/usr/lib/os-release') {
		while (<$fd>) {
			chomp;
			if (/^\s*([A-Z_]*)\s*=\s*\"([^"]*)\"/ or
			    /^\s*([A-Z_]*)\s*=\s*(\S+)/) {
				$info->{$1} = $2;
			}
		}
		(my $like) = split ' ', $info->{'ID_LIKE'};
		$info->{'os_id'} = $info->{'ID'} if !$like
		    or $info->{'ID'} =~ /(debian)|(rhel)|(suse)/;
		$info->{'os_id'} = $like unless $info->{'os_id'};
		$info->{'pm'} = $os_pm{$info->{'os_id'}};
		if ($info->{'ID'} =~ /(rhel)|(centos)/) {
			$info->{'dist_type'} = 'el';
			$info->{'dist_version'} = $info->{'VERSION_ID'};
		}
		if ($info->{'ID'} eq 'fedora') {
			$info->{'dist_type'} = 'fc';
			$info->{'dist_version'} = $info->{'VERSION_ID'};
			$info->{'pm'} = 'dnf';
			$info->{'pm'} = 'yum' unless ($info->{'pm_path'} = which 'dnf');
		}
	} elsif (-e '/etc/debian_version') {
		$info->{'os_id'} = 'debian';
		$info->{'ID'} = 'debian';
		$info->{'pm'} = 'apt-get';
		chomp ($info->{'VERSION_ID'} = <$fd>)
			if open $fd, '<', '/etc/debian_version';
	} elsif (-e '/etc/SuSE-release') {
		$info->{'os_id'} = 'suse';
		$info->{'ID'} = 'suse';
		$info->{'pm'} = 'zypper';
		if (open my $fd, '<', '/etc/SuSE-release') {
			chomp ($info->{'distro'} = <$fd>);
			my $patchlevel;
			while (<$fd>) {
				chomp;
				$info->{'VERSION_ID'} = $_ if s/^VERSION\s*=\s*//;
				$patchlevel = $_ if s/^PATCHLEVEL\s*=\s*//;
			}
			$info->{'VERSION_ID'} .= ".$patchlevel" if $info->{'VERSION_ID'} && $patchlevel;
		}
	} elsif (-e '/etc/fedora-release') {
		$info->{'os_id'} = 'rhel';
		$info->{'ID'} = 'fedora';
		$info->{'dist_type'} = 'fc';
		$info->{'pm'} = 'dnf';
		$info->{'pm'} = 'yum' unless ($info->{'pm_path'} = which 'dnf');
	} elsif (-e '/etc/redhat-release') {
		$info->{'os_id'} = 'rhel';
		$info->{'ID'} = 'rhel';
		$info->{'pm'} = 'yum';
	}
	unless ($info->{'pm'}) {
		# Try to guess based on the package manager.
		foreach ('dnf', 'zypper', 'yum', 'apt-get') {
			my $pm_path = which $_;
			next unless $pm_path;
			$info->{'pm'} = $_;
			$info->{'pm_path'} = $pm_path;
			$info->{'os_id'} = $pm_os{$_} unless $info->{'os_id'};
			last;
		}
		return undef unless $info->{'pm'};
		$info->{'ID'} = $info->{'os_id'} unless $info->{'ID'};
	}
	$info->{'pm_path'} = which $info->{'pm'} unless $info->{'pm_path'};

	$info->{'arch'} = `uname -m`;
	die "uname: $!\n" if $?;
	chomp $info->{'arch'};

	if ($info->{'os_id'} eq 'rhel' and !$info->{'dist_type'}) {
		my $distro;
		$distro = `rpm -q --whatprovides redhat-release`;
		$distro = `rpm -q --whatprovides centos-release` if $?;
		$distro = `rpm -q --whatprovides cloudlinux-release` if $?;
		$distro = `rpm -q --whatprovides sl-release` if $?;
		chomp $distro;
		$info->{'dist_type'} = 'el' if $distro =~ /^(redhat|centos|cloudlinux|sl|oraclelinux)-/;
		$info->{'dist_type'} = 'el' if $distro =~ /^system-release-/;
		$info->{'dist_type'} = 'fc' if $distro =~ /^fedora-/;
		$info->{'dist_version'} = $1 if $distro =~ /\w+-release(?:-server|-workstation|-client)?-(\d+)/;
		$info->{'dist_version'} = 7 if $distro =~ /^system-release-/; # Amazon Linux
		$info->{'distro'} = $distro if $distro;
	}
	return $info;
}

sub which
{
	my $file = shift;
	my @dirs = split /:/, $ENV{PATH};
	@dirs = qw(/sbin /usr/sbin /bin /usr/bin) unless @dirs;
	foreach my $dir (@dirs) {
		return "$dir/$file" if -f "$dir/$file" && -x "$dir/$file";
	}
	return undef;
}
