'use strict';

import angular from "angular";

export default angular.module('fastx.Filters',[])
.filter('num', function() {
    return function(input) {
      return parseInt(input, 10);
    }
})
.filter('bytes', function() {
	return function(bytes, precision) {
		if (isNaN(parseFloat(bytes)) || !isFinite(bytes)) return '-';
		if (typeof precision === 'undefined') precision = 1;
		var units = ['bytes', 'kB', 'MB', 'GB', 'TB', 'PB'],
			number = Math.floor(Math.log(bytes) / Math.log(1024));
		return (bytes / Math.pow(1024, Math.floor(number))).toFixed(precision) +  ' ' + units[number];
	}
})
.filter('uptime', function(){
	return function(seconds){
		if (isNaN(parseFloat(seconds)) || !isFinite(seconds)) return '';
		var overflow;
		var t = {
			seconds:0,
			minutes:0,
			hours:0,
			days:0,
			years:0
		};
		t.seconds = Math.floor(seconds % 60);
		t.minutes = Math.floor(seconds / 60) % 60;
		t.hours = Math.floor(seconds / (60 * 60)) % 24;
		t.days =  Math.floor(seconds / (60 * 60 * 24)) % 365;
		t.years =  Math.floor(seconds / (60 * 60 * 24 * 365));
		var str ='';
		if(t.years  ){str+= t.years + ' y ';}
		if(t.days   ){str+= t.days + ' d ';}
		if(t.hours  ){str+= t.hours + ' h ';}
		if(t.minutes){str+= t.minutes + ' m ';}
		//if(t.seconds){str+= t.seconds + ' s ';}
		return str;
	}
})
.filter('uptimeL', function(){
	return function(seconds){
		if (isNaN(parseFloat(seconds)) || !isFinite(seconds)) return '';
		var overflow;
		var t = {
			seconds:0,
			minutes:0,
			hours:0,
			days:0,
			years:0
		};
		t.seconds = Math.floor(seconds % 60);
		t.minutes = Math.floor(seconds / 60) % 60;
		t.hours = Math.floor(seconds / (60 * 60)) % 24;
		t.days =  Math.floor(seconds / (60 * 60 * 24)) % 365;
		t.years =  Math.floor(seconds / (60 * 60 * 24 * 365));
		var str ='';
		if(t.years  ){str+= [t.years ,' ','year',t.years >1?'s ':' '].join('');}
		if(t.days   ){str+= [t.days ,' ','day',t.days >1?'s ':' '].join('');}
		if(t.hours  ){str+= [t.hours ,' ','hour',t.hours >1?'s ':' '].join('');}
		if(t.minutes){str+= t.minutes + ' min ';}
		//if(t.seconds){str+= t.seconds + ' sec ';}
		return str;
	}
})
.filter('capitalize',function(){
	return function(c){
		  return (!!c) ? c.charAt(0).toUpperCase() + c.substr(1).toLowerCase() : '';
	}
});
