'use strict';

import angular from "angular";


import "../directives";

import "../Api";
import "../Token";
import "../../lib/angular-foundation.min";

export default angular.module('fastx.Share',[
'mm.foundation',
'Token',
'fastx.Api',
'fastx.Directives'
])
.factory('SharePostMessage',['$window',function($window){
	function uuid() {
		var d = new Date().getTime();
		var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function(c) {
			var r = (d + Math.random()*16)%16 | 0;
			d = Math.floor(d/16);
			return (c=='x' ? r : (r&0x3|0x8)).toString(16);
		});
		return uuid;
	};
	var s = {};
	var msgStore = {};
	$window.addEventListener('message',function(e){
		if(e.origin !== parent.location.origin){return;}
		var o = msgStore[e.data.msgId];
		delete msgStore[e.data.msgId];
		switch(e.data.action){
			case 'success':
				if(o.success){o.success(e.data.data);}
				break;
			case 'error':
				if(o.error){o.error(e.data.data);}
				break;
			default:
				break;
		}
	});
	function message(action,o,success,error){
		var uid = uuid();
		msgStore[uid] = {
			success:success,
			error:error
		};
		window.postMessage({
			action:action,
			data:o,
			msgId:uid
		},'*');
	}
	s.list = function(o,success,error){
		message('shareList',o,success,error);
	}
	s.update = function(o,success,error){
		message('shareUpdate',o,success,error);
	}
	s.remove = function(o,success,error){
		message('shareRemove',o,success,error);
	}
	s.reset = function(o,success,error){
		message('shareReset',o,success,error);
	}
	s.email = function(o,success,error){
		message('shareEmail',o,success,error);
	}
	return s;
}])
.factory('Share',['Api','Token',function(Api,Token){
	var s = {};
	var postMessage;
	s.list = function(o,success,error){
		Api.post('api/share/list',{},o,success,error)
	}
	s.update = function(o,success,error){
		Api.post('api/share/edit',{},o,success,error);
	}
	s.remove = function(o,success,error){
		Api.post('api/share/delete',{},o,success,error);
	}

	s.reset = function(o,success,error){
		Api.post('api/share/reset',{},o,success,error);
	}

	s.email = function(o,success,error){
		Api.post('api/share/email',{},o,success,error);
	}
	return s;
}])

.controller("ShareDialogCtrl",['$scope','Share','SharePostMessage',
			'id','cbOff','$location','postMessage',
	function($scope,share,sharePostMessage,
			 id,cbOff,$location,postMessage){
	var Share = postMessage?sharePostMessage:share;
	var users = [];
	$scope.users = [];
	$scope.granter = false;
	$scope.granterOpts = [
		{key:'Granter',value:true},
		{key:'Viewer',value:false}
	];
	$scope.advanced = false;

	$scope.keys = {};
	$scope.share = {
		viewers:{},
		granters:{}
	};
	$scope.session = {id:id};
	function error(e){
		$scope.text = e.error.message;
		$scope.submitClass = 'alert';
	}
	function filterKeys(d){
		$scope.text = '';
		var i;
		if(!$scope.link.type){
			$scope.link.type = '';
		}
		for(i=0;i<d.length;i++){
			if(d[i].perms.indexOf('granter') >=0){
				$scope.share.granters = d[i];
				$scope.share.granting = true;
				$scope.share.granters.url = createUrl({key:$scope.share.granters.key,id:id});
			}else{
				$scope.share.viewers = d[i];
				$scope.share.viewers.url = createUrl({key:$scope.share.viewers.key,id:id});
			}
		}
		$scope.keys = d;
	}
	Share.list({id:id},function(d){
		$scope.text ='';
		if(d.keys.length){
			$scope.share.sharing = true;
		}
		filterKeys(d.keys);
	},error);

	$scope.$watch('share.sharing',function(n,o){
// 		if(n === o){return;}
		if(n){
			if(!$scope.keys.length){
				Share.update({id:id,perms:['viewer']},function(d){
					filterKeys(d.keys);},error);
			}
		}else{
			$scope.share.granting = false;
			$scope.advanced = false;
			delete $scope.share.viewers;
			delete $scope.share.granters;
			delete $scope.link.type;
			Share.reset({id:id},function(d){filterKeys(d.keys);},error);

		}
	});
	$scope.$watch('share.granting',function(n,o){
		if(n === o){return;}
		if(n){
			if(!$scope.share.granters || !Object.keys($scope.share.granters).length){
				Share.update({id:id,perms:['granter']},function(d){
					filterKeys(d.keys);},error);
			}
		}else{
			delete $scope.share.granters;
			delete $scope.link.type;
			Share.remove({id:id,perms:'granter'},function(d){filterKeys(d.keys);},error);
		}
	});

	function createUrl(o){
		var uri = btoa(JSON.stringify(o));
		return [window.location.href.replace(/\/[^\/]+$/,""),'/share#',encodeURIComponent(uri)].join('');
	}

	$scope.link ={
		type: ''
	}
	$scope.email = function(users,link){
		var emails = [];
		if(!users.length){return;}
		users.forEach(function(c,i,a){
			emails.push(c.text);
		});
		Share.email({
			emails:emails,
			url:(link === 'granter')?$scope.share.granters.url:$scope.share.viewers.url
		},function(){
			$scope.users =[];
			$scope.text = '';
		},function(e){
			$scope.text = e.error.message;
			$scope.submitClass = 'alert';
		});
	}
	$scope.$watch('link.type',function(n,o){
		if(n === o){return;}
		if(n === '' && $scope.share.viewers){
			$scope.link.url = $scope.share.viewers.url;
		}else if(n === 'granter' && $scope.share.granters){
			$scope.link.url = $scope.share.granters.url;
		}
	});
}])
