'use strict';
import angular from "angular";

import "angular-xeditable";

import "../../../common/filters";
import "../../../common/command-module";
import "./screenshot";
import "./sharing";

import "../../../lib/angular-foundation.min";
import "../../../lib/angular-components/angular-resizable.min";

export default angular.module('fastx.session-info',[
	'fastx.command-module',
	'angularResizable',
	'fastx.Filters',
	'xeditable',
	'fastx.session-info-screenshot',
	'fastx.session-info-sharing',
	'mm.foundation'
])
.controller('SessionInfoCtrl',['$scope','CommandModule','$modal',function($scope,CommandModule,$modal){

	$scope.elapsed = function(t){
		return Date.now() - Date.parse(t);
	}
	$scope.showLastDisconnected = function(lastDisconnect){
		var e = $scope.elapsed(lastDisconnect);
		return (e < 120000);
	}

	$scope.disconnect = function(id){
		CommandModule.disconnect({id:id},function(){
		},function(e){
			//SiteUtils.errorDialog($scope,e.error);
		});
	}
	$scope.addTime = function (date,ms) {
		return (new Date(date)).getTime() + ms;
	}
	$scope.timeFormat = function (min) {
		var str = [];
		var d = Math.floor(min/1440);
		var h;
		var m = min%60;

		if(d) {
			str.push([d,'d'].join(''));
			min -= d*1440;
		}
		h = Math.floor(min/60);
		if (h) {
			str.push([h,'h'].join(''));
		}
		if (m) {
			str.push([m,'m'].join(''));
		}
		return str.join(', ');
	}

}])
.controller('ParamsCtrl',['$scope','CommandModule',function($scope,CommandModule){
	var keyTitle = function(i){
		return i.charAt(0).toUpperCase() + i.slice(1).split(/([A-Z]+[^A-Z]*)/).join(" ");
	}

	$scope.keyTitle = keyTitle;


	$scope.reset = function(){
		$scope.newParam = {
			error:'',
			key:'',
			value:''
		}
	}
	$scope.addParam = function(k,v,selected){
		$scope.newParam.error ='';
		if(["name","clientVersion","clientTransport"].indexOf(k) >= 0){
			$scope.newParam.error ='Key name is reserved';
			return false
		}

		var p = {
			id:selected.id,
			params:{}
		};
		p.params[k] = v;
		CommandModule.editSession(p,function(){

		},function(e){
			$scope.newParam.error = e.message;
		});
		return true;
	}
	$scope.setParam = function(k,$data,selected){
		var p = {
			id:selected.id,
			params:{}
		};
		p.params[k]= $data?$data:"";
		CommandModule.editSession(p);
	}
}])
.controller('NameCtrl',['$scope','CommandModule',function($scope,CommandModule){
	$scope.editName = function(selected,$data){
		CommandModule.editSession({
				id:selected.data.id,
				params:{
					name:($data)?$data:''
				}
			},
			function(){
				//$timeout($scope.getSessions,5000);
			},function(e){
				return e.message;
			});
	}
}])
.controller('LogCtrl',['$scope','$modal',function($scope,$modal){
    $scope.viewLogDialog = function(session){
        var mi = $modal.open({
            controller:'ViewXrdpLogsDialogCtrl',
            templateUrl:'js/fastx/session/info/view-log-dialog.html',
            size:'small',
            resolve:{
                session:function(){return session;}
            }
        });
        mi.result.then(function(a){console.log('dd',a);},function(b){console.log('rrr',b);});
    }
}])
.controller('ViewXrdpLogsDialogCtrl',['$scope','CommandModule','session',function($scope,CommandModule,session){
    $scope.refresh = function(){
        CommandModule.log({
            id:session.data.id,
        },function(d){

            $scope.logs = Object.keys(d).map(function(c,i){
                return {v:c,k:d[c]};
            });
            $scope.log = $scope.logs[0].k;
        },function(e){
            $scope.alert ={type:'alert',msg: e.error.message,close:function(){delete $scope.alert;}};
        });
    }
    $scope.refresh();
}])
;
