'use strict';
import angular from "angular";

import "../../user-menu/bookmark";
import "../../../common/command-module";
import "../../../common/utils";

import "../../../lib/angular-foundation.min";

export default angular.module('fastx.start-session',[
	'fastx.Bookmark',
	'mm.foundation',
	'fastx.utils',
	'fastx.command-module'])
.controller('StartSessionCtrl',['$scope','$modal',function($scope,$modal){
	$scope.newSession  = function(){
		var mi = $modal.open({
			templateUrl:'js/fastx/session/start/start.html',
			controller:'StartSessionDialogCtrl',
			size:'large'
			//windowClass:'start-dialog',
		});
		mi.result.then(function(){},function(){});
	}
}])
.controller('StartBookmarkCtrl',['$scope','Bookmark',function($scope,Bookmark){
	var autocompleteSource = [];
	$scope.bookmarks = {};
	$scope.tags = [];
	$scope.selectedTag;
	$scope.tagFilter = function (c) {
			return c.tags.indexOf ($scope.selectedTag.key) >= 0;
	}
	Bookmark.list(function(data){
		data = Object.assign ({
			suggestions: [],
			bookmarks: [],
			history: []
		}, data);

		var globalLabel = 'Global Bookmarks';
		var myLabel = 'My Bookmarks';
		data.suggestions = data.suggestions.map (function (c) {
			c.name = c.name || c.command;
			c.tags = c.tags || [];
			c.tags.push (globalLabel);
			return c;
		});

		data.bookmarks = data.bookmarks.map (function (c) {
			c.name = c.name || c.command;
			c.tags = c.tags || [];
			c.tags.push (myLabel);
			return c;
		});

		var bookmarks = data.suggestions.concat(data.bookmarks);
		for (var i in bookmarks) {
			bookmarks [i].tags.forEach (function (c) {
				if ([ globalLabel, myLabel ].indexOf (c) >= 0) { return; }
				if(!$scope.tags.find(function (t) {
					return t.key === c;
				}) ) {
					$scope.tags.push ({
						key: c,
						value: c
					});
				}
			});
		}
		$scope.tags.sort (function (a,b) {
			if (a.key > b.key) {
				return 1;
			}
			if (a.key < b.key) {
				return -1;
			}
			if (a.key === b.key) {
				return 0;
			}
		});



		$scope.bookmarks.bookmarks = bookmarks;
		$scope.tags.push ({ key: globalLabel, value: globalLabel });
		$scope.tags.push ({ key: myLabel, value: myLabel });
		$scope.selectedTag = $scope.tags [0];


		if (data.history.length) {
			data.history.forEach(function(c){
				autocompleteSource.push({
					label:c.name || c.command,
					value:c.command
				});
			});
		}
		$scope.bookmarks.history = data.history;

		$('#startCommand').autocomplete({
			source:autocompleteSource,
			classes:'autocomplete',
			minLength:0,
			change:function(event,ui){
				if(ui.item){
					$scope.$apply(function(){
							$scope.params.command = ui.item.value;
							$scope.params.name = ui.item.label;
					});
				}
			}
		}).focus(function(){
        //Use the below line instead of triggering keydown
        $(this).autocomplete("search","");
		});
	},function(e){
		$scope.bookmarks = {};
		$scope.alert = {type:'alert',msg:e.error.message, close:function(){delete $scope.alert;}};
		console.log('Cannot load bookmarks', e);
	});
}])
.controller('StartSessionDialogCtrl',['$scope','$modalInstance','$modal',
			'SiteUtils','CommandModule','Bookmark','$window','$timeout','$rootScope','GoToSession',
	function($scope,$modalInstance,$modal,SiteUtils,CommandModule,Bookmark,$window,$timeout,$rootScope,GoToSession){
	$scope.params = {
		command:'',
		geometry:'',
		name:'',
		profile:''
	}
	$scope.lastCommand = '';

	$scope.profiles = [];
	CommandModule.getProfiles(function(p){
		$scope.profiles = p;
	},function(e){
		$scope.alert = {type:'alert',msg:e.message};
	});


	$scope.$watchCollection('params',function(n,o){
		if(n === o){return;}
		if(n.geometry !== o.geometry) {
			$scope.params.windowMode = (n.geometry === 'rootless')?"rootless":"";
		}

		if (n.id === o.id) {
			$scope.active = null
		}
	});
	$scope.updateCommand = function(o){
		$scope.params = Object.assign({},o);

		if(o.geometry === 'rootless'){
			$scope.params.windowMode = 'rootless';
		}else{
			$scope.params.windowMode = '';
		}
		if(o.windowMode === 'rootless'){
			$scope.params.geometry = 'rootless';
		}
		$scope.active = o.id;
	}
	$scope.clearActive = function(a){
		delete $scope.params.name;
		delete $scope.params.profile;
		$scope.active ='';
	}

	function startParams(p,active){
		if( p.windowMode === 'rootless'){
			p.geometry = 'rootless';
		}else{
			p.geometry = ($window.innerWidth).toString() + 'x'  + ($window.innerHeight).toString();
		}
		return {
			command:p.command,
			geometry:p.geometry,
			name:p.name,
			profile:p.profile,
			params:{
				shortcut:active || undefined,
				clientVersion:'web-' + 	$window.localStorage['version']
			}
		};
	}

	$scope.scheduleSession = function(params,active){
		$scope.disabled = true;
		params = startParams(params,active);
		$modalInstance.close();
			var mi = $modal.open({
					templateUrl:'js/fastx/session/start/schedule.html',
					controller:'ScheduleStartDialogCtrl',
					resolve:{
						params:function(){
							return params;
						}
					}
			});
		mi.result.then(function(){},function(){});
	}
	$scope.startSession = function(params,active){
		$scope.disabled = true;
		params = startParams(params,active);
		CommandModule.start(params,function(data){
			$window.localStorage[data.id] = JSON.stringify(data);

			$scope.id = data.id;
			$timeout(function(){
				$modalInstance.close();
				GoToSession($scope.id);
			},1200);//add a delay for session to start
			$scope.disabled = false;
		},function(e){
			$scope.id = e.id;
			$scope.alert = { type:'alert',msg:e.error.message,close:function(){delete $scope.alert;} };
			$scope.disabled = false;
		});
	}
}])
.controller('ScheduleStartDialogCtrl',['$scope','$modalInstance','CommandModule','params',function($scope,$modalInstance,CommandModule,params){
	$scope.data = {}
	CommandModule.schedule(params,function(data){
		$scope.data = data;
	},function(e){
		$scope.data.stdout = '';
		$scope.data.stderr = e.error.message;
	});
	$scope.close = $modalInstance.close;
}])
.controller('ViewLogCtrl',['$scope','$modal',function($scope,$modal){
	$scope.viewLog = function(session,log){
		var mi = $modal.open({
			templateUrl:'js/fastx/session/view-log-dialog.html',
			controller:'ViewLogDialogCtrl',
			scope:$scope
		});
		mi.result.then(function(){},function(){});
	}
}])
