"use strict";

import angular from "angular";

import "../common/directives";
import "../common/utils";
import "../common/connection/util";
import "../common/command-module";

angular.module('fastx.connect.share',[
	'fastx.utils',
	'fastx.command-module',
	'fastx.connection-util',
	'fastx.Directives'
])
.run(['$window','ClosedDialog',function($window,ClosedDialog){
	$window.addEventListener('message',function(e){
		switch(e.data.type){
			case 'websocket-closed':
				ClosedDialog(e.data);
				break;
			default:
		}
	});
}])
.config(['$routeProvider','$locationProvider',function($routeProvider, $locationProvider) {
    $routeProvider.otherwise({
	//	templateUrl:'/js/share/index.html'
	});
	$locationProvider.html5Mode({
		enabled: true,
		requireBase: false
	});
}])
.factory('Share',['$window','$q','Api','NameDialog','$location',function($window,$q,Api,NameDialog,$location){
	var f = {};

	/* returns a promise with the data; on reject, returns id if exists ****/
	f.getShareQuery = function(){
		var q = $q.defer();
		var hash = $location.hash();

		try{
			if(!hash.length){
				$q.reject(new Error('Invalid Hash string'));
			}
			var fastx =  JSON.parse(atob(hash));
			if(typeof fastx !== 'object'){
				q.reject({message:'Hash string is invalid'});
			}
			q.resolve(Object.assign(fastx,$location.search()));
		}catch(e){
			q.reject({message:'Could not parse hash string'});
		}
		return q.promise;
	}

	f.share = function(o){
		var q = $q.defer();
		f.getShareQuery().then(function(o){
			var n = NameDialog('sharename');
			n.then(function(username){
				Api.post('api/connect/share',{},o,function(fastx){
					try{
						fastx.username = username;
						fastx.id = o.id;
						q.resolve(fastx);
					}catch(e){
						q.reject(e);
					}
				},q.reject);
			},q.reject);
		},q.reject);
		return q.promise;
	}

	return f;
}])
.directive('fxShare',['Share','ClosedDialog','$window',function(Share,ClosedDialog,$window,fastx){
	function link($scope,$element,$attrs){
		Share.share().then(function(o){
			var iframe = document.createElement('iframe');
			iframe.src = '/xrdp' + $window.location.search + '#' + btoa(JSON.stringify(o));
			iframe.setAttribute('style','margin:0px;vertical-align:bottom;display:block;width:100%;border:0px none transparent;padding 0px;overflow:hidden');
			iframe.name = 'xrdp';
			iframe.width = $window.innerWidth;
			iframe.height = $window.innerHeight;
			iframe.setAttribute('seamless','seamless');
			$window.addEventListener('resize',function(e){
				iframe.width = $window.innerWidth;
				iframe.height = $window.innerHeight;
			});
			$element.html(iframe);
		},ClosedDialog)
	}
	return {
		link:link
	}
}])
.controller('AppCtrl',['$scope','Connect','ClosedDialog',function($scope,Connect,ClosedDialog){

}]);
angular.element(document).ready(function() {
	angular.bootstrap(document, ['fastx.connect.share']);
});
