"use strict";

import "../common/directives";
import "../common/utils";
import "../common/connection/util";
import '../common/share/share'
import '../fastx/user-menu/bookmark.js'
import "../common/command-module";

import "../lib/angular-foundation.min";

angular.module('fastx.shortcut',[
	'fastx.utils',
	'fastx.command-module',
	'fastx.connection-util',
	'fastx.Directives',
	'fastx.Share',
	'fastx.Bookmark',
	"mm.foundation"
])
.run(['$window','ClosedDialog',function($window,ClosedDialog){
	$window.addEventListener('message',function(e){
		switch(e.data.type){
			case 'websocket-closed':
				ClosedDialog(e.data);
				break;
			default:
		}
	});
}])
.config(['$routeProvider','$locationProvider',function($routeProvider, $locationProvider) {
  $routeProvider
   .otherwise({
	   templateUrl:'js/shortcut/shortcut.html'
	});
   $locationProvider.html5Mode({
		enabled: true,
		requireBase: false
	});
}])
.factory('ShareMessages',['$window','Share',function($window,Share,ClosedDialog){
	var origin,iwin;
	function success(msgId,d){
		iwin.postMessage({
			action:'success',
			msgId:msgId,
			data:d
		},origin);
	}
	function error(msgId,d){
		iwin.postMessage({
			action:'error',
			msgId:msgId,
			data:d
		},origin);
	}
	$window.addEventListener('message',function(e){
		//if(e.origin !== origin){return;}
		switch(e.data.action){
			case 'shareList':
				Share.list(e.data.data,
					function(d){success(e.data.msgId,d)},
					function(e){error(e.data.msgId,e)}
					  );
				break;
			case 'shareUpdate':
				Share.update(e.data.data,
					function(d){success(e.data.msgId,d)},
					function(e){error(e.data.msgId,e)}
					  );
				break;
			case 'shareRemove':
				Share.remove(e.data.data,
					function(d){success(e.data.msgId,d)},
					function(e){error(e.data.msgId,e)}
					  );
				break;
			case 'shareReset':
				Share.reset(e.data.data,
					function(d){success(e.data.msgId,d)},
					function(e){error(e.data.msgId,e)}
				);
				break;
			case 'shareEmail':
				Share.email(e.data.data,
					function(d){success(e.data.msgId,d)},
					function(e){error(e.data.msgId,e)}
				);
				break;
			default:
				break;
		}
	});
	function m(o,win){
		origin = o;
		iwin = win;
	}
	return m;
}])
.factory('checkLogin',['Token','$interval','$q','$window',function(Token,$interval,$q,$window){
	return function(){
		var q = $q.defer();
		if(!Token.token){
			$window.location.href = 'auth?redirectUrl=' + encodeURIComponent(location.href);
			q.reject();
		}else{
			$interval(function(){
				Token.refresh();
			},60000);
			Token.refresh(q.resolve,q.reject);
		}
		return q.promise;
	}
}])

.factory('Connect',['$window','$q','Api','NameDialog','checkLogin','$location','Token','$http','$modal',
		 function($window,$q,Api,NameDialog,checkLogin,$location,Token,$http,$modal){
	var f = {};
	var v;

	/* returns a promise with the data; on reject, returns id if exists ****/
	f.getQuery = function(){
		var q = $q.defer();
		var hash = $location.hash();
		if(!hash.length){
			q.reject();
		}else{
			q.resolve({id:hash});
		}
		return q.promise;
	}
	function shortcut(o,resolve,reject){
		console.log('Making API connection');
		Api.post('api/connect/shortcut',{},{
			id:o.id,
			params:{
				clientVersion:($window.localStorage['version'])?'web-' + $window.localStorage['version']:undefined
			}
		},function(fastx){
			document.getElementsByTagName("title")[0].innerHTML = o.params?(o.params.name || o.id):o.id;
			fastx.username = fastx.login;
			fastx.id = o.id;
			if(fastx.baseUrl){
				var h = $http({
					method:'HEAD',
					url:fastx.baseUrl + '/xrdp'
				});
				h.then(function(res){
					resolve(fastx);
				},function(e){
					delete fastx.baseUrl;
					resolve(fastx);
				});
			}else{
				resolve(fastx);
			}
		},reject);
	}
	f.connect = function(){
		var q = $q.defer();
		var c = checkLogin();
		c.then(function(){
			f.getQuery().then(function(o){
				shortcut(o,q.resolve,q.reject);
			}).catch(function(e){
				var mi = $modal.open({
					controller:"BookmarkShortcutCtrl",
					templateUrl:"js/shortcut/bookmark.html"
				});
			});
		});
		c.catch(function(e){
			$window.location.href = 'auth?redirectUrl=' + encodeURIComponent(location.href);
		});
		return q.promise;
	}
	return f;
}])
.directive('fxConnect',['Connect','ClosedDialog','$window','ShareMessages',
		   function(Connect,ClosedDialog,$window,ShareMessages){
	function link($scope,$element,$attrs){
		Connect.connect().then(function(o){
			console.log('Creating iframe');
			var iframe = document.createElement('iframe');
			iframe.src = (o.baseUrl?o.baseUrl +'/':'' ) + 'xrdp' + $window.location.search + '#' + btoa(JSON.stringify(o));
			iframe.setAttribute('style','margin:0px;vertical-align:bottom;display:block;width:100%;border:0px none transparent;padding 0px;overflow:hidden');
			iframe.name = 'xrdp';
			iframe.width = $window.innerWidth;
			iframe.height = $window.innerHeight ;
			iframe.setAttribute('seamless','seamless');
			$element.html(iframe);
			$window.addEventListener('resize',function(e){
				iframe.width = $window.innerWidth;
				iframe.height = $window.innerHeight ;
			});
			ShareMessages(o.baseUrl,iframe.contentWindow);
		},ClosedDialog);
	}
	return {
		link:link
	}
}])
.controller('AppCtrl',['$scope','Connect','ClosedDialog',function($scope,Connect,ClosedDialog){

}])
.controller('BookmarkShortcutCtrl',['$scope', 'Bookmark', '$modalInstance', function($scope, Bookmark, $modalInstance) {
	Bookmark.list(function (b) {
		$scope.bookmarks = b.suggestions
	},$modalInstance.dismiss);
}])
.controller('ConnectionErrorCtrl',['$scope','CommandModule','$window','$rootScope',function($scope,CommandModule,$window,$rootScope){
	//$scope.canReconnect = ([-32007].indexOf($scope.error.code) < 0);
	$scope.reconnect = function(){
		var cb = function(e){$window.location.reload();}
		CommandModule.disconnect({id:$rootScope.id},cb,cb);
	}
	$scope.close = ($window.opener)?function(){
		$window.close();
	}:null;
}]);
angular.element(document).ready(function() {
	angular.bootstrap(document, ['fastx.shortcut']);
});
