'use strict';
import angular from "angular";
import "../../lib/angular-foundation.min";

import "ng-tags-input";

import "../../common/Api";
import "../../common/Token";
import "../../common/utils";

export default angular.module('System.Network',[
	'fastx.Api',
	'Token',
	'fastx.utils',
	'mm.foundation',
	'ngTagsInput'
])
.factory('System.Network',['Api','Token',function(Api,Token){
	var n = {};
	n.getNetworkConfig = function(success,error){
		Api.get('system/network',{},success,error);
	}
	n.setNetworkConfig = function(o,success,error){
		Api.post('system/network',{},o,success,error);
	}
	return n;
}])
.controller('NetworkConfigCtrl',['$scope','System.Network','$window','RandomString',function($scope,Network,$window,RandomString){
	$scope.isAdmin = true;
	($scope.getConfig = function(){
		Network.getNetworkConfig(function(d){
			$scope.network = angular.extend({
				loadBalance:{
					metric:'',
					desc:false
				},
				serverCN:$window.location.hostname.replace(/.*?\./,'*.'),
				keys:{},
				rejectUnauthorized:false,
				enableDirectMode:false,
			},d);
			if(!Object.keys($scope.network.keys).length){
				$scope.network.keys[RandomString(32)] = {
					disableCluster:true
				};
			}

			for(var i in $scope.network.keys){
				$scope.network.keys[i].id = i;
				if(!$scope.network.keys[i].hasOwnProperty('disableCluster')){
					$scope.network.keys[i].disableCluster = false;
				}

				$scope.network.keys[i].urls = $scope.network.keys[i].urls?$scope.network.keys[i].urls.filter(function(c){
					return c.url;
				}):[];
			}
			if(typeof $scope.network.loadBalance === 'string'){
				$scope.network.loadBalance = {
					metric:$scope.network.loadBalance,
					desc:false
				}
			}
			$scope.v = {
				lb:''
			}
			$scope.v.lb = (['sessions','random','local',''].indexOf(
				$scope.network.loadBalance.metric) >=0)?$scope.network.loadBalance.metric:'custom';
			$scope.$watch('v.lb',function(n,o){
				if(n===o){return;}
				switch(n){
					case '':
					case 'sessions':
					case 'random':
					case 'local':
						$scope.network.loadBalance.metric =n;
						$scope.network.loadBalance.desc = false;
						break;
					case 'custom':
						if(['sessions','random','local',''].indexOf(o) >=0){
							$scope.network.loadBalance.metric ='';
							$scope.network.loadBalance.desc = false;
						}
						break;
				}
			});
		});
	})();

	$scope.submit = function(c){
		var t,id,urls,urlNames;
		for(var i in c.keys){
			if(!i){
				delete c.keys[i];
				continue;
			}
			if(c.keys[i].urls){
				urls = [],urlNames=[];
				c.keys[i].urls.forEach(function(my){
					if(urlNames.indexOf(my.url) < 0){
						urlNames.push(my.url);
						urls.push(my);
					}
				});
				c.keys[i].urls = urls;
			}
			if(c.keys[i].id === i){
				delete c.keys[i].id;
			}else{
				id = c.keys[i].id;
				delete c.keys[i].id;
				t = c.keys[i];
				delete c.keys[i];
				c.keys[id] = t;
			}
		}

		Network.setNetworkConfig(c,function(d){
			$scope.alert ={type:'success',msg:'Changes Saved Successfully',close:function(){delete $scope.alert;}};
			$scope.getConfig();
		},function(e){
			$scope.alert ={type:'alert',msg: e.error.message,close:function(){delete $scope.alert;}};
			//$scope.getConfig();
		});
	}
}])
.controller('ServerKeysCtrl',['$scope','RandomString','$window',function($scope,RandomString,$window){
	$scope.hasKeys = function(){
		if($scope.network && $scope.network.keys){
			return Object.keys($scope.network.keys).length;
		}
	}
	$scope.addKey = function(){
		$scope.network.keys[RandomString(32)] = {
			allowIncomingConnections:false,
			urls:[]
		}
	}
	$scope.addUrl = function(index){
		if(!$scope.network.keys[index].urls){
			$scope.network.keys[index].urls = [];
		}
		if(!$scope.network.keys[index].urls.length){
			$scope.network.keys[index].urls.push({
				url:[$window.location.protocol,'//',$window.location.host].join('')
			});
		}else{
			$scope.network.keys[index].urls.push({url:''});
		}

	}

	$scope.deleteUrl = function(index,$index){
		$scope.network.keys[index].urls.splice($index,1);
	}

	if(!$scope.hasKeys() && $scope.network){
		$scope.network.keys = {};
		$scope.addKey();
	}

}])
.controller('ServerKeyCtrl',['$scope',function($scope){
	$scope.deleteKey = function(id){
		delete $scope.network.keys[id];
	}
	$scope.$watch('key.id',function(n,o){
		//console.log(n,o);
	});
}])
.controller('SSLCtrl',['$scope','$window',function($scope,$window){
	$scope.certOptions = [
		{key:'Accept self-signed certificates',value:false},
		{key:'Accept SSL certificate is from trusted Certificate Authorities',value:true},
	];
	$scope.$watch('network.rejectUnauthorized',function(n,o){
		if(n === o){return;}
		if(!n){
			delete $scope.network.serverCN;
		}else{
			$scope.network.serverCN = $window.location.hostname.replace(/.*?\./,'*.')
		}
	});
}])
.controller('LoadBalanceCtrl',['$scope','$modal',function($scope,$modal){
	$scope.sudoDialog = function(n){
		if(n){
			$modal.open({
				controller:'SudoDialogCtrl',
				templateUrl:'js/system/system-config/sudo-dialog.html',
				windowClass:'big-modal'
			});
		}
	}
	$scope.lbOptions = [
		{k:'Select Server With the Highest Value',v:false},
		{k:'Select Server With the Lowest Value',v:true}
	];

}])
.controller('SudoDialogCtrl',[function(){

}]);
