
'use strict';
import angular from "angular";

import "../../lib/angular-foundation.min";

import "../../common/Api";
import "../../common/Token";

import "./command";
import "./session";

export default angular.module('System.Profile',[
	'fastx.Api',
	'Token',
	'mm.foundation',
	'System.Session',
	'System.Command'
])
.factory('System.Profile',['Api','Token',function(Api,Token){
	var a = {};
	a.edit = function(o,success,error){
		Api.post('system/session/profile/edit',{},o,success,error);
	}
	a.remove = function(o,success,error){
		Api.post('system/session/profile/delete',{},o,success,error);
	}
	a.get = function(success,error){
		Api.get('system/session/profile',{},success,error);
	}
	a.getDefault = function(success,error){
		Api.get('system/session/profile/default',{},success,error);
	}
	a.setDefault = function(o,success,error){
		Api.post('system/session/profile/default',{},o,success,error);
	}
	a.set = function(o,success,error){
		Api.post('system/session/profile',{},o,success,error);
	}
	a.getUserData = function(success,error){
		Api.get('api/session/profile',{},success,error);
	}
	return a;
}])
.controller('ProfileDeleteDialogCtrl',['$scope','$modalInstance',function($scope,$mi){
	$scope.close = function(){
		$mi.close();
	}
}])
.controller('ProfileDialogCtrl',['$scope','$modalInstance','profile','System.Profile',function($scope,$mi,profile,Profile){
	$scope.scheduleOpen = profile.schedule.script && profile.schedule.script.length;
	$scope.profile = profile;
	$scope.userData = Object.keys($scope.profile.schedule.userData || {}).map(function(c,i,a){
		var s = $scope.profile.schedule.userData[c];
		return {
			name:c,
			type:s.type,
			init:s.init,
			values:s.values
		}
	});
	$scope.save = function(p,userData){
		var ud = {};
		userData.forEach(function(c,i,a){
			ud[c.name] = Object.assign({},c);
			delete ud[c.name].name;
		});
		p.schedule.userData = ud;
		if(p.isDefault){
			Profile.setDefault(p,function(d){
				$mi.close();
			},function(e){
				$scope.alert ={type:'alert',msg: e.error.message,close:function(){delete $scope.alert;}};
			});
		}else{
			Profile.edit(p,function(d){
				$mi.close(d);
			},function(e){
				$scope.alert ={type:'alert',msg: e.error.message,close:function(){delete $scope.alert;}};
			});
		}
	}
	$scope.logLevels = [
            {k:'Fatal',v:1},
            {k:'Error',v:2},
            {k:'Warn',v:3},
            {k:'Info',v:4},
            {k:'Trance',v:5},
            {k:'Debug',v:6},
            {k:'Verbose',v:7},
        ];

}])
.controller('SystemProfileCtrl',['$scope','System.Profile','$modal',function($scope,Profile,$modal){
	Profile.getDefault(function(p){
		p.isDefault = true;
		p.name = 'Default Profile';
		p.description = 'Default Profile';
		$scope.default = p;
	},function(e){

	});
	($scope.getProfiles = function(){
		Profile.get(function(p){
			$scope.profiles = p;
		},function(e){
			$scope.alert ={type:'alert',msg: e.error.message,close:function(){delete $scope.alert;}};
		});
	})();

	$scope.add = function(p){
		p = p ||{};
		$scope.edit(Object.assign({},p,{name:'',description:'',id:'',isDefault:false}));
	}

	$scope.remove = function(p){
		var $mi = $modal.open({
			templateUrl:'js/system/profile/profile-delete-dialog.html',
			controller:'ProfileDeleteDialogCtrl',
			size:'tiny',
		});
		$mi.result.then(function(np){
			Profile.remove({id:p.name},function(p){
				$scope.profiles = p;
			},function(){
				$scope.alert ={type:'alert',msg: e.error.message,close:function(){delete $scope.alert;}};
			});
		},function(){});
	}
	$scope.edit = function(p){
		var $mi = $modal.open({
			templateUrl:'js/system/profile/profile-dialog.html',
			controller:'ProfileDialogCtrl',
			windowClass:'profile-modal',
			resolve:{
				profile:function(){
					return Object.assign({
						name:'',
						description:'',
						session:{
							"clipmgr":true,
							"sharing":true,
							"offlineSharing":false,
							"iglx":false,
							"listenTCP":false,
							"videoRam":0x20000,
							"cpuPercentage":50,
							"disconnectTimeout":0,
							"maxSessions":0,
                                                        "logVerbose":3
						},
						schedule:{
							script:"",
							userData:{}
						}
					},p);
				}
			}
		});
		$mi.result.then(function(p){
			if(p){
				$scope.profiles = p;
			}
		},function(){});
	}
}])
.controller('ScheduleCtrl',['$scope',function($scope){
	$scope.addUserData = function(userData){
		userData.push({
			name:'',
			type:'',
			init:null,
			value:[]
		});
	}
}]);
