'use strict'


import angular from 'angular';

import '../../lib/foundation/foundation.js';
import '../../lib/foundation/foundation.slider.js';

import "../../lib/angular-foundation.min";
import "../../common/utils";



export default angular.module('fastx.Cfg',['mm.foundation','fastx.utils'])
.factory('UpdateEvent',['$window','Api',function($window,Api){
	var defaults;
	Api.get('api/xrdp/config',{},function(c){
		defaults = c.defaults;
	})
	var cfgUpdateEvent = function (e){};
	function initialCfgUpdateEvent(e){
		var local = {};
		try{
			local = JSON.parse($window.localStorage['snCfg']);
		}catch(e){}
		var cfg = {
			FPSLimitMax:60,
			CompressionMax:7,
			BandwidthThrottleAdjust:true
		};
		angular.extend(cfg,$window.FastXDefaults);
		angular.extend(cfg,local);
		if(!cfg.BandwidthThrottleAdjust){
			document.dispatchEvent(new CustomEvent('snSendSetConfigOptionsRequest',{detail:cfg}));
		}
		document.removeEventListener('snCfgUpdate',initialCfgUpdateEvent);
		document.addEventListener('snCfgUpdate',cfgUpdateEvent);
	}
	document.addEventListener('snCfgUpdate',initialCfgUpdateEvent,{once:true});
	document.addEventListener('snCfgResponse',function(e){
	});

	return {
		defaults:function(){
			return defaults
		},
		update:function UpdateEvent(cb){
			document.removeEventListener('snCfgUpdate',cfgUpdateEvent);
			cfgUpdateEvent = cb;
			document.addEventListener('snCfgUpdate',cfgUpdateEvent);
		}
	}
}])
.run(['$window','UpdateEvent',function($window,UpdateEvent){

}])
.factory('WebGLSupport',[function(){
	function webgl_support() {
		return false;
		try{
			var canvas = document.createElement( 'canvas' );
			return !! window.WebGLRenderingContext && (
				canvas.getContext( 'webgl' ) || canvas.getContext( 'experimental-webgl' ) );
		}catch( e ) { return false; }
	};
	return webgl_support;
}])
.controller('CfgCtrl',['$rootScope','$scope','$modal','Cfg',function($rootScope,$scope,$modal,Cfg){
	$scope.a = 1;
	$scope.cfg = function(){
		$rootScope.cb.disableInput(true);
	}

}])
.controller('CfgDialogCtrl',['$scope','$rootScope','WebGLSupport','Cfg','UpdateEvent','$window',
			function($scope,$rootScope,WebGLSupport,Cfg,UpdateEvent,$window){
	$scope.webglsupport = WebGLSupport();
	$scope.Cfg = new Cfg();
	$scope.minF = 1;
	$scope.maxF =  $scope.Cfg.FPSLimitMax;

	$scope.levels = ['Lowest', 'Lower','Low','Medium','Medium High','High','Higher','Highest'];

	$scope.minC = 0;
	$scope.maxC = $scope.levels.length - 1;
	$scope.tC = function(n){
		return $scope.maxC - n;
	}
	//FrameAcknowledgeWindow
	$scope.minW = 1;
	$scope.maxW = 60;
	$scope.tF = function(n){
		if(n===0){return 0;}
		return n;
	}

	$scope.minM = 1;
	$scope.maxM = 200;
	$scope.tM = function(n){
		return n;
	}
	$scope.defaults = UpdateEvent.defaults();

	UpdateEvent.update(function(e){
		$scope.$apply(function(){
			angular.extend($scope.Cfg,e.detail);
		});
	});

	$scope.changeDecoder = function(d){
		document.dispatchEvent(new CustomEvent("fxDecoder",{
			detail:d?"WASM":"JS"
		}));
		$window.localStorage["decoder"] = d?"WASM":"JS";
	}
	if("WebAssembly" in $window){
		$scope.wasm = true;
		$scope.decoder = ($window.localStorage["decoder"] === "WASM");
		$scope.changeDecoder($scope.decoder);
	}
	$scope.changeDecoder($scope.decoder || false);

	$scope.setDefault =function(n){
		$scope.defaultSettings = n;
		$scope.Cfg.BandwidthThrottleAdjust = (n ==='auto');
		if(!$scope.Cfg.BandwidthThrottleAdjust && n !=='custom'){
			$scope.Cfg.Compression = $scope.defaults[n].Compression;
			$scope.Cfg.FPSlimit = $scope.defaults[n].FPSLimit;
			$scope.Cfg.FrameAcknowledgeWindow = $scope.defaults[n].FrameAcknowledgeWindow;
			$scope.Cfg.MouseQuantum = $scope.defaults[n].MouseQuantum;
		}
		$scope.Cfg.setOptions($scope.Cfg);
		$window.localStorage['xrdpDefaults'] = n;
	}

	$scope.update = function(cfg,newVal){
		if($scope.disabled()){return;}

		if(!$scope.Cfg.BandwidthThrottleAdjust ){
			$scope.Cfg[cfg] = newVal;
			$scope.Cfg.setOptions($scope.Cfg);
		}
	}

	$scope.defaultSettings = $window.localStorage['xrdpDefaults'];
	if(isNaN($scope.defaultSettings)){
		$scope.defaultSettings =$window.localStorage['xrdpDefaults'] || 'auto';
	}
	$scope.disabled = function(){
		return $scope.defaultSettings !== 'custom';
	}
}])

.factory('Cfg',['$window','$rootScope',function($window,$rootScope){
	var cfg = function (){
		var c;
		try{
			c = JSON.parse($window.localStorage['snCfg']);
		}catch(e){
			c = {};
		}
		c = angular.extend({
			BandwidthThrottleAdjust:true,
			FPSLimitMax:60,
			FPSLimit:30,
			FrameAcknowledgeWindow:10,
			CompressionMax:7,
			WebGL:true
		},c);

		c.store = function(o){
			$window.localStorage['snCfg'] = JSON.stringify(angular.extend(c,o));
		}
		c.setOptions = function(o){
			var t = {
				Compression:0,
				FPSLimit:60,
				FrameAcknowledgeWindow:2,
				MouseQuantum:1
			};

			var i;
			try{
				i=Object.assign({},t,JSON.parse($window.localStorage['snCfg']));
			}catch(e){
				i=Object.assign({},t);
			}
			if(o){
				o = Object.assign({},t,o);
			}
			if(o.BandwidthThrottleAdjust){
				o.FrameAcknowledgeWindow = 2;
				o.MouseQuantum = 20;
			}
			if(	!o.BandwidthThrottleAdjust &&
				o.Compression &&
				o.Compression === i.Compression &&
				o.FPSLimit &&
				o.FPSLimit === i.FPSLimit &&
				o.FrameAcknowledgeWindow &&
				o.FrameAcknowledgeWindow === i.FrameAcknowledgeWindow &&
				o.MouseQuantum &&
				o.MouseQuantum === i.MouseQuantum){
					return;
				}
			document.dispatchEvent(new CustomEvent('snSendSetConfigOptionsRequest',{detail:o}));
			if($rootScope.cb && $rootScope.cb.setMouseMoveTimer){
				$rootScope.cb.setMouseMoveTimer(o.MouseQuantum);
			}
			c.store(o);
		}
		c.getOptions = function(){
			return c;
		}
		return c;
	}

	return cfg;
}])

.directive('slider',['$timeout',function($timeout){


	function link(scope,elem,attrs){
		var updating;
		/* moves he slider when the model updates */
		function updateView(n){
			var e = angular.element(elem);
			var px = Math.abs( (attrs['reverse']?e[0].clientWidth:0) - n/attrs['max'] * e[0].clientWidth);
			var a =angular.element(elem).children();
			(a.filter(function(c,i){
				return angular.element(a[c]).hasClass('range-slider-active-segment');
			})).css('width', px +'px');
		}


		scope.$watch(attrs['model'],function(n,o){
			if(n === o){return;}
			updateView(n);
			if(!updating){
				updating = true;
				$timeout(function(){
					scope.Cfg.setOptions(scope.Cfg);
					updating = false;
				},3000);
			}

		});

		/* updates the model */
		var fn = function(v){
			var t = function(e){return e;}

			if(attrs['translate']){
				t = scope[attrs['translate']];
			}
			//allow for objects with dot notation
			var str = 'scope' + (((attrs['model'].split('.')).map(function(o){
				return ['["',o,'"]'].join('');
			})).join('')) + ' = t(v) ;';
			eval(str);
		}
		var u;
		function updateModelStart(e){
			u = scope.defaultSettings ==='custom';
		}
		function updateModelEnd(e){
			u= false;

		}
		function updateModel(e){
			if(!u){return;}

			var val = Math.ceil(Math.round(100*(e.offsetX/angular.element(elem)[0].clientWidth),100));
			if(val > 100){val = 100;}
			$(e.target).find('input').val(val);
			var v  =  Math.ceil(Math.max(val/100*attrs['max'],attrs['min']));
            scope.$apply(function(){fn(v);});

			//$(handle).css('transform','translateX(' +  off  + 'px)' );
			//$( $(e.target).parent().find('.range-slider-active-segment')[0]).css('width',val + '%');

		}
		$(document).foundation();
		$(document).foundation('slider','reflow');
		$(elem).on('change.fndtn.slider',function(e){
			var phase = scope.$root.$$phase;
			var val = $(e.target).find('input').val();
			if(val === 'NaN'){
				return;
			}
            if (phase == '$apply' || phase == '$digest') {
             } else {
				 var v  =  Math.ceil(Math.max(val/100*attrs['max'],attrs['min']));
                 scope.$apply(function(){fn(v);});
             }
		});
		$(elem).on('mousedown',updateModelStart);
		$(elem).on('mousemove',updateModel);
		$(document).on('mouseup',updateModelEnd);
		$(elem).on('click',function(e){updateModelStart(e);updateModel(e);updateModelEnd(e)});
		$(elem).foundation('slider','set_value',attrs['init']);
	}
	return {
		link:link
	}
}]);
