'use strict'

export default FastXObject;

FastXObject.$inject = ['$window','$rootScope','TouchEvents','$modal','Clipboard','Rail', '$http'];

function FastXObject ($window,$rootScope,TouchEvents,$modal,Clipboard,Rail,$http) {
	/*defines the object that will be passed around*/
	function getKeyboardLayout(layout){
		var l = parseInt($window.localStorage['layout'],10) ||  layout;
		//dvorak - US keyboard hack
		if(!$window.KeyboardEvent.prototype.hasOwnProperty('code')){
			if([0x00010409].indexOf(l) >= 0){
				return 0x00000409;
			}
		}
		return l;
	}
	return new Promise (function (resolve, reject) {
	 $http.get ('/xrdp/config').then (function (d) {
		 if (d.status !== 200) {
			 reject (new Error (d.statusText));
		 }
		 if (!d.data.result) {
			 if (!d.data.error) {
				 reject (new Error ('Configuration not found'));
			 } else {
				 reject (new Error (d.data.error.message));
			 }
		 }
		var fastx ={};
		fastx.keyboardLayout = getKeyboardLayout(d.data.result.keyboard.layout);
		fastx.keyboardType = parseInt(window.localStorage['keyboardType'],10) ||  d.data.result.keyboard.layout;

		fastx.ready = function(canvas,cb){
			$rootScope.cb = cb;
			$rootScope.canvas = canvas;

			/**************** Callbacks *********/
			document.addEventListener('snSendSetConfigOptionsRequest',function(e){
				delete e.detail.ScreenHeight;
				delete e.detail.ScreenWidth;
				if(e.detail.BandwidthThrottleAdjust){
					delete e.detail.Compression;
					delete e.detail.FPSLimit;

				}else{
					delete e.detail.BandwidthThrottle;
				}
				if(e.detail.MouseQuantum){
					cb.setMouseMoveTimer(e.detail.MouseQuantum);
				}
				try{
					var c = JSON.parse($window.localStorage['snCfg']);
					$window.localStorage['snCfg'] = JSON.stringify(angular.extend(c,e.detail));
				}catch(e){
					$window.localStorage['snCfg'] = JSON.stringify(angular.extend(e.detail));
				}

				cb.sendSetConfigOptionsRequest(e.detail);
			});
			document.addEventListener('snSendResizeRequest',function(e){
				if(e.detail){
					cb.sendResizeRequest(e.detail.width || $window.innerWidth,
						e.detail.height || $window.innerHeight);
				}else{
					cb.sendResizeRequest($window.innerWidth,$window.innerHeight);
				}
			});
			document.addEventListener('snClipboardData',function(e){
				Clipboard.data = e.detail;
			});
			document.addEventListener('snRequestRemoteClipboardData',function(e){
				cb.requestDataFromRemoteClipboard();
			});
			document.addEventListener('snSetRemoteClipboardData',function(e){
				cb.setClipboardData(e.detail || '');
				cb.sendDataToRemoteClipboard(0x0D);
			});

			if('ontouchstart' in $window){
				TouchEvents(canvas,cb);
			}
			document.addEventListener('snClipboardCapsSent',function(){
				$rootScope.clipboardReady = true;
			});

			document.addEventListener('snShareReady',function(e){
				$rootScope.myClient = e.detail;
			});
			document.addEventListener('snShareClients',function(e){
				$rootScope.clients = e.detail;
			});
			document.addEventListener('snShareMouseEvent',function(e){

			});
			document.addEventListener('snSharePassword',function(e){
				var str = JSON.stringify({
						RdpPort:fastx.port,
						password:e.detail.password,
						id:fastx.id
					});
					var key = btoa(str);
					$rootScope.$broadcast('share:url',[window.location.origin,'/share?',key].join(''));
			});

			function enableRailListener(e){
				console.log('RAIL enabled');
				$rootScope.railInit = true;
				Rail.init(canvas,cb);
				document.removeEventListener('snRailEnabled',enableRailListener);
			}
			document.addEventListener('snRailEnabled',enableRailListener,{once:true});
			$rootScope.$apply(function(){
				$rootScope.xrdpReady = true;
			});
		}
			resolve (fastx);
	 	});
 	});
}
