'use strict';
var DrawQueue = require("xrdp/canvas/draw-queue");
var capabilitySettings = require('xrdp/capability-settings');

var canvases = {};
var bitmapId = 0xFFFF;

function current(){
    return canvases[bitmapId];
}
function primary(canvas){
    if(canvas){
        canvases[0xFFFF] = new DrawQueue(canvas.width,canvas.height);
    }
    return canvases[0xFFFF];
}
function get(id){
    return canvases[id];
}
function store(id,dq){
    if( (id !== 0xFFFF) && (id >= capabilitySettings.capabilities[0x11].offscreenCacheEntries) ){
        throw new Error('Offscreen Surface Cache ID greater that max cache Entries');
    }
    canvases[id] = dq;
}
function remove(id){
    if(id === 0xFFFF){
        throw new Error('Cannot delete primary surface');
    }
    delete canvases[id];
}
function clear(){
    var p = canvases[0xFFFF];
    canvases = {
        0xFFFF:p
    };
}
function create(w,h){
    return  new DrawQueue(w,h);
}
var o = {
    get:get,
    store:store,
    remove:remove,
    clear:clear,
    create:create,
    primary:primary,
    current:current
};
Object.defineProperty(o,'bitmapId',{
    get:function(){return bitmapId;},
    set:function(id){bitmapId = id;}
});

module.exports = o;
