'use strict';     
    var THREE       = require('three.min');
    var vs          = require('text!xrdp/shaders/shader.vert');
    var fs          = require('text!xrdp/shaders/shader.frag');
    
    
	function ceilPow2( aSize ){
		return Math.pow( 2, Math.ceil( Math.log( aSize ) / Math.log( 2 ) ) ); 
	}
	
	function RDPWindow(w,h){
		this.h = ceilPow2(h);
		this.w = ceilPow2(w);

		var c = document.createElement('canvas');
		var ctx = c.getContext('2d');
		this.imdata = ctx.createImageData(64,64);
		
		this.renderer = new THREE.WebGLRenderer();
		this.offscreenTarget = new THREE.WebGLRenderTarget(this.h,this.w,{
			depthBuffer:false,
			stencilBuffer:false,
		});
		this.renderTarget = new THREE.WebGLRenderTarget(this.h,this.w,{
			depthBuffer:false,
			stencilBuffer:false,
		});
		
		var mesh = new THREE.Mesh(
			new THREE.PlaneGeometry(this.w,this.h),
			new THREE.MeshBasicMaterial({
				map:this.renderTarget.texture
			})
		);
		this.renderedScene = new THREE.Scene();
		this.renderedScene.add(mesh);
		this.scene = new THREE.Scene();
		var x = Math.ceil(this.h/64);
		var y;
		var i,j=0;
		var mesh,row,uniforms;
		while(x--){
			y  = Math.ceil(this.w/64);
			i=0;
			row =[];
			while(y--){
				tile = new THREE.PlaneGeometry(64,64);
				tile.translate((j + 32) -this.w/2,this.h/2 - (i + 32),0);
				uniforms = {
						imdata:{value:this.DataTexture(4096*4)},
						//crTex:{value:this.DataTexture(4096*4)},
						//cbTex:{value:this.DataTexture(4096*4)},
						clip:{value:this.ClipTexture([0,0,64,64],0,0)}
				};
				mesh = new THREE.Mesh(tile,new THREE.ShaderMaterial({
					uniforms:uniforms,
					vertexShader:vs,
					fragmentShader:fs,
					transparent:true
				}));
				i+=64;
				this.scene.add(mesh);
			}
			j+=64;
			
		}
		console.log(this.scene);

		this.frameMesh = new THREE.Mesh(new THREE.PlaneGeometry(this.w,this.h),
			new THREE.MeshBasicMaterial({
				map:this.offscreenTarget.texture
			}));
		this.frameMesh.translateZ(-1);
		
		var that = this;
		this.scene.add(this.frameMesh);
		this.resize(w,h);	
		
		
		var that = this;
		var time,prevTime =0;
		function startRender(){
			requestAnimationFrame( startRender );
			that.renderer.render(that.renderedScene,that.camera);
		}
		this.startRender = startRender;
	}
	RDPWindow.prototype.resize= function(w,h){
		this.w = w;
		this.h = h;
		
		this.renderer.setSize( this.w,this.h );
		this.renderer.setViewport(0,0,this.w,this.h);
		this.camera = new THREE.OrthographicCamera(-this.w/2,this.w/2,this.h/2,-this.h/2,0.1,1000);
		this.camera.position.z = 0.1;
	}
	
	RDPWindow.prototype.DataTexture = function(tex){
		var y  = new THREE.DataTexture(new Uint8Array(tex),64,64,THREE.RGBAFormat,THREE.UnsignedByteType);
		y.needsUpdate = true;
		y.flipY = true;
		return y;
	}
	RDPWindow.prototype.clip = function(c2x,regions,x,y){
		c2x.save();
		c2x.clearRect(0,0,64,64);
		c2x.beginPath();
		while(r = regions.shift()){
			c2x.rect(r[0] -x,r[1] - y,r[2],r[3]);
		}
		c2x.clip();
		c2x.fill();
		c2x.restore();
		var c = new THREE.Texture(c2x.canvas);
		c.needsUpdate = true;
		return c;
	}
	RDPWindow.prototype.ClipTexture = function(regions,x,y){
		var r;
		var c2x = document.createElement('canvas').getContext('2d');
		c2x.canvas.width = 64;
		c2x.canvas.height = 64;
		c2x.fillStyle = '#FFFFFF';
		return this.clip(c2x,regions,x,y);
		
	}
	RDPWindow.prototype.tile = function(data){
		var x = Math.floor( (data.rect[0] + data.x)/64) *Math.floor(this.w/64) + Math.floor( (data.rect[1] + data.y)/64);
		this.scene.children[x].material.uniforms.imdata.value.image.data =  new Uint8Array(data.imdata);
		this.scene.children[x].material.uniforms.imdata.value.needsUpdate = true;
		
		var ctx = this.scene.children[x].material.uniforms.clip.value.image.getContext('2d');
		ctx.save();
		ctx.clearRect(0,0,64,64);
		ctx.fillStyle = '#FFFFFF';
		ctx.beginPath();
		var r;
		while(r = data.regions.shift()){
			ctx.rect(r[0] -data.x,r[1] - data.y,r[2],r[3]);
		}
		ctx.clip();
		ctx.fill();
		ctx.restore();
		
		this.scene.children[x].material.uniforms.clip.value.needsUpdate = true;
	}
	RDPWindow.prototype.process = function(data){
		//console.log(data.type);
		switch(data.type){
			case 'tile':
				this.tile(data);
				break;
			case 'framebegin':
				break;
			case 'frameend':
				this.render();
				break;
			case 'copy':
				//console.log('copy');
				this.ctx.putImageData(this.ctx.getImageData(
					data.src[0],
					data.src[1],
					data.w,data.h
					),data.dst[0],data.dst[1]
				);
				break;
			default:
				break;
		}
	}
	
	
	RDPWindow.prototype.getCanvas = function(){
		return this.renderer.domElement;
	}
	RDPWindow.prototype.render = function(){
		this.scene.needsUpdate = true;
		this.renderer.render(this.renderedScene,this.camera,this.offscreenTarget);
		this.renderer.render(this.scene,this.camera,this.renderTarget);
	}
	
	function start(w,h) {
		var rdp = new RDPWindow(w,h);
		rdp.startRender();
		Object.defineProperty(rdp,"canvas",{
			get:function(){
				return rdp.getCanvas();
			}
		});
		return rdp;
	}
	
module.exports = {
    start:start
}