'use strict';

    var DataStream  = require('DataStream');
    var Offscreen   = require('xrdp/cache/offscreen-bitmap');
    var Screen      = require('xrdp/canvas/screen');


	function writeBitmapCodecRemoteFX(codecId,captureFlags,icaps){
		var icapLen = 8;
		var dsCaps = new DataStream();
		dsCaps.writeUint16(0xCBC0);
		dsCaps.writeUint32(0x0008);//length
		dsCaps.writeUint16(0x0001);
		dsCaps.writeUint16(0xCBC1);
		dsCaps.writeUint32(13 + icapLen*icaps.length);
		dsCaps.writeUint8(0x01);
		dsCaps.writeUint16(0xCFC0);
		dsCaps.writeUint16(icaps.length);//number icaps
		dsCaps.writeUint16(icapLen);//icap size

		//arr[0] = codec mode 0x02 | 0x00,
		//arr[1] = entropy alg 0x01 | 0x04

		var icap = null;
		while(icaps.length){
		icap = icaps.shift();
		dsCaps.writeUint16(0x0100);
		dsCaps.writeUint16(0x0040);
		dsCaps.writeUint8(icap[0]);
		dsCaps.writeUint8(0x01);
		dsCaps.writeUint8(0x01);
		dsCaps.writeUint8(icap[1]);
		}
		//RFX Image GUID
		var ds = new DataStream();
		//begin guid
		ds.writeUint32(0x76772F12);
		ds.writeUint16(0xBD72);
		ds.writeUint16(0x4463);
		ds.writeUint8(0xAF);
		ds.writeUint8(0xB3);
		ds.writeUint8(0xB7);
		ds.writeUint8(0x3C);
		ds.writeUint8(0x9C);
		ds.writeUint8(0x6F);
		ds.writeUint8(0x78);
		ds.writeUint8(0x86);
		//end guid

		ds.writeUint8(codecId);//codec id
		ds.writeUint16(12 + dsCaps.byteLength);

		ds.writeUint32(12 + dsCaps.byteLength);//caps container length
		ds.writeUint32(captureFlags);
		ds.writeUint32(dsCaps.byteLength);
		ds.writeUint8Array(new Uint8Array(dsCaps.buffer));
		return ds.buffer;
    }
	function writeBitmapCodecImageRemoteFX(codecId,captureFlags,icaps){
		var icapLen = 8;
		var dsCaps = new DataStream();
		dsCaps.writeUint16(0xCBC0);
		dsCaps.writeUint32(0x0008);//length
		dsCaps.writeUint16(0x0001);
		dsCaps.writeUint16(0xCBC1);
		dsCaps.writeUint32(13 + icapLen*icaps.length);
		dsCaps.writeUint8(0x01);
		dsCaps.writeUint16(0xCFC0);
		dsCaps.writeUint16(icaps.length);//number icaps
		dsCaps.writeUint16(icapLen);//icap size

		//arr[0] = codec mode 0x02 | 0x00,
		//arr[1] = entropy alg 0x01 | 0x04

		var icap = null;
		while(icaps.length){
		icap = icaps.shift();
		dsCaps.writeUint16(0x0100);
		dsCaps.writeUint16(0x0040);
		dsCaps.writeUint8(icap[0]);
		dsCaps.writeUint8(0x01);
		dsCaps.writeUint8(0x01);
		dsCaps.writeUint8(icap[1]);
		}
		//RFX Image GUID
		var ds = new DataStream();
		//begin guid
		ds.writeUint32(0x2744CCD4);
		ds.writeUint16(0x9D8A);
		ds.writeUint16(0x4E74);
		ds.writeUint8(0x80);
		ds.writeUint8(0x3C);
		ds.writeUint8(0x0E);
		ds.writeUint8(0xCB);
		ds.writeUint8(0xEE);
		ds.writeUint8(0xA1);
		ds.writeUint8(0x9C);
		ds.writeUint8(0x54);
		//end guid

		ds.writeUint8(codecId);//codec id
		ds.writeUint16(12 + dsCaps.byteLength);

		ds.writeUint32(12 + dsCaps.byteLength);//caps container length
		ds.writeUint32(captureFlags);
		ds.writeUint32(dsCaps.byteLength);
		ds.writeUint8Array(new Uint8Array(dsCaps.buffer));
		return ds.buffer;
	}
	function writeBitmapCodecNsCodec(codecId,allowDynamicFidelity,allowSubsampling,colorLossLevel){
		var ds = new DataStream();

		//begin guid
		ds.writeUint32(0xCA8D1BB9);
		ds.writeUint16(0x000F);
		ds.writeUint16(0x154F);
		ds.writeUint8(0x58);
		ds.writeUint8(0x9F);
		ds.writeUint8(0xAE);
		ds.writeUint8(0x2D);
		ds.writeUint8(0x1A);
		ds.writeUint8(0x87);
		ds.writeUint8(0xE2);
		ds.writeUint8(0xD6);
		//end guid

		ds.writeUint8(codecId);//codec id

		ds.writeUint16(0x03);//length

		ds.writeUint8(allowDynamicFidelity);
		ds.writeUint8(allowSubsampling);
		ds.writeUint8(colorLossLevel);
		return ds.buffer;
	}

	function init(settings){
		if(settings.height){
			capabilities[0x0002].desktopHeight = settings.height;
		}
		if(settings.width){
			capabilities[0x0002].desktopWidth = settings.width;
		}

		if(settings.keyboard){
			if(settings.keyboard.layout){
				capabilities[0x000D].keyboardLayout = settings.keyboard.layout;
			}
			if(settings.keyboard.type){
				capabilities[0x000D].keyboardType = settings.keyboard.type;
			}
		}

		if(settings.codecIds['nsCodec']){
			capabilities[0x001D].bitmapCodecs['nsCodec'] = {
				codecId:settings.codecIds['nsCodec'],
				buffer:writeBitmapCodecNsCodec(settings.codecIds['nsCodec'],0x01,0x01,0x07)
			}
		}
		if(settings.codecIds['rfx']){
			capabilities[0x001D].bitmapCodecs['rfx'] ={
				codecId:settings.codecIds['rfx'],
				buffer:writeBitmapCodecRemoteFX(settings.codecIds['rfx'],0x00, [[0x02,0x01],[0x02,0x04]])
			}
		}
		if(settings.codecIds['rfxImage']){
			capabilities[0x001D].bitmapCodecs['rfxImage'] = {
				codecId:settings.codecIds['rfxImage'],
				buffer:writeBitmapCodecImageRemoteFX(settings.codecIds['rfxImage'],0x00, [[0x02,0x01],[0x02,0x04]])
			}
		}
	}
	var capabilities = {
		0x0001:{//general
			extraFlags:0x0400 | 0x0001 | 0x0004 |0x0008 | 0x0010,
			refreshRectSupport:1,
			suppressOutputSupport:1
		},
		0x0002:{//bitmap
			preferredBitsPerPixel:0xCA01,
			desktopWidth:1024,
			desktopHeight:768,
			desktopResizeFlag:1,
			drawingFlags:0x04 | 0x08
		},
		0x0003:{//order
			orderFlags:0x0020 | 0x0002 | 0x0008,
			orderSupport:[
			1/*dstBlt*/,		1/*patBlt*/,		0/*srcBlt*/,		1/*memBlt*/,
			1/*mem3Blt*/,		0/*unused*/,		0/*unused*/,		0/*drawNineGrid*/,
			1/*lineTo*/,		0/*multi9Grid*/,	0/*unused*/,		0/*saveBitmap*/,
			0/*unused*/,		0/*unused*/,		0/*unused*/,		1/*multiDstBlt*/,
			1/*multiPatBlt*/,	1/*multiSrcBlt*/,	1/*multiOpaRect*/,	0/*fastIndex*/,
			1/*polygonSC*/,		1/*polygonCB*/,		1/*polyline*/,		0/*unused*/,
			0/*fastGlyph*/,		0/*ellipseSC*/,		0/*ellipseCB*/,		0/*glyphIndex*/,
			0/*unused*/,		0/*unused*/,		0/*unused*/,		0/*unused*/
			],
			orderSupportExtra: 0x0002 | 0x0004
		},

		0x0004:{//bitmapCacheRev1
			cache0Entries:200,
			cache0MaximumCellSize:1024,
			cache1Entries:600,
			cache1MaximumCellSize:4096,
			cache2Entries:65535,
			cache2MaximumCellSize:16384
		},
		0x0005:{//control
		},
		0x0007:{//activation
		},
		0x0008:{//pointer
			colorPointerCacheSize:0x14,
			pointerCacheSize:0x14
		},
		0x0009:{//share
		},
		0x000A:{//colorCache
		},
		0x000C:{//sound
			soundFlags:0x0001
		},
		0x000D:{//input
			inputFlags: 0x0001 | 0x0004 | 0x0008 |  0x0020,
			keyboardLayout: 0x00000409,
			keyboardType:0x00000004,
			keyboardSubtype:0x00,
			keyboardFunctionKeyNum:12,
			imeFileName:''
		},
		0x000E:{//font
		},
		0x000F:{//brush
			brushSupportLevel:0x00000002
		},
		0x0010:{//glyphCache
			cacheDefinitions:[
				[0xFE,0x04],[0xFE,0x04],[0xFE,0x08],[0xFE,0x08], [0xFE,0x10],
				[0xFE,0x20],[0xFE,0x40],[0xFE,0x80],[0xFE,0x100],[0x40,0x100]
			],
			fragCache:[0x100,0x100],
			glyphSupportLevel:0x0002
		},
		0x0011:{//offscreen cache
			offscreenSupportLevel:0x00000001,
			offscreenCacheSize:0x1E00,
			offscreenCacheEntries:0x07D0
		},
		0x0013:{//bitmapCacheRev2
			cacheFlags:0x0002,
			cellCaches://set k to 1 for persistent cache
				[{numEntries:0x0258,k:0},
				{numEntries:0x0258,k:0},
				{numEntries:0x0800,k:0},
				{numEntries:0x1000,k:0},
				{numEntries:0x0800,k:0}]
		},
		0x0014:{//virtual channels
			flags:0,
			vCChunkSize:0x0640
		},
		0x0015:{//drawNineGrid Cache
			drawNineGridSupportLevel:0x00000002,
			drawNineGridCacheSize:2560,
			drawNineGridCacheEntries:260
		},
		0x0016:{//draw GDI Plus
			drawGDIPlusSupportLevel:0x00000001,
			gdipVersion:0x00000001,
			drawGdipCacheLevel:0x00000001,
			gdipCacheEntries:{
				gdipGraphicsCacheEntries:10,
				gdipBrushCacheEntries:5,
				gdipPenCacheEntries:5,
				gdipImageCacheEntreis:10,
				gdipImageAttributesCacheEntries:2
			},
			gdipCacheChunkSize:{
				gdipGraphicsCacheChunkSize:512,
				gdipObjectBrushCacheChunkSize:2048,
				gdipObjectPenCacheChunkSize:1024,
				gdipObjectImageAttributesCacheChunkSize:64
			},
			gdipImageCacheProperties:{
				gdipObjectImageCacheChunkSize:4096,
				gdipObjectImageCacheTotalSize:256,
				gdipObjectImageCacheMaxSize:128
			}
		},
		0x0017:{//RAIL
			S:1,
			L:1,
			I:0
		},
		0x0018:{//window
			wndSupportLevel:0x00000002,
			numIconCaches:3,
			numIconEntries:13
		},
		0x0019:{//compDesk
			compDeskSupportLevel:0x0001
		},
		0x001A:{//multi Fragment Update
			maxRequestSize:0xFFFFFFFF
		},
		0x001B:{//large pointer
			largePointerSupportFlags:0x00000001
		},
		0x001C:{//surface Commands
			cmdFlags:0x00000002 | 0x00000010 | 0x00000040
		},
		0x001D:{//bitmap Codecs
			bitmapCodecs:{}
		},
		0x001E:{//Frame acknowledge
			maxUnacknowledgedFrameCount:2
		},
	}
	var o = {};
	Object.defineProperty(o,'capabilities',{
		get:function(){return capabilities}
	});
module.exports = {
		init:init,
		capabilities:o.capabilities
	}
