    var DataStream      = require('DataStream');
    var virtualChannel  = require('./virtual-channel');

	function readClipCaps(dv){
		var capsetNum = dv.getUint16(0,true);
		var capsets = [];
		var cdv = new DataView(dv.buffer,dv.byteOffset +4,dv.byteLength-4);
		var type,len;
		while(capsetNum--){
			type = cdv.getUint16(0,true);
			len = cdv.getUint16(2,true);
			switch(type){
				case 0x0001:
					capsets.push({
						type:type,
						version:cdv.getUint32(4,true),
						flags:cdv.getUint32(8,true)
					});
					break;
				default:
					console.log('Unknown Clipboard capability set type');
					return undefined;
			}
			cdv = new DataView(cdv.buffer,cdv.byteOffset + len,cdv.byteLength - len);
		}
		return capsets;
	}
	function readFormatDataRequest(dv){
		return dv.getUint32(0,true);
	}

	function writeHeader(buffer,type,flags){
		var u8 = new Uint8Array(buffer.byteLength +8);
		u8.set(new Uint8Array(buffer),8);
		var dv = new DataView(u8.buffer);
		dv.setUint16(0,type,true);
		dv.setUint16(2,flags,true);
		dv.setUint32(4,buffer.byteLength,true);
		return dv.buffer;
	}

	function writeCapabilities(){
		var ds = new DataStream();
		ds.writeUint16(1);
		ds.writeUint16(0);
		ds.writeUint16(0x0001);//capset type
		ds.writeUint16(12);
		ds.writeUint32(0x00000002);
		ds.writeUint32(0x00000002 | 0x00000010);
		return writeHeader(ds.buffer,0x0007,0x0000);
	}
	function writeTempDirectory(){
		var ds = new DataStream();
		ds.writeString('c:\\temp\\clipdata',null,520);
		return writeHeader(ds.buffer,0x0006,0x0000);
	}
	function writeFormatList(){
		var ds = new DataStream();
		ds.writeUint32(0x01);//ascii
		ds.writeUint16(0);//name is null
		ds.writeUint32(0x0D);//unicode
		ds.writeUint16(0);//name is null
		return writeHeader(ds.buffer,0x0002,0x0000);
	}
	function writeFormatListResponse(ok){
		Clipboard.prototype.listResponseFailure = ok?false:true;
		return writeHeader(new ArrayBuffer(0),0x0003,ok?0x0001:0x0002);
	}
	function writeFormatDataRequest(id){
		var dv = new DataView(new ArrayBuffer(4));
		dv.setUint32(0,id,true);
		return writeHeader(dv.buffer,0x0004,0x0000);
	}
	function writeFormatDataResponse(ok,dataBuffer){
		if(Clipboard.prototype.listResponseFailure){ok=false;}
		return writeHeader(ok?dataBuffer:new ArrayBuffer(0),0x0005,ok?0x0001:0x0002);
	}

	var Clipboard = function(socket,userChannelId,channelId,canvas){
		Clipboard.clipboardData = '';
		Clipboard.canvas = canvas;
		Clipboard.vc = new virtualChannel(socket,userChannelId,channelId);
	}
	Clipboard.prototype.listResponseFailure = false;
	Clipboard.cb = {};

	Clipboard.cb.setClipboardData = function setClipboardData(text){
		Clipboard.clipboardData = text;
	}
	Clipboard.cb.getClipboardData = function getClipboardData(){
		return Clipboard.clipboardData;
	}
	Clipboard.cb.sendDataToRemoteClipboard = function sendDataToRemoteClipboard(id){
		Clipboard.vc.send(writeFormatList());
		var ds = new DataStream();
		switch(id){
			case 0x01://ascii
				ds.writeCString(Clipboard.clipboardData);
				break;
			case 0x0D://unicode
				ds.writeUCS2String(Clipboard.clipboardData);
				ds.writeUint16(0);
				break;
		}
		Clipboard.vc.send(writeFormatDataResponse(true,ds.buffer));
	}

	Clipboard.cb.requestDataFromRemoteClipboard = function requestDataFromRemoteClipboard(){
		Clipboard.vc.send(writeFormatDataRequest(0x01));//request ascii
	}

	Object.defineProperty(Clipboard.prototype,'callbacks',{get:function(){return Clipboard.cb;}});
	Clipboard.prototype.processMessage = function(dv){
		var data = Clipboard.vc.processMessage(dv);
		if(!data){ return undefined;}
		switch(data.type){
			case 0x0001://CB_MONITOR_READY
				if(data.flags !== 0){
					console.log("Invalid Clipboard Monitor Ready Flags");
					return undefined;
				}
				Clipboard.vc.send(writeCapabilities());
				//temp dirs not supported in browser
				Clipboard.vc.send(writeFormatList());
				break;
			case 0x0002://CB_FORMAT_LIST
				var longNames = {};
				var i=0,id,char;
				while(i < dv.byteLength){
					id = dv.getUint32(i,true);
					longNames[id] = '';
					i+=4;
					do{
						char = dv.getUint16(i,true)
						longNames[id] += String.fromCharCode(char);
						i+=2;
					}while(char)
				}
				Clipboard.vc.send(writeFormatListResponse(true));
				break;
			case 0x0003://CB_FORMAT_LIST_RESPONSE
				if( (data.flags & 0x0002) ){
					Clipboard.prototype.listResponseFailure = true;
					return;
				}else if( !(data.flags & 0x0001)){
					return;
				}
				break;
			case 0x0004://CB_FORMAT_DATA_REQUEST
				//server is requesting the data in the id
				Clipboard.cb.sendDataToRemoteClipboard(readFormatDataRequest(data.data));
				break;
			case 0x0005://CB_FORMAT_DATA_RESPONSE
				if(data.flags & 0x0002){
					//failure
				}else if(data.flags & 0x0001){
					//success
					var ds = new DataStream(data.data);
					Clipboard.cb.setClipboardData(ds.readCString());
					document.dispatchEvent(new CustomEvent('snClipboardData',{
							detail:Clipboard.clipboardData
						}));
				}else{
					//unknown
					return undefined;
				}
				break;
			case 0x0007:
				if(data.flags !== 0){
					return undefined;
				}
				document.dispatchEvent(new CustomEvent('snClipboardCapsSent',{detail:null}));
				if(!readClipCaps(data.data).data){return undefined;}
				break;
			case 0x0006://CB_TEMP_DIRECTORY
			case 0x0008://CB_FILECONTENTS_REQUEST
			case 0x0009://CB_FILE_CONTENTS_RESPONSE
			case 0x000A://CB_LOCK_CLIPDATA
			case 0x000B://CB_UNLOCK_CLIPDATA
				console.log('type not supported in browser', data);
				break;
			default:
				console.log('Unknown clipboard message type',data);
				return undefined;
		}
		return 1;
	}
module.exports = Clipboard;
