'use strict';

    var scancodes   = require('./RDPScancodes');
    var fastPathInput    = require('./fast-path-input');

    var polling,timeout,pos;

	var Hook = function(socket,canvas,disableKeyboard,disableMouse){
		Hook.s = socket;
		Hook.canvas = canvas;
		Hook.disableKeyboard = disableKeyboard;
		Hook.disableMouse = disableMouse;
		Hook.disabled = false;
		Hook.canvas.addEventListener('contextmenu',disableContextMenu);
		if(!disableKeyboard){
			var input = document.createElement('input');
			input.id = 'myText';
			input.style.cssText='position:absolute;left:-999999px;top:-999999px;' ;
			document.body.appendChild(input);
			input.focus();
			document.addEventListener('mousedown',function(e){
				if(['TEXTAREA','INPUT','SELECT','BUTTON','A'].indexOf(e.target.tagName) >=0){return;}
				input.focus();
			});
			input.addEventListener('keydown',keyDownEvent);
			input.addEventListener('keyup',keyUpEvent);
			if(window.navigator.platform === 'iPad'){
				input.addEventListener('input',function(e){
					var v = input.value.split('');
					var c;
					while(c = v.shift()){
						virtualPress(c);
					}
					input.value ='';
				});
			}
			window.addEventListener ('blur',resetKeyStates);
      window.addEventListener ('focus',setNumlock)
      document.addEventListener("focusout",function(e){
          sendModifiersUp(e);
      });
      document.addEventListener("focusin",function(e){
          sendModifiersDown(e);
      });
		}
		if(!disableMouse){
			Hook.canvas.addEventListener('mousemove',mouseMoveEvent);
			Hook.canvas.addEventListener('mousedown',function(e){
                input.focus();
                mouseDownEvent(e);
            });
			Hook.canvas.addEventListener('mouseup',mouseUpEvent);
			Hook.canvas.addEventListener('wheel',mouseWheelEvent);
		}
	}
	function disableInput(bool){
		Hook.disabled = bool;
	}

	function removeAllListeners(){
		Hook.canvas.removeEventListener('mousemove',mouseMoveEvent);
		Hook.canvas.removeEventListener('mousedown',mouseDownEvent);
		Hook.canvas.removeEventListener('mouseup',mouseUpEvent);
		Hook.canvas.removeEventListener('wheel',mouseWheelEvent);
		document.removeEventListener('keydown',keyDownEvent);
		document.removeEventListener('keyup',keyUpEvent);
		window.removeEventListener('blur',resetKeyStates);
		Hook.canvas.removeEventListener('contextmenu',disableContextMenu);
	}

	Object.defineProperty(Hook,'socket',{
		get:function(){
			if(Hook.disabled){
				return {send:function(){}};
			}else{
				return Hook.s;
			}
		}
	});
	Object.defineProperty(Hook.prototype,'callbacks',{
		get:function(){
			return {
                sendModifiersDown:sendModifiersDown,
                sendModifiersUp:sendModifiersUp,
				sendMouseDown:sendMouseDown,
				sendMouseUp:sendMouseUp,
				sendMouseMove:sendMouseMove,
				sendMouseWheel:sendMouseWheel,
				sendKeyDown:keyDown,
				sendKeyUp:keyUp,
				resetKeyStates:resetKeyStates,
				getMousePos:getMousePos,
				disableInput:disableInput,
				setMouseMoveTimer:setTimer,
				removeAllListeners:removeAllListeners,
				mouseWheelEvent:mouseWheelEvent
			}
		}
	});

	function setTimer(ms){
		polling = ms;
	}
	function timeoutEvent(ms){
		return setTimeout(function(){
			sendMouseMove(pos);
			timeout = null; //clear timeout
		},ms);
	}
	function getMousePos(e){
		var rect = Hook.canvas.getBoundingClientRect();
		return  [
			Math.max(e.clientX - rect.left,0),
			Math.max(e.clientY - rect.top,0)
		];
	}
	function setMousePos(e){
		sendMouseMove(getMousePos(e));
	}
    function sendModifiersDown(e){
        if(e.altKey){
            keyDown({
                code:"AltLeft",
                location:KeyboardEvent.DOM_KEY_LOCATION_STANDARD
            });
        }
        if(e.ctrlKey){
            keyDown({
                code:"ControlLeft",
                location:KeyboardEvent.DOM_KEY_LOCATION_STANDARD
            });
        }
        if(e.metaKey){
            keyDown({
                code:"MetaRight",
                location:KeyboardEvent.DOM_KEY_LOCATION_STANDARD
            });
        }
        if(e.shiftKey){
            keyDown({
                code:"ShiftLeft",
                location:KeyboardEvent.DOM_KEY_LOCATION_STANDARD
            });
        }
    }
    function sendModifiersUp(e){
        if(!e.altKey){
            keyUp({
                code:"AltLeft",
                location:KeyboardEvent.DOM_KEY_LOCATION_STANDARD
            });
        }
        if(!e.ctrlKey){
            keyUp({
                code:"ControlLeft",
                location:KeyboardEvent.DOM_KEY_LOCATION_STANDARD
            });
        }
        if(!e.metaKey){
            keyUp({
                code:"MetaRight",
                location:KeyboardEvent.DOM_KEY_LOCATION_STANDARD
            });
        }
        if(!e.shiftKey){
            keyUp({
                code:"ShiftLeft",
                location:KeyboardEvent.DOM_KEY_LOCATION_STANDARD
            });
        }
    }
	function mouseMoveEvent(e){
		pos = getMousePos(e);
		if(polling){
			if(!timeout){
				sendMouseMove(pos);
				timeout = timeoutEvent(polling);
			}
		}else{
			sendMouseMove(pos);
		}
	}
	function mouseDownEvent(e){

		e.preventDefault();
		e.stopPropagation();
		pos = getMousePos(e);
        sendModifiersDown(e);
		sendMouseDown(e.which,pos);
	}
	function mouseUpEvent(e){
		e.preventDefault();
		e.stopPropagation();
		pos = getMousePos(e);
        //sendModifiersUp(e);
		sendMouseUp(e.which,pos);
	}
	function mouseWheelEvent(e){
		e.preventDefault();
		e.stopPropagation();
		var delta = -400*(e.deltaY);
		pos = getMousePos(e);
		sendMouseWheel(delta,pos);
	}
	function disableContextMenu(e){
		e.preventDefault();
		e.stopPropagation();
		return false;
	}
	function keyDownEvent(e){
		e.stopPropagation();
		e.preventDefault();
		if(window.navigator.platform === 'iPad'){
			iPadDown(e);
			//iPadUp(e);
			return;
		}
    setNumlock(e);
    if (e.code === 'NumLock') { return; }
		keyDown(e);
	}
	function keyUpEvent(e){
		e.stopPropagation();
		e.preventDefault();
		if(window.navigator.platform === 'iPad'){
			//iPadUp(e);
			return;
		}
		keyUp(e);
	}
  var numlock;
  function setNumlock (e) {
    if (!e.getModifierState) { return; }
    var k;
    if (typeof numlock === 'undefined') {
        numlock = e.getModifierState ('NumLock');
        k = new KeyboardEvent ('NumLock', {
          code: 'NumLock'
        });

        if (numlock) {
          keyDown (k);
        } else {
          keyUp (k);
        }
        return;
    }
    if (numlock !== e.getModifierState ('NumLock')) {
      k = new KeyboardEvent ('NumLock', {
        code: 'NumLock'
      });
      if (numlock) {
        keyUp (k);
      } else {
        keyDown (k);
      }
    }
    numlock =  e.getModifierState ('NumLock');
  }
	function resetKeyStates(){
        sendModifiersUp({
            altKey:true,
            ctrlKey:true,
            metaKey:true,
            shiftKey:true
        });
		Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.sync(false,false,false,false)]));
	}

	function mapButton(button){
		switch(button){
		case 1:
		return 1;
		case 2:
		return 3;
		case 3:
		return 2;
		case 4:
		return 4;
		case 5:
		return 5;
		}
	}

	function sendMouseDown(button,pos){
		Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.mouseButton(mapButton(button),pos,true)]));
	}
	function sendMouseUp(button,pos){
		Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.mouseButton(mapButton(button),pos,false)]));
	}
	function sendMouseMove(pos){
		Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.mouseMove(pos)]));
	}
	function sendMouseWheel(delta,pos){
		Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.mouseWheel(delta,pos,false)]));
	}


	function keyDown(e) {
		var info = scancodes(e);
		if (!info) return;
		if(e.repeat){
			keyUp(e);
		}
		switch (e.location) {
		case KeyboardEvent.DOM_KEY_LOCATION_STANDARD:
			if ('is_lock' in info && navigator.userAgent.indexOf("Mac OS X") != -1) {
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode'],false)]));
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode'],true)]));
			}
			else
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode'],false)]));
			break;
		case KeyboardEvent.DOM_KEY_LOCATION_LEFT:
			if ('scancode_left' in info)
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode_left'],false)]));
			else
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode'],false)]));
			break;
		case KeyboardEvent.DOM_KEY_LOCATION_RIGHT:
			if ('scancode_right' in info)
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode_right'],false)]));
			else
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode'],false)]));
			break;
		case KeyboardEvent.DOM_KEY_LOCATION_NUMPAD:
			if ('scancode_lock' in info)
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode_lock'],false)]));
			else
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode'],false)]));
			break;
		}
	}
	function keyUp(e) {
		var info = scancodes(e);

		if (!info) return;

		switch (e.location) {
		case KeyboardEvent.DOM_KEY_LOCATION_STANDARD:
			if ('is_lock' in info && navigator.userAgent.indexOf("Mac OS X") != -1) {
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode'],false)]));
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode'],true)]));
			}
			else {
				// Windows Chrome modifier bug
				if ('scancode_left' in info)
					Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode_left'],true)]));
				if ('scancode_right' in info)
					Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode_right'],true)]));

				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode'],true)]));
			}
			break;
		case KeyboardEvent.DOM_KEY_LOCATION_LEFT:
			if ('scancode_left' in info)
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode_left'],true)]));
			else
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode'],true)]));
			break;
		case KeyboardEvent.DOM_KEY_LOCATION_RIGHT:
			if ('scancode_right' in info)
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode_right'],true)]));
			else
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode'],true)]));
			break;
		case KeyboardEvent.DOM_KEY_LOCATION_NUMPAD:
			if ('scancode_lock' in info)
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode_lock'],true)]));
			else
				Hook.socket.send(fastPathInput.writeInputEventPdu([fastPathInput.key(info['scancode'],true)]));
			break;
		}
	}

	function iPadDown(e){

		if(1|| (e.keyCode >=48 && e.keyCode <= 90) ||
			([8,9,13,20,46,186,187,188,189,190,192,219,220,221,222].indexOf(e.keyCode) >= 0)
		){
			switch(e.keyCode){
				case 8:
					virtualPress('Backspace');
					break;
				case 9:
					virtualPress('Tab');
					break;
				case 13:
					virtualPress('Enter');
					break;
				case 20:
					virtualPress('CapsLock');
					break;
				case 46:
					virtualPress('Delete');
					break;
				default:
					virtualPress(e.key);
			}
			return; //let the input event handle it
		}
	}
	function iPadUp(e){
		if( (e.keyCode >=48 && e.keyCode <= 90) ||
				([8,9,13,20,46,186,187,188,189,190,192,219,220,221,222].indexOf(e.keyCode) >= 0)
		){
			keyUp(e);
			return; //let the input event handle it
		}
	}
	function virtualPress(c){
		var code,shifted;

		if(c.length > 1){
			code = c;
		}else if(c >='0' && c <='9'){
			code = 'Digit' +c;
		}else if(c >= 'A' && c <='Z'){
			code = 'Key' + c.toUpperCase();
			shifted = true;
		}else if(c >= 'a' && c <='z'){
			code = 'Key' + c.toUpperCase();
		}else{
			var keys = {
				' ':{code:'Space'},
				'!':{code:'Digit1',shifted:true},
				'@':{code:'Digit2',shifted:true},
				'#':{code:'Digit3',shifted:true},
				'$':{code:'Digit4',shifted:true},
				'%':{code:'Digit5',shifted:true},
				'^':{code:'Digit6',shifted:true},
				'&':{code:'Digit7',shifted:true},
				'*':{code:'Digit8',shifted:true},
				'(':{code:'Digit9',shifted:true},
				')':{code:'Digit0',shifted:true},
				'[':{code:'BracketLeft'},
				']':{code:'BracketRight'},
				'{':{code:'BracketLeft',shifted:true},
				'}':{code:'BracketRight',shifted:true},
				';':{code:'Semicolon'},
				':':{code:'Semicolon',shifted:true},
				',':{code:'Comma'},
				'.':{code:'Period'},
				'<':{code:'Comma',shifted:true},
				'>':{code:'Period',shifted:true},
				'/':{code:'Slash'},
				'?':{code:'Slash',shifted:true},
				"\\":{code:'Backslash'},
				"|":{code:"Backslash",shifted:true},
				"-":{code:'Minus'},
				"_":{code:'Minus',shifted:true},
				"=":{code:'Equal'},
				"+":{code:'Equal',shifted:true},
				"\'":{code:'Quote'},
				"\"":{code:'Quote',shifted:true},
				"`":{code:'Backquote'},
				"~":{code:'Backquote',shifted:true},
			}
			if(c in keys){
				code = keys[c].code;
				shifted = keys[c].shifted;
			}else{
				console.log(c);
			}
		}


		if(shifted){
			keyDown({
				code:'ShiftLeft',
				location:KeyboardEvent.DOM_KEY_LOCATION_STANDARD
			});
		}
		keyDown({
			code:code,
			location:KeyboardEvent.DOM_KEY_LOCATION_STANDARD
		});
		keyUp({
			code:code,
			location:KeyboardEvent.DOM_KEY_LOCATION_STANDARD
		});
		if(shifted){
			keyUp({
				code:'ShiftLeft',
				location:KeyboardEvent.DOM_KEY_LOCATION_STANDARD
			});
		}
	}
module.exports = Hook;
