'use strict';

    var tpkt            = require('xrdp/core/tpkt');
    var x224            = require('xrdp/core/x224');
    var mcs             = require('xrdp/core/mcs');
    var securityHeaderReader  = require('xrdp/security/header/reader');
    var securityHeaderWriter  = require('xrdp/security/header/writer');
    var DataStream      = require('DataStream');


    var seqNum;


	function MessageChannel(socket, Client){
		this._networkCharacteristicsTimer = Date.now();
		this.clientAutoDetectResponsePDU = function(buf){
			return tpkt.writePacket(x224.writeDataPDU(mcs.writeSDrqHeader(
				Client.store.userChannelId,
				Client.store.messageChannelId,0x01,0x03,
				securityHeaderWriter.writeSecurityHeaderAndData(0x2000,
				Client.store.receivedServerData.encryptionLevel,
				Client.store.sessionKeys,new Uint8Array(buf)))));
		}
		this.encryptionLevel = Client.store.receivedServerData.encryptionLevel;
		this.encryptionMethods = Client.settings.encryptionMethods;
		this.send = function(buf){
			socket.send(buf);
		}
		Object.defineProperty(this,'_networkCharacteristicSequenceNumberStore',{
			get:function(){
				return Client.store.networkCharacteristicsSequenceNumber;
			},set:function(n){
				Client.store.networkCharacteristicsSequenceNumber = n;
			}
		});
		Object.defineProperty(this,'_byteCountStore',{
			get:function(){
				return Client.store.networkCharacteristicsByteCount;
			},set:function(n){
				Client.store.networkCharacteristicsByteCount = n;
			}
		});
	}

	function rttMeasureResponse(){
		var buf = new ArrayBuffer(0x06);
		var dv = new DataView(buf);
		dv.setUint8(0,0x06);
		dv.setUint8(1,0x01);
		dv.setUint16(2,seqNum,true);
		dv.setUint16(4,0x0000,true);
		return buf;
	}
	function bandwidthMeasureResults(respType,bw,ms){
		var buf = new ArrayBuffer(0x0E);
		var dv = new DataView(buf);
		dv.setUint8(0,0x0E);
		dv.setUint8(1,0x01);
		dv.setUint16(2,seqNum,true);
		dv.setUint16(4,respType,true);
		dv.setUint32(6,ms,true);
		dv.setUint32(10,bw,true);
		return buf;
	}
	function networkCharacteristicsSync(bw,rtt){
		var buf = new ArrayBuffer(0x0E);
		var dv = new DataView(buf);
		dv.setUint8(0,0x0E);
		dv.setUint8(1,0x01);
		dv.setUint16(2,seqNum,true);
		dv.setUint16(4,0x0018,true);
		dv.setUint32(6,bw,true); //kb per sec
		dv.setUint32(10,rtt,true);//ms
		return buf
	}
	function rttMeasureRequest(_self,reqType){
		_self.send(_self.clientAutoDetectResponsePDU(rttMeasureResponse()));
	}
	function bandwidthMeasureStart(_self,reqType){
		switch(reqType){
			case 0x0014:
				_self._byteCountStore =0;
				_self._networkCharacteristicsTimer = Date.now();
				break;
			case 0x0114:
				_self._byteCountStore =0;
				_self._networkCharacteristicSequenceNumberStore = seqNum;
				_self._networkCharacteristicsTimer = Date.now();

				break;
			case 0x1014:

				break;
		}
	}
	function bandwidthMeasurePayload(_self,dv,that){

	}
	function bandwidthMeasureStop(_self,reqType,payloadLength){
		switch(reqType){
			case 0x002B:
				_self._byteCountStore += 8 + payloadLength;
				_self.send(
					_self.clientAutoDetectResponsePDU(
						bandwidthMeasureResults(0x0003,_self._byteCountStore,Date.now() - _self._networkCharacteristicsTimer)
					));
				break;
			case 0x0429:
				//TODO enable multitransport option - send over Reliable UDP

				// autoDetectReqPduData  -- https://msdn.microsoft.com/en-us/library/jj678772.aspx
				_self.send(
					_self.clientAutoDetectResponsePDU(
						bandwidthMeasureResults(0x0003,_self._byteCountStore,Date.now() - _self._networkCharacteristicsTimer)
					));
				break;
			case 0x0629:
				if(_self._networkCharacteristicSequenceNumberStore !== seqNum){return;}
				//TODO send over lossy UDP
		}
	}
	function networkCharacteristicResult(_self,reqType,dv){
		var o = {
			baseRTT:null,
			averageRTT:null,
			bandwidth:null
		};
		switch(reqType){
			case 0x0840:
				o.baseRTT = dv.getUin32(0,true);
				o.averageRTT = dv.getUint32(4,true);
				break;
			case 0x0880:
				o.bandwidth = dv.getUin32(0,true);
				o.averageRTT = dv.getUint32(4,true);
				break;
			case 0x08C0:
				o.baseRTT = dv.getUin32(0,true);
				o.bandwidth = dv.getUint32(4,true);
				o.averageRTT = dv.getUin32(8,true);
				break;
		}
		document.dispatchEvent(new CustomEvent('rdpNetworkCharacteristics',{detail:o}));
	}

	function process(dv){
		var sec = securityHeaderReader.process(dv,this.encryptionLevel,this.encryptionMethods);

		var len = sec.data.getUint8(0);
		var type = sec.data.getUint8(1);
		seqNum = sec.data.getUint16(2,true);
		var reqType = sec.data.getUint16(4,true);
		switch(reqType){
			case 0x0001://autodetect_request https://msdn.microsoft.com/en-us/library/hh880899.aspx
			case 0x1001:
				rttMeasureRequest(this,reqType);
				break;
			case 0x0014:
			case 0x0114:
			case 0x1014:
				bandwidthMeasureStart(this,reqType)
				break;
			case 0x0002: //https://msdn.microsoft.com/en-us/library/hh881104.aspx
				bandwidthMeasurePayload(this,new DataView(sec.data.buffer,
						sec.data.byteOffset +8,sec.data.byteLength - 8 - sec.data.getUint16(6)));
				break;
			case 0x002B: //https://msdn.microsoft.com/en-us/library/hh881042.aspx
			case 0x0429: //https://msdn.microsoft.com/en-us/library/hh881042.aspx
			case 0x0629: //https://msdn.microsoft.com/en-us/library/hh881042.aspx
				if(reqType === 0x002B){
					bandwidthMeasureStop(this,reqType,sec.data.getUint16(6));
				}else{
					bandwidthMeasureStop(this,reqType);
				}
				break;
			case 0x0840: //https://msdn.microsoft.com/en-us/library/jj217487.aspx
			case 0x0880: //https://msdn.microsoft.com/en-us/library/jj217487.aspx
				if(len === 0x0E){
					networkCharacteristicResult(this,reqType,new DataView(sec.data.buffer,
						sec.data.byteOffset +6,sec.data.byteLength - 6));
				}
				break;
			case 0x08C0: //https://msdn.microsoft.com/en-us/library/jj217487.aspx
				if(len === 0x12){
					networkCharacteristicResult(this,reqType,new DataView(sec.data.buffer,
						sec.data.byteOffset +6,sec.data.byteLength - 6));
				}
				break;
			default:
				break;
		}
		return len;
	}
	MessageChannel.prototype.process = process;

module.exports = MessageChannel;
