'use strict';
    var virtualChannel      = require('./virtual-channel');
    var DataStream          = require('DataStream');
    var StringView          = require('StringView');


    var Rail = function(socket,userChannelId,channelId,canvas){
		Rail.canvas = canvas;
		Rail.vc = new virtualChannel(socket,userChannelId,channelId);
	}
	function readPDUHeader(dv){
		return {
			flags:dv.getUint32(4,true),
			data:new DataView(dv.buffer,dv.byteOffset + 8,dv.getUint32(0,true))
		}
	}
	function readRailHeader(dv){
		var o ={};
		o.type = dv.getUint16(0,true);
		o.data = new DataView(dv.buffer,dv.byteOffset + 2,dv.getUint16(2,true) - 4);
		return o;
	}
	function readHeader(dv){
		var d = readPDUHeader(dv);
		if(d.data){
			return readRailHeader(d.data);
		}
	}
	function writeHeaderAndData(orderType,buf){
		var ds = new DataStream();
		ds.writeUint16(orderType);
		ds.writeUint16(buf.byteLength + 4);
		ds.writeUint8Array(new Uint8Array(buf));
		return ds.buffer;
	}
	function writeHandshakePDU(buildNumber){
		var ds = new DataStream();
		ds.writeUint32(buildNumber);
		return writeHeaderAndData(0x0005,ds.buffer);
	}
	function writeClientInformationPDU(flags){
		var buf = new ArrayBuffer(4);
		var dv = new DataView(buf);
		dv.setUint32(0,flags,true)
		return writeHeaderAndData(0x000B,buf);
	}

	function setBool(spi,bool){
		var buf = new ArrayBuffer(1);
		var dv = new DataView(buf);
		dv.setUint8(0,(bool)?1:0);
		return writeSystemParametersUpdatePDU(spi,buf);
	}
	function setRect(spi,rect){
		var buf = new ArrayBuffer(8);
		var dv = new DataView(buf);
		dv.setUint16(0,rect[0],true);
		dv.setUint16(2,rect[1],true);
		dv.setUint16(4,rect[2],true);
		dv.setUint16(6,rect[3],true);
		return writeSystemParametersUpdatePDU(spi,buf);
	}
	function setDragFullWindows(bool){
		return setBool(0x00000025,bool);
	}
	function setKeyboardCues(bool){
		return setBool(0x0000100B,bool);
	}
	function setKeyboardPref(bool){
		return setBool(0x00000045,bool);
	}
	function setMouseButtonSwap(bool){
		return setBool(0x00000021,bool);
	}
	function setWorkArea(rect){
		return setRect(0x0000002F,rect);
	}
	function setDisplayChange(rect){
		return setRect(0x0000F001,rect);
	}
	function setTaskbarPos(rect){
		return setRect(0x0000F000,rect);
	}
	function setHighContrast(flags,colorScheme){
		var ds = new DataStream();
		ds.writeUint32(flags);
		ds.writeUint32( (colorScheme.length)?colorScheme.length *2 + 2:0);
		if(colorScheme.length){
			ds.writeUint16(colorScheme.length *2);
			ds.writeUCS2String(colorScheme);
		}
		return writeSystemParametersUpdatePDU(0x00000043,ds.buffer);
	}

	function writeSystemParametersUpdatePDU(param,body){
		var buf = new ArrayBuffer(4 + body.byteLength);
		var dv = new DataView(buf);
		dv.setUint32(0,param,true);
		(new Uint8Array(buf)).set(new Uint8Array(body),4);
		return writeHeaderAndData(0x0003,buf);
	}

	Rail.cb = {rail:{}};

	function moveResize(id,rect){
		var buf = new ArrayBuffer(12);
		var dv = new DataView(buf);
		dv.setUint32(0,id,true);
		dv.setUint16(4,rect[0],true);
		dv.setUint16(6,rect[1],true);
		dv.setUint16(8,rect[2],true);
		dv.setUint16(10,rect[3],true);
		Rail.vc.send(writeHeaderAndData(0x0008,buf));
	}
	Rail.cb.rail.execute = function(){
		throw new Error('Execute Not Supported ... yet');
	}
	Rail.cb.rail.activate = function(id){
		var buf = new ArrayBuffer(5);
		var dv = new DataView(buf);
		dv.setUint32(0,id,true);
		dv.setUint8(4,1);
		Rail.vc.send(writeHeaderAndData(0x0002,buf));
	}
	Rail.cb.rail.deactivate = function(id){
		var buf = new ArrayBuffer(5);
		var dv = new DataView(buf);
		dv.setUint32(0,id,true);
		dv.setUint8(4,0);
		Rail.vc.send(writeHeaderAndData(0x0002,buf));
	}
	Rail.cb.rail.sysCommand = function(id,command){
		if([0xF000,0xF010,0xF020,0xF030,0xF060,0xF100,0xF120,0xF160].indexOf(command) < 0){
			throw new Error('Invalid System Command');
		}
		var buf = new ArrayBuffer(6);
		var dv = new DataView(buf);
		dv.setUint32(0,id,true);
		dv.setUint16(4,command,true);
		Rail.vc.send(writeHeaderAndData(0x0004,buf));
	}

	Rail.cb.rail.sysSizeWindow = function(id){
		Rail.cb.rail.sysCommand(id,0xF000);
	}
	Rail.cb.rail.sysMoveWindow = function(id){
		Rail.cb.rail.sysCommand(id,0xF010);
	}
	Rail.cb.rail.minimizeWindow = function(id){
		Rail.cb.rail.sysCommand(id,0xF020);
	}
	Rail.cb.rail.maximizeWindow = function(id){
		Rail.cb.rail.sysCommand(id,0xF030);
	}
	Rail.cb.rail.closeWindow = function(id){
		Rail.cb.rail.sysCommand(id,0xF060);
	}
	Rail.cb.rail.keyMenu = function(id){
		Rail.cb.rail.sysCommand(id,0xF100);
	}
	Rail.cb.rail.restoreWindow = function(id){
		Rail.cb.rail.sysCommand(id,0xF120);
	}
	Rail.cb.rail.sysMenuDefault = function(id){
		Rail.cb.rail.sysCommand(id,0xF160);
	}

	Rail.cb.rail.notify = function(){}
	Rail.cb.rail.windowMove = moveResize;
	Rail.cb.rail.windowResize = moveResize;
	Rail.cb.rail.status = function(){

	}
	Rail.cb.rail.sysMenu = function(id,left,top){
		var buf = new ArrayBuffer(8);
		var dv = new DataView(buf);
		dv.setUint32(0,id,true);
		dv.setUint16(4,left,true);
		dv.setUint16(6,top,true);
		Rail.vc.send(writeHeaderAndData(0x000C,buf));
	}
	Rail.cb.rail.getAppIdReq = function(id){
		var buf = new ArrayBuffer(4);
		var dv = new DataView(buf);
		dv.setUint32(0,id,true);
		Rail.vc.send(writeHeaderAndData(0x000E,buf));
	}
	Rail.cb.rail.languageImeInfo = function(){
		throw new Error('IME Not Supported');
	}
	Rail.cb.rail.compartmentIfo = function(){
		throw new Error('IME Not Supported');
	}

	Object.defineProperty(Rail.prototype,'callbacks',{
		get:function(){return Rail.cb;}
	});

	Rail.prototype.processMessage = function processMessage(dv){
		var data = readHeader(dv);
		if(!data){return;}
		switch(data.type){
			case 0x0003://sysparam
				console.log('sys param update');
				break;
			case 0x0005://handshake
				Rail.vc.send(writeHandshakePDU(0x0000b10d));
				Rail.vc.send(writeClientInformationPDU(0x00000001 | 0x00000004));
				Rail.vc.send(setMouseButtonSwap(0));
				Rail.vc.send(setKeyboardPref(0))
				Rail.vc.send(setDragFullWindows(0));
				Rail.vc.send(setKeyboardCues(0));
				Rail.vc.send(setWorkArea([0,0,6000,3000]));
				document.dispatchEvent(new CustomEvent('snRailEnabled'));

				break;
			case 0x0009: //local move-size
				console.log('move/size');
				break;
			case 0x000a: //minmax info
				console.log('min/max info');
				break;
			case 0x000d://lang bar
				console.log('langbar info');
				break;
			case 0x0080: //exec result
				console.log('exec result');
				break;
			case 0x000F://get app id response
				console.log('get App id response');
				break;
			case 0x0012:// compartment status info
				console.log('compartment status');
				break;
			case 0x0013: //handshake ex
				console.log('handshake ex');
				Rail.vc.send(writeHandshakePDU(0x0000b10d));
				break;
			case 0x0014://zorder sync
				console.log('z-order');
				break;
			default:
				console.log('Unknown rail option',data);
				break;
		}
	}
module.exports = Rail;
