'use strict';
    var virtualChannel      = require('./virtual-channel');
    var DataStream          = require('DataStream');
    var StringView          = require('StringView');


	function writeHeader(buffer,type,flags){
		flags = flags || 0x00;
		var u8 = new Uint8Array(buffer.byteLength +8);
		u8.set(new Uint8Array(buffer),8);
		var dv = new DataView(u8.buffer);
		dv.setUint16(0,type,true);
		dv.setUint16(2,flags,true);
		dv.setUint32(4,buffer.byteLength,true);
		return dv.buffer;
	}

	function optionsUpdatedNotify(dv){
		var str = new StringView(dv.buffer,'UTF-8',dv.byteOffset,dv.byteLength);
		var o = JSON.parse(str.toString());
		document.dispatchEvent(new CustomEvent('snCfgUpdate',{
			detail:o
		}));
	}

	function setOptionsResponse(dv){
		var str = new StringView(dv.buffer,'UTF-8',dv.byteOffset,dv.byteLength);
		var o = JSON.parse(str.toString());
		document.dispatchEvent(new CustomEvent('snCfgResponse',{
			detail:o
		}));
	}



	var Sncfg = function(socket,userChannelId,channelId,canvas){
		Sncfg.canvas = canvas;
		Sncfg.vc = new virtualChannel(socket,userChannelId,channelId);
	}

	Sncfg.cb = {};

	Sncfg.cb.sendSetConfigOptionsRequest = function sendSetConfigOptionsRequest(o){
		var sv = new StringView(JSON.stringify(o),'UTF-8');
		Sncfg.vc.send(writeHeader(sv.buffer,2));
	}
	Object.defineProperty(Sncfg.prototype,'callbacks',{
		get:function(){return Sncfg.cb;}
	});
	Sncfg.prototype.processMessage = function processMessage(dv){
		var data = Sncfg.vc.processMessage(dv);
		if(!data){return;}
		switch(data.type){
			case 1://optionsUpdatedNotify
				optionsUpdatedNotify(data.data);
				break;
			case 2://setOptionsRequest
				console.log('Cannot send a request to a client');
				break;
			case 3://setOptionsResponse
				setOptionsResponse(data.data);
				break;
			default:
				console.log('Unknown sncfg option');
				break;
		}
	}

module.exports = Sncfg;
