'use strict';
    var virtualChannel      = require('./virtual-channel');
    var DataStream          = require('DataStream');
    var StringView          = require('StringView');


    var Snrail = function(socket,userChannelId,channelId,canvas){
		Snrail.canvas = canvas;
		Snrail.vc = new virtualChannel(socket,userChannelId,channelId);
	}
	function writeHeaderAndData(orderType,buf){
		var ds = new DataStream();
		ds.writeUint16(orderType);
		ds.writeUint16(0);
		ds.writeUint16(buf.byteLength + 4);
		ds.writeUint8Array(new Uint8Array(buf));
		return ds.buffer;
	}
	Snrail.cb = {snrail:{}};

	/*
	 * SNRAIL_WINDOW_MOVE (0x05)
	 * Sent from the client to the server when a window is moved. This is almost identical
	 * to the RAIL_WINDOW_MOVE message except that the positions are signed and allow windows moved off screen
	 *
	*/
	Snrail.cb.snrail.move = function(id,rect){
		var buf = new ArrayBuffer(20);
		var dv = new DataView(buf);
		dv.setUint32(0,id,true);
		dv.setInt32(4,rect[0],true);//left
		dv.setInt32(8,rect[1],true);//top
		dv.setInt32(12,rect[2],true);//right
		dv.setInt32(16,rect[3],true);//bottom
		Snrail.vc.send(writeHeaderAndData(0x0005,buf));
	}
	/*
	 * SNRAIL_WINDOW_RAISE (0x06)
	 * Sent when a window is raised to the top. The RAIL protocol only supports
	 * the RAIL_ACTIVATE message which activates the window and raises it to the top as well
	 */
	Snrail.cb.snrail.raise = function(id){
		var buf = new ArrayBuffer(4);
		var dv = new DataView(buf);
		dv.setUint32(0,id,true);
		Snrail.vc.send(writeHeaderAndData(0x0006,buf));
	}
	/*
	 * SNRAIL_WINDOW_ZORDER (0x07)
	 * Move a window in the zorder below the window with id aboveId
	*/
	Snrail.cb.snrail.zorder = function(id,aboveId){
		var buf = new ArrayBuffer(8);
		var dv = new DataView(buf);
		dv.setUint32(0,id,true);
		dv.setUint32(4,aboveId,true);
		Snrail.vc.send(writeHeaderAndData(0x0007,buf));
	}

	/*
	 * SNRAIL_WINDOW_ACTIVATE (0x08)
	 * Activate the window. The RAIL protocol only supports the RAIL_ACTIVATE
	 * message which activates the window and raises it to the top as well
	 */
	Snrail.cb.snrail.activate = function(id){
		var buf = new ArrayBuffer(4);
		var dv = new DataView(buf);
		dv.setUint32(0,id,true);
		Snrail.vc.send(writeHeaderAndData(0x0008,buf));
	}

	/*
	 * SNRAIL_MONITOR_LAYOUT (0x09)
	 * Sent by the client to the server to describe the layout of attached monitors. The server will
	 * set the RANDR monitors accordingly and allows programs to position windows
	 * on the monitors and align popup menus with monitor borders.
	 */
	//NOT Implemented.  Only 1 monitor in the Browser

	Object.defineProperty(Snrail.prototype,'callbacks',{
		get:function(){return Snrail.cb;}
	});
module.exports = Snrail;
