'use strict';

    var virtualChannel      = require('./virtual-channel');
    var DataStream          = require('DataStream');
    var StringView          = require('StringView');


	function generatePassword(){
		if(window.crypto){
			var array = new Uint8Array(16);
			window.crypto.getRandomValues(array);
			var str = [];
			var i = array.length;
			while(i--){
				str.push(array[i].toString(16));
			}
			return str.join('');
		}else{//IE 10
			var n = 16;
			var str = [];
			while(n--){
				str.push(Math.floor(Math.random()*15).toString(16));
			}
			return str.join('');
		}
	}

	function readPDUHeader(dv){
		return {
			flags:dv.getUint32(4,true),
			data:new DataView(dv.buffer,dv.byteOffset + 8,dv.getUint32(0,true))
		}
	}
	function readHeader(dv){
		var o = {};
		o.type = dv.getUint16(0,true);
		o.flags = dv.getUint16(2,true);
		o.data = new DataView(dv.buffer,dv.byteOffset + 8,dv.getUint32(4,true));
		return o;
	}
	function writeHeader(type,flags,dataBuf){
		var ds = new DataStream();
		ds.writeUint16(type);
		ds.writeUint16(flags);
		ds.writeUint32(dataBuf.byteLength);
		ds.writeUint8Array(new Uint8Array(dataBuf));
		return ds.buffer;
	}
	function clientId(dv){
		Snshare.myClientId = dv.getUint32(0,true);
		document.dispatchEvent(new CustomEvent('snShareId',{
			detail:{
				shareId:Snshare.myClientId
			}
		}));
	}
	function serverStatus(dv){
		var offset =1;
		Snshare.isSharing = dv.getUint8(0);
		Snshare.clients = [];
		var name,c,client;

		while(offset < dv.byteLength){
			name = [];
			client = {
				id:dv.getUint32(offset,true),
				flags:dv.getUint16(offset+4,true),
				hasControl:Boolean(dv.getUint16(offset+4,true) & 0x03)
			};
			offset+=6;
			while(c = dv.getUint16(offset,true)){
				name.push(String.fromCharCode(c));
				offset +=2;
			};
			offset+=2;
			client.name = name.join('');
			Snshare.clients.push(client);
		}
		document.dispatchEvent(new CustomEvent('snShareData',{
			detail:{
				isSharing:Snshare.isSharing,
				clients:Snshare.clients,
			}
		}));
	}
	function startSharing(dv){
		console.log('start sharing');
	}
	function endSharing(dv){
		console.log('end sharing');
	}
	function kickUser(dv){
		console.log('kick sharing');
	}
	function giveControl(dv){
		console.log('giveContorl sharing');
	}
	function disconnectInfo(dv){
		console.log('disconnectInfo sharing');
	}

	var Snshare = function(socket,userChannelId,channelId,canvas,fastx){
		Snshare.socket = socket;
		Snshare.vc = new virtualChannel(socket,userChannelId,channelId);
		Snshare.isSharing =0;
		Snshare.clients = [];
		Snshare.canvas = canvas;
		Snshare.fastx = fastx;

	}
	Snshare.cb = {sharing:{}};
	Object.defineProperty(Snshare.cb,'isSharing',{
		get:function(){return Snshare.isSharing;}
	});
	Object.defineProperty(Snshare.cb,'clients',{
		get:function(){return Snshare.clients;}
	});
	Object.defineProperty(Snshare.cb,'myClientId',{
		get:function(){return Snshare.myClientId;}
	});

	Object.defineProperty(Snshare.prototype,'callbacks',{
		get:function(){return Snshare.cb;}
	});


	Snshare.prototype.processMessage = function(dv){
		var pdu = readPDUHeader(dv);
		if(!pdu.data){return;}
		var header = readHeader(pdu.data);
		if(!header.type){return;}
		switch(header.type){
			case 1://client id
				clientId(header.data);
				break;
			case 2://server status
				serverStatus(header.data);
				break;
			case 3: //start sharing
				startSharing(header.data);
				break;
			case 4: //end sharing
				endSharing(header.data);
				break;
			case 5: //kick user
				kickUser(header.data);
				break;
			case 6: //give control
				giveControl(header.data);
				break;
			case 7: //disconnect info
				disconnectInfo(header.data);
				break;
			default:
				console.log('Unknown share type');
				break;
		}
	}


	Snshare.cb.sharing.start = function(){
		if(Snshare.myClientId !== 0){return;}
		var password = generatePassword();

		var ds = new DataStream();
		ds.writeUCS2String(password);
		Snshare.vc.send(writeHeader(3,0,ds.buffer));
		var str = JSON.stringify({
				RdpPort:Snshare.fastx.port,
				password:password,
				id:Snshare.fastx.id
			});

		//var key = (new StringView(str,'UTF-8')).toBase64();
		var key = btoa(str);
		return [window.location.origin,'/share?',key].join('');
	}
	Snshare.cb.sharing.end = function(){
		if(Snshare.myClientId !== 0){return;}
		Snshare.vc.send(writeHeader(4,0,new ArrayBuffer(0)));
	}

	Snshare.cb.sharing.kick = function(id){
		if(Snshare.myClientId !== 0){return;}
		var buf = new ArrayBuffer(4);
		var dv = new DataView(buf);
		dv.setUint32(0,id,true);
		Snshare.vc.send(writeHeader(5,0,buf));
	}

	Snshare.cb.sharing.control = function(id){
		if(Snshare.myClientId !== 0){return;}
		var buf = new ArrayBuffer(4);
		var dv = new DataView(buf);
		dv.setUint32(0,id,true);
		Snshare.vc.send(writeHeader(6,0,buf));
	}
module.exports = Snshare;
