'use strict';

    var virtualChannel      = require('./virtual-channel');
    var DataStream          = require('DataStream');
    var StringView          = require('StringView');

	function generatePassword(){
		if(window.crypto){
			var array = new Uint8Array(16);
			window.crypto.getRandomValues(array);
			var str = [];
			var i = array.length;
			while(i--){
				str.push(array[i].toString(16));
			}
			return str.join('');
		}else{//IE 10
			var n = 16;
			var str = [];
			while(n--){
				str.push(Math.floor(Math.random()*15).toString(16));
			}
			return str.join('');
		}
	}

	function readPDUHeader(dv){
		return {
			flags:dv.getUint32(4,true),
			data:new DataView(dv.buffer,dv.byteOffset + 8,dv.getUint32(0,true))
		}
	}
	function readHeader(dv){
		var o = {};
		o.type = dv.getUint16(0,true);
		o.flags = dv.getUint16(2,true);
		o.data = new DataView(dv.buffer,dv.byteOffset + 8,dv.getUint32(4,true));
		return o;
	}
	function writeHeader(type,flags,dataBuf){
		var ds = new DataStream();
		ds.writeUint16(type);
		ds.writeUint16(flags);
		ds.writeUint32(dataBuf.byteLength);
		ds.writeUint8Array(new Uint8Array(dataBuf));
		return ds.buffer;
	}

	function readPerms(perm){

	}
	function client(dv){
		var o ={},strLen;
		var ds = new DataStream(dv.buffer,dv.byteOffset);
		o.id = ds.readUint32();
		strLen = ds.readUint16();
		o.name = ds.readString(strLen,'UTF8');
		var perms = ds.readUint16();
		o.isMaster = perms & 0x0001;
		o.isGranter = perms & 0x0002;
		var flags = ds.readUint16();
		o.keyboard = flags & 0x0001;
		o.mouse = flags & 0x0002;

		return {
			data:o,
			dv:new DataView(dv.buffer,dv.byteOffset + 10 + strLen,dv.byteLength -10 -strLen)
		}
	}

	function sharingReady(dv){
		var o ={};
		o.id = dv.getUint32(0,true);
		var perms = dv.getUint16(4,true);
		o.isMaster = perms & 0x0001;
		o.isGranter = perms & 0x0002;
		document.dispatchEvent(new CustomEvent('snShareReady',{
			detail:o
		}));
	}

	function clientList(dv){

		var len = dv.getUint16(0,true);
		var clients = [],c;

		dv = new DataView(dv.buffer,dv.byteOffset +2,dv.byteLength -2);

		while(len--){
			c = client(dv);
			clients.push(c.data);
			dv = c.dv;
		}

		document.dispatchEvent(new CustomEvent('snShareClients',{detail:clients}));
	}
	function password(dv){
		var ds = new DataStream(dv.buffer,dv.byteOffset);
		document.dispatchEvent(new CustomEvent('snSharePassword',{
			detail:{password:ds.readString(ds.readUint16(),'UTF8')}
		}));
	}

	function mouseEvent(dv){
		var o ={};
		var flags = dv.getUint32(4,true);
		o.pos = [dv.getUint16(0,true),dv.getUint16(2,true)];
		o.keyboard = flags & 0x00000001;
		o.mouse = flags &  0x00000002;
		document.dispatchEvent(new CustomEvent('snShareMouseEvent',{detail:o}));
	}

	var Snshare = function(socket,userChannelId,channelId,canvas,fastx){
		Snshare.socket = socket;
		Snshare.vc = new virtualChannel(socket,userChannelId,channelId);
		Snshare.canvas = canvas;
		Snshare.fastx = fastx;

	}
	Snshare.cb = {sharing:{}};

	Object.defineProperty(Snshare.prototype,'callbacks',{
		get:function(){return Snshare.cb;}
	});
	Snshare.cb.sharing.getClientList = function(){
		Snshare.vc.send(writeHeader(0x02,0,new ArrayBuffer(0)));
	}
	Snshare.cb.sharing.getPassword = function(granter,timeout){
		var ab = new ArrayBuffer(6);
		var dv = new DataView(ab);
		var perms =0;
		if(granter){
			perms &= 0x02;
		}
		dv.setUint16(0,perms,true);
		dv.setUint32(2,timeout,true);
		Snshare.vc.send(writeHeader(0x04,0,ab));
	}
	Snshare.cb.sharing.kickClient = function(id){
		var ab = new ArrayBuffer(4);
		var dv = new DataView(ab);
		dv.setUint32(0,id,true);
		Snshare.vc.send(writeHeader(0x06,0,ab));
	}
	Snshare.cb.sharing.grantControl = function(id){
		var ab = new ArrayBuffer(4);
		var dv = new DataView(ab);
		dv.setUint32(0,id,true);
		Snshare.vc.send(writeHeader(0x07,0,ab));
	}

	Snshare.prototype.processMessage = function(dv){
		var pdu = readPDUHeader(dv);
		if(!pdu.data){return;}
		var header = readHeader(pdu.data);
		if(!header.type){return;}
		switch(header.type){
			case 0x01://SHARING_READY
				sharingReady(header.data);
				break;
			case 0x03: //CLIENT_LIST
				clientList(header.data);
				break;
			case 0x05: //PASSWORD
				password(header.data);
				break;
			case 0x08: //MOUSE_EVENT
				mouseEvent(header.data);
				break;
			case 0x02://GET_CLIENT_LIST
			case 0x04: //GET_PASSWORD
			case 0x06: //KICK_CLIENT
			case 0x07: //GRANT_CONTROL
				console.log('Server sent a client message');
				break;
			default:
				console.log('Unknown share type');
				break;
		}
	}
module.exports = Snshare;
