'use strict';

    var DataStream      = require('DataStream');
    var mcs             = require('xrdp/core/mcs');
    var tpkt            = require('xrdp/core/tpkt');
    var x224            = require('xrdp/core/x224');

	function readHistoryData(dv,flags,totalLength,o){
		if(flags & 0x01){//first
			o.history = new ArrayBuffer(totalLength);
			o.historyOffset = 0;
		}
		var len = Math.min(dv.byteLength,totalLength - o.historyOffset);
		(new Uint8Array(o.history)).set(new Uint8Array(dv.buffer,dv.byteOffset,dv.byteLength),o.historyOffset,len);
		o.historyOffset += dv.byteLength;
		if(flags & 0x02){//last
			return new DataView(o.history);
		}
	}
	function readPDUHeader(dv,o){
		var totalLength = dv.getUint32(0,true);
		var flags = dv.getUint32(4,true);
		var data = readHistoryData(new DataView(dv.buffer,dv.byteOffset + 8,dv.byteLength - 8),flags,totalLength,o);
		if(data){
			return {
				flags:flags,
				data:data
			}
		}
	}
	function writeNextChunk(buffer,bytesSent,flags,o){
		if(bytesSent >= buffer.byteLength){return;}
		flags = flags || 0x00;
		var seg = 0x00;
		if(bytesSent === 0){
			seg |= 0x01
		}
		if(buffer.byteLength - bytesSent <= o.chunkSize){
			seg |= 0x02; //last in sequence
		}
		flags |= seg;
		if(buffer.byteLength > o.chunkSize){
			flags |= 0x10; //show protocol flag
		}
		var len = Math.min(o.chunkSize,buffer.byteLength - bytesSent);
		var buf = new ArrayBuffer(len+8)
		var dv = new DataView(buf);
		dv.setUint32(0,buffer.byteLength,true);//total Length
		dv.setUint32(4,flags,true);
		(new Uint8Array(buf)).set(new Uint8Array(buffer,bytesSent,len),8);
		return tpkt.writePacket(x224.writeDataPDU(
			mcs.writeSDrqHeader(o.userChannelId,o.virtualChannelId,0x01,0x03,
			new Uint8Array(buf))));
		}
	function send(buffer,flags,o){
		var s,bytesSent =0;
		while(s = writeNextChunk(buffer,bytesSent,flags,o)){
			o.socket.send(s);
			bytesSent += Math.min(o.chunkSize,buffer.byteLength - bytesSent);
		}
	}

	function processMessage(dv,o){
		var data = readPDUHeader(dv,o);
		if(data){
			return readMessageHeader(data.data);
		}
	}
	function readMessageHeader(dv){
		if(!dv){return;}
		var o = {};
		o.type = dv.getUint16(0,true);
		o.flags = dv.getUint16(2,true);
		if(dv.byteOffset + 8 < dv.byteLength){
			o.data = new DataView(dv.buffer,dv.byteOffset + 8,dv.getUint32(4,true));
		}
		return o;
	}

module.exports = function(s,ucId,vcId,chunk){
    this.options = {
            socket:s,
            virtualChannelId:vcId,
            userChannelId:ucId,
            chunkSize:chunk || 1600,
            history:new ArrayBuffer(0),
            historyOffset:0,
    }
    var that = this
    return {
            send:function(buffer,flags){
                    send(buffer,flags,that.options);
            },
            processMessage:function(dv){
                    return processMessage(dv,that.options);
            }
    }
}
