'use strict';

var channelJoin     = require('./channel-join');
var tpkt            = require('xrdp/core/tpkt');
var x224            = require('xrdp/core/x224');

function readResult(result){
    var reason = '';
    switch(result){
            case 0:break;//success
            case 1:reason += 'Domain Merging';break;
            case 2:reason += 'Domain Not Hierarchical';break;
            case 3:reason += 'No Such Channel';break;
            case 4:reason += 'No Such Domain';break;
            case 5:reason += 'No Such User';break;
            case 6:reason += 'Not Admitted';break;
            case 7:reason += 'Other UserId';break;
            case 8:reason += 'Parameters Unacceptible';break;
            case 9:reason += 'Token Not Available';break;
            case 10:reason += 'Token Not Possessed';break;
            case 11:reason += 'Too Many Channels';break;
            case 12:reason += 'Too Many Tokens';break;
            case 13:reason += 'Too Many Users';break;
            case 14:reason += 'Unspecified Failure';break;
            case 15: reason += 'User Rejected';break;
            default:reason += 'Invalid Reason Code';
    }
    if(reason.length){throw new Error('Error in result. Reason: ' + reason);}
    return result;
}
function readDomainMCSPDUChoice(byte){
    return byte >> 2;
}
function decodeAttachUserConfirm(dv,store){
    var byte = dv.getUint8(0);
    if(readDomainMCSPDUChoice(byte) !== 11){
            throw new Error('bad confirm choice');
    }
    if(!(byte >> 1 & 0x01)){
            throw new Error('initiator not present');
    }
    readResult(dv.getUint8(1) >> 4 + (byte & 0x01))
    store.userChannelId = dv.getUint16(2);
}
function processAttachUser(socket,Client,dv){
    var data = tpkt.parsePacket(dv);
    if(!data.payload){return 0;}
    data = x224.parseConnectPDU(data.payload);
    if(!data.data){return 0;}

    decodeAttachUserConfirm(data.data,Client.store);
    socket.send(channelJoin.request(Client.store.userChannelId,Client.store.userChannelId + 1001));
    return dv.getUint16(2,false);
}

module.exports = {
    process:processAttachUser
}
