'use strict';

    var DataStream      = require('DataStream');
    var clientInfo      = require('./client-info');
    var tpkt            = require('xrdp/core/tpkt');
    var x224            = require('xrdp/core/x224');
    var mcs             = require('xrdp/core/mcs');
    var Security        = require('xrdp/security/header/util');

	function readResult(result){
		var reason = '';
		switch(result){
			case 0:break;//success
			case 1:reason += 'Domain Merging';break;
			case 2:reason += 'Domain Not Hierarchical';break;
			case 3:reason += 'No Such Channel';break;
			case 4:reason += 'No Such Domain';break;
			case 5:reason += 'No Such User';break;
			case 6:reason += 'Not Admitted';break;
			case 7:reason += 'Other UserId';break;
			case 8:reason += 'Parameters Unacceptible';break;
			case 9:reason += 'Token Not Available';break;
			case 10:reason += 'Token Not Possessed';break;
			case 11:reason += 'Too Many Channels';break;
			case 12:reason += 'Too Many Tokens';break;
			case 13:reason += 'Too Many Users';break;
			case 14:reason += 'Unspecified Failure';break;
			case 15: reason += 'User Rejected';break;
			default:reason += 'Invalid Reason Code';
		}
		if(reason.length){console.log('Error in result. Reason: ' + reason);}
		return result;
	}
	function writeDomainMCSPDUChoice(num){
		if(num >= 0 && num <= 42){
			return num << 2;
		}
	}
	function readDomainMCSPDUChoice(byte){
		return byte >> 2;
	}
	function channelJoinRequest(initiator,channelId){
			var ds = new DataStream();
			ds.writeUint8(writeDomainMCSPDUChoice(14));
			ds.writeUint16(initiator,DataStream.BIG_ENDIAN);
			ds.writeUint16(channelId,DataStream.BIG_ENDIAN);
			return tpkt.writePacket(x224.writeDataPDU(ds.buffer));
		}
	function decodeChannelJoinConfirm(dv,channelId){
			var ds = new DataStream(dv.buffer,dv.byteOffset);
			var byte = ds.readUint8();
			if(readDomainMCSPDUChoice(byte) !== 15){
				console.log('bad channel Join confirm choice');
				return undefined;
			}
			if(!(byte >> 1 & 0x01)){
				console.log('channelId not present');
				return undefined;
			}
			if(readResult(ds.readUint8() >> 4 + (byte & 0x01))){return undefined;}
			ds.readUint16();//ignore initiator
			ds.readUint16();//ignore requested
			if(ds.readUint16(DataStream.BIG_ENDIAN) !== channelId){
				console.log('bad channelId');
				return undefined;
			}
			return 1;
		}

	function getClientRandom(){
		var clientRandom = new Uint8Array(32);
		var cryptoObj = window.crypto || window.msCrypto; // for IE 11
		cryptoObj.getRandomValues(clientRandom);
		return clientRandom;
	}
	function generateSessionKeys(clientRandom,serverRandom,encryptionMethod){
		return Security.generateSessionKeys({
			clientRandom:clientRandom,
			serverRandom:serverRandom,
			encryptionMethod:encryptionMethod
		});
	}
	function messageChannel(socket,Client,connection){
		if(Client.store.messageChannelId){
			socket.send(channelJoinRequest(Client.store.userChannelId,Client.store.messageChannelId));
			connection.state = 'channel_connection_message_channel';
		}else{
			sendSecurityAndClientInfo(socket,Client,connection);
		}
	}
	function sendSecurityAndClientInfo(socket,Client,connection){
		if(Client.store.receivedServerData.encryptionLevel || Client.store.receivedServerData.encryptionMethod){
			var clientRandom = getClientRandom();
			Client.store.sessionKeys = generateSessionKeys(
				clientRandom,
				new Uint8Array(Client.store.receivedServerData.serverRandom.buffer,
					Client.store.receivedServerData.serverRandom.byteOffset,
					Client.store.receivedServerData.serverRandom.byteLength),
				Client.store.receivedServerData.encryptionMethod
			);
			socket.send(securityExchange(
				Client.store.userChannelId,
				Client.store.ioChannelId,
				Client.store.receivedServerData.serverCertificate.data.exp,
				Client.store.receivedServerData.serverCertificate.data.mod,
				Client.store.receivedServerData.serverCertificate.data.keyLen,
				clientRandom
			));
		}
		socket.send(clientInfo(
			Client.store.userChannelId,
			Client.store.ioChannelId,
			Client.store.receivedServerData.encryptionLevel,
			Client.store.sessionKeys,
			Client.settings));
		connection.state = 'licensing';
	}
	function securityExchange(userChannelId,ioChannelId,exp,mod,keyLen,clientRandom){
		var b = new BigInteger(clientRandom.map(function(n){
			return (n.toString(16).length ===2)?n.toString(16):('0'+n.toString(16));
		}).join(''),16);
		var mp = b.modPow(exp,mod);
		var e = new Uint8Array(mp.toByteArray().reverse());
		var ds = new DataStream();
		ds.writeUint16(0x0001);//flags
		ds.writeUint16(0x0000);//flagsHi
		ds.writeUint32(keyLen);//random size
		ds.writeUint8Array(new Uint8Array(e));//encrypted 32 byte random number
		if(keyLen - e.length){
			ds.writeUint8Array(new Uint8Array(keyLen - e.length) );
		}
		return tpkt.writePacket(x224.writeDataPDU(mcs.writeSDrqHeader(userChannelId,ioChannelId,0x01,0x03,new Uint8Array(ds.buffer))));
	}

	function processChannelConnectionUserChannel(socket,Client,connection,dv){
		var data = tpkt.parsePacket(dv);
		if(!data.payload){return 0;}
		data = x224.parseConnectPDU(data.payload);
		if(!data.data){return 0;}

		if(decodeChannelJoinConfirm(data.data,Client.store.userChannelId + 1001)){
			socket.send(channelJoinRequest(Client.store.userChannelId,Client.store.ioChannelId));
			connection.state = 'channel_connection_io_channel';
		}else{socket.close(1000,'Could not decode Channel Join Confirm');}
		return dv.getUint16(2,false);
	}

	function processChannelConnectionIOChannel(socket,Client,connection,dv){
		var data = tpkt.parsePacket(dv);
		if(!data.payload){return 0;}
		data = x224.parseConnectPDU(data.payload);
		if(!data.data){return 0;}

		if(decodeChannelJoinConfirm(data.data,Client.store.ioChannelId)){
				if(connection.channelsJoined < Client.store.staticVirtualChannelIds.length){
					socket.send(channelJoinRequest(Client.store.userChannelId,Client.store.staticVirtualChannelIds[connection.channelsJoined]));
					connection.state = 'channel_connection_virtual_channel';
				}else{
					messageChannel(socket,Client,connection);
				}
		}else{socket.close(1000,'Could not decode Channel Join Confirm');}
		return  dv.getUint16(2,false);
	}

	function processChannelConnectionMessageChannel(socket,Client,connection,dv){
		var data = tpkt.parsePacket(dv);
		if(!data.payload){return 0;}
		data = x224.parseConnectPDU(data.payload);
		if(!data.data){return 0;}
		if(decodeChannelJoinConfirm(data.data,Client.store.messageChannelId)){
			sendSecurityAndClientInfo(socket,Client,connection);
		}else{socket.close();}
		return dv.getUint16(2,false);
	}


	function processChannelConnectionVirtualChannel(socket,Client,connection,dv){
		var data = tpkt.parsePacket(dv);
		if(!data.payload){return 0;}
		data = x224.parseConnectPDU(data.payload);
		if(!data.data){return 0;}

		if(decodeChannelJoinConfirm(data.data,Client.store.staticVirtualChannelIds[connection.channelsJoined])){
			connection.channelsJoined++;
			if(connection.channelsJoined < Client.store.staticVirtualChannelIds.length){
				socket.send(channelJoinRequest(Client.store.userChannelId,Client.store.staticVirtualChannelIds[connection.channelsJoined]));
			}else{
				messageChannel(socket,Client,connection);
			}
		}else{socket.close();}
		return dv.getUint16(2,false);
	}

	function process(socket,Client,connection,dv){

		switch(connection.state){
			case 'channel_connection_io_channel':
				return processChannelConnectionIOChannel(socket,Client,connection,dv);
				break;
			case 'channel_connection_user_channel':
				return processChannelConnectionUserChannel(socket,Client,connection,dv);
				break;
			case 'channel_connection_message_channel':
				return processChannelConnectionMessageChannel(socket,Client,connection,dv);
				break;
			case 'channel_connection_virtual_channel':
				return processChannelConnectionVirtualChannel(socket,Client,connection,dv);
				break;
		}
	}

module.exports = {
		process:process,
		request:channelJoinRequest
	}
