'use strict';

    var DataStream          = require('DataStream');
    var tpkt                = require('xrdp/core/tpkt');
    var x224                = require('xrdp/core/x224');
    var mcs                 = require('xrdp/core/mcs');
    var securityHeader      = require('xrdp/security/header/writer');


	/********** clientInfo functions************/
		function writeInfoPacket(ds,flags,codePageId,domain,userName,password,alternateShell,workingDir){
			if(flags & 0x00000010 ){
				//unicode
				ds.writeUint16(codePageId);//ignore
				ds.writeUint16(codePageId);
			}else{
				ds.writeUint16(codePageId);//ignore
				ds.writeUint16(codePageId);
			}
			ds.writeUint32(flags);
			if(flags & 0x00000010 ){
				ds.writeUint16(domain.length *2);
				ds.writeUint16(userName.length *2);
				ds.writeUint16(password.length *2);
				ds.writeUint16(alternateShell.length *2);
				ds.writeUint16(workingDir.length *2);
				if(domain.length){ ds.writeUCS2String(domain); }
				ds.writeUint16(0x0000);
				if(userName.length){ ds.writeUCS2String(userName); }
				ds.writeUint16(0x0000);
				if(password.length){ ds.writeUCS2String(password); }
				ds.writeUint16(0x0000);
				if(alternateShell.length){ ds.writeUCS2String(alternateShell); }
				ds.writeUint16(0x0000);
				if(workingDir.length){ ds.writeUCS2String(workingDir); }
				ds.writeUint16(0x0000);
			}else{
				ds.writeUint16(domain.length);
				ds.writeUint16(userName.length);
				ds.writeUint16(password.length);
				ds.writeUint16(alternateShell.length);
				ds.writeUint16(workingDir.length);
				ds.writeCString(domain);
				ds.writeCString(userName);
				ds.writeCString(password);
				ds.writeCString(alternateShell);
				ds.writeCString(workingDir);
			}
		}
		function writeExtraInfo(ds,isUnicode,clientAddressFamily,clientAddress,clientDir,performanceFlags,autoReconnectCookie){
			ds.writeUint16(clientAddressFamily);
			if(isUnicode){
				ds.writeUint16(clientAddress.length *2 +4);
				ds.writeUCS2String(clientAddress);
				ds.writeUint32(0x00000000);
				ds.writeUint16(clientDir.length *2 +4);
				ds.writeUCS2String(clientDir);
				ds.writeUint32(0x00000000);
			}else{
				ds.writeUint16(clientAddress.length +1);
				ds.writeCString(clientAddress);
				ds.writeUint16(clientDir.length +1);
				ds.writeCString(clientDir);
			}

			//write time zone timeZoneInformation
			ds.writeUint32( 0x01e0);//480 mninute bias


			ds.writeUCS2String("Pacific Standard Time",DataStream.LITTLE_ENDIAN,32);

			//daylight savings transition
			ds.writeUint16(0x0000);//year
			ds.writeUint16(11);//November
			ds.writeUint16(0);//sunday
			ds.writeUint16(1);//Nov 2
			ds.writeUint16(2);//2am
			ds.writeUint16(0);//minute
			ds.writeUint16(0);//second
			ds.writeUint16(0);//millisecond

			ds.writeUint32(0);//standard bias

			ds.writeUCS2String("Pacific Daylight Time",DataStream.LITTLE_ENDIAN,32);

			//daylight savings transition
			ds.writeUint16(0x0000);//year
			ds.writeUint16(3);//November
			ds.writeUint16(0);//sunday
			ds.writeUint16(9);//Nov 2
			ds.writeUint16(2);//2am
			ds.writeUint16(0);//minute
			ds.writeUint16(0);//second
			ds.writeUint16(0);//millisecond

			ds.writeUint32(0xffffffc4);//Daylight Bias (-60 minutes)

			ds.writeUint32(0x00000000);//client ID -- ignore

			ds.writeUint32(performanceFlags || 0 );

			if(autoReconnectCookie){
				ds.writeUint16(0x001C);
				ds.writeUint8Array(autoReconnectCookie);
			}else{
				ds.writeUint16(0);//not a reconnectCookie
				return 1;

				ds.writeUint32(0x00000000);
			}

			if(dynamicDstTimeZoneKeyName){
				ds.writeUint16(0);
				ds.writeUint16(0);
				ds.writeUint16(dynamicDstTimeZoneKeyName.length *2);
				ds.writeUCS2String(dynamicDstTimeZoneKeyName);
			}else{
				return 1;
			}
			if(daylightSavingsTimeDisabled){
				ds.writeUint16(daylightSavingsTimeDisabled);
			}else{
				return 1;
			}
			return 1;
		}
		function clientInfo(userChannelId,ioChannelId,encryptionLevel,sessionKeys,settings){
			var ds = new DataStream();
			var dsInfoPacket = new DataStream();
			//INFO_MOUSE | INFO_MOUSE_HAS_WHEEL | INFO_NOAUDIOPLAYBACK
			writeInfoPacket(dsInfoPacket,
				0x00000001 | 0x00000002 | 0x00000010 | 0x00000020 | 0x00000040 | 0x00000100|  0x00080000 | 0x00010000,
				0x0000 |  (settings.channels.indexOf('rail') >=0 ?0x00008000:0) ,
				'',
				settings.userName || '',
				settings.password || '',
				'',
				''
			);

			writeExtraInfo(dsInfoPacket,true,0x0002,'127.0.0.1','C:\\Windows\\System32\\mstscax.dll',
				0x00000001 | 0x00000004 | 0x0000002
			);

			ds.writeUint8Array(securityHeader.writeSecurityHeaderAndData(0x0040,encryptionLevel,sessionKeys,new Uint8Array(dsInfoPacket.buffer)));
			return tpkt.writePacket(x224.writeDataPDU(mcs.writeSDrqHeader(userChannelId,ioChannelId,0x01,0x03,new Uint8Array(ds.buffer))));
		}
module.exports = clientInfo;
