'use strict';
/*
 * Capability Set definitions
 * http://msdn.microsoft.com/en-us/library/cc240486.aspx
*/

    var capset      = require('./capability-set');
    var settings    = require('xrdp/capability-settings');

module.exports = function(capStream){
     var capsetCount = 0;
    if(settings.capabilities[0x0001]){
		capsetCount += capset.writeGeneral(capStream,
			settings.capabilities[0x0001].extraFlags,
			settings.capabilities[0x0001].refreshRectSupport,
			settings.capabilities[0x0001].suppressOutputSupport);
	}
	if(settings.capabilities[0x0002]){
		capsetCount += capset.writeBitmap(capStream,
			settings.capabilities[0x0002].preferredBitsPerPixel,
			settings.capabilities[0x0002].desktopWidth,
			settings.capabilities[0x0002].dekstopHeight,
			settings.capabilities[0x0002].desktopResizeFlag,
			settings.capabilities[0x0002].drawingFlags);
		//http://msdn.microsoft.com/en-us/library/cc240556.aspx
	}
	if(settings.capabilities[0x0003]){
		capsetCount += capset.writeOrder(capStream,
			settings.capabilities[0x0003].orderFlags,
			settings.capabilities[0x0003].orderSupport,
			settings.capabilities[0x0003].orderSupportExtra);
	}
	if(settings.capabilities[0x0004]){
		capsetCount += capset.writeBitmapCacheRev1(capStream,
			settings.capabilities[0x0004].cache0Entries,
			settings.capabilities[0x0004].cache0MaximumCellSize,
			settings.capabilities[0x0004].cache1Entries,
			settings.capabilities[0x0004].cache1MaximumCellSize,
			settings.capabilities[0x0004].cache2Entries,
			settings.capabilities[0x0004].cache2MaximumCellSize);
	}
	if(settings.capabilities[0x0005]){
		capsetCount += capset.writeControl(capStream);
	}
	if(settings.capabilities[0x0006]){
		capsetCount += capset.writeActivation(capStream);
	}
	if(settings.capabilities[0x0008]){
		capsetCount += capset.writePointer(capStream,
			settings.capabilities[0x0008].colorPointerCacheSize,
			settings.capabilities[0x0008].pointerCacheSize);
	}
	if(settings.capabilities[0x0009]){
		capsetCount += capset.writeShare(capStream);
	}
	if(settings.capabilities[0x000A]){
		capsetCount += capset.writeColorCache(capStream);
	}

		/*
	if(settings.capabilities[0x000C]){
		capsetCount += capset.writeGlyphCache(capStream,
			settings.capabilities[0x0010].cacheDefinitions,
			settings.capabilities[0x0010].fragCache,
			settings.capabilities[0x0010].glyphSupportLevel);
	}
		*/
	if(settings.capabilities[0x000C]){
		capsetCount += capset.writeSound(capStream,
			settings.capabilities[0x000C].soundFlags);
	}
	if(settings.capabilities[0x000D]){
		capsetCount += capset.writeInput(capStream,
			settings.capabilities[0x000D].inputFlags,
			settings.capabilities[0x000D].keyboardLayout,
			settings.capabilities[0x000D].keyboardType,
			settings.capabilities[0x000D].keybaordSubtype,
			settings.capabilities[0x000D].keyboardFunctionKeyNum,
			settings.capabilities[0x000D].imeFileName);
	}
	if(settings.capabilities[0x000E]){
		capsetCount += capset.writeFont(capStream);
	}
	if(settings.capabilities[0x000F]){
		capsetCount += capset.writeBrush(capStream,
			settings.capabilities[0x000F].brushSupportLevel);//brush color full
	}
	if(settings.capabilities[0x0011]){
		capsetCount += capset.writeOffscreenCache(capStream,
			settings.capabilities[0x0011].offscreenSupportLevel,
			settings.capabilities[0x0011].offscreenCacheSize,
			settings.capabilities[0x0011].offscreenCacheEntries);
	}
	if(settings.capabilities[0x0013]){
		capsetCount += capset.writeBitmapCacheRev2(capStream,
			settings.capabilities[0x0013].cacheFlags,
			settings.capabilities[0x0013].cellCaches);
	}
	if(settings.capabilities[0x0014]){
		capsetCount += capset.writeVirtualChannel(capStream,
			settings.capabilities[0x0014].flags,
			settings.capabilities[0x0014].vCChunkSize);
	}
	if(settings.capabilities[0x0015]){
		capsetCount += capset.writeDrawNineGridCache(capStream,
			settings.capabilities[0x0015].drawNineGridSupportLevel,
			settings.capabilities[0x0015].drawNineGridCacheSize,
			settings.capabilities[0x0015].drawNineGridCacheEntries);
	}
	if(settings.capabilities[0x0016]){
		capsetCount += capset.writeDrawGdiPlus(capStream,
			settings.capabilities[0x0016].drawGDIPlusSupportLevel,
			settings.capabilities[0x0016].gdipVersion,
			settings.capabilities[0x0016].drawGdipCacheLevel,
			settings.capabilities[0x0016].gdipCacheEntries,
			settings.capabilities[0x0016].gdipCacheChunkSize,
			settings.capabilities[0x0016].gdipImageCacheProperties);
	}
	if(settings.capabilities[0x0017]){
		capsetCount += capset.writeRail(capStream,
			settings.capabilities[0x0017].S,
			settings.capabilities[0x0017].L,
			settings.capabilities[0x0017].I);
	}
	if(settings.capabilities[0x0018]){
		capsetCount += capset.writeWindow(capStream,
			settings.capabilities[0x0018].wndSupportLevel,
			settings.capabilities[0x0018].numIconCaches,
			settings.capabilities[0x0018].numIconEntries);
	}
	if(settings.capabilities[0x0019]){
		capsetCount += capset.writeCompDesk(capStream,
			settings.capabilities[0x0019].compDeskSupportLevel);
	}
	if(settings.capabilities[0x001A]){
		capsetCount += capset.writeMultiFragmentUpdate(capStream,
			settings.capabilities[0x001A].maxRequestSize);
	}

	if(settings.capabilities[0x001B]){
		capsetCount += capset.writeLargePointer(capStream,
			settings.capabilities[0x001B].largePointerSupportFlags);
	}
	if(settings.capabilities[0x001C]){
		capsetCount += capset.writeSurfaceCommands(capStream,
			settings.capabilities[0x001C].cmdFlags);
	}
	if(settings.capabilities[0x001D]){
		capsetCount += capset.writeBitmapCodecs(capStream,
			settings.capabilities[0x001D].bitmapCodecs);
	}
	if(settings.capabilities[0x001E]){
		capsetCount += capset.writeFrameAcknowledge(capStream,
		settings.capabilities[0x001E].maxUnacknowledgedFrameCount);
	}

   if(! capsetCount){
     console.log('Error writing capability sets');
     return undefined;
   }
   return capsetCount;
  }
