'use strict';

    var DataStream  = require('DataStream');
    var tpkt        = require('xrdp/core/tpkt');
    var x224        = require('xrdp/core/x224');
    var per         = require('xrdp/core/per');
    var ber         = require('xrdp/core/ber');


	function decodeConnectionConfirm(dv){
			dv = new DataView(dv.buffer,dv.byteOffset +1,dv.byteLength-1);
			if(dv.byteLength === 0){return 1;}
			var flags = dv.getUint8(1);
			if(dv.getUint16(2,true) !== 0x0008){
				throw new Error('Invalid length',dv.byteLength);
			}
			switch(dv.getUint8(0)){
				case 0x02: //TYPE_RDP_NEG_RSP
					return {
						protocol:dv.getUint32(4,true),
						flags:flags
					};
					break;
				case 0x03: //TYPE_RDP_NEG_FAILURE
					if(flags !== 0x00){
						throw new Error('invalid flags');
					}
					console.log(dv.getUint32(4,true));
					break;
				default:
					throw new Error('invalid response type');
			}
			return {};
		}

	function processConnectionConfirm(socket,Client,dv){
			var data = tpkt.parsePacket(dv);
			if(!data.payload){return 0;}
			data = x224.parseConnectPDU(data.payload);
			if(!data.data){return 0;}

			if(data.code !== 0xD0){
				throw new Error('Process Init, Invalid Client State');
			}
			var o = decodeConnectionConfirm(data.variable);
			if(o.protocol){//security https://msdn.microsoft.com/en-us/library/cc240506.aspx
				socket.send(connectInitialGccConferenceCreate(Client.settings,o.flags));
			}else{
				socket.send(connectInitialGccConferenceCreate(Client.settings,o.flags));
			}
			return dv.getUint16(2,false);
		}
		/***** connectInitialGccConferenceCreate functions************/
		function csCore(d,settings){
			var ds = new DataStream();
			ds.writeUint32(0x00080004); //version
			ds.writeUint16(settings.width);
			ds.writeUint16(settings.height);
			ds.writeUint16(0xCA01); //color depth:  RNS_UD_COLOR_8BPP
			ds.writeUint16(0xAA03); //SAS sequence: RNS_UD_SAS_DEL
			ds.writeUint32(settings.keyboard.layout);
			ds.writeUint32(2600);//build number
			ds.writeUCS2String("FastXJS",true,15);
			ds.writeUint16(0x0000); //null terminator
			ds.writeUint32(settings.keyboard.type);
			ds.writeUint32(settings.keyboard.subtype);
			ds.writeUint32(settings.keyboard.functionKeyNum);
			ds.writeUCS2String("",true,31); //IME keyboard filename
			ds.writeUint16(0x0000); //null terminator
			ds.writeUint16(0xCA01);//postBeta2ColorDepth
			ds.writeUint16(0x01); //client prodid
			ds.writeUint32(0);//serial
			ds.writeUint16(0x0010);//high color 24
			ds.writeUint16(settings.supportedColorDepths);
			ds.writeUint16(settings.earlyCapabilitiesFlags);
			ds.writeUCS2String("",true,32);
			ds.writeUint8(0x07);//connection Type
			ds.writeUint8(0);//pad octet
			ds.writeUint32(0);//server Selected PROTOCOL

			d.writeUint16(0xC001);//header
			d.writeUint16(ds.byteLength +4);
			d.writeUint8Array(new Uint8Array(ds.buffer));
		};

		function security(ds,encryptionFlags,isFrenchLocale){
			ds.writeUint8Array([0x02,0xC0,12,0x00]);
			if(isFrenchLocale){
				ds.writeUint32(0);
				ds.writeUint32(encryptionFlags);
			}else{
				ds.writeUint32(encryptionFlags);
				ds.writeUint32(0);
			}
		};
		function channel(name,options){
			var d = new DataStream(12);
			if(name.length > 7) throw new Error("Invalid Channel Name");
			d.writeCString(name,7);
			d.writeUint8(0);
			d.writeUint32(options);
			return d.buffer;
		}
		function network(ds,channels){
			ds.writeUint16(0xC003);
			ds.writeUint16(channels.length*12+8);
			ds.writeUint32(channels.length);//channel count
			for(var i=0;i<channels.length;i++){
				ds.writeUint8Array(new Uint8Array(channel(channels[i],0xc0a000000)));
			}
		};
		function cluster(ds,flags,id){
			var len = (flags & 0x00000002)?12:8;
			ds.writeUint8Array([0x04,0xC0,len,0x00]);
			ds.writeUint32(flags);
			if(len === 12){
				ds.writeUint32(id);
			}
		}
		function monitor(ds){
			ds.writeUint8Array([0x05,0xC0,32,0x00]);
			ds.writeUint32(0);//reserved flags
			ds.writeUint32(1);//1 monitor
			ds.writeUint32(0);//left
			ds.writeUint32(0);//top
			ds.writeUint32(window.screen.width);//right
			ds.writeUint32(window.screen.height);//bottom
			ds.writeUint32(1);//primary monitor
		}
		function messageChannel(ds){
			ds.writeUint16(0xC006);
			ds.writeUint16(8);
			ds.writeUint32(0);
		}
		function connectInitialGccConferenceCreate(settings,flags){
			//begin Gcc Conference Create BER Encoded
			var ds = new DataStream(),
			cids = new DataStream(),
			userData = new DataStream(),
			out = new DataStream();
			ds.writeUint8Array([0x00,0x05, 0x00, 0x14, 0x7C, 0x00, 0x01]); //gccData --gcccrq
			//create the userdata
			csCore(userData,settings);
			security(userData,13,settings.isFrenchLocale);//Client.settings.encryptionMethods);
			network(userData,settings.channels);
			//cluster(userData,0x00000000);//redirection not supported
			if(flags & 0x00000001){
				//monitor(userData);
				messageChannel(userData);
			}

			//create the conference Create Request
			//per length
			per.writeLength(ds,userData.byteLength + 12);
			ds.writeUint8Array([0x00, 0x08, 0x00, 0x10, 0x00, 0x01, 0xC0, 0x00, 0x44, 0x75, 0x63, 0x61]); //connectPdu - DUCA
			per.writeOctetString(ds, new Uint8Array(userData.buffer),0);//write userdata
			//begin Connect-Initial -- mcsCi
			cids.writeUint8Array([4,1,1, 4,1,1, 1,1,0xFF,
			0x30, 0x19, 2,1,34, 2,1,2, 2,1,0, 2,1,1, 2,1,0, 2,1,1, 2,2,0xFF,0xFF, 2,1,2,//target
			0x30, 0x19, 2,1,1, 2,1,1, 2,1,1, 2,1,1, 2,1,0, 2,1,1, 2,2,0x04,0x20, 2,1,2,//minimum
			0x30, 0x1C, 2,2,0xFF,0xFF, 2,2,0xFC,0x17, 2,2,0xFF,0xFF, 2,1,1, 2,1,0, 2,1,1, 2,2,0xFF,0xFF, 2,1,2]);//maximum
			ber.writeOctetString(cids,new Uint8Array(ds.buffer));
			ber.writeApplicationTag(out,101,cids.byteLength);
			out.writeUint8Array(new Uint8Array(cids.buffer));
			return tpkt.writePacket(x224.writeDataPDU(out.buffer));
		}

module.exports = {
		process:processConnectionConfirm
	}
