'use strict';


    var mcs                 = require('xrdp/core/mcs');
    var tpkt                = require('xrdp/core/tpkt');
    var x224                = require('xrdp/core/x224');
    var securityHeaderReader      = require('xrdp/security/header/reader');

	function decodeLicenseErrorValidClient(dv,encryptionLevel,encryptionMethod){
		var o = securityHeaderReader.process(dv,encryptionLevel,encryptionMethod);
		if( !(o.flags & 0x0080)){
			throw new Error('Did not receive a License PDU');
		}
		if(o.data.getUint8(0) !== 0xFF){
			throw new Error('Invalid Preamble Header');
		}
		if(o.data.getUint16(2,true) !== o.data.byteLength){
			throw new Error('Invalid License Data Message Size');
		}
		//ignore error code
		if(o.data.getUint32(8,true) !== 0x00000002){
			throw new Error('Invalid State Transition');
		};
		if(o.data.getUint16(12,true) !== 0x0004){
			throw new Error('Invalid License Error Blob');
		}
		if(o.data.getUint16(14,true) !== 0x0000){
			throw new Error('License error blob is not empty');
		}
	}
	function processLicensing(socket,Client,dv){
		var data = tpkt.parsePacket(dv);
		if(!data.payload){return 0;}
		data = x224.parseConnectPDU(data.payload);
		if(!data.data){return 0;}

		data = mcs.parseSDin(data.data);
		if(!data.userData){return 0;}
		decodeLicenseErrorValidClient(data.userData,
			Client.store.receivedServerData.encryptionLevel,
			Client.store.receivedServerData.encryptionMethod);
		return dv.getUint16(2,false);
	}
module.exports = {
    process:processLicensing
}
