'use strict';
/*
 * MCS
 */

    var DataStream  = require('DataStream');
    var per         = require('./per');

module.exports = {
	writeSDrqHeader: function(initiator, channelId, dataPriority, segmentation, userData) {
		var ds = new DataStream;

		ds.writeUint8(25 << 2);
		ds.writeUint16(initiator,DataStream.BIG_ENDIAN);
		ds.writeUint16(channelId,DataStream.BIG_ENDIAN);
		ds.writeUint8(dataPriority << 6 | segmentation << 4);
		per.writeOctetString(ds,userData,0);

		return ds.buffer;
	},
	parseSDrq: function(dv) {
		if (dv.getUint8(0) >>> 2 != 25) {
			throw new Error("Bad Send Data Request choice");
		}

		var o = {};

		o.initiator = dv.getUint16(1,false) + 1001;
		o.channelId = dv.getUint16(3,false);

		if (o.initiator > 65535) {
			throw new Error("Bad Send Data Request Initiator");
		}

		if (o.channelId > 65535) {
			throw new Error("Bad Send Data Request Channel Id");
		}

		o.dataPriority = (dv.getUint8(5) & 0xC0) >>> 6;
		o.segmentation = (dv.getUint8(5) & 0x03) >>> 4;

		var pl = per.readLength(dv,6);

		if (!pl) {
			throw new Error("Bad Send Data Request User Data length");
		}

		o.userData = new DataView(dv.buffer,dv.byteOffset + 6 + pl.consumed,pl.length);

		return o;
	},
	writeSDinHeader: function(initiator, channelId, dataPriority, segmentation, userData) {
		var ds = new DataStream;

		ds.writeUint8(26 << 2);
		ds.writeUint16(initiator,DataStream.BIG_ENDIAN);
		ds.writeUint16(channelId,DataStream.BIG_ENDIAN);
		ds.writeUint8(dataPriority << 6 | segmentation << 4);
		per.writeOctetString(ds,userData,0);

		return ds.buffer;
	},
	parseSDin: function(dv) {
		if (dv.getUint8(0) >>> 2 != 26) {
			console.log(dv.getUint8(0) >>> 2,dv.getUint8(0));
			throw new Error("Bad Send Data Indication choice");
		}

		var o = {};
		o.type = 26;
		o.initiator = dv.getUint16(1,false) + 1001;
		o.channelId = dv.getUint16(3,false);

		o.dataPriority = (dv.getUint8(5) & 0xC0) >>> 6;
		o.segmentation = (dv.getUint8(5) & 0x03) >>> 4;

		var pl = per.readLength(dv,6);

		if (!pl) {
			throw new Error("Bad Send Data Request User Data length");
		}

		o.userData = new DataView(dv.buffer,dv.byteOffset + 6 + pl.consumed,pl.length);

		return o;
	},
	parseDpum: function(dv) {
		if (dv.byteLength < 2) {
			throw new Error("Bad Disconnect Ultimatum length");
		}

		if (dv.getUint8(0) >>> 2 != 8) {
			throw new Error("Bad Disconnect Ultimatum choice");
		}

		var o = {};

		o.reason = dv.getUint8(1);

		return o;
	},
	parseDisconnectProviderUltimatum: function(dv){
		if (dv.getUint8(0) >>> 2 != 8) {
			throw new Error("Bad Disconnect Provider Ultimate choice");
		}

		var o = {};
		o.reason = dv.getUint8(1);
		o.type = 8;
		return o;
	},
	parse:function(dv){
		switch(dv.getUint8(0) >>>2){
			case 8:
				return this.parseDisconnectProviderUltimatum(dv);
			case 26:
				return this.parseSDin(dv);
			default:
				throw new Error("Unkniown MCS choice " + (dv.getUint8(0) >>> 2) );
		}
	}
}
