'use strict';
/*
 * X.224
 */

var DataStream      = require('DataStream');

module.exports = {
	writeCrqPDU: function(data) {
		if (data.byteLength > 249) throw new Error("Bad CRQ length");

		var ds = new DataStream;

		ds.writeUint8(6 + data.byteLength);
		ds.writeUint8(0xE0);
		ds.writeUint16(0,DataStream.BIG_ENDIAN);
		ds.writeUint16(0,DataStream.BIG_ENDIAN);
		ds.writeUint8(0);
		ds.writeUint8Array(new Uint8Array(data));

		return ds.buffer;
	},
	writeCcfPDU: function(data) {
		if (data.byteLength > 249) throw new Error("Bad CCF length");

		var ds = new DataStream;

		ds.writeUint8(6 + data.byteLength);
		ds.writeUint8(0xD0);
		ds.writeUint16(0,DataStream.BIG_ENDIAN);
		ds.writeUint16(0,DataStream.BIG_ENDIAN);
		ds.writeUint8(0);
		ds.writeUint8Array(new Uint8Array(data));

		return ds.buffer;
	},
	parseConnectPDU: function(dv) {
		var headerLength = dv.getUint8(0);
		var code = dv.getUint8(1) & 0xF0;
		var o = { code: code };

		switch (code) {
		case 0xE0: // Connection Request
			break;
		case 0xD0: // Connection Confirm
			if (dv.getUint8(1) & 0x0F) {
				throw new Error("Bad Connection Confirm PDU");
			}

			o.dstRef = dv.getUint16(2,false);
			o.srcRef = dv.getUint8(4);
			o.classOption = dv.getUint8(5);
			o.variable = new DataView(dv.buffer,dv.byteOffset + 6,
				headerLength - 5);
			break;
		case 0x80: // Disconnect Request
			break;
		case 0xC0: // Disconnect Confirm
			break;
		case 0xF0: // Data
			o.tpduNr = dv.getUint8(1);
			break;
		case 0x10: // Expedited Data
			break;
		case 0x60: // Data Acknowledge
			break;
		case 0x20: // Expedited Data Acknowledge
			break;
		case 0x50: // Reject
			break;
		case 0x70: // Error
			break;
		default:
			throw new Error("Bad Connect PDU type");
			return undefined;
		}
		if(dv.byteLength - (1 + headerLength) ){
			o.data = new DataView(dv.buffer,dv.byteOffset + (1 + headerLength),
				dv.byteLength - (1 + headerLength));
		}else{
			o.data = new DataView(new ArrayBuffer(0));
		}
		return o;
	},
	writeDataPDU: function(data) {
		var ds = new DataStream;

		ds.writeUint8(0x02);
		ds.writeUint8(0xF0);
		ds.writeUint8(0x80);
		ds.writeUint8Array(new Uint8Array(data));

		return ds.buffer;
	},
	parseDataPDU: function(dv) {
		if (dv.getUint8(0) != 0x02) {
			throw new Error("Bad Data header length");
		}

		if (dv.getUint8(1) != 0xF0) {
			throw new Error("Bad Data DT ROA");
		}

		if (dv.getUint8(2) != 0x80) {
			throw new Error("Bad Data TPDU-NR and EOT");
		}

		var o = {};

		o.data = new DataView(dv.buffer,dv.byteOffset + 3,dv.byteLength - 3);

		return o;
	}
}
