'use strict';
    
    var DataStream = require('DataStream');    
    
	function decompress(dv,bpp){
		console.log('TODO: decompress bitmap data');
		return dv;
	}
	
	function create8(im,data){
		var j=0;
		for(var i =0; i<data.length; i++){
			im.data[j] 		= data.getUint8(i);
			im.data[j + 1] 	= data.getUint8(i);
			im.data[j + 2] 	= data.getUint8(i);
			im.data[j + 3] 	= 255;
			j+=4;
		}
	}
	function create16(im,data){
		var j=0;
		for(var i =0; i<data.byteLength; i+=2){
			im.data[j] 		= data.getUint16(i) & 0x001F;
			im.data[j + 1] 	= (data.getUint16(i) & 0x07E0) >>> 5;
			im.data[j + 2] 	= (data.getUint16(i) & 0xF800) >>> 10;
			im.data[j + 3] 	= 255;
			j+=4;
		}
	}
	function create24(im,data){
		var j=0;
		for(var i =0; i<data.byteLength; i+=3){
			im.data[j] 		= data.getUint8(i);
			im.data[j + 1] 	= data.getUint8(i + 1);
			im.data[j + 2] 	= data.getUint8(i + 2);
			im.data[j + 3] 	= 255;
			j+=4;
		}
	}
	function create32(im,data){
		for(var i=0; i<data.byteLength;i++){
			im.data[i] = data.getUint8(i);
		}
	}
	function create(bitmap){
		var c = document.createElement('canvas');
		c.width = bitmap.width;
		c.height = bitmap.height;
		var ctx = c.getContext('2d');
		var im = ctx.createImageData(bitmap.width,bitmap.height);
		
		var j = 0;
		switch(bitmap.bpp){
			case 8:
				create8(im,bitmap.data);
				break;
			case 16:
				create16(im,bitmap.data);
				break;
			case 24:
				create24(im,bitmap.data);
				break;
			case 32:
				create32(im,bitmap.data);
				break;
		}
		ctx.putImageData(im,0,0);
		return c;
	}
	
	function bitmapData(ds){
		var o ={
			rect:[
				ds.readUint16(),
				ds.readUint16(),
				ds.readUint16(),
				ds.readUint16()
			],
			width:ds.readUint16(),
			height:ds.readUint16(),
			bpp:ds.readUint16()
		};
		var flags = ds.readUint16();
		
		if((flags & 0x0400) &&!(flags & 0x0001)){
			throw new Error('BitmapData: Compression header is present and Data is not Compressed');
		}
		var len = ds.readUint16();
		var compressionHeader ={};
		if(flags & 0x0001){//bitmap data is compressed
			if(!(flags & 0x0400)){//get compression header
				if(ds.readUint16() !== 0){
					throw new Error('Compression header FirstRowSize not = 0');
				}
				compressionHeader.mainBodySize = ds.readUint16();
				compressionHeader.scanWidth = ds.readUint16();
				if(compressionHeader.scanWidth % 4){
					throw new Error('Compression header scanwidth not divisible by 4');
				}
				compressionHeader.uncompressedSize = ds.readUint16();
				len -= 8;
			}
			o.data = decompress(new DataView(ds.readUint8Array(len).buffer),o.bpp);
		}else{
			o.data = new DataView(new DataView(ds.readUint8Array(len).buffer));
		}
		return o;
	}
	
	function process(dv){
		var ds = new DataStream(dv.buffer,dv.byteOffset);
		if(ds.readUint16() !== 0x0001){
			throw new Error('UpdateType must be  UPDATETYPE_BITMAP (0x0001)');
		}
		var n = ds.readUint16();
		var o = {
			rectangles:[]
		};
		
		while(n--){
			o.rectangles.push(bitmapData(ds));
		}
		return o;
	}
	function processBitmapData(dv){
		return bitmapData(new DataStream(dv.buffer,dv.byteOffset));
	}
	
module.exports = {
		create:create,
		process:process,
		processBitmapData:processBitmapData
	}