'use strict';

    var Offscreen       = require('xrdp/cache/offscreen-bitmap');
    var Screen          = require('xrdp/canvas/screen');

function altSec(o){
		switch(o.orderType){
			case 0x00:
				/* The Switch Surface Alternate Secondary Drawing Order is used by the server to
				 * instruct the client to switch the target drawing surface either to the primary
				 * drawing surface (screen desktop) or to an entry in the Offscreen Bitmap Cache.
				 * Support for offscreen bitmap caching is specified in the Offscreen Bitmap Cache
				 * Capability Set (see [MS-RDPBCGR] section 2.2.7.1.9).
				 */
				Offscreen.bitmapId = o.bitmapId;
				break;
			case 0x01:
				/* The Create Offscreen Bitmap Alternate Secondary Drawing Order is used by the server
				 * to instruct the client to create a bitmap of a particular width and height in the
				 * Offscreen Bitmap Cache. Support for offscreen bitmap caching is specified in the
				 * Offscreen Bitmap Cache Capability Set (see [MS-RDPBCGR] section 2.2.7.1.9).
				 */
				Offscreen.store(o.bitmapId,Offscreen.create(o.width,o.height));
				if(o.deleteList.length){
					var l;
					while(l = o.deleteList.shift()){
						Offscreen.remove(l);
					}
				}
				break;
			case 0x02:
				/* The Stream Bitmap First Alternate Secondary Drawing Order is used by the server
				 * to send the client the first block in a streamed bitmap and information describing
				 * the bitmap (such as color depth, width, and height).
				 */
				var bitmapStream = {
					width:o.bitmapWidth,
					height:o.bitmapHeight,
					bpp:o.bitmapBpp,
					data:o.bitmapBlock
				}
				if(o.bitmapFlags & 0x01){//END
					//create a NineGrid
				}
				break;
			case 0x03:
				/* The Stream Bitmap Next Alternate Secondary Drawing Order is used by the server to
				 * send the client intermediate and final blocks in a streamed bitmap.
				 */
				break;
			case 0x04:
				/* The Create NineGrid Bitmap Alternate Secondary Drawing Order is used by the server
				 * to instruct the client to create a NineGrid bitmap of a particular width and height
				 * in the NineGrid Bitmap Cache (the color depth MUST be 32 bpp). Support for NineGrid
				 * drawing is specified in the DrawNineGrid Cache Capability Set (section 2.2.1.2).
				 */

				break;
			case 0x05:
				/* The Draw GDI+ Cache First Alternate Secondary Drawing Order contains the first batch of
				 * GDI+ 1.1 cacheable drawing primitives that comprise a rendering update sent by the server
				 * to the client. Support for GDI+ 1.1 rendering and primitive caching is specified in
				 * the Draw GDI+ Capability Set (section 2.2.1.3).
				 */
				break;
			case 0x06:
				/* The Draw GDI+ Cache Next Alternate Secondary Drawing Order contains the second or
				/* subsequent batch of GDI+ 1.1 cacheable drawing primitives that comprise a rendering
				 * update sent by the server to the client. The first primitive in the sequence MUST
				 * have been transmitted with the Draw GDI+ Cache First Alternate Secondary Drawing Order.
				 * Support for GDI+ 1.1 rendering and primitive caching is specified in the Draw
				 * GDI+ Capability Set (section 2.2.1.3).
				 */
				break;
			case 0x07:
				/* The Draw GDI+ Cache End Alternate Secondary Drawing Order contains the final batch of
				 * GDI+ 1.1 cacheable drawing primitives that comprise a rendering update sent by
				 * the server to the client. Support for GDI+ 1.1 rendering and primitive caching
				 * is specified in the Draw GDI+ Capability Set (section 2.2.1.3).
				 */

				break;
			case 0x08:
				//Draw GDI+ First Alternate Secondary Drawing Order (see section 2.2.2.2.1.3.6.2).
				break;
			case 0x09:
				//Draw GDI+ Cache Next Alternate Secondary Drawing Order (see section 2.2.2.2.1.3.6.3).
				break;
			case 0x0A:
				//Draw GDI+ Cache End Alternate Secondary Drawing Order (see section 2.2.2.2.1.3.6.4).
				break;
			case 0x0B:
				//Windowing Alternate Secondary Drawing Order (see [MS-RDPERP] section 2.2.1.3).
				Screen.canvas.dispatchEvent(new CustomEvent('snWindowOrder',{detail:o}));
				break;
			case 0x0C:
				//Desktop Composition Alternate Secondary Drawing Order (see [MS-RDPEDC] section 2.2.1.1).
				break;
			case 0x0D:
				/* The Frame Marker Alternate Secondary Drawing Order is used by the server to indicate
				 * to the client the beginning and end of a logical frame of graphics data.
				 * Breaking graphics data up into logical frames indicates to the client which orders
				 * SHOULD be rendered as a logical unit, hence helping to prevent screen tearing.
				 * Support for frame markers is specified in the Order Capability Set
				 * (see [MS-RDPBCGR] section 2.2.7.1.3).
				 */
				console.log('Frame Marker')
				break;
			default:
				console.log('Unknown Alternate Secondary order type');
		}
	}
module.exports = altSec;
