'use strict';


    var Offscreen       = require('xrdp/cache/offscreen-bitmap');

	function color(arr){
		return ['rgba(',arr[0],',',arr[1],',',arr[2],',',arr[3],')'].join('');
	}
	function pattern(brush){
		return color([128,0,0,255]);
	}
	function fillMode(mode){
		switch(mode){
			case 0x01:
				return "evenodd";
				break;
			case 0x02:
				return "nonzero";
				break;
		}
	}
	function clipping(cfx,rects){
		var r;
		while(r = rects.shift()){
			ctx.rect(r[0],r[1],r[2]-r[0],r[3]-r[1]);
		}
		ctx.clip();
	}
	function primary(o){
        Offscreen.current().done(Offscreen.current().enqueue(), function () {
    		var ctx = Offscreen.current().canvas.getContext('2d');
    		ctx.save();

    		switch(o.orderType){
    			case 0x00://dstBlt -- The DstBlt Primary Drawing Order is used to paint a rectangle by using a destination-only raster operation
    				var dst = ctx.getImageData(o.nLeftRect,o.nTopRect,o.nWidth,o.nHeight);
    				ctx.fillStyle = rop(ctx,dst,null,null,o.bRop);
    				ctx.fillRect(o.nLeftRect,o.nTopRect,o.nWidth,o.nHeight);
    				break;
    			case 0x01://The PatBlt Primary Drawing Order is used to paint a rectangle by using a specified brush and three-way raster operation.
    				var dst = ctx.getImageData(o.nLeftRect,o.nTopRect,o.nWidth,o.nHeight);
    				break;
    			case 0x02:
    				/* The ScrBlt Primary Drawing Order is used to perform a bit-block transfer from a
    				 * source region to a destination region. The source surface is always the primary
    				 * drawing surface, while the target surface is the current target
    				 * surface—specified by the Switch Surface Alternate Secondary Drawing Order
    				 * (section 2.2.2.2.1.3.3).
    				 **/
    				ctx.drawImage(Offscreen.primary().canvas,o.nXSrc,o.nYSrc,o.nWidth,o.nHeight,o.nLeftRect,o.nTopRect,o.nWidth,o.nHeight);
    				//TODO brop;
    				break;
    			case 0x07:
    				/* The DrawNineGrid Primary Drawing Order encodes a single
    				 * NineGrid drawing command with a single bounding rectangle.
    				 */
    				console.log('DrawNineGrid not Implemented',o);
    				break;
    			case 0x08:
    				/* The MultiDrawNineGrid Primary Drawing Order encodes a single NineGrid
    				 * drawing command with multiple clipping rectangles.
    				 */
    				console.log('MultiDrawNineGrid not Implemented',o);
    				break;
    			case 0x09:
    				/* The LineTo Primary Drawing Order encodes a single line drawing order
    				 * that is restricted to solid color lines, one pixel wide.
    				 */
    				ctx.beginPath();
    				ctx.strokeStyle = color(o.PenColor.concat(255));
    				ctx.moveTo(o.nXStart,o.nYStart);
    				ctx.lineTo(o.nXEnd,o.nYEnd);
    				ctx.stroke();
    				break;
    			case 0x0A://The OpaqueRect Primary Drawing Order is used to paint a rectangle by using an opaque brush
    				ctx.fillStyle = color([o.RedOrPaletteIndex,o.Green,o.Blue,255]);
    				ctx.fillRect(o.nLeftRect,o.nTopRect,o.nWidth,o.nHeight);
    				break;
    			case 0x0B:
    				/* The SaveBitmap Primary Drawing Order encodes a rectangle
    				 * of the screen image for saving or restoring by the client.
    				 * */
    				console.log('SaveBitmap not Implemented',o);
    				break;
    			case 0x0D:
    				/* The MemBlt Primary Drawing Order is used to render a bitmap stored in the bitmap cache
    				 * or offscreen bitmap cache to the screen.
    				 */
    				console.log('MemBlt not Implemented',o);
    				break;
    			case 0x0E:
    				/* The Mem3Blt Primary Drawing Order is used to render a bitmap stored in the bitmap cache
    				 * or offscreen bitmap cache to the screen by using a specified brush and
    				 * three-way raster operation.
    				 */
    				console.log('Mem3Blt not Implemented',o);
    				break;
    			case 0x0F://The MultiDstBlt Primary Drawing Order is used to paint multiple rectangles by using a destination-only raster operation.
    				var dst = ctx.getImageData(o.nLeftRect,o.nTopRect,o.nWidth,o.nHeight);
    				clipping(ctx,o.rectangles);
    				ctx.fillStyle = rop(ctx,dst,null,null,o.bRop,[o.nLeftRect,o.nTopRect,o.nWidth,o.nHeight]);
    				ctx.fillRect(o.nLeftRect,o.nTopRect,o.nWidth,o.nHeight);
    				break;
    			case 0x10:
    				/* The MultiPatBlt Primary Drawing Order is used to paint multiple
    				 * rectangles by using a specified brush and three-way raster operation.
    				 */
    				console.log('MultiPat not Implemented',o);
    				break;
    			case 0x11:
    				/* The MultiScrBlt Primary Drawing Order is used to perform multiple bit-block transfers
    				 * from source regions to destination regions of the screen.
    				 */
    				clipping(ctx,o.rectangles);
    				ctx.drawImage(Offscreen.primary().canvas,o.nXSrc,o.nYSrc,o.nWidth,o.nHeight,o.nLeftRect,o.nTopRect,o.nWidth,o.nHeight);
    				break;
    			case 0x12://The MultiOpaqueRect Primary Drawing Order is used to paint multiple rectangles by using an opaque brush.
    				clipping(ctx,o.rectangles);
    				ctx.fillStyle = color([o.RedOrPaletteIndex,o.Green,o.Blue,255]);
    				ctx.fillRect(o.nLeftRect,o.nTopRect,o.nWidth,o.nHeight);
    				break;
    			case 0x13:
    				/* The FastIndex Primary Drawing Order encodes a set of glyph indices at a specified position.
    				 * This order is an improved version of the GlyphIndex (section 2.2.2.2.1.1.2.13)
    				 * Primary Drawing Order. The regular GlyphIndex order contains five brush fields
    				 * that the FastIndex order does not: BrushOrgX, BrushOrgY, BrushStyle,
    				 * BrushHatch, and BrushExtra. These extra fields MUST all be implicitly assumed
    				 * to exist and contain default values of zero (implying that a solid color
    				 * brush will be used).
    				 */
    				console.log('Fast Index not Implemented',o);
    				break;
    			case 0x14:
    				/* The PolygonSC Primary Drawing Order encodes a solid-color polygon consisting of
    				 * two or more vertices connected by straight lines.
    				 */
    				var l;
    				ctx.beginPath();
    				ctx.fillStyle = color(o.PenColor.concat(255));
    				if(o.points.length){
    					ctx.moveTo(o.xStart,o.yStart);
    					while(l = points.shift()){
    						ctx.lineTo(l[0],l[1]);
    					}
    				}
    				ctx.closePath();
    				ctx.fill(fillMode(o.FillMode));
    				break;
    			case 0x15:
    				/* The PolygonCB Primary Drawing Order encodes a color brush polygon consisting
    				 * of two or more vertices connected by straight lines.
    				 * */
    				var l;
    				ctx.beginPath();
    				ctx.fillStyle = pattern(o.Brush);
    				//o.brop2;
    				if(o.points.length){
    					ctx.moveTo(o.xStart,o.yStart);
    					while(l = points.shift()){
    						ctx.lineTo(l[0],l[1]);
    					}
    				}
    				ctx.closePath();
    				ctx.fill(fillMode(o.FillMode));
    				break;
    			case 0x16:
    				/* The Polyline Primary Drawing Order encodes a solid color polyline
    				 * consisting of two or more vertices connected by straight lines.
    				 */
    				var l;
    				ctx.beginPath();
    				ctx.strokeStyle = color(o.PenColor.concat(255));
    				if(o.points.length){
    					ctx.moveTo(o.xStart,o.yStart);
    					while(l = points.shift()){
    						ctx.lineTo(l[0],l[1]);
    					}
    				}
    				ctx.stroke();
    				break;
    			case 0x18:
    				/* The FastGlyph Primary Drawing Order encodes a single glyph at a specified position.
    				 * This primary drawing order is a fast way of outputting a single glyph and bypasses
    				 * having to send a Cache Glyph Secondary Drawing Order (see sections 2.2.2.2.1.2.5
    				 * and 2.2.2.2.1.2.6) followed by a GlyphIndex (section 2.2.2.2.1.1.2.13) or
    				 * FastIndex (section 2.2.2.2.1.1.2.14) Primary Drawing Order.
    				 */
    				console.log('FastGlyph not Implemented',o);
    				break;
    			case 0x19:
    				/* The EllipseSC Primary Drawing Order encodes a single, solid-color ellipse. */
    				console.log('Ellipse not Implemented',o);
    				break;
    			case 0x1A:
    				/* The EllipseCB Primary Drawing Order encodes a color brush ellipse. */
    				console.log('Ellipse not Implemented',o);
    				break;
    			case 0x1B:
    				/* The GlyphIndex Primary Drawing Order encodes a set of glyph indices at a specified position.*/
    				console.log('GlyphIndex not Implemented',o);
    				break;
    			default:
    				console.log('Unknown Primary order type');
    		}
    		ctx.restore();
       });
	}
module.exports = primary;
