'use strict';

var PrimaryOrder            = require('./primary');
var SecondaryOrder          = require('./secondary');
var AlternateSecondaryOrder = require('./alternate-secondary');

function readControlFlags (byte) {
	var o = {};
	if(byte & 0x01){
		if(byte & 0x02){
			o.type = 'secondary';
		}else{
			o.type ='primary';
			o.orderTypeSpecified = (byte & 0x08);
			o.boundingRect = (byte & 0x04);
			o.useZeroBoundsDelta = (byte & 0x20);
			o.useDeltaCoords = (byte & 0x10);
			o.fieldFlagBytes = 2*((byte & 0x80) >>> 7) + ((byte & 0x40) >>> 6);
		}
	}else if(byte & 0x02){
		o.type ='alternate_secondary';
		o.orderType = (byte & 0xFC) >> 2;
	}else{
		throw new Error('Unknown Drawing Order');
		return undefined;
	}
	return o;
}

function process (dv) {
	var o = [],flags,order;
	var numOrders = dv.getUint16(0,true);
	dv = new DataView(dv.buffer,dv.byteOffset +2);
	var byte;
	while(numOrders--){
		order = {};
		byte = dv.getUint8(0);
		if(byte & 0x01){
			if(byte & 0x02){//secondary
				order = SecondaryOrder.process(dv);
			}else{//primary drawing order
				order = PrimaryOrder.process(dv);
			}
		}else if(byte & 0x02){//'alternate_secondary'
			order = AlternateSecondaryOrder.process(dv);
		}else{
			throw new Error('Unknown Drawing Order');
			return undefined;
		}
		o.push(order.data);
		dv = order.dv;
	}

	return o;
}

module.exports = {
    process:process
}
