'use strict';
	var DataStream              = require('DataStream');
        var packedSignedFormat      = require('./packed-signed-format');  

	function PrimaryOrder(){
		this.orderType;
		this.bounds = [0,0,0,0];
		this.delta = false;
		this.fieldFlags =0;;
		this.orderHistory ={};

		return this.process;
	}
	PrimaryOrder.prototype.getFieldFlags = function(ds){
		var that = this;
		function getFields(fieldNum){
			var bit0 = that.controlFlags & 0x40;
			var bit1 = that.controlFlags & 0x80;
			var maxBytes = Math.ceil((fieldNum +1)/8);
			that.fieldFlags =0;
			var shift =0;
			if(!bit1 && !bit0){
				//dont subtract
			}else if(!bit1 && bit0){
				maxBytes--;
			}else if(bit1 && !bit0){
				msxBytes-=2;
			}else if(bit1 && bit0){
				maxBytes-=3;
			}
			if(maxBytes>0){
				while(maxBytes--){
					that.fieldFlags += ds.readUint8() << shift;
					shift+=8;
				}
			}
		}
		switch(this.orderType){
			case 0x00://TS_ENC_DSTBLT_ORDER
				getFields(5);
				break;
			case 0x01://TS_ENC_PATBLT_ORDER
				getFields(12);
				break
			case 0x02:
				getFields(7);
				break;
			case 0x07://TS_ENC_DRAWNINEGRID_ORDER
				getFields(5);
				break;
			case 0x08://TS_ENC_MULTI_DRAWNINEGRID_ORDER
				getFields(7);
				break;
			case 0x09://TS_ENC_LINETO_ORDER
				getFields(10);
				break;
			case 0x0A://TS_ENC_OPAQUERECT_ORDER
				getFields(7);
				break;
			case 0x0B://TS_ENC_SAVEBITMAP_ORDER
				getFields(6);
				break;
			case 0x0D://TS_ENC_MEMBLT_ORDER
				getFields(9);
				break;
			case 0x0E://TS_ENC_MEM3BLT_ORDER
				getFields(16);
				break;
			case 0x0F://TS_ENC_MULTIDSTBLT_ORDER
				getFields(7);
				break;
			case 0x10://TS_ENC_MULTIPATBLT_ORDER
				getFields(14);
				break;
			case 0x11://TS_ENC_MULTISCRBLT_ORDER
				getFields(9);
				break;
			case 0x12://TS_ENC_MULTIOPAQUERECT_ORDER
				getFields(9);
				break;
			case 0x13://TS_ENC_FAST_INDEX_ORDER
				getFields(15);
				break;
			case 0x14://TS_ENC_POLYGON_SC_ORDER
				getFields(7);
				break;
			case 0x15://TS_ENC_POLYGON_CB_ORDER
				getFields(13);
				break;
			case 0x16://TS_ENC_POLYLINE_ORDER
				getFields(7);
				break;
			case 0x18://TS_ENC_FAST_GLYPH_ORDER
				getFields(15);
				break;
			case 0x19://TS_ENC_ELLIPSE_SC_ORDER
				getFields(7);
				break;
			case 0x1A://TS_ENC_ELLIPSE_CB_ORDER
				getFields(13);
				break;
			case 0x1B://TS_ENC_INDEX_ORDER
				getFields(22);
				break;
			default:
				console.log('Unknown Order Type',o);
		}
	}
	PrimaryOrder.prototype.getBounds = function(ds){
		var descriptionByte = ds.readUint8();
		if(descriptionByte & 0x01){//left
			this.bounds[0] = ds.readUint16();
		}else if(descriptionByte & 0x10){//delta_left
			this.bounds[0] += ds.readInt8();
		}

		if(descriptionByte & 0x02){//top
			this.bounds[1] = ds.readUint16();
		}else if(descriptionByte & 0x20){//delta_top
			this.bounds[1] += ds.readInt8();
		}

		if(descriptionByte & 0x04){//right
			this.bounds[2] = ds.readUint16();
		}else if(descriptionByte& 0x40){//delta_right
			this.bounds[2] += ds.readInt8();
		}

		if(descriptionByte & 0x08){//bottom
			this.bounds[3] = ds.readUint16();
		}else if(descriptionByte & 0x80){//delta_bottom
			this.bounds[3] += ds.readInt8();
		}
	}
	PrimaryOrder.prototype.decodeHeader = function(ds){
		var offset =0;
		this.controlFlags = ds.readUint8();
		if(this.controlFlags & 0x08){//orderType field
			this.orderType = ds.readUint8();
		}
		this.getFieldFlags(ds);
		if(this.controlFlags & 0x04){//bounding rect
			if( !(this.controlFlags & 0x20)){
				this.getBounds(ds);
			}

		}
		this.delta = this.controlFlags & 0x10;
	}
	PrimaryOrder.prototype.getOrderHistory = function(orderType,field){
		if(!this.orderHistory[orderType]){
			this.orderHistory[orderType] = {};
		};
		if(!this.orderHistory[orderType].hasOwnProperty(field)){
			this.orderHistory[orderType][field] = 0;
		}
		return this.orderHistory[orderType][field];
	}
	PrimaryOrder.prototype.setOrderHistory = function(order,o){
		if(!this.orderHistory.hasOwnProperty(order)){//create the order history
			this.orderHistory[order] = {};
		}
		Object.assign(this.orderHistory[order],o);
	}
	PrimaryOrder.prototype.coordField = function(ds,order,type){
		var h = this.getOrderHistory(order,type);
		if(this.delta){
			h += ds.readInt8();
		}else{
			h = ds.readInt16();
		}
		return h;
	}
	PrimaryOrder.prototype.genericColor = function(ds){
		return [ds.readUint8(),ds.readUint8(),ds.readUint8()];
	}
	PrimaryOrder.prototype.oneByteHeader = function(ds){
		return new DataView(ds.readUint8Array(ds.readUint8()).buffer);
	}
	PrimaryOrder.prototype.twoByteHeader = function(ds){
		return new DataView(ds.readUint8Array(ds.readUint16()).buffer);
	}
	PrimaryOrder.prototype.deltaEncodedRectangle = function(dv,numRects){

		var rect = [0,0,0,0];//base rect [left,top,width,height];
		var rects = [];
		var ds = new DataStream(dv.buffer,dv.byteOffset);
		var zeroBitsArr = ds.readUint8Array(Math.ceil(numRects/2));
		var i =0,odd=0;

		while(numRects--){
			if( odd % 2 ){//odd
				rect[0] += (zeroBitsArr[i] & 0x08)?0:packedSignedFormat(ds);
				rect[1] += (zeroBitsArr[i] & 0x04)?0:packedSignedFormat(ds);
				rect[2] = (zeroBitsArr[i] & 0x02)?0:packedSignedFormat(ds);
				rect[3] = (zeroBitsArr[i] & 0x01)?0:packedSignedFormat(ds);
				i++;
			}else{
				rect[0] += (zeroBitsArr[i] & 0x80)?0:packedSignedFormat(ds);
				rect[1] += (zeroBitsArr[i] & 0x40)?0:packedSignedFormat(ds);
				rect[2] = (zeroBitsArr[i] & 0x20)?0:packedSignedFormat(ds);
				rect[3] = (zeroBitsArr[i] & 0x10)?0:packedSignedFormat(ds);
			}
			odd++;
			rects.push([rect[0],rect[1],rect[0]+rect[2],rect[1]+rect[3]]);
		}
		return rects;

	}
	PrimaryOrder.prototype.deltaEncodedPoints = function(dv,numPoints,pos){
		var o ={};
		var ds = new DataStream(dv.buffer,dv.byteOffset);
		var zeroBitsArr = ds.readUint8Array(Math.ceil(numPoints/4));
		var pt = [0,0];
		var pts = [];
		var i=0,odd=0;
		while(numPoints > odd){
			switch(odd % 4){
				case 0:
					pt[0] = (zeroBitsArr[i] & 0x80)?0:packedSignedFormat(ds);
					pt[1] = (zeroBitsArr[i] & 0x40)?0:packedSignedFormat(ds);
					break;
				case 1:
					pt[0] = (zeroBitsArr[i] & 0x20)?0:packedSignedFormat(ds);
					pt[1] = (zeroBitsArr[i] & 0x10)?0:packedSignedFormat(ds);
					break;
				case 2:
					pt[0] = (zeroBitsArr[i] & 0x08)?0:packedSignedFormat(ds);
					pt[1] = (zeroBitsArr[i] & 0x04)?0:packedSignedFormat(ds);
					break;
				case 3:
					pt[0] = (zeroBitsArr[i] & 0x02)?0:packedSignedFormat(ds);
					pt[1] = (zeroBitsArr[i] & 0x01)?0:packedSignedFormat(ds);
					i++;
					break;
			}
			pos[0] +=pt[0];
			pos[1] +=pt[1];
			pts.push([pos[0],pos[1]]);
			odd++;
		}
		return pts;
	}
	PrimaryOrder.prototype.package = function(ds,o){
		this.setOrderHistory(o.orderType,o);
		o.orderClass = 0x01;
		return o;
	}
	PrimaryOrder.prototype.dstBlt = function(ds){
		var o = {
			orderType:0x00
		};
		o.nLeftRect = (this.fieldFlags & 0x01)?
			this.coordField(ds,o.orderType,'nLeftRect'):
			this.getOrderHistory(o.orderType,'nLeftRect');

		o.nTopRect =(this.fieldFlags & 0x02)?
			this.coordField(ds,o.orderType,'nTopRect'):
			this.getOrderHistory(o.orderType,'nTopRect');

		o.nWidth = (this.fieldFlags & 0x04)?
			 this.coordField(ds,o.orderType,'nWidth'):
			 this.getOrderHistory(o.orderType,'nWidth');

		o.nHeight = (this.fieldFlags & 0x08)?
			 this.coordField(ds,o.orderType,'nHeight'):
			 this.getOrderHistory(o.orderType,'nHeight');

		o.bRop = (this.fieldFlags & 0x10)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'bRop');

		return this.package(ds,o);
	}
	PrimaryOrder.prototype.patBlt = function(ds){
		var o = {
			orderType:0x01
		};
		o.nLeftRect = (this.fieldFlags & 0x01)?//left
			 this.coordField(ds,o.orderType,'nLeftRect'):
			 this.getOrderHistory(o.orderType,'nLeftRect');
		o.nTopRect = (this.fieldFlags & 0x02)?//top
			 this.coordField(ds,o.orderType,'nTopRect'):
			 this.getOrderHistory(o.orderType,'nTopRect');
		o.nWidth = (this.fieldFlags & 0x04)?//width
			this.coordField(ds,o.orderType,'nWidth'):
			this.getOrderHistory(o.orderType,'nWidth');
		o.nHeight = (this.fieldFlags & 0x08)?//height
			this.coordField(ds,o.orderType,'nHeight'):
			this.getOrderHistory(o.orderType,'nHeight');
		o.bRop =(this.fieldFlags & 0x10)?//bRpd
				ds.readUint8():
				this.getOrderHistory(o.orderType,'bRop');
		o.BackColor = (this.fieldFlags & 0x20)?
			this.genericColor(ds):
			this.getOrderHistory(o.orderType,'BackColor');
		o.ForeColor = (this.fieldFlags & 0x40)?
			this.genericColor(ds):
			this.getOrderHistory(o.orderType,'ForeColor');
		o.BrushOrgX =  (this.fieldFlags & 0x80)?
			ds.readInt8():
			this.getOrderHistory(o.orderType,'BrushOrgX');
		o.BrushOrgY = (this.fieldFlags & 0x0100)?
			 ds.readInt8():
			 this.getOrderHistory(o.orderType,'BrushOrgY');
		o.BrushStyle = (this.fieldFlags & 0x0200)?
			ds.readUint8():
			this.getOrderHistory(o.orderType,'BrushStyle');
		o.BrushHatch =(this.fieldFlags & 0x0400)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'BrushHatch');
		o.BrushExtra = (this.fieldFlags & 0x0800)?
			 new DataView(ds.readUint8Array(7).buffer):
			 new DataView(new ArrayBuffer(7));
		return this.package(ds,o);
	}
	PrimaryOrder.prototype.srcBlt = function(ds){
		var o = {
			orderType:0x02
		};
		o.nLeftRect = (this.fieldFlags & 0x01)?
			this.coordField(ds,o.orderType,'nLeftRect'):
			 this.getOrderHistory(o.orderType,'nLeftRect');
		o.nTopRect = (this.fieldFlags & 0x02)?
			 this.coordField(ds,o.orderType,'nTopRect'):
			 this.getOrderHistory(o.orderType,'nTopRect');
		o.nWidth = (this.fieldFlags & 0x04)?
			 this.coordField(ds,o.orderType,'nWidth'):
			  this.getOrderHistory(o.orderType,'nWidth');
		o.nHeight =  (this.fieldFlags & 0x08)?
			this.coordField(ds,o.orderType,'nHeight'):
			 this.getOrderHistory(o.orderType,'nHeight');
		o.bRop =(this.fieldFlags & 0x10)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'bRop');
		o.nXSrc =(this.fieldFlags & 0x20)?
			 this.coordField(ds,o.orderType,'nXSrc'):
			  this.getOrderHistory(o.orderType,'nXSrc');
		o.nYSrc = (this.fieldFlags & 0x40)?
			this.coordField(ds,o.orderType,'nYSrc'):
			this.getOrderHistory(o.orderType,'nYSrc');
		return this.package(ds,o);
	}
	PrimaryOrder.prototype.drawNineGrid = function(ds){
		var o = {
			orderType:0x07
		};
		o.srcLeft = (this.fieldFlags & 0x01)?
			this.coordField(ds,o.orderType,'srcLeft'):
			this.getOrderHistory(o.orderType,'srcLeft');
		o.srcTop = (this.fieldFlags & 0x02)?
			this.coordField(ds,o.orderType,'srcTop'):
			this.getOrderHistory(o.orderType,'srcTop');
		o.srcRight = (this.fieldFlags & 0x04)?
			 this.coordField(ds,o.orderType,'srcRight'):
			 this.getOrderHistory(o.orderType,'srcRight');
		o.srcBottom=(this.fieldFlags & 0x08)?
			 this.coordField(ds,o.orderType,'srcBottom'):
			 this.getOrderHistory(o.orderType,'srcBottom');
		o.bitmapId =(this.fieldFlags & 0x10)?
			 ds.readUint16():
			 this.getOrderHistory(o.orderType,'bitmapId');
		return this.package(ds,o);
	}

	PrimaryOrder.prototype.lineTo = function(ds){
		var o = {
			orderType:0x09
		};
		o.BackMode = (this.fieldFlags & 0x01)?
			 ds.readUint16():
			 this.getOrderHistory(o.orderType,'BackMode');
		o.nXStart = (this.fieldFlags & 0x02)?
			 this.coordField(ds,o.orderType,'nXStart'):
			  this.getOrderHistory(o.orderType,'nXStart');
		o.nYStart =(this.fieldFlags & 0x04)?
			 this.coordField(ds,o.orderType,'nYStart'):
			  this.getOrderHistory(o.orderType,'nYStart');
		o.nXEnd =(this.fieldFlags & 0x08)?
			 this.coordField(ds,o.orderType,'nXEnd'):
			 this.getOrderHistory(o.orderType,'nXEnd');
		o.nYEnd =(this.fieldFlags & 0x10)?
			 this.coordField(ds,o.orderType,'nYEnd'):
			 this.getOrderHistory(o.orderType,'nYEnd');
		o.BackColor =(this.fieldFlags & 0x20)?
			 this.genericColor(ds):
			 this.getOrderHistory(o.orderType,'BackColor');
		o.bRop2 =(this.fieldFlags & 0x40)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'bRop2');
		o.PenStyle =(this.fieldFlags & 0x80)?
			 ds.readUint8():
			  this.getOrderHistory(o.orderType,'PenStyle');
		o.PenWidth =(this.fieldFlags & 0x0100)?
			ds.readUint8():
			 this.getOrderHistory(o.orderType,'PenWidth');
		o.PenColor =(this.fieldFlags & 0x0200)?
			 this.genericColor(ds):
			  this.getOrderHistory(o.orderType,'PenColor');
		return this.package(ds,o);
	}
	PrimaryOrder.prototype.opaqueRect = function(ds){
		var o = {
			orderType:0x0A
		};
		o.nLeftRect =  (this.fieldFlags & 0x01)?//left
			this.coordField(ds,o.orderType,'nLeftRect'):
			this.getOrderHistory(o.orderType,'nLeftRect');
		o.nTopRect = (this.fieldFlags & 0x02)?//top
			 this.coordField(ds,o.orderType,'nTopRect'):
			 this.getOrderHistory(o.orderType,'nTopRect');
		o.nWidth = (this.fieldFlags & 0x04)?//width
			 this.coordField(ds,o.orderType,'nWidth'):
			 this.getOrderHistory(o.orderType,'nWidth');
		o.nHeight = (this.fieldFlags & 0x08)?//height
			 this.coordField(ds,o.orderType,'nHeight'):
			  this.getOrderHistory(o.orderType,'nHeight');
		o.RedOrPaletteIndex = (this.fieldFlags & 0x10)?//red
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'RedOrPaletteIndex');
		o.Green =(this.fieldFlags & 0x20)?//green
			ds.readUint8():
			 this.getOrderHistory(o.orderType,'Green');
		o.Blue = (this.fieldFlags & 0x40)?//blue
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'Blue');
		return this.package(ds,o);
	}
	PrimaryOrder.prototype.saveBitmap = function(ds){
		var o = {
			orderType:0x0B
		};
		o.SavedBitmapPosition =(this.fieldFlags & 0x01)?
			 ds.readUint32():
			 this.getOrderHistory(o.orderType,'SavedBitmapPosition');
		o.nLeftRect =(this.fieldFlags & 0x02)?
			 this.coordField(ds,o.orderType,'nLeftRect'):
			  this.getOrderHistory(o.orderType,'nLeftRect');
		o.nTopRect =(this.fieldFlags & 0x04)?
			 this.coordField(ds,o.orderType,'nTopRect'):
			  this.getOrderHistory(o.orderType,'nTopRect');
		o.nRightRect =(this.fieldFlags & 0x08)?
			 this.coordField(ds,o.orderType,'nRightRect'):
			  this.getOrderHistory(o.orderType,'nRightRect');
		o.nBottomRect =(this.fieldFlags & 0x10)?
			 this.coordField(ds,o.orderType,'nBottomRect'):
			  this.getOrderHistory(o.orderType,'nBottomRect');
		o.Operation = (this.fieldFlags & 0x20)?
			 this.coordField(ds,o.orderType,'Operation'):
			 this.getOrderHistory(o.orderType,'Operation');
		return this.package(ds,o);
	}
	PrimaryOrder.prototype.memBlt = function(ds){
		var o = {
			orderType:0x0D
		};
		if(this.fieldFlags & 0x01){
			o.BitmapCacheId = ds.readUint8();
			o.ColorTableCacheIndex = ds.readUint8();
		}else{
			o.BitmapCacheId = this.getOrderHistory(o.orderType,'BitmapCacheId');
			o.ColorTableCacheIndex = this.getOrderHistory(o.orderType,'ColorTableCacheIndex');
		}
		o.nLeftRect = (this.fieldFlags & 0x02)?
			 this.coordField(ds,o.orderType,'nLeftRect'):
			  this.getOrderHistory(o.orderType,'nLeftRect');
		o.nTopRect = (this.fieldFlags & 0x04)?
			 this.coordField(ds,o.orderType,'nTopRect'):
			 this.getOrderHistory(o.orderType,'nTopRect');
		o.nWidth = (this.fieldFlags & 0x08)?
			this.coordField(ds,o.orderType,'nWidth'):
			 this.getOrderHistory(o.orderType,'nWidth');
		o.nHeight =(this.fieldFlags & 0x10)?
			 this.coordField(ds,o.orderType,'nHeight'):
			 this.getOrderHistory(o.orderType,'nHeight');
		o.bRop =(this.fieldFlags & 0x20)?
			 ds.readUint8():
			  this.getOrderHistory(o.orderType,'bRop');
		o.nXSrc = (this.fieldFlags & 0x40)?
			this.coordField(ds,o.orderType,'nXSrc'):
			this.getOrderHistory(o.orderType,'nXSrc');
		o.nYSrc =(this.fieldFlags & 0x80)?
			 this.coordField(ds,o.orderType,'nYSrc'):
			 this.getOrderHistory(o.orderType,'nYSrc');
		o.cacheIndex = (this.fieldFlags & 0x0100)?
			ds.readUint16():
			this.getOrderHistory(o.orderType,'cacheIndex');
		return this.package(ds,o);
	}
	PrimaryOrder.prototype.mem3Blt = function(ds){
		var o = {
			orderType:0x0E
		};
		if(this.fieldFlags & 0x01){
			o.BitmapCacheId = ds.readUint8();
			o.ColorTableCacheIndex = ds.readUint8();
		}else{
			o.BitmapCacheId = this.getOrderHistory(o.orderType,'BitmapCacheId');
			o.ColorTableCacheIndex = this.getOrderHistory(o.orderType,'ColorTableCacheIndex');
		}
		o.nLeftRect = (this.fieldFlags & 0x02)?
			this.coordField(ds,o.orderType,'nLeftRect'):
			this.getOrderHistory(o.orderType,'nLeftRect');
		o.nTopRect =(this.fieldFlags & 0x04)?
			 this.coordField(ds,o.orderType,'nTopRect'):
			 this.getOrderHistory(o.orderType,'nTopRect');
		o.nWidth=(this.fieldFlags & 0x08)?
			 this.coordField(ds,o.orderType,'nWidth'):
			 this.getOrderHistory(o.orderType,'nWidth');
		o.nHeight = (this.fieldFlags & 0x10)?
			this.coordField(ds,o.orderType,'nHeight'):
			 this.getOrderHistory(o.orderType,'nHeight');
		o.bRop = (this.fieldFlags & 0x20)?
			ds.readUint8():
			this.getOrderHistory(o.orderType,'bRop');
		o.nXSrc =(this.fieldFlags & 0x40)?
			 this.coordField(ds,o.orderType,'nXSrc'):
			 this.getOrderHistory(o.orderType,'nXSrc');
		o.nYSrc =(this.fieldFlags & 0x80)?
			 this.coordField(ds,o.orderType,'nYSrc'):
			 this.getOrderHistory(o.orderType,'nYSrc');
		o.BackColor = (this.fieldFlags & 0x0100)?
			this.genericColor(ds):
			 this.getOrderHistory(o.orderType,'BackColor');
		o.ForeColor =(this.fieldFlags & 0x0200)?
			 this.genericColor(ds):
			  this.getOrderHistory(o.orderType,'ForeColor');
		o.BrushOrgX =(this.fieldFlags & 0x0400)?
			 ds.readInt8():
			  this.getOrderHistory(o.orderType,'BrushOrgX');
		o.BrushOrgY =(this.fieldFlags & 0x0800)?
			 ds.readInt8():
			 this.getOrderHistory(o.orderType,'BrushOrgY');
		o.BrushStyle =(this.fieldFlags & 0x1000)?
			 ds.readUint8():
			  this.getOrderHistory(o.orderType,'BrushStyle');
		o.BrushHatch =(this.fieldFlags & 0x2000)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'BrushHatch');
		o.BrushExtra =(this.fieldFlags & 0x4000)?
			 new DataView(dsreadUint8Array(7).buffer):
			 this.getOrderHistory(o.orderType,'BrushExtra');
		o.cacheIndex =(this.fieldFlags & 0x8000)?
			 ds.readUint16():
			  this.getOrderHistory(o.orderType,'cacheIndex');
		return this.package(ds,o);
	}

	PrimaryOrder.prototype.fastIndex = function(ds){
		var o = {
			orderType:0x13
		};
		o.cacheId = (this.fieldFlags & 0x01)?
			ds.readUint8():
			this.getOrderHistory(o.orderType,'cacheId');
		o.fDrawing = (this.fieldFlags & 0x02)?
			 ds.readUint16():
			 this.getOrderHistory(o.orderType,'fDrawing');
		o.BackColor = (this.fieldFlags & 0x04)?
			 this.genericColor(ds):
			  this.getOrderHistory(o.orderType,'BackColor');
		o.ForeColor =(this.fieldFlags & 0x08)?
			 this.genericColor(ds):
			  this.getOrderHistory(o.orderType,'ForeColor');
		o.BkLeft =(this.fieldFlags & 0x10)?
			 this.coordField(ds,o.orderType,'BkLeft'):
			  this.getOrderHistory(o.orderType,'BkLeft');
		o.BkTop =(this.fieldFlags & 0x20)?
			 this.coordField(ds,o.orderType,'BkTop'):
			 this.getOrderHistory(o.orderType,'BkTop');
		o.BkRight = (this.fieldFlags & 0x40)?
			this.coordField(ds,o.orderType,'BkRight'):
			 this.getOrderHistory(o.orderType,'BkRight');
		o.BkBottom = (this.fieldFlags & 0x80)?
			this.coordField(ds,o.orderType,'BkBottom'):
			this.getOrderHistory(o.orderType,'BkBottom');
		o.OpLeft =(this.fieldFlags & 0x0100)?
			 this.coordField(ds,o.orderType,'OpLeft'):
			 this.getOrderHistory(o.orderType,'OpLeft');
		o.OpTop =(this.fieldFlags & 0x0200)?
			this.coordField(ds,o.orderType,'OpTop'):
			 this.getOrderHistory(o.orderType,'OpTop');
		o.OpRight = (this.fieldFlags & 0x0400)?
			this.coordField(ds,o.orderType,'OpRight'):
			 this.getOrderHistory(o.orderType,'OpRight');
		o.OpBottom = (this.fieldFlags & 0x0800)?
			 this.coordField(ds,o.orderType,'OpBottom'):
			  this.getOrderHistory(o.orderType,'OpBottom');
		o.X =(this.fieldFlags & 0x1000)?
			 this.coordField(ds,o.orderType,'X'):
			  this.getOrderHistory(o.orderType,'X');
		o.Y = (this.fieldFlags & 0x2000)?
			this.coordField(ds,o.orderType,'Y'):
			 this.getOrderHistory(o.orderType,'Y');
		o.VariableBytes = (this.fieldFlags & 0x4000)?
			this.oneByteHeader(ds):
			 this.getOrderHistory(o.orderType,'VariableBytes');
		return this.package(ds,o);
	}
	PrimaryOrder.prototype.polygonSC = function(ds){
		var o = {
			orderType:0x14
		};
		o.xStart = (this.fieldFlags & 0x01)?
			this.coordField(ds,o.orderType,'xStart'):
			 this.getOrderHistory(o.orderType,'xStart');
		o.yStart = (this.fieldFlags & 0x02)?
			this.coordField(ds,o.orderType,'yStart'):
			this.getOrderHistory(o.orderType,'yStart');
		o.bRop2 =(this.fieldFlags & 0x04)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'bRop2');
		o.FillMode =(this.fieldFlags & 0x08)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'FillMode');
		o.BrushColor =(this.fieldFlags & 0x10)?
			 this.genericColor(ds):
			 this.getOrderHistory(o.orderType,'BrushColor');
		o.NumDeltaEntries = (this.fieldFlags & 0x20)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'NumDeltaEntries');
		o.points =(this.fieldFlags & 0x40)?
			 this.deltaEncodedPoints(this.oneByteHeader(ds),
			o.NumDeltaEntries,[o.xStart,o.yStart]):
			this.getOrderHistory(o.orderType,'points');
		return this.package(ds,o);
	}
	PrimaryOrder.prototype.polygonCB = function(ds){
		var o = {
			orderType:0x15
		};
		o.xStart = (this.fieldFlags & 0x01)?
			this.coordField(ds,o.orderType,'xStart'):
			this.getOrderHistory(o.orderType,'xStart');
		o.yStart =(this.fieldFlags & 0x02)?
			 this.coordField(ds,o.orderType,'yStart'):
			 this.getOrderHistory(o.orderType,'yStart');
		o.bRop2 =(this.fieldFlags & 0x04)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'bRop2');
		o.FillMode =(this.fieldFlags & 0x08)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'FillMode');
		o.BackColor =(this.fieldFlags & 0x10)?
			 this.genericColor(ds):
			 this.getOrderHistory(o.orderType,'BackColor');
		o.ForeColor = (this.fieldFlags & 0x20)?
			this.genericColor(ds):
			this.getOrderHistory(o.orderType,'ForeColor');
		o.BrushOrgX =(this.fieldFlags & 0x40)?
			 ds.readInt8():
			 this.getOrderHistory(o.orderType,'BrushOrgX');
		o.BrushOrgY =(this.fieldFlags & 0x80)?
			 ds.readInt8():
			 this.getOrderHistory(o.orderType,'BrushOrgY');
		o.BrushStyle =(this.fieldFlags & 0x0100)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'BrushStyle');
		o.BrushHatch =(this.fieldFlags & 0x0200)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'BrushHatch');
		o.BrushExtra =(this.fieldFlags & 0x0400)?
			 new DataView(ds.readUint8Array(7).buffer):
			 this.getOrderHistory(o.orderType,'BrushExtra');
		o.NumDeltaEntries =(this.fieldFlags & 0x0800)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'NumDeltaEntries');
		o.points =(this.fieldFlags & 0x1000)?
			 this.deltaEncodedPoints(this.oneByteHeader(ds),o.NumDeltaEntries,[o.xStart,o.yStart]):
			 this.getOrderHistory(o.orderType,'points');
		return this.package(ds,o);
	}
	PrimaryOrder.prototype.polyline = function(ds){
		var o = {
			orderType:0x16
		};
		o.xStart = (this.fieldFlags & 0x01)?
			this.coordField(ds,o.orderType,'xStart'):
			this.getOrderHistory(o.orderType,'xStart');
		o.yStart =(this.fieldFlags & 0x02)?
			 this.coordField(ds,o.orderType,'yStart'):
			 this.getOrderHistory(o.orderType,'yStart');
		o.bRop2 =(this.fieldFlags & 0x04)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'bRop2');
		o.BrushCacheEntry =(this.fieldFlags & 0x08)?
			 ds.readUint16():
			 this.getOrderHistory(o.orderType,'BrushCacheEntry');
		o.PenColor = (this.fieldFlags & 0x10)?
			this.genericColor(ds):
			this.getOrderHistory(o.orderType,'PenColor');
		o.NumDeltaEntries =(this.fieldFlags & 0x20)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'NumDeltaEntries');
		o.points =(this.fieldFlags & 0x40)?
			 this.deltaEncodedPoints(this.oneByteHeader(ds),o.NumDeltaEntries,[o.xStart,o.yStart]):
			this.getOrderHistory(o.orderType,'points');
		return this.package(ds,o);
	}

	PrimaryOrder.prototype.ellipseSC = function(ds){
		var o = {
			orderType:0x19
		};
		o.LeftRect =(this.fieldFlags & 0x01)?
			 this.coordField(ds,o.orderType,'LeftRect'):
			 this.getOrderHistory(o.orderType,'LeftRect');
		o.TopRect =(this.fieldFlags & 0x02)?
			 this.coordField(ds,o.orderType,'TopRect'):
			 this.getOrderHistory(o.orderType,'TopRect');
		o.RightRect = (this.fieldFlags & 0x04)?
			this.coordField(ds,o.orderType,'RightRect'):
			this.getOrderHistory(o.orderType,'RightRect');
		o.BottomRect =(this.fieldFlags & 0x08)?
			 this.coordField(ds,o.orderType,'BottomRect'):
			 this.getOrderHistory(o.orderType,'BottomRect');
		o.bRop2 =(this.fieldFlags & 0x10)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'bRop2');
		o.FillMode = (this.fieldFlags & 0x20)?
			ds.readUint8():
			this.getOrderHistory(o.orderType,'FillMode');
		o.Color =(this.fieldFlags & 0x40)?
			 this.genericColor(ds):
			 this.getOrderHistory(o.orderType,'Color');
		return this.package(ds,o);
	}
	PrimaryOrder.prototype.ellipseCB = function(ds){
		var o = {
			orderType:0x1A
		};
		o.LeftRect = (this.fieldFlags & 0x01)?
			this.coordField(ds,o.orderType,'LeftRect'):
			 this.getOrderHistory(o.orderType,'LeftRect');
		o.TopRect =(this.fieldFlags & 0x02)?
			 this.coordField(ds,o.orderType,'TopRect'):
			  this.getOrderHistory(o.orderType,'TopRect');
		o.RightRect = (this.fieldFlags & 0x04)?
			this.coordField(ds,o.orderType,'RightRect'):
			 this.getOrderHistory(o.orderType,'RightRect');
		o.BottomRect =(this.fieldFlags & 0x08)?
			 this.coordField(ds,o.orderType,'BottomRect'):
			  this.getOrderHistory(o.orderType,'BottomRect');
		o.bRop2 =(this.fieldFlags & 0x10)?
			 ds.readUint8():
			  this.getOrderHistory(o.orderType,'bRop2');
		o.FillMode = (this.fieldFlags & 0x20)?
			ds.readUint8():
			 this.getOrderHistory(o.orderType,'FillMode');
		o.BackColor =(this.fieldFlags & 0x40)?
			 this.genericColor(ds):
			  this.getOrderHistory(o.orderType,'BackColor');
		o.ForeColor = (this.fieldFlags & 0x80)?
			this.genericColor(ds):
			 this.getOrderHistory(o.orderType,'ForeColor');
		o.BrushOrgX =(this.fieldFlags & 0x0100)?
			 ds.readInt8():
			  this.getOrderHistory(o.orderType,'BrushOrgX');
		o.BrushOrgY = (this.fieldFlags & 0x0200)?
			ds.readInt8():
			 this.getOrderHistory(o.orderType,'BrushOrgY');
		o.BrushStyle =(this.fieldFlags & 0x0400)?
			 ds.readUint8():
			  this.getOrderHistory(o.orderType,'BrushStyle');
		o.BrushHatch = (this.fieldFlags & 0x0800)?
			ds.readUint8():
			 this.getOrderHistory(o.orderType,'BrushHatch');
		o.BrushExtra =(this.fieldFlags & 0x1000)?
			 new DataView(ds.readUint8Array(7)):
			 this.getOrderHistory(o.orderType,'BrushExtra');
		return this.package(ds,o);
	}

	PrimaryOrder.prototype.multiDstBlt = function(ds){
		var o = {
			orderType:0x0F
		};
		 o.nLeftRect =(this.fieldFlags & 0x01)?
			 this.coordField(ds,o.orderType,'nLeftRect'):
			 this.getOrderHistory(o.orderType,'nLeftRect');
		o.nTopRect = (this.fieldFlags & 0x02)?
			this.coordField(ds,o.orderType,'nTopRect'):
			this.getOrderHistory(o.orderType,'nTopRect');
		o.nWidth = (this.fieldFlags & 0x04)?
			 this.coordField(ds,o.orderType,'nWidth'):
			 this.getOrderHistory(o.orderType,'nWidth');
		o.nHeight = (this.fieldFlags & 0x08)?
			 this.coordField(ds,o.orderType,'nHeight'):
			 this.getOrderHistory(o.orderType,'nHeight');
		o.bRop = (this.fieldFlags & 0x10)?
			ds.readUint8():
			this.getOrderHistory(o.orderType,'bRop');

		o.nDeltaEntries = (this.fieldFlags & 0x20)?
			ds.readUint8():
			this.getOrderHistory(o.orderType,'nDeltaEntries');
		o.rectangles = (this.fieldFlags & 0x40)?
			 this.deltaEncodedRectangle(this.twoByteHeader(ds),o.nDeltaEntries):
			 this.getOrderHistory(o.orderType,'rectangles');

		return this.package(ds,o);
	}
	PrimaryOrder.prototype.multiPatBlt = function(ds){
		var o = {
			orderType:0x10
		};
		o.nLeftRect = (this.fieldFlags & 0x01)?
			this.coordField(ds,o.orderType,'nLeftRect'):
			 this.getOrderHistory(o.orderType,'nLeftRect');
		o.nTopRect = (this.fieldFlags & 0x02)?
			this.coordField(ds,o.orderType,'nTopRect'):
			 this.getOrderHistory(o.orderType,'nTopRect');
		o.nWidth =(this.fieldFlags & 0x04)?
			 this.coordField(ds,o.orderType,'nWidth'):
			 this.getOrderHistory(o.orderType,'nWidth');
		o.nHeight =(this.fieldFlags & 0x08)?
			 this.coordField(ds,o.orderType,'nHeight'):
			  this.getOrderHistory(o.orderType,'nHeight');
		o.bRop =(this.fieldFlags & 0x10)?
			 ds.readUint8():
			  this.getOrderHistory(o.orderType,'bRop');
		o.BackColor = (this.fieldFlags & 0x20)?
			this.genericColor(ds):
			 this.getOrderHistory(o.orderType,'BackColor');
		o.ForeColor = (this.fieldFlags & 0x40)?
			this.genericColor(ds):
			 this.getOrderHistory(o.orderType,'ForeColor');
		o.BrushOrgX =(this.fieldFlags & 0x80)?
			 ds.readInt8():
			  this.getOrderHistory(o.orderType,'BrushOrgX');
		o.BrushOrgY = (this.fieldFlags & 0x0100)?
			ds.readInt8():
			 this.getOrderHistory(o.orderType,'BrushOrgY');
		o.BrushStyle =(this.fieldFlags & 0x0200)?
			 ds.readUint8():
			  this.getOrderHistory(o.orderType,'BrushStyle');
		o.BrushHatch = (this.fieldFlags & 0x0400)?
			ds.readUint8():
			 this.getOrderHistory(o.orderType,'BrushHatch');
		o.BrushExtra =(this.fieldFlags & 0x0800)?
			 new DataView(ds.readUint8Array(7)):
			  this.getOrderHistory(o.orderType,'BrushExtra');
		o.nDeltaEntries =(this.fieldFlags & 0x1000)?
			 ds.readUint8():
			  this.getOrderHistory(o.orderType,'nDeltaEntries');
		o.rectangles = (this.fieldFlags & 0x2000)?
			this.deltaEncodedRectangle(this.twoByteHeader(ds),o.nDeltaEntries):
			 this.getOrderHistory(o.orderType,'rectangles');
		return this.package(ds,o);
	}
	PrimaryOrder.prototype.multiSrcBlt = function(ds){
		var o = {
			orderType:0x11
		};
		o.nLeftRect =(this.fieldFlags & 0x01)?
			 this.coordField(ds,o.orderType,'nLeftRect'):
			 this.getOrderHistory(o.orderType,'nLeftRect');
		o.nTopRect =(this.fieldFlags & 0x02)?
			 this.coordField(ds,o.orderType,'nTopRect'):
			  this.getOrderHistory(o.orderType,'nTopRect');
		o.nWidth =(this.fieldFlags & 0x04)?
			 this.coordField(ds,o.orderType,'nWidth'):
			  this.getOrderHistory(o.orderType,'nWidth');
		o.nHeight = (this.fieldFlags & 0x08)?
			this.coordField(ds,o.orderType,'nHeight'):
			 this.getOrderHistory(o.orderType,'nHeight');
		o.bRop =(this.fieldFlags & 0x10)?
			 ds.readUint8():
			  this.getOrderHistory(o.orderType,'bRop');
		o.nXSrc =(this.fieldFlags & 0x20)?
			 this.coordField(ds,o.orderType,'nXSrc'):
			  this.getOrderHistory(o.orderType,'nXSrc');
		o.nYSrc = (this.fieldFlags & 0x40)?
			this.coordField(ds,o.orderType,'nYSrc'):
			 this.getOrderHistory(o.orderType,'nYSrc');
		o.nDeltaEntries =(this.fieldFlags & 0x80)?
			 ds.readUint8():
			  this.getOrderHistory(o.orderType,'nDeltaEntries');
		o.rectangles = (this.fieldFlags & 0x0100)?
			this.deltaEncodedRectangle(this.twoByteHeader(ds),o.nDeltaEntries):
			 this.getOrderHistory(o.orderType,'rectangles');
		return this.package(ds,o);
	}
	PrimaryOrder.prototype.multiOpaqueRect = function(ds){
		var o = {
			orderType:0x12
		};
		o.nLeftRect = (this.fieldFlags & 0x01)?
			this.coordField(ds,o.orderType,'nLeftRect'):
			this.getOrderHistory(o.orderType,'nLeftRect');
		o.nTopRect = (this.fieldFlags & 0x02)?
			 this.coordField(ds,o.orderType,'nTopRect'):
			 this.getOrderHistory(o.orderType,'nTopRect');
		o.nWidth = (this.fieldFlags & 0x04)?
			this.coordField(ds,o.orderType,'nWidth'):
			 this.getOrderHistory(o.orderType,'nWidth');
		o.nHeight = (this.fieldFlags & 0x08)?
			 this.coordField(ds,o.orderType,'nHeight'):
			 this.getOrderHistory(o.orderType,'nHeight');
		o.RedOrPaletteIndex =(this.fieldFlags & 0x10)?
			 ds.readUint8():
			  this.getOrderHistory(o.orderType,'RedOrPaletteIndex');
		o.Green = (this.fieldFlags & 0x20)?
			ds.readUint8():
			this.getOrderHistory(o.orderType,'Green');
		o.Blue =  (this.fieldFlags & 0x40)?
			ds.readUint8():
			this.getOrderHistory(o.orderType,'Blue');
		o.nDeltaEntries = (this.fieldFlags & 0x80)?
			ds.readUint8():
			this.getOrderHistory(o.orderType,'nDeltaEntries');
		o.rectangles = (this.fieldFlags & 0x0100)?
			this.deltaEncodedRectangle(this.twoByteHeader(ds),o.nDeltaEntries):
			this.getOrderHistory(o.orderType,'rectangles');

		return this.package(ds,o);
	}
	PrimaryOrder.prototype.multiDrawNineGrid = function(ds){
		var o = {
			orderType:0x08
		};
		o.srcLeft =(this.fieldFlags & 0x01)?
			 this.coordField(ds,o.orderType,'srcLeft'):
			 this.getOrderHistory(o.orderType,'srcLeft');
		o.srcTop =(this.fieldFlags & 0x02)?
			 this.coordField(ds,o.orderType,'srcTop'):
			 this.getOrderHistory(o.orderType,'srcTop');
		o.srcRight = (this.fieldFlags & 0x04)?
			this.coordField(ds,o.orderType,'srcRight'):
			this.getOrderHistory(o.orderType,'srcRight');
		o.srcBottom = (this.fieldFlags & 0x08)?
			this.coordField(ds,o.orderType,'srcBottom'):
			this.getOrderHistory(o.orderType,'srcBottom');
		o.bitmapId = (this.fieldFlags & 0x10)?
			ds.readUint16():
			this.getOrderHistory(o.orderType,'bitmapId');
		o.nDeltaEntries = (this.fieldFlags & 0x20)?
			 ds.readUint8():
			 this.getOrderHistory(o.orderType,'nDeltaEntries');
		o.rectangles =(this.fieldFlags & 0x040)?
			 this.deltaEncodedRectangle(this.twoByteHeader(ds),o.nDeltaEntries):
			 this.getOrderHistory(o.orderType,'rectangles');
		return this.package(ds,o);
	}


	PrimaryOrder.prototype.fastGlyph = function(ds){
		console.log("TODO FIX FAST GLYPH");
		function varBytes(dv,ulCharInc){
			var ds = new DataStream(dv.buffer,dv.byteOffset);
			var len = dv.byteLength;
			var b,d,s;
			var glyphs = [];
			while(len--){
				b = ds.readUint8();
				if(b === 0xFF){//add
					s = ds.readUint8();//index to store cache
					if(dv.byteLength - 3 || ds.readUint8()){
						console.log('invalid final byte Length for glyphCache');
					}
				}else{
					if(b = 0xFE){// use previously stored fragment
						s = ds.readUint8(); //index to the store
					}
					if(!ulCharInc){
						d = ds.readUint8();
						if(d == 0x80){
							d = ds.readUint16();
						}
					}else{
						d = ulCharInc
					}
					glyphs.push({
						index:b,
						delta:d
					});
				}
				switch(b){
					case 0xFE://Use Fragment

						break;
					case 0xFF: //Add
						break;
					default://glyph Index

						break;
				}
			}
		}
		var o = {
			orderType:0x18
		};
		if(this.fieldFlags & 0x01){
			o.cacheId = ds.readUint8();
		}if(this.fieldFlags & 0x02){
			o.fDrawing = ds.readUint16();
		}if(this.fieldFlags & 0x04){
			o.BackColor = this.genericColor(ds);
		}if(this.fieldFlags & 0x08){
			o.ForeColor = this.genericColor(ds);
		}if(this.fieldFlags & 0x10){
			o.BkLeft = this.coordField(ds,o.orderType,'BkLeft');
		}if(this.fieldFlags & 0x20){
			o.BkTop = this.coordField(ds,o.orderType,'BkTop');
		}if(this.fieldFlags & 0x40){
			o.BkRight = this.coordField(ds,o.orderType,'BkRight');
		}if(this.fieldFlags & 0x80){
			o.BkBottom = this.coordField(ds,o.orderType,'BkBottom');
		}if(this.fieldFlags & 0x0100){
			o.OpLeft = this.coordField(ds,o.orderType,'OpLeft');
		}if(this.fieldFlags & 0x0200){
			o.OpTop = this.coordField(ds,o.orderType,'OpTop');
		}if(this.fieldFlags & 0x0400){
			o.OpRight = this.coordField(ds,o.orderType,'OpRight');
		}if(this.fieldFlags & 0x0800){
			o.OpBottom = this.coordField(ds,o.orderType,'OpBottom');
		}if(this.fieldFlags & 0x1000){
			o.X = this.coordField(ds,o.orderType,'X');
		}if(this.fieldFlags & 0x2000){
			o.Y = this.coordField(ds,o.orderType,'Y');
		}if(this.fieldFlags & 0x4000){
			o.VariableBytes = this.oneByteHeader(ds);
		}
		return this.package(ds,o);
	}
	PrimaryOrder.prototype.glyphIndex = function(ds){
		console.log('TODO FIX GLYPH INDEX');
		var o = {
			orderType:0x18
		};
		if(this.fieldFlags & 0x01){
			o.cacheId  = ds.readUint8();
		}if(this.fieldFlags & 0x02){
			o.flAccel = ds.readUint8();
		}if(this.fieldFlags & 0x04){
			o.ulCharInc = ds.readUint8();
		}if(this.fieldFlags & 0x08){
			o.fOpRedundant = ds.readUint8();
		}if(this.fieldFlags & 0x10){
			o.BackColor = this.genericColor(ds);
		}if(this.fieldFlags & 0x20){
			o.ForeColor = this.genericColor(ds);
		}if(this.fieldFlags & 0x40){
			o.BkLeft = ds.readInt16();
		}if(this.fieldFlags & 0x80){
			o.BkTop = ds.readInt16();
		}if(this.fieldFlags & 0x0100){
			o.BkRight = ds.readInt16();
		}if(this.fieldFlags & 0x0200){
			o.BkBottom = ds.readInt16();
		}if(this.fieldFlags & 0x0400){
			o.OpLeft = ds.readInt16();
			if(o.fOpRedundant && o.OpLeft){
				console.log('must be set to 0!!!');
			}
		}if(this.fieldFlags & 0x0800){
			o.OpTop = ds.readInt16();
			if(o.fOpRedundant && o.OpTop){
				console.log('must be set to 0!!!');//must be set to 0!!!
			}
		}if(this.fieldFlags & 0x1000){
			o.OpRight= ds.readInt16();
			if(o.fOpRedundant && o.OpRight){
				console.log('must be set to 0!!!');//must be set to 0!!!
			}
		}if(this.fieldFlags & 0x2000){
			o.OpBottom = ds.readInt16();
			if(o.fOpRedundant && o.OpBottom){
				console.log('must be set to 0!!!');//must be set to 0!!!
			}
		}if(this.fieldFlags & 0x4000){
			o.BrushOrgX = ds.readUint8();
		}if(this.fieldFlags & 0x8000){
			o.BrushOrgY = ds.readUint8();
		}if(this.fieldFlags & 0x00010000){
			o.BrushStyle = ds.readUint8();
		}if(this.fieldFlags & 0x00020000){
			o.BrushHatch = ds.readUint8();
		}if(this.fieldFlags & 0x00040000){
			o.BrushExtra = new DataView(ds.readUint8Attay(7).buffer);
			ds.byteOffset+=7;
		}if(this.fieldFlags & 0x00080000){
			o.X = ds.readInt16();
		}if(this.fieldFlags & 0x00100000){
			o.Y = ds.readInt16();
		}if(this.fieldFlags & 0x00200000){
			o.VariableBytes = this.oneByteHeader(ds);
		}
		return this.package(ds,o);
	}



	var po = new PrimaryOrder();
	//https://msdn.microsoft.com/en-us/library/cc241586.aspx
	function process(dv){

		var ds = new DataStream(dv.buffer,dv.byteOffset);
		po.decodeHeader(ds);
		var o ={};
		switch(po.orderType){
			case 0x00://TS_ENC_DSTBLT_ORDER
				o = po.dstBlt(ds);
				break;
			case 0x01://TS_ENC_PATBLT_ORDER
				o = po.patBlt(ds);
				break
			case 0x02:
				o = po.srcBlt(ds);
				break;
			case 0x07://TS_ENC_DRAWNINEGRID_ORDER
				o = po.drawNineGrid(ds);
				break;
			case 0x08://TS_ENC_MULTI_DRAWNINEGRID_ORDER
				o = po.multiDrawNineGrid(ds);
				break;
			case 0x09://TS_ENC_LINETO_ORDER
				o = po.lineTo(ds);
				break;
			case 0x0A://TS_ENC_OPAQUERECT_ORDER
				o = po.opaqueRect(ds);
				break;
			case 0x0B://TS_ENC_SAVEBITMAP_ORDER
				o = po.saveBitmap(ds);
				break;
			case 0x0D://TS_ENC_MEMBLT_ORDER
				o = po.memBlt(ds);
				break;
			case 0x0E://TS_ENC_MEM3BLT_ORDER
				o = po.mem3Blt(ds);
				break;
			case 0x0F://TS_ENC_MULTIDSTBLT_ORDER
				o = po.multiDstBlt(ds);
				break;
			case 0x10://TS_ENC_MULTIPATBLT_ORDER
				o = po.multiPatBlt(ds);
				break;
			case 0x11://TS_ENC_MULTISCRBLT_ORDER
				o = po.multiSrcBlt(ds);
				break;
			case 0x12://TS_ENC_MULTIOPAQUERECT_ORDER
				o = po.multiOpaqueRect(ds);
				break;
			case 0x13://TS_ENC_FAST_INDEX_ORDER
				o = po.fastIndex(ds);
				break;
			case 0x14://TS_ENC_POLYGON_SC_ORDER
				o = po.polygonSC(ds);
				break;
			case 0x15://TS_ENC_POLYGON_CB_ORDER
				o = po.polygonCB(ds);
				break;
			case 0x16://TS_ENC_POLYLINE_ORDER
				o = po.polyline(ds);
				break;
			case 0x18://TS_ENC_FAST_GLYPH_ORDER
				o = po.fastGlyph(ds);
				break;
			case 0x19://TS_ENC_ELLIPSE_SC_ORDER
				o = po.ellipseSC(ds);
				break;
			case 0x1A://TS_ENC_ELLIPSE_CB_ORDER
				o = po.ellipseCB(ds);
				break;
			case 0x1B://TS_ENC_INDEX_ORDER
				o = po.glyphIndex(ds);
				break;
			default:
				console.log('Unknown Order Type',o);
		}
		return {
			data:o,
			dv:new DataView(ds.buffer,ds.byteOffset,ds.byteLength-ds.position)
		};
	}

module.exports = {
    process:process
}
