'use strict';
    var DataStream          = require('DataStream');
    var capabilitySettings  = require('xrdp/capability-settings');
    var Surface             = require('xrdp/fast-path/surface/surface');

	function decode(dv,codecId){
		return dv;
	}
	function SecondaryOrder(){

	}
	//https://msdn.microsoft.com/en-us/library/cc241611.aspx
	SecondaryOrder.prototype.decodeHeader = function(dv){
		var h ={};
		h.controlFlags = dv.getUint8(0);
		h.orderLength = dv.getUint16(1,true) + 13;//add the 13 for historical reasons (from the doc)
		h.extraFlags = dv.getUint16(3,true);
		h.orderType = dv.getUint8(5);

		return {header:h,dv:new DataView(dv.buffer,dv.byteOffset+6)};
	}
	SecondaryOrder.prototype.decodeTwoByteUnsigned = function(ds){
		var val2=0;
		var val1 = ds.readUint8() & 0xFE;
		if(ds.readUint8() & 0x01){//c is present
			val2 = ds.readUint8();
		}
		return val1+(0xFF00*val2);
	}
	SecondaryOrder.prototype.decodeTwoByteSigned = function(ds){
		var val2 =0;
		var val1 = (ds.readUint8() & 0xFC);
		var neg = (ds.readUint8() & 0x02)?1:-1;
		if(ds.readUint8() & 0x01){//c is present
			val2 = ds.readUint8();
		}
		return (neg*(val1+(0xFF00*val2)));
	}
	SecondaryOrder.prototype.decodeFourByteUnsigned = function(ds){
		var val1=0,val2=0,val3=0,val4=0;
		val1 = (ds.readUint8() & 0xFC);
		switch(ds.readUint8() & 0x11){
			case 0:
				break;
			case 1:
				val2 = ds.readUint8();
				break;
			case 2:
				val2 = ds.readUint8();
				val3 = ds.readUint8();
				break;
			case 3:
				val2 = ds.readUint8();
				val3 = ds.readUint8();
				val4 = ds.readUint8();
				break;
		}
		return val1+(0xFF00*val2)+(0x00FF0000*val3)+(0xFF000000*val4);
	}
	SecondaryOrder.prototype.package = function(o,dv,offset){
		o.orderClass = 0x03;
		return {
			data:o,
			dv:new DataView(dv.buffer,dv.byteOffset+offset)
		}
	}
	SecondaryOrder.prototype.cacheBitmapRev1 = function(dv,header){
		var o =  {orderType:header.orderType};
		var offset = 0;
		o.cacheId  = dv.getUint8(offset++);
		if([0,1,2].indexOf(o.cacheId) < 0){
			throw new Error('Invalid Cache Id');
		}
		offset++;//pad octet
		o.width = dv.getUint8(offset++);
		o.height = dv.getUint8(offset++);
		switch(dv.getUint8(offset++)){
			case 0x08:
				o.bpp = 8;
				break;
			case 0x10:
				o.bpp = 16;
				break;
			case 0x18:
				o.bpp = 24;
				break;
			case 0x20:
				o.bpp =32;
				break;
			default:
				throw new Error('Invalid Bits per pixel');
		}
		var len =  dv.getUint16(offset);
		offset+=2;
		o.cacheIndex = dv.getUint16(offset);
		offset+=2;
		o.bitmap = Bitmap.processBitmapData(new DataView(dv.buffer,dv.byteOffset+offset,len));
		return this.package(o,dv,offset);
	}
	SecondaryOrder.prototype.cacheColorTable = function(dv,header){
		var o =  {orderType:0x01};
		var offset = 0;
		o.cacheIndex = dv.getUint8(offset++);
		if(o.cacheIndex > 5){
			throw new Error('Color Table Cache Index must be 0 - 5 inclusive');
		}
		var i = dv.getUint16(offset,true);
		if(i !== 256){
			throw new Error('Color Table Number Colors must be 256');
		}
		offset+=2;
		o.colors = [];
		while(i--){
			o.colors.push([dv.getUint8(offset),dv.getUint8(offset+1),dv.getUint8(offset+2)]);
			offset+=4;
		}
		return {
			data:o,
			dv:new DataView(dv.buffer,dv.byteOffset+offset)
		}
	}
	SecondaryOrder.prototype.cacheBitmapRev2 = function(dv,header){
		var o =  {orderType:0x02};
		var ds = new DataStream(dv.buffer,dv.byteOffset);
		o.cacheId = (header.extraFlags & 0x03);
		switch( (header.extraFlags & 0x78) >>> 3){
			case 0x03:
				o.bpp = 8;
				break;
			case 0x04:
				o.bpp = 16;
				break;
			case 0x05:
				o.bpp = 24;
				break;
			case 0x06:
				o.bpp = 32;
				break;
			default:
				throw new Error('Invalid bit depth');
		}
		var flags = (header.extraFlags & 0xFF80) >>> 7;
		if(flags & 0x02){
			o.key = ds.readUint32().toString(16);
			o.key = ds.readUint32().toString(16) + o.key;//prepend
		}
		o.width = this.decodeTwoByteUnsigned(ds);
		if(flags & 0x01){
			o.height = o.width;
		}
		var len = this.decodeFourByteUnsigned(ds);
		o.cacheIndex = this.decodeTwoByteUnsigned(ds);
		if(flags & 0x10){//ignore CacheIndex
			if(o.cacheIndex !== 32767){
				throw new Error('Invalid Cache Index on noncached bitmap');
			}
		}else{
			if(o.cacheIndex > capabilitySettings[0x0013].cellCaches[header.cacheId].numEntries){
				throw new Error('Cache Index is out of bounds');
			}
		}
		o.bitmap = Bitmap.processBitmapData(new DataView(ds.buffer,ds.position + ds.byteOffset,len));
		return this.package(o,dv,ds.position);
	}
	SecondaryOrder.prototype.cacheGlyphRev1 = function(dv,header){
		var o =  header;
		var offset = 0;
		o.cacheId = dv.getUint8(offset++);
		o.cGlyphs = dv.getUint8(offset++);
		o.glyphData = [];
		var glyphs = o.cGlyphs;
		var g ={},len;
		while(glyphs--){
			g ={};
			g.cacheIndex = dv.getUint16(offset,true);
			offset+=2;
			g.x = dv.getInt16(offset,true);
			offset+=2;
			g.y = dv.getInt16(offset,true);
			offset+=2;
			o.cx = dv.getUint16(offset,true);
			offset+=2;
			o.cy = dv.getUint16(offset,true);
			offset+=2;
			len= Math.ceil((o.cx*o.cy)/4)*4;//nearest multiple of 4
			o.aj = new DataView(dv.buffer,dv.byteOffset+offset,len);
			offset += len;
			o.glyphData.push(g);
		}
		if(header.extraFlags & 0x0100){//unicode
			o.unicodeCharacters = new DataView(dv.buffer,dv.byteOffset+offset,2*o.cGlyphs);
			offset+=2*o.cGlyphs;
		}
		return this.package(o,dv,offset);

	}
	SecondaryOrder.prototype.cacheGlyphRev2 = function(dv,header){
		var o =  {orderType:0x03};
		var ds = new DataStream(dv.buffer,dv.byteOffset);
		o.cacheId = header.extraFlags & 0x0F;
		o.flags =  (header.extraFlags & 0xF0) >>> 0x04;
		o.cGlyphs = (header.extraFlags) >>> 0x08;
		o.glyphData = {};
		o.glyphData.cacheIndex = ds.readUint8();
		o.glyphData.x = this.decodeTwoByteSigned(ds);
		o.glyphData.y = this.decodeTwoByteSigned(ds);
		o.glyphData.cx = this.decodeTwoByteUnsigned(ds);
		o.glyphData.cy = this.decodeTwoByteUnsigned(ds);
		var size =  Math.ceil(((o.glyphData.cx + 7) / 8) * o.glyphData.cy);
		while(size % 4 !== 0){//round up to the nearest mult of 4
			size++;
		}
		o.glyphData.aj = new DataView(ds.buffer,ds.byteOffset+ds.position,size);
		ds.seek(ds.position+size);
		if(o.flags & 0x01){
			o.unicodeCharacters = new DataView(ds.buffer,ds.byteOffset+ds.position,o.cGlyphs *2);
			ds.seek(ds.position+ o.cGlyphs*2);
		}

		return this.package(o,dv,offset);
	}
	SecondaryOrder.prototype.cacheBrush = function(dv,header){
		var o = {
			orderType:0x07
		};
		var offset = 0;
		o.cacheEntry = dv.getUint8(offset++);
		if(o.cacheEntry > 63){
			throw new Error('Invalid brush Cache entry');
		}
		var bpp = dv.getUint8(offset++);
		if([0x01,0x03,0x04,0x05,0x06].indexOf(bpp) < 0){
			throw new Error('Invalid Bits per pixel value');
		}else if(bpp === 0x01){
			o.bpp = 1;
		}else{
			o.bpp = Math.pow(2,bpp);
		}
		o.width = dv.getUint8(offset++);
		o.height = dv.getUint8(offset++);
		offset++;//style not used
		var len  = dv.getUint8(offset++);
		o.brushData = new DataView(dv.buffer,dv.byteOffset+offset,len);
		offset += len;
		return this.package(o,dv,offset);
	}
	SecondaryOrder.prototype.cacheBitmapRev3 = function(dv,header){
		var o =  {
			orderType:0x08
		};
		var offset = 0;
		o.cacheId = (header.extraFlags & 0x0007);
		switch((header.extraFlags & 0x0078) >>>3){
			case 0x03:
				o.bpp = 8;
				break;
			case 0x004:
				o.bpp = 16;
				break;
			case 0x05:
				o.bpp = 24;
				break;
			case 0x06:
				o.bpp = 32;
				break;
			default:
				throw new Error('Invalid Bits per pixel value');
		}

		var flags = (header.extraFlags & 0xFF80) >>> 7;
		o.cacheIndex = dv.getUint16(offset,true);
		offset +=2;
		if(flags & 0x10){//ignore CacheIndex
			if(o.cacheIndex !== 32767){
				throw new Error('Invalid Cache Index on noncached bitmap');
			}
		}else{
			if(o.cacheIndex > capabilitySettings[0x0013].cellCaches[header.cacheId].numEntries){
				throw new Error('Cache Index is out of bounds');
			}
		}
		o.key = dv.getUint32(offset +4,true).toString(16) + dv.getUint32(offset,true).toString(16);
		offset += 8;
		o.bitmap = Surface.readBitmapData(new DataView(dv.buffer,dv.byteOffset+offset));
		offset += o.bitmap.compressedLength + 12;
		return this.package(o,dv,offset);
	}
	var so = new SecondaryOrder();
	function process(dv){
		//process header first
		var o = so.decodeHeader(dv);
		switch(o.header.orderType){
			case 0x00://cache_bitmap_uncompressed
				return so.cacheBitmapRev1(o.dv,o.header);
				break;
			case 0x01://cache_color_table
				console.log(o);
				return so.cacheColorTable(o.dv,o.header);
				break;
			case 0x02://cache_bitmap_compressed
				return so.cacheBitmapRev1(o.dv,o.header);
				break;
			case 0x03://cache_glyph
				return so.cacheGlyphRev2(o.dv,o.header);
				break;
			case 0x04://cache_bitmap_uncompressed_rev2
				return so.cacheBitmapRev2(o.dv,o.header);
				break;
			case 0x05://cache_bitmap_compressed_rev2
				return so.cacheBitmapRev2(o.dv,o.header);
				break;
			case 0x07://cache_brush
				return so.cacheBrush(o.dv,o.header);
				break;
			case 0x08://cache_bitmap_compressed_rev3
				return so.cacheBitmapRev3(o.dv,o.header);
				break;
			default:
				console.log('Unknown Secondary Order',o.header);
		}
	}
module.exports ={
    process:process
}
