'use strict';

const DataStream = require('DataStream');
const capabilitySettings = require('xrdp/capability-settings');
const FrameMarker = require("./frame-marker")
var rfx = require('xrdp/rfx/rfx');

var frameId;
var frameQueue = {};

function decompress(o){
	switch(o.codecId){
		case 0:
			return dv;
		case capabilitySettings.capabilities[0x001D].bitmapCodecs['rfx'].codecId:
			rfx.decompress(o,FrameMarker.currentFrame());
			break;
		case capabilitySettings.capabilities[0x001D].bitmapCodecs['rfxImage'].codecId:
			return dv;
			break;
		case capabilitySettings.capabilities[0x001D].bitmapCodecs['nsCodec'].codecId:
			return dv;
			break;
		default:
			throw new Error('Codec Id not found');
	}
}

function extendedBitmapData(ds){
    var o = {};
    o.rect = [
        ds.readUint16(),
		ds.readUint16(),
		ds.readUint16(),
		ds.readUint16()];
	o.bpp = ds.readUint8();
	var flags = ds.readUint8();
	ds.seek(ds.position +1);
	o.codecId = ds.readUint8();
	o.width = ds.readUint16();
	o.height = ds.readUint16();
	var len = ds.readUint32();
	if(flags & 0x01){
		var compressedHeader = {};
		compressedHeader.highUniqueId = ds.readUint32();
		compressedHeader.lowUniqueId = ds.readUint32();
		compressedHeader.ms = ds.readUint8Array(8);
		compressedHeader.sec  = ds.readUint8Array(8);
		o.compressedHeader = compressedHeader;
	}
	o.data = new DataView((ds.readUint8Array(len).buffer));
	decompress(o);
}

function setSurfaceBits(dv,extra){
	var ds = new DataStream(dv.buffer,dv.byteOffset);
	if(ds.readUint16() !== 0x0001){
		throw new Error('Invalid Command Type');
	}
	extendedBitmapData(ds);
}
function streamSurfaceBits(dv){
	var ds = new DataStream(dv.buffer,dv.byteOffset);
	if(ds.readUint16() !== 0x0006){
		throw new Error('Invalid Command Type');
	}
	extendedBitmapData(ds);
}
function frameMarker(dv){
    if(dv.getUint16(2,true)===0x00){
        //FRAME BEGIN
        FrameMarker.frameBegin(dv.getUint32(4,true));
    }else{
        //FRAME END
        FrameMarker.frameEnd();
    }
}
function readBitmapData(dv,o){
	extendedBitmapData(new DataStream(dv.buffer,dv.byteOffset),o);
}

function readSufaceCommand(dv){
	switch(dv.getUint16(0,true)){
		case 0x0001: //set_surface_bits
			/*The Set Surface Bits Command is used to transport encoded bitmap data destined for a
			 * rectangular region of the primary drawing surface from an RDP server to an RDP client.
			 */
			 setSurfaceBits(dv);
			 break
		case 0x0006: //stream_surface_Bits
			/* The Stream Surface Bits Command is used to transport encoded bitmap data destined for a
			 * rectangular region of the primary drawing surface from an RDP server to an RDP client.
			 */
			streamSurfaceBits(dv);
			break;
		case 0x0004: //frame_marker
			frameMarker(dv);
			break;
		default:
			console.log('Unknown Surface Command type');
			break;
	}
}

module.exports = {
    readBitmapData:readBitmapData,
    readSufaceCommand:readSufaceCommand
}
