'use strict';
/*
 * A read only bitstream in the "backward" direction.
 */

var ReadStream = require('xrdp/rfx/decompress/js/ReadStream');

var BackwardReadStream = function(buf) {
	this.base = ReadStream;
	this.base(buf);
};

BackwardReadStream.prototype = new ReadStream;

BackwardReadStream.prototype.peek = function(bits) {
	var remaining = this.remaining;
	var index = this.index;
	var pending = this.pending;
	var result = 0;
	var done = 0;

	while (bits && remaining) {
		var i = Math.min(bits,pending);
		var mask = ((1 << i) - 1) << (8 - pending);

		result |= ((this.arr[index] & mask) >>> (8 - pending)) << done;

		bits -= i;
		done += i;

		if (pending == i) {
			remaining--;
			index++;
			pending = 8;
		}
		else pending -= i;
	}

	return result;
}

BackwardReadStream.prototype.read = function(bits) {
	var result = 0;
	var done = 0;

	while (bits && this.remaining) {
		var i = Math.min(bits,this.pending);
		var mask = ((1 << i) - 1) << (8 - this.pending);

		result |= ((this.arr[this.index] & mask) >>> (8 - this.pending)) << done;

		bits -= i;
		done += i;

		if (this.pending == i) {
			this.remaining--;
			this.index++;
			this.pending = 8;
		}
		else this.pending -= i;
	}

	return result;
}

module.exports = BackwardReadStream;
