'use strict';
/*
 * A read only bitstream in the "forward" direction.
 */
    var ReadStream  = require("xrdp/rfx/decompress/js/ReadStream");
    var ffs         = require("xrdp/rfx/decompress/js/ffs");

var ForwardReadStream = function(buf) {
	this.base = ReadStream;
	this.base(buf);
};

ForwardReadStream.prototype = new ReadStream;

ForwardReadStream.prototype.peek = function(bits) {
	var remaining = this.remaining;
	var index = this.index;
	var pending = this.pending;
	var result = 0;

	while (bits && remaining) {
		var tmp = this.arr[index];
		tmp &= (1 << pending) - 1;

		var i = Math.min(bits,pending);

		result = (result << i) | (tmp >>> (pending - i));

		bits -= i;

		if (pending == i) {
			remaining--;
			index++;
			pending = 8;
		}
		else pending -= i;
	}

	return result;
}

ForwardReadStream.prototype.read = function(bits) {
	var result = 0;

	while (bits && this.remaining) {
		var tmp = this.arr[this.index];
		tmp &= (1 << this.pending) - 1;

		var i = Math.min(bits,this.pending);

		result = (result << i) | (tmp >>> (this.pending - i));

		bits -= i;

		if (this.pending == i) {
			this.remaining--;
			this.index++;
			this.pending = 8;
		}
		else this.pending -= i;
	}

	return result;
}

ForwardReadStream.prototype.readRun = function(bit) {
	var count = 0;

	while (this.remaining) {
		var val = bit ? ~this.arr[this.index] : this.arr[this.index];

		val &= (1 << this.pending) - 1;

		var fs = ffs[val];

		count += this.pending - fs;

		if (fs > 1) this.pending = fs - 1;
		else {
			this.remaining--;
			this.index++;
			this.pending = 8;
		}

		if (fs) break;
	}

	return count;
}

ForwardReadStream.prototype.readBoundedRun = function(bit, max) {
	var count = 0;

	while (max && this.remaining) {
		var i = Math.max(this.pending - max,0);
		var val = bit ? ~this.arr[this.index] : this.arr[this.index];

		val &= (1 << this.pending) - 1;
		val |= (1 << i) - 1;

		var fs = ffs[val];

		max -= this.pending - fs;
		count += this.pending - fs;

		if (fs > 1) this.pending = max ? fs - 1 : fs;
		else {
			this.remaining--;
			this.index++;
			this.pending = 8;
		}

		if (fs) break;
	}

	return count;
}

module.exports = ForwardReadStream;
