'use strict';

/*
 * RemoteFX tile compression.
 */

function simd_module(stdlib, foreign, heap) {
	"use asm";

	var u1base = new stdlib.Uint8Array(heap);
	var i4 = stdlib.SIMD.int32x4;
	var i4gt = i4.greaterThan;
	var i4select = i4.select;
	var i4add = i4.add;
	var i4sub = i4.sub;
	var i4mul = i4.mul;
	var i4or = i4.or;
	var i4lshift = i4.shiftLeftByScalar;
	var i4rshift = i4.shiftRightArithmeticByScalar;
	var i4load = i4.load;
	var i4store = i4.store;
	var i4shuffle = i4.shuffle;
	var i4zero = i4(0,0,0,0);
	var i4one = i4(1,1,1,1);
	var i4y1 = i4(4096,4096,4096,4096);
	var i4cb1 = i4(-22544,-22544,-22544,-22544);
	var i4cb2 = i4(115998,115998,115998,115998);
	var i4cr1 = i4(91947,91947,91947,91947);
	var i4cr2 = i4(-46792,-46792,-46792,-46792);
	var i4max = i4(255,255,255,255);

	function toRGB(src_y, src_cb, src_cr, src_rgb) {
		src_y = src_y | 0;
		src_cb = src_cb | 0;
		src_cr = src_cr | 0;
		src_rgb = src_rgb | 0;

		var i = 0;
		var fy = i4(0,0,0,0);
		var fcb = i4(0,0,0,0);
		var fcr = i4(0,0,0,0);
		var fr = i4(0,0,0,0);
		var fg = i4(0,0,0,0);
		var fb = i4(0,0,0,0);
		var frgb = i4(0,0,0,0);

		i = 1024;
		while ((i | 0) > 0) {
			i = (i - 1) | 0;

			fy = i4load(u1base,src_y);
			fcb = i4load(u1base,src_cb);
			fcr = i4load(u1base,src_cr);

			fy = i4lshift(i4add(fy,i4y1),16);
			fr = i4add(fy,i4mul(fcr,i4cr1));
			fg = i4add(fy,i4add(i4mul(fcb,i4cb1),i4mul(fcr,i4cr2)));
			fb = i4add(fy,i4mul(fcb,i4cb2));

			// shift back
			fr = i4rshift(fr,21);
			fg = i4rshift(fg,21);
			fb = i4rshift(fb,21);

			// clamp to 255
			fr = i4select(i4gt(fr,i4max),i4max,fr);
			fg = i4select(i4gt(fg,i4max),i4max,fg);
			fb = i4select(i4gt(fb,i4max),i4max,fb);

			// clamp to 0
			fr = i4select(i4gt(fr,i4zero),fr,i4zero);
			fg = i4select(i4gt(fg,i4zero),fg,i4zero);
			fb = i4select(i4gt(fb,i4zero),fb,i4zero);

			frgb = i4or(fr,i4or(i4lshift(fg,8),i4or(i4lshift(fb,16),i4lshift(i4max,24))));
			i4store(u1base,src_rgb,frgb);

			src_y = (src_y + 16) | 0;
			src_cb = (src_cb + 16) | 0;
			src_cr = (src_cr + 16) | 0;
			src_rgb = (src_rgb + 16) | 0;
		}
	}

	function idwt_1(r_st, c_st, sw) {
		r_st = r_st | 0;
		c_st = c_st | 0;
		sw = sw | 0;

		var n = 0;
		var l = 0;
		var h = 0;
		var e = 0;

		var fr1 = i4(0,0,0,0);
		var fr2 = i4(0,0,0,0);
		var fr3 = i4(0,0,0,0);
		var fr4 = i4(0,0,0,0);
		var fs1 = i4(0,0,0,0);
		var fs2 = i4(0,0,0,0);
		var fl1 = i4(0,0,0,0);
		var fl2 = i4(0,0,0,0);
		var fl3 = i4(0,0,0,0);
		var fl4 = i4(0,0,0,0);
		var fh0 = i4(0,0,0,0);
		var fh1 = i4(0,0,0,0);
		var fh2 = i4(0,0,0,0);
		var fh3 = i4(0,0,0,0);
		var fh4 = i4(0,0,0,0);
		var fe0 = i4(0,0,0,0);
		var fe1 = i4(0,0,0,0);
		var fe2 = i4(0,0,0,0);
		var fe3 = i4(0,0,0,0);
		var fe4 = i4(0,0,0,0);

		// even
		l = r_st;
		h = (r_st + (sw << 2)) | 0;
		e = c_st;

		fr1 = i4load(u1base,l);
		fr2 = i4load(u1base,l + 256);
		fr3 = i4load(u1base,l + 512);
		fr4 = i4load(u1base,l + 768);
		fs1 = i4shuffle(fr1,fr2,0,4,1,5);
		fs2 = i4shuffle(fr3,fr4,0,4,1,5);
		fl1 = i4shuffle(fs1,fs2,0,1,4,5);
		fl2 = i4shuffle(fs1,fs2,2,3,6,7);
		fs1 = i4shuffle(fr1,fr2,2,6,3,7);
		fs2 = i4shuffle(fr3,fr4,2,6,3,7);
		fl3 = i4shuffle(fs1,fs2,0,1,4,5);
		fl4 = i4shuffle(fs1,fs2,2,3,6,7);

		fr1 = i4load(u1base,h);
		fr2 = i4load(u1base,h + 256);
		fr3 = i4load(u1base,h + 512);
		fr4 = i4load(u1base,h + 768);
		fs1 = i4shuffle(fr1,fr2,0,4,1,5);
		fs2 = i4shuffle(fr3,fr4,0,4,1,5);
		fh1 = i4shuffle(fs1,fs2,0,1,4,5);
		fh2 = i4shuffle(fs1,fs2,2,3,6,7);
		fs1 = i4shuffle(fr1,fr2,2,6,3,7);
		fs2 = i4shuffle(fr3,fr4,2,6,3,7);
		fh3 = i4shuffle(fs1,fs2,0,1,4,5);
		fh4 = i4shuffle(fs1,fs2,2,3,6,7);

		i4store(u1base,e,i4sub(fl1,i4rshift(i4add(i4lshift(fh1,1),i4one),1)));
		i4store(u1base,e + 512,i4sub(fl2,i4rshift(i4add(i4add(fh1,fh2),i4one),1)));
		i4store(u1base,e + 1024,i4sub(fl3,i4rshift(i4add(i4add(fh2,fh3),i4one),1)));
		i4store(u1base,e + 1536,i4sub(fl4,i4rshift(i4add(i4add(fh3,fh4),i4one),1)));

		n = (sw - 4) | 0;
		while ((n | 0) > 0) {
			n = (n - 4) | 0;

			l = (l + 16) | 0;
			h = (h + 16) | 0;
			e = (e + 2048) | 0;

			fr1 = i4load(u1base,l);
			fr2 = i4load(u1base,l + 256);
			fr3 = i4load(u1base,l + 512);
			fr4 = i4load(u1base,l + 768);
			fs1 = i4shuffle(fr1,fr2,0,4,1,5);
			fs2 = i4shuffle(fr3,fr4,0,4,1,5);
			fl1 = i4shuffle(fs1,fs2,0,1,4,5);
			fl2 = i4shuffle(fs1,fs2,2,3,6,7);
			fs1 = i4shuffle(fr1,fr2,2,6,3,7);
			fs2 = i4shuffle(fr3,fr4,2,6,3,7);
			fl3 = i4shuffle(fs1,fs2,0,1,4,5);
			fl4 = i4shuffle(fs1,fs2,2,3,6,7);

			fr1 = i4load(u1base,h);
			fr2 = i4load(u1base,h + 256);
			fr3 = i4load(u1base,h + 512);
			fr4 = i4load(u1base,h + 768);
			fh0 = fh4;
			fs1 = i4shuffle(fr1,fr2,0,4,1,5);
			fs2 = i4shuffle(fr3,fr4,0,4,1,5);
			fh1 = i4shuffle(fs1,fs2,0,1,4,5);
			fh2 = i4shuffle(fs1,fs2,2,3,6,7);
			fs1 = i4shuffle(fr1,fr2,2,6,3,7);
			fs2 = i4shuffle(fr3,fr4,2,6,3,7);
			fh3 = i4shuffle(fs1,fs2,0,1,4,5);
			fh4 = i4shuffle(fs1,fs2,2,3,6,7);

			i4store(u1base,e,i4sub(fl1,i4rshift(i4add(i4add(fh0,fh1),i4one),1)));
			i4store(u1base,e + 512,i4sub(fl2,i4rshift(i4add(i4add(fh1,fh2),i4one),1)));
			i4store(u1base,e + 1024,i4sub(fl3,i4rshift(i4add(i4add(fh2,fh3),i4one),1)));
			i4store(u1base,e + 1536,i4sub(fl4,i4rshift(i4add(i4add(fh3,fh4),i4one),1)));
		}

		// odd
		h = (r_st + (sw << 2)) | 0;
		e = c_st;

		fe4 = i4load(u1base,e);

		n = (sw - 4) | 0;
		while ((n | 0) > 0) {
			n = (n - 4) | 0;

			fr1 = i4load(u1base,h);
			fr2 = i4load(u1base,h + 256);
			fr3 = i4load(u1base,h + 512);
			fr4 = i4load(u1base,h + 768);
			fs1 = i4shuffle(fr1,fr2,0,4,1,5);
			fs2 = i4shuffle(fr3,fr4,0,4,1,5);
			fh1 = i4shuffle(fs1,fs2,0,1,4,5);
			fh2 = i4shuffle(fs1,fs2,2,3,6,7);
			fs1 = i4shuffle(fr1,fr2,2,6,3,7);
			fs2 = i4shuffle(fr3,fr4,2,6,3,7);
			fh3 = i4shuffle(fs1,fs2,0,1,4,5);
			fh4 = i4shuffle(fs1,fs2,2,3,6,7);

			fe0 = fe4;
			fe1 = i4load(u1base,e + 512);
			fe2 = i4load(u1base,e + 1024);
			fe3 = i4load(u1base,e + 1536);
			fe4 = i4load(u1base,e + 2048);

			i4store(u1base,e + 256,i4add(i4lshift(fh1,1),i4rshift(i4add(fe0,fe1),1)));
			i4store(u1base,e + 768,i4add(i4lshift(fh2,1),i4rshift(i4add(fe1,fe2),1)));
			i4store(u1base,e + 1280,i4add(i4lshift(fh3,1),i4rshift(i4add(fe2,fe3),1)));
			i4store(u1base,e + 1792,i4add(i4lshift(fh4,1),i4rshift(i4add(fe3,fe4),1)));

			h = (h + 16) | 0;
			e = (e + 2048) | 0;
		}

		fr1 = i4load(u1base,h);
		fr2 = i4load(u1base,h + 256);
		fr3 = i4load(u1base,h + 512);
		fr4 = i4load(u1base,h + 768);
		fs1 = i4shuffle(fr1,fr2,0,4,1,5);
		fs2 = i4shuffle(fr3,fr4,0,4,1,5);
		fh1 = i4shuffle(fs1,fs2,0,1,4,5);
		fh2 = i4shuffle(fs1,fs2,2,3,6,7);
		fs1 = i4shuffle(fr1,fr2,2,6,3,7);
		fs2 = i4shuffle(fr3,fr4,2,6,3,7);
		fh3 = i4shuffle(fs1,fs2,0,1,4,5);
		fh4 = i4shuffle(fs1,fs2,2,3,6,7);

		fe0 = fe4;
		fe1 = i4load(u1base,e + 512);
		fe2 = i4load(u1base,e + 1024);
		fe3 = i4load(u1base,e + 1536);

		i4store(u1base,e + 256,i4add(i4lshift(fh1,1),i4rshift(i4add(fe0,fe1),1)));
		i4store(u1base,e + 768,i4add(i4lshift(fh2,1),i4rshift(i4add(fe1,fe2),1)));
		i4store(u1base,e + 1280,i4add(i4lshift(fh3,1),i4rshift(i4add(fe2,fe3),1)));
		i4store(u1base,e + 1792,i4add(i4lshift(fh4,1),fe3));
	}

	function idwt_2(src, ext) {
		src = src | 0;
		ext = ext | 0;

		var r_st = 0;
		var c_st = 0;
		var i = 0;

		r_st = src;
		c_st = ext;
		i = 4;
		while ((i | 0) > 0) {
			i = (i - 1) | 0;
			idwt_1(r_st,c_st,8);
			r_st = (r_st + 1024) | 0;
			c_st = (c_st + 16) | 0;
		}

		r_st = ext;
		c_st = src;
		i = 4;
		while ((i | 0) > 0) {
			i = (i - 1) | 0;
			idwt_1(r_st,c_st,8);
			r_st = (r_st + 1024) | 0;
			c_st = (c_st + 16) | 0;
		}

		r_st = src;
		c_st = ext;
		i = 8;
		while ((i | 0) > 0) {
			i = (i - 1) | 0;
			idwt_1(r_st,c_st,16);
			r_st = (r_st + 1024) | 0;
			c_st = (c_st + 16) | 0;
		}

		r_st = ext;
		c_st = src;
		i = 8;
		while ((i | 0) > 0) {
			i = (i - 1) | 0;
			idwt_1(r_st,c_st,16);
			r_st = (r_st + 1024) | 0;
			c_st = (c_st + 16) | 0;
		}

		r_st = src;
		c_st = ext;
		i = 16;
		while ((i | 0) > 0) {
			i = (i - 1) | 0;
			idwt_1(r_st,c_st,32);
			r_st = (r_st + 1024) | 0;
			c_st = (c_st + 16) | 0;
		}

		r_st = ext;
		c_st = src;
		i = 16;
		while ((i | 0) > 0) {
			i = (i - 1) | 0;
			idwt_1(r_st,c_st,32);
			r_st = (r_st + 1024) | 0;
			c_st = (c_st + 16) | 0;
		}
	}

	function dd(src, ext, fac1, fac2, fac3, fac4, fac5, fac6, fac7, fac8, fac9, fac10) {
		src = src | 0;
		ext = ext | 0;
		fac1 = fac1 | 0;
		fac2 = fac2 | 0;
		fac3 = fac3 | 0;
		fac4 = fac4 | 0;
		fac5 = fac5 | 0;
		fac6 = fac6 | 0;
		fac7 = fac7 | 0;
		fac8 = fac8 | 0;
		fac9 = fac9 | 0;
		fac10 = fac10 | 0;

		var n = 0;
		var i = 0;
		var w = 0;
		var h = 0;
		var m = 0;
		var fs = i4(0,0,0,0);

		m = src;

		// HL1
		n = (ext + 128) | 0;
		h = 32;
		while ((h | 0) > 0) {
			h = (h - 1) | 0;
			i = n;
			w = 8;
			while ((w | 0) > 0) {
				w = (w - 1) | 0;
				fs = i4load(u1base,m);
				i4store(u1base,i,i4lshift(fs,fac9));
				i = (i + 16) | 0;
				m = (m + 16) | 0;
			}
			n = (n + 256) | 0;
		}

		// LH1
		n = (ext + 8192) | 0;
		h = 32;
		while ((h | 0) > 0) {
			h = (h - 1) | 0;
			i = n;
			w = 8;
			while ((w | 0) > 0) {
				w = (w - 1) | 0;
				fs = i4load(u1base,m);
				i4store(u1base,i,i4lshift(fs,fac8));
				i = (i + 16) | 0;
				m = (m + 16) | 0;
			}
			n = (n + 256) | 0;
		}

		// HH1
		n = (ext + 8320) | 0;
		h = 32;
		while ((h | 0) > 0) {
			h = (h - 1) | 0;
			i = n;
			w = 8;
			while ((w | 0) > 0) {
				w = (w - 1) | 0;
				fs = i4load(u1base,m);
				i4store(u1base,i,i4lshift(fs,fac10));
				i = (i + 16) | 0;
				m = (m + 16) | 0;
			}
			n = (n + 256) | 0;
		}

		// HL2
		n = (ext + 64) | 0;
		h = 16;
		while ((h | 0) > 0) {
			h = (h - 1) | 0;
			i = n;
			w = 4;
			while ((w | 0) > 0) {
				w = (w - 1) | 0;
				fs = i4load(u1base,m);
				i4store(u1base,i,i4lshift(fs,fac6));
				i = (i + 16) | 0;
				m = (m + 16) | 0;
			}
			n = (n + 256) | 0;
		}

		// LH2
		n = (ext + 4096) | 0;
		h = 16;
		while ((h | 0) > 0) {
			h = (h - 1) | 0;
			i = n;
			w = 4;
			while ((w | 0) > 0) {
				w = (w - 1) | 0;
				fs = i4load(u1base,m);
				i4store(u1base,i,i4lshift(fs,fac5));
				i = (i + 16) | 0;
				m = (m + 16) | 0;
			}
			n = (n + 256) | 0;
		}

		// HH2
		n = (ext + 4160) | 0;
		h = 16;
		while ((h | 0) > 0) {
			h = (h - 1) | 0;
			i = n;
			w = 4;
			while ((w | 0) > 0) {
				w = (w - 1) | 0;
				fs = i4load(u1base,m);
				i4store(u1base,i,i4lshift(fs,fac7));
				i = (i + 16) | 0;
				m = (m + 16) | 0;
			}
			n = (n + 256) | 0;
		}

		// HL3
		n = (ext + 32) | 0;
		h = 8;
		while ((h | 0) > 0) {
			h = (h - 1) | 0;
			i = n;
			w = 2;
			while ((w | 0) > 0) {
				w = (w - 1) | 0;
				fs = i4load(u1base,m);
				i4store(u1base,i,i4lshift(fs,fac3));
				i = (i + 16) | 0;
				m = (m + 16) | 0;
			}
			n = (n + 256) | 0;
		}

		// LH3
		n = (ext + 2048) | 0;
		h = 8;
		while ((h | 0) > 0) {
			h = (h - 1) | 0;
			i = n;
			w = 2;
			while ((w | 0) > 0) {
				w = (w - 1) | 0;
				fs = i4load(u1base,m);
				i4store(u1base,i,i4lshift(fs,fac2));
				i = (i + 16) | 0;
				m = (m + 16) | 0;
			}
			n = (n + 256) | 0;
		}

		// HH3
		n = (ext + 2080) | 0;
		h = 8;
		while ((h | 0) > 0) {
			h = (h - 1) | 0;
			i = n;
			w = 2;
			while ((w | 0) > 0) {
				w = (w - 1) | 0;
				fs = i4load(u1base,m);
				i4store(u1base,i,i4lshift(fs,fac4));
				i = (i + 16) | 0;
				m = (m + 16) | 0;
			}
			n = (n + 256) | 0;
		}

		// LL3
		n = ext;
		h = 8;
		while ((h | 0) > 0) {
			h = (h - 1) | 0;
			i = n;
			w = 2;
			while ((w | 0) > 0) {
				w = (w - 1) | 0;
				fs = i4load(u1base,m);
				i4store(u1base,i,i4lshift(fs,fac1));
				i = (i + 16) | 0;
				m = (m + 16) | 0;
			}
			n = (n + 256) | 0;
		}

		// copy back
		n = src;
		m = ext;
		i = 1024;
		while ((i | 0) > 0) {
			i = (i - 1) | 0;
			fs = i4load(u1base,m);
			i4store(u1base,n,fs);
			n = (n + 16) | 0;
			m = (m + 16) | 0;
		}
	}

	return {
		toRGB: toRGB,
		idwt_2: idwt_2,
		dd: dd
	};
}

function toRGB(yArr, cbArr, crArr, rgbData, x, y, width, height, step) {
	var start = y * step + (x << 2), i, j = 0;
	var h = height;

	while (h--) {
		var w = width;

		i = start;
		while (w--) {
			var y = yArr[j] + 4096;
			var cb = cbArr[j];
			var cr = crArr[j];

			y <<= 16;

			var r = y + cr * 91947;
			var g = y + cb * -22544 + cr * -46792;
			var b = y + cb * 115998;

			rgbData[i] = r >> 21;
			rgbData[i + 1] = g >> 21;
			rgbData[i + 2] = b >> 21;
			rgbData[i + 3] = 255;

			i += 4;
			j++;
		}
		start += step;
	}
}

function idwt_1(src, src_s, dst, dst_s, sw) {
	var n;

	// even
	var l = src_s;
	var h = src_s + sw;
	var e = dst_s;

	dst[e] = src[l] - (((src[h] << 1) + 1) >> 1);

	n = sw - 1;
	while (n--) {
		l++;
		h++;
		e += 128;

		dst[e] = src[l] - ((src[h - 1] + src[h] + 1) >> 1);
	}

	// odd
	var h = src_s + sw;
	var e = dst_s;

	n = sw - 1;
	while (n--) {
		dst[e + 64] = (src[h] << 1) + ((dst[e] + dst[e + 128]) >> 1);

		h++;
		e += 128;
	}

	dst[e + 64] = (src[h] << 1) + dst[e];
}

function idwt_2(arr, scratch) {
	for (var n = 0; n < 16; n++) idwt_1(arr,n << 6,scratch,n,8);
	for (var n = 0; n < 16; n++) idwt_1(scratch,n << 6,arr,n,8);
	for (var n = 0; n < 32; n++) idwt_1(arr,n << 6,scratch,n,16);
	for (var n = 0; n < 32; n++) idwt_1(scratch,n << 6,arr,n,16);
	for (var n = 0; n < 64; n++) idwt_1(arr,n << 6,scratch,n,32);
	for (var n = 0; n < 64; n++) idwt_1(scratch,n << 6,arr,n,32);
}

function dd(arr, scratch, facs) {
	var n = 0;
	var start, i;
	var w, h;
	var fac;

	// HL1
	start = 32;
	fac = facs[8];
	h = 32;
	while (h--) {
		i = start;
		w = 32;
		while (w--) scratch[i++] = arr[n++] << (fac - 1);
		start += 64;
	}

	// LH1
	start = 2048;
	fac = facs[7];
	h = 32;
	while (h--) {
		i = start;
		w = 32;
		while (w--) scratch[i++] = arr[n++] << (fac - 1);
		start += 64;
	}

	// HH1
	start = 2080;
	fac = facs[9];
	h = 32;
	while (h--) {
		i = start;
		w = 32;
		while (w--) scratch[i++] = arr[n++] << (fac - 1);
		start += 64;
	}

	// HL2
	start = 16;
	fac = facs[5];
	h = 16;
	while (h--) {
		i = start;
		w = 16;
		while (w--) scratch[i++] = arr[n++] << (fac - 1);
		start += 64;
	}

	// LH2
	start = 1024;
	fac = facs[4];
	h = 16;
	while (h--) {
		i = start;
		w = 16;
		while (w--) scratch[i++] = arr[n++] << (fac - 1);
		start += 64;
	}

	// HH2
	start = 1040;
	fac = facs[6];
	h = 16;
	while (h--) {
		i = start;
		w = 16;
		while (w--) scratch[i++] = arr[n++] << (fac - 1);
		start += 64;
	}

	// HL3
	start = 8;
	fac = facs[2];
	h = 8;
	while (h--) {
		i = start;
		w = 8;
		while (w--) scratch[i++] = arr[n++] << (fac - 1);
		start += 64;
	}

	// LH3
	start = 512;
	fac = facs[1];
	h = 8;
	while (h--) {
		i = start;
		w = 8;
		while (w--) scratch[i++] = arr[n++] << (fac - 1);
		start += 64;
	}

	// HH3
	start = 520;
	fac = facs[3];
	h = 8;
	while (h--) {
		i = start;
		w = 8;
		while (w--) scratch[i++] = arr[n++] << (fac - 1);
		start += 64;
	}

	// LL3
	start = 0;
	fac = facs[0];
	h = 8;
	while (h--) {
		i = start;
		w = 8;
		while (w--) scratch[i++] = arr[n++] << (fac - 1);
		start += 64;
	}

	arr.set(scratch);
}

function differentialDecode(arr) {
	// LL3
	var prev = arr[4032];

	for (var n = 4033; n < 4096; n++) {
		arr[n] += prev;
		prev = arr[n];
	}
}

function clamp(v, min, max) {
	if (v < min) return min;
	if (v > max) return max;
	return v;
}

function toYCbCr(rgbData, x, y, width, height, step, yArr, cbArr, crArr) {
	var start = y * step + (x << 2), i, j = 0;
	var h = height;

	while (h--) {
		var w = width;

		i = start;
		while (w--) {
			var r = rgbData[i];
			var g = rgbData[i + 1];
			var b = rgbData[i + 2];
			var y = (r * 9798 + g * 19235 + b * 3735) >> 10;
			var cb = (r * -5535 + g * -10868 + b * 16403) >> 10;
			var cr = (r * 16377 + g * -13714 + b * -2663) >> 10;

			yArr[j] = clamp(y - 4096,-4096,4095);
			cbArr[j] = clamp(cb,-4096,4095);
			crArr[j] = clamp(cr,-4096,4095);

			i += 4;
			j++;
		}
		start += step;
	}
}

function flip(src, dst) {
	var i = 0;
	for (var h = 0; h < 64; h++) {
		var j = h;
		var w = 64;
		while (w--) {
			dst[j] = src[i];
			i++;
			j += 64;
		}
	}
}

function dwt_1(src, src_s, dst, dst_s, sw) {
	var n;

	// high
	var h = dst_s + (sw << 6);
	var e = src_s;

	n = sw - 1;
	while (n--) {
		dst[h] = (src[e + 1] - ((src[e] + src[e + 2]) >> 1)) >> 1;

		h += 64;
		e += 2;
	}

	dst[h] = (src[e + 1] - src[e]) >> 1;

	// low
	var l = dst_s;
	var h = dst_s + (sw << 6);
	var e = src_s;

	dst[l] = src[e] + dst[h];

	n = sw - 1;
	while (n--) {
		l += 64;
		h += 64;
		e += 2;

		dst[l] = src[e] + ((dst[h - 64] + dst[h]) >> 1);
	}
}

function dwt_2(arr, scratch) {
	flip(arr,scratch);
	for (var n = 0; n < 64; n++) dwt_1(scratch,n << 6,arr,n,32);
	for (var n = 0; n < 64; n++) dwt_1(arr,n << 6,scratch,n,32);
	for (var n = 0; n < 32; n++) dwt_1(scratch,n << 6,arr,n,16);
	for (var n = 0; n < 32; n++) dwt_1(arr,n << 6,scratch,n,16);
	for (var n = 0; n < 16; n++) dwt_1(scratch,n << 6,arr,n,8);
	for (var n = 0; n < 16; n++) dwt_1(arr,n << 6,scratch,n,8);
	flip(scratch,arr);
}

function ld(arr, scratch, facs) {
	var n = 0;
	var start, i;
	var w, h;
	var fac;
	var half;

	// HL1
	start = 32;
	fac = facs[8];
	half = 1 << (fac - 2);
	h = 32;
	while (h--) {
		i = start;
		w = 32;
		while (w--) scratch[n++] = (arr[i++] + half) >> (fac - 1);
		start += 64;
	}

	// LH1
	start = 2048;
	fac = facs[7];
	half = 1 << (fac - 2);
	h = 32;
	while (h--) {
		i = start;
		w = 32;
		while (w--) scratch[n++] = (arr[i++] + half) >> (fac - 1);
		start += 64;
	}

	// HH1
	start = 2080;
	fac = facs[9];
	half = 1 << (fac - 2);
	h = 32;
	while (h--) {
		i = start;
		w = 32;
		while (w--) scratch[n++] = (arr[i++] + half) >> (fac - 1);
		start += 64;
	}

	// HL2
	start = 16;
	fac = facs[5];
	half = 1 << (fac - 2);
	h = 16;
	while (h--) {
		i = start;
		w = 16;
		while (w--) scratch[n++] = (arr[i++] + half) >> (fac - 1);
		start += 64;
	}

	// LH2
	start = 1024;
	fac = facs[4];
	half = 1 << (fac - 2);
	h = 16;
	while (h--) {
		i = start;
		w = 16;
		while (w--) scratch[n++] = (arr[i++] + half) >> (fac - 1);
		start += 64;
	}

	// HH2
	start = 1040;
	fac = facs[6];
	half = 1 << (fac - 2);
	h = 16;
	while (h--) {
		i = start;
		w = 16;
		while (w--) scratch[n++] = (arr[i++] + half) >> (fac - 1);
		start += 64;
	}

	// HL3
	start = 8;
	fac = facs[2];
	half = 1 << (fac - 2);
	h = 8;
	while (h--) {
		i = start;
		w = 8;
		while (w--) scratch[n++] = (arr[i++] + half) >> (fac - 1);
		start += 64;
	}

	// LH3
	start = 512;
	fac = facs[1];
	half = 1 << (fac - 2);
	h = 8;
	while (h--) {
		i = start;
		w = 8;
		while (w--) scratch[n++] = (arr[i++] + half) >> (fac - 1);
		start += 64;
	}

	// HH3
	start = 520;
	fac = facs[3];
	half = 1 << (fac - 2);
	h = 8;
	while (h--) {
		i = start;
		w = 8;
		while (w--) scratch[n++] = (arr[i++] + half) >> (fac - 1);
		start += 64;
	}

	// LL3
	start = 0;
	fac = facs[0];
	half = 1 << (fac - 2);
	h = 8;
	while (h--) {
		i = start;
		w = 8;
		while (w--) scratch[n++] = (arr[i++] + half) >> (fac - 1);
		start += 64;
	}

	arr.set(scratch);
}

function differentialEncode(arr) {
	// LL3
	var prev = arr[4032];

	for (var n = 4033; n < 4096; n++) {
		var tmp = arr[n];

		arr[n] -= prev;
		prev = tmp;
	}
}

var RFXTileDecompressorSIMD = function() {
	this.heap = new Int32Array(16384);
	this.simd = simd_module(self,{},this.heap.buffer);
}

RFXTileDecompressorSIMD.prototype.decompress = function(yArr, cbArr, crArr, yFacs, cbFacs, crFacs,rgbData) {
	yFacs = yFacs || [6,6,6,6,7,7,8,8,8,9];
	cbFacs = cbFacs || [6,6,6,6,7,7,8,8,8,9];
	crFacs = crFacs || [6,6,6,6,7,7,8,8,8,9];

	// ARLGR decode
	//decode(4096,yData,this.heap.subarray(0,4096));
//	decode(4096,cbData,this.heap.subarray(4096,8192));
	//decode(4096,crData,this.heap.subarray(8192,12288));

	// differential decode
	differentialDecode(this.heap.subarray(0,4096));
	differentialDecode(this.heap.subarray(4096,8192));
	differentialDecode(this.heap.subarray(8192,12288));

	// dequantize, scale, and delinearize
	this.simd.dd(0,49152,yFacs[0] - 1,yFacs[1] - 1,yFacs[2] - 1,yFacs[3] - 1,yFacs[4] - 1,yFacs[5] - 1,yFacs[6] - 1,yFacs[7] - 1,yFacs[8] - 1,yFacs[9] - 1);
	this.simd.dd(16384,49152,cbFacs[0] - 1,cbFacs[1] - 1,cbFacs[2] - 1,cbFacs[3] - 1,cbFacs[4] - 1,cbFacs[5] - 1,cbFacs[6] - 1,cbFacs[7] - 1,cbFacs[8] - 1,cbFacs[9] - 1);
	this.simd.dd(32768,49152,crFacs[0] - 1,crFacs[1] - 1,crFacs[2] - 1,crFacs[3] - 1,crFacs[4] - 1,crFacs[5] - 1,crFacs[6] - 1,crFacs[7] - 1,crFacs[8] - 1,crFacs[9] - 1);

	// IDWT
	this.simd.idwt_2(0,49152);
	this.simd.idwt_2(16384,49152);
	this.simd.idwt_2(32768,49152);

	// color conversion
	this.simd.toRGB(0,16384,32768,49152);
	rgbData.set(new Uint8Array(this.heap.buffer,49152));
}

var RFXTileDecompressor = function() {

	this.scratch = new Int16Array(4096);
}

RFXTileDecompressor.prototype.decompress = function(yArr, cbArr, crArr,  yFacs, cbFacs, crFacs,rgbData) {
	yFacs = yFacs || [6,6,6,6,7,7,8,8,8,9];
	cbFacs = cbFacs || [6,6,6,6,7,7,8,8,8,9];
	crFacs = crFacs || [6,6,6,6,7,7,8,8,8,9];

	this.yArr = yArr;
	this.cbArr = cbArr;
	this.crArr = crArr;


	// differential decode
	differentialDecode(this.yArr);
	differentialDecode(this.cbArr);
	differentialDecode(this.crArr);

	// dequantize, scale, and delinearize
	dd(this.yArr,this.scratch,yFacs);
	dd(this.cbArr,this.scratch,cbFacs);
	dd(this.crArr,this.scratch,crFacs);

	// IDWT
	idwt_2(this.yArr,this.scratch);
	idwt_2(this.cbArr,this.scratch);
	idwt_2(this.crArr,this.scratch);

	// color conversion
	toRGB(this.yArr,this.cbArr,this.crArr,rgbData,0,0,64,64,256);
}

var RFXTileCompressor = function() {
	this.yArr = new Int16Array(4096);
	this.cbArr = new Int16Array(4096);
	this.crArr = new Int16Array(4096);
	this.scratch = new Int16Array(4096);
}

RFXTileCompressor.prototype.compress = function(rgbData, x, y, step, encode, yFacs, cbFacs, crFacs) {
	yFacs = yFacs || [6,6,6,6,7,7,8,8,8,9];
	cbFacs = cbFacs || [6,6,6,6,7,7,8,8,8,9];
	crFacs = crFacs || [6,6,6,6,7,7,8,8,8,9];

	// color conversion
	toYCbCr(rgbData,x,y,64,64,step,this.yArr,this.cbArr,this.crArr);

	// DWT
	dwt_2(this.yArr,this.scratch);
	dwt_2(this.cbArr,this.scratch);
	dwt_2(this.crArr,this.scratch);

	// linearize, quantize, and scale
	ld(this.yArr,this.scratch,yFacs);
	ld(this.cbArr,this.scratch,cbFacs);
	ld(this.crArr,this.scratch,crFacs);

	// differential encode
	differentialEncode(this.yArr);
	differentialEncode(this.cbArr);
	differentialEncode(this.crArr);

	// ARLGR encode
	return { yData: encode(this.yArr), cbData: encode(this.cbArr), crData: encode(this.crArr) };
}

module.exports = {
	decompressor: RFXTileDecompressor,
	decompressorSIMD: RFXTileDecompressorSIMD,
	compressor: RFXTileCompressor
}
