'use strict';
/*
 * ARLGR encoding.
 */

    var ForwardReadStream       = require("xrdp/rfx/decompress/js/ForwardReadStream");
    var ForwardWriteStream      = require("xrdp/rfx/decompress/js/ForwardWriteStream");
    var ffs                     = require("xrdp/rfx/decompress/js/ffs");

function getMinBits(val) {
	var tt;

	if (tt = val >>> 8) return ffs[tt] + 8;
	else return ffs[val];
}

function get2MagSignFromInt(input) {
	return input >= 0 ? (input << 1) : (-input << 1) - 1;
}

// This could be optimized further by removing the inner call.
function writeBit(ws, n, bit) {
	var mask = bit ? 0xffffffff : 0;

	while (n) {
		var i = Math.min(n,32);

		ws.write(i,mask);
		n -= i;
	}
}

function decode1(terms, buf, arr) {
	var rs = new ForwardReadStream(buf);
	var i = 0;
	var k = 1;
	var kp = 8;
	var kr = 1;
	var krp = 8;

	while (i < terms) {
		if (k) {
			var run = rs.readRun(0);
			var zeros = 0;

			while (run--) {
				zeros += 1 << k;

				kp = Math.min(kp + 4,80);
				k = kp >> 3;
			}

			if (k) zeros += rs.read(k);

			zeros = Math.min(zeros,terms - i);

			while (zeros--) arr[i++] = 0;

			if (i < terms) {
				var sign = rs.read(1);
				var vk = rs.readRun(1);
				var mag = (kr ? (vk << kr) | rs.read(kr) : vk) + 1;

				arr[i++] = (-2 * sign + 1) * mag;

				kp = Math.max(kp - 6,0);
				k = kp >> 3;

				if (!vk) {
					krp = Math.max(krp - 2,0);
					kr = krp >> 3;
				}
				else if (vk > 1) {
					krp = Math.min(krp + vk,80);
					kr = krp >> 3;
				}
			}
		}
		else {
			var vk = rs.readRun(1);
			var mag = kr ? (vk << kr) | rs.read(kr) : vk;

			if (mag) {
				arr[i++] = (-2 * (mag & 1) + 1) * ((mag + 1) >> 1);

				kp = Math.max(kp - 3,0);
				k = kp >> 3;
			}
			else {
				arr[i++] = 0;

				kp = Math.min(kp + 3,80);
				k = kp >> 3;
			}

			if (!vk) {
				krp = Math.max(krp - 2,0);
				kr = krp >> 3;
			}
			else if (vk > 1) {
				krp = Math.min(krp + vk,80);
				kr = krp >> 3;
			}
		}
	}
}

function decode3(terms, buf, arr) {
	var rs = new ForwardReadStream(buf);
	var i = 0;
	var k = 1;
	var kp = 8;
	var kr = 1;
	var krp = 8;

	while (i < terms) {
		if (k) {
			var run = rs.readRun(0);
			var zeros = 0;

			while (run--) {
				zeros += 1 << k;

				kp = Math.min(kp + 4,80);
				k = kp >> 3;
			}

			if (k) zeros += rs.read(k);

			zeros = Math.min(zeros,terms - i);

			while (zeros--) arr[i++] = 0;

			if (i < terms) {
				var sign = rs.read(1);
				var vk = rs.readRun(1);
				var mag = (kr ? (vk << kr) | rs.read(kr) : vk) + 1;

				arr[i++] = (-2 * sign + 1) * mag;

				kp = Math.max(kp - 6,0);
				k = kp >> 3;

				if (!vk) {
					krp = Math.max(krp - 2,0);
					kr = krp >> 3;
				}
				else if (vk > 1) {
					krp = Math.min(krp + vk,80);
					kr = krp >> 3;
				}
			}
		}
		else {
			var vk = rs.readRun(1);
			var mag = kr ? (vk << kr) | rs.read(kr) : vk;
			var val1 = mag ? rs.read(getMinBits(mag)) : 0;
			var val2 = mag - val1;

			arr[i++] = (-2 * (val1 & 1) + 1) * ((val1 + 1) >> 1);
			if (i < terms) arr[i++] = (-2 * (val2 & 1) + 1) * ((val2 + 1) >> 1);

			if (val1 && val2) {
				kp = Math.max(kp - 6,0);
				k = kp >> 3;
			}
			else if (!val1 && !val2) {
				kp = Math.min(kp + 6,80);
				k = kp >> 3;
			}

			if (!vk) {
				krp = Math.max(krp - 2,0);
				kr = krp >> 3;
			}
			else if (vk > 1) {
				krp = Math.min(krp + vk,80);
				kr = krp >> 3;
			}
		}
	}
}

function encode1(arr) {
	var ws = new ForwardWriteStream;
	var i = 0;
	var k = 1;
	var kp = 8;
	var krp = 8;

	while (i < arr.length) {
		if (k) {
			var input;
			var zeros = 0;

			while (i < arr.length && !(input = arr[i++])) zeros++;

			var run = 0;

			while (zeros >= 1 << k) {
				zeros -= 1 << k;
				run++;

				kp = Math.min(kp + 4,80);
				k = kp >> 3;
			}

			if (run) writeBit(ws,run,0);
			writeBit(ws,1,1);
			if (k) ws.write(k,zeros);

			if (input) {
				writeBit(ws,1,input < 0 ? 1 : 0);

				var mag = (input < 0 ? -input : input) - 1;
				var kr = krp >> 3;
				var vk = mag >> kr;

				if (vk) writeBit(ws,vk,1);
				writeBit(ws,1,0);
				if (kr) ws.write(kr,mag);

				kp = Math.max(kp - 6,0);
				k = kp >> 3;

				if (!vk) krp = Math.max(krp - 2,0);
				else if (vk > 1) krp = Math.min(krp + vk,80);
			}
		}
		else {
			var twoMs = get2MagSignFromInt(arr[i++]);
			var kr = krp >> 3;
			var vk = twoMs >> kr;

			if (vk) writeBit(ws,vk,1);
			writeBit(ws,1,0);
			if (kr) ws.write(kr,twoMs);

			if (twoMs) {
				kp = Math.max(kp - 3,0);
				k = kp >> 3;
			}
			else {
				kp = Math.min(kp + 3,80);
				k = kp >> 3;
			}

			if (!vk) krp = Math.max(krp - 2,0);
			else if (vk > 1) krp = Math.min(krp + vk,80);
		}
	}

	return ws.buffer;
}

function encode3(arr) {
	var ws = new ForwardWriteStream;
	var i = 0;
	var k = 1;
	var kp = 8;
	var krp = 8;

	while (i < arr.length) {
		if (k) {
			var input;
			var zeros = 0;

			while (i < arr.length && !(input = arr[i++])) zeros++;

			var run = 0;

			while (zeros >= 1 << k) {
				zeros -= 1 << k;
				run++;

				kp = Math.min(kp + 4,80);
				k = kp >> 3;
			}

			if (run) writeBit(ws,run,0);
			writeBit(ws,1,1);
			if (k) ws.write(k,zeros);

			if (input) {
				writeBit(ws,1,input < 0 ? 1 : 0);

				var mag = (input < 0 ? -input : input) - 1;
				var kr = krp >> 3;
				var vk = mag >> kr;

				if (vk) writeBit(ws,vk,1);
				writeBit(ws,1,0);
				if (kr) ws.write(kr,mag);

				kp = Math.max(kp - 6,0);
				k = kp >> 3;

				if (!vk) krp = Math.max(krp - 2,0);
				else if (vk > 1) krp = Math.min(krp + vk,80);
			}
		}
		else {
			var twoMs1 = get2MagSignFromInt(arr[i++]);
			var twoMs2 = get2MagSignFromInt(i < arr.length ? arr[i++] : 0);
			var sum2Ms = twoMs1 + twoMs2;
			var kr = krp >> 3;
			var vk = sum2Ms >> kr;

			if (vk) writeBit(ws,vk,1);
			writeBit(ws,1,0);
			if (kr) ws.write(kr,sum2Ms);
			if (sum2Ms) ws.write(getMinBits(sum2Ms),twoMs1);

			if (twoMs1 && twoMs2) {
				kp = Math.max(kp - 6,0);
				k = kp >> 3;
			}
			else if (!twoMs1 && !twoMs2) {
				kp = Math.min(kp + 6,80);
				k = kp >> 3;
			}

			if (!vk) krp = Math.max(krp - 2,0);
			else if (vk > 1) krp = Math.min(krp + vk,80);
		}
	}

	return ws.buffer;
}

module.exports = {
	decoder: function(mode) {
		switch (mode) {
		case 1: return decode1;
		case 4: return decode3;
		default: return undefined;
		}
	},
	encoder: function(mode) {
		switch (mode) {
		case 1: return encode1;
		case 4: return encode3;
		default: return undefined;
		}
	}
}
