'use strict';
/*
 * http://msdn.microsoft.com/en-us/library/ff635269.aspx
 *
 * TS_RFX_BLOCKT
 * TS_RFX_CODEC_CHANNELT
 * TS_RFX_CHANNELT
 * TS_RFX_CODEC_VERSIONT
 * TS_RFX_CODEC_QUANT
 * TS_RFX_RECT
 */

const channels = require("./channels");
const context = require("./context");
const region = require("./region");
const sync = require("./sync");
const versions = require("./versions");
const frameBegin = require("./frame-begin");
const frameEnd = require("./frame-end");
const tileSet = require("./tileset");

var readHeader = function(dv){//https://msdn.microsoft.com/en-us/library/ff635385.aspx
    var headerLength = 6;
    var o = {};
    o.blockType = dv.getUint16(0,true);
    o.blockLen = dv.getUint32(2,true);
    if([0xCCC3,0xCCC4,0xCCC5,0xCCC6,0xCCC7].indexOf(o.blockType) >= 0){
        //codec channel T extension
        if(dv.getUint8(6) !== 0x01){
            throw new Error("Invalid CodecId");
        }
        var channelId = dv.getUint8(7);
        if( !((o.blockType === 0xCCC3 && channelId === 0xFF) || channelId === 0x00)){
            throw new Error("Invalid Channel Id");
        }
        headerLength +=2;
    }
    if(dv.byteOffset + headerLength < dv.buffer.byteLength){
        o.data = new DataView(dv.buffer,dv.byteOffset+ headerLength,o.blockLen - headerLength);
    }
    return o;
}

function Frame () {
    var that = this;
    var regions = [];
    var tilesetPromise;

    that.regions = function (r) {
        regions = r
    }
    that.tileset = function (tsp) {
        tilesetPromise = tsp;
    }
    that.end = function () {
        return  new Promise(function(resolve,reject) {
            tilesetPromise.then(function (tiles) {
                resolve({
                    regions:regions,
                    tiles:tiles
                });
            });
        });
    }
}

var processMessage = (function(){
    var f;

    return function(dv,promises) {
    	var o = readHeader(dv);
    	switch (o.blockType) {
    		case 0xCCC0://Sync
    			sync(o.data);
    			break;
    		case 0xCCC1://versions
    			versions(o.data);
    			break;
    		case 0xCCC2:
    			channels(o.data);
    			break;
    		//Codec ChannelT
    		case 0xCCC3://rfx.WBT_CONTEXT
    			context(o.data);
    			break;
    		case 0xCCC4://rfx.WBT_FRAME_BEGIN
                frameBegin(o.data);
                f = new Frame();
                break;
    		case 0xCCC5://rfx.WBT_FRAME_END:
                frameEnd(o.data);
                promises.push(f.end());
    			break;
    		case 0xCCC6://rfx.WBT_REGION:
                f.regions(region(o.data));
    			break;
    		case 0xCCC7://rfx.WBT_EXTENSION -- tileset:
                f.tileset(tileSet(o.data));
    			break;
    		default:
    			throw new Error("Invalid BlockType");
    	}
    	return o.blockLen;
    }
})();

var process = function(o){
	var bytesRead =0;
	var dv,promises =[];
    if(o.data){
		do{
            dv = new DataView(o.data.buffer,o.data.byteOffset + bytesRead);
            bytesRead += processMessage(dv,promises);
		}while(bytesRead < o.data.byteLength && o.data.byteOffset + bytesRead < dv.buffer.byteLength);
	}
    return promises;
}
module.exports = {
    process:process
};
