//const decompress  = require("xrdp/rfx/decompress/decompress");

const RFX_WORKER = "js/xrdp/rfx-worker.bundle.js";

var myDecoder = window.localStorage["decoder"] || (("WebAssembly" in window)?"WASM":"JS");
document.addEventListener("fxDecoder",function(e){
	myDecoder = e.detail;
});

function WorkerPool (n) {
	var that = this;
	this._i = 0;
	this._pool = [];
	this._pending = {};

	for (var i =0; i< n; i++) {
		this._pool.push(new Worker(RFX_WORKER));
		this._pool[i].onmessage = function(e){
			that._pending[e.data.tileId](e.data);
			delete that._pending[e.data.tileId];
		}
	}
}
WorkerPool.prototype.message = function(data,cb) {
	this._pool[ this._i % this._pool.length ].postMessage(Object.assign({
		tileId:this._i,
	},data));
	this._pending[this._i] = cb;
	this._i++;
}
var workers = new WorkerPool(8);

const tile = function(dv,worker,extra){
	var o = {};
    if(dv.getUint16(0,true) !== 0xCAC3){
        throw new Error("BlockType must be CBT_TILE 0xCAC3");
    }
    o.blockLen = dv.getUint32(2,true);
	o.quantIdxY = dv.getUint8(6);
	o.quantIdxCb = dv.getUint8(7);
	o.quantIdxCr = dv.getUint8(8);
	o.xIdx = dv.getUint16(9,true);
	o.yIdx = dv.getUint16(11,true);
    this.yLen = dv.getUint16(13,true);
    this.cbLen = dv.getUint16(15,true);
	o.yData  = new DataView(dv.buffer,dv.byteOffset + 19,this.yLen);
	o.cbData = new DataView(dv.buffer,dv.byteOffset + 19 + this.yLen ,this.cbLen);
	o.crData = new DataView(dv.buffer,dv.byteOffset + 19 + this.yLen + this.cbLen,dv.getUint16(17,true));
    return o;
}

const tileSet = function(dv){
    if (dv.getUint16(0,true) !== 0xCAC2){
        throw new Error("Invalid subtype");
    }
    var i,quants = [];
    var entropy = (dv.getUint16(4,true) >>> 10) & 0x0F;
    var numQuants = dv.getUint8(6);
    //http://msdn.microsoft.com/en-us/library/ff635311.aspx
    for (i = 0; i < numQuants; i++) {
        quants.push([
            (dv.getUint8(14 + 5*i) & 0x0F),//ll3
            (dv.getUint8(14 + 5*i) >>> 4), //lh3
            (dv.getUint8(15 + 5*i) & 0x0F),//hl3
            (dv.getUint8(15 + 5*i) >>> 4), //hh3
            (dv.getUint8(16 + 5*i) & 0x0F),//lh2
            (dv.getUint8(16 + 5*i) >>> 4), //hl2
            (dv.getUint8(17 + 5*i) & 0x0F),//hh2
            (dv.getUint8(17 + 5*i) >>> 4), //lh1
            (dv.getUint8(18 + 5*i) & 0x0F),//hl1
            (dv.getUint8(18 + 5*i) >>> 4)]);//hh1
    }
    var offset =0,t;
    var numTiles = dv.getUint16(8,true);
    var tileData = new DataView(dv.buffer,dv.byteOffset + 14 +  numQuants*5,dv.getUint32(10,true));
	var ps = [];
    for(i = 0;i<numTiles;i++){
        t = tile(new DataView(tileData.buffer,tileData.byteOffset + offset));
		ps.push(new Promise( function(resolve,reject){
			workers.message({
				t:t,
				entropy:entropy,
				quants:quants,
				engine:myDecoder
			},resolve);

		}));
        offset += t.blockLen;
    }
    return Promise.all(ps);
}
module.exports = tileSet;
