"use strict";
const messages = require("./messages/messages");
const Offscreen = require("xrdp/cache/offscreen-bitmap");
const render = require("./main/render");


function putTile (c) {
    var tile;
    while(tile = c.tileset.tiles.shift()){
        render.tile(c.tileset.regions,c.rect,tile);
    }
}

function frameEnd (id,data,cb) {
    Promise.all(data.partials.map(function(c){
        return c.promise;
    })).then(function(tilesetPromises) {
        if(!tilesetPromises.length){
            Offscreen.current().done(data.queue,function(){
                cb(id);
            });
            return;
        }
        Promise.all(tilesetPromises.reduce(function(a,b){
            return a.concat(b);
        })).then(function (tilesets) {
            var sets = data.partials.map(function(c,i){
                return {
                    rect:c.rect,
                    tileset:tilesets[i]
                };
            });
            Offscreen.current().done(data.queue,function(){
                render.begin(Offscreen.current().canvas);
                sets.forEach(function(c){
                    putTile(c);
                });
                render.end(id);
                cb(id);
            });
        });
    });
}
function frameBegin (frameMarker) {
    frameMarker.data = {
       frameBegin:true,
       queue:Offscreen.current().enqueue(),
       partials:[]
   }
   frameMarker.frameEnd = frameEnd;
}

function decompress (o,frameMarker) {
    if (!frameMarker) {
        //outside of frame render to screen
        Offscreen.current().done(Offscreen.current.enqueue(), function () {
            render.begin(Offscreen.current().canvas);

            render.end();
        });
   } else {
      if(!frameMarker.data.frameBegin){
           //first Frame;
           frameBegin(frameMarker);
      }
       var p = messages.process(o);
       frameMarker.data.partials.push({
           rect:o.rect,
           promise:p
       });
   }
}
module.exports = {
    decompress:decompress
}
