'use strict';
	/* 
	 * https://msdn.microsoft.com/en-us/library/cc240785.aspx
	 */
	var cr,sr;
	
	function md5(str){return KJUR.crypto.Util.md5(str);}
	function sha1(str){return KJUR.crypto.Util.sha1(str);}
	
	function u8toString(u8){
		var u16 = Array.prototype.slice.call(new Uint16Array(u8)).map(function(n){
			return String.fromCharCode(n);
		});
		return u16.join('');
		
	}
	function stringToU8(str){
		var arr = (str.split('')).map(function(c){return c.charCodeAt(0)});
		return new Uint8Array(new Uint16Array(arr));
	}
	function saltedHash(S,I){
		// SaltedHash(S, I) = MD5(S + SHA(I + S + ClientRandom + ServerRandom))
		var ab = new Uint8Array(I.length + S.length + cr.length + sr.length);
		ab.set(I,0);
		ab.set(S,I.length);
		ab.set(cr,I.length + S.length);
		ab.set(sr,I.length + S.length + cr.length);
		return stringToU8(md5(u8toString(S) + sha1(u8toString(ab))));
		
	}
	function preMasterSecret(clientRandom,serverRandom){
		var ab = new Uint8Array(384/8);
		ab.set(new Uint8Array(clientRandom.buffer,clientRandom.byteOffset,192/8),0);
		ab.set(new Uint8Array(serverRandom.buffer,serverRandom.byteOffset,192/8),192/8);
		return ab;
	}
	function preMasterHash(I,pms){
		return saltedHash(pms,I);
	}
	function masterHash(I,ms){
		return saltedHash(ms,I);
	}
	function masterSecret(pms){
		var a,b,c;
		a = preMasterHash(new Uint8Array([0x41]),pms);
		b = preMasterHash(new Uint8Array([0x42,0x42]),pms);
		c = preMasterHash(new Uint8Array([0x43,0x43,0x43]),pms);
		var ab = new Uint8Array(a.length+b.length+c.length);
		ab.set(a,0);
		ab.set(b,a.length);
		ab.set(c,b.length);
		return ab;
	}
	
	function sessionKeyBlob(pms){
		//MasterHash(0x58) + MasterHash(0x5959) + MasterHash(0x5A5A5A)
		var a,b,c;
		var ms = masterSecret(pms)
		a = masterHash(new Uint8Array([0x58]),ms);
		b = masterHash(new Uint8Array([0x50,0x59]),ms);
		c = masterHash(new Uint8Array([0x5A,0x5A,0x5A]),ms);
		var ab = new Uint8Array(a.length + b.length + c.length);
		ab.set(a,0);
		ab.set(b,a.length);
		ab.set(c,a.length + b.length);
		return ab;
	}
	
	function finalHash(K){
		var ab = new Uint8Array(K.length+cr.length+sr.length);
		ab.set(K,0);
		ab.set(cr,K.length);
		ab.set(sr,K.length+cr.length);
		return stringToU8(md5(u8toString(ab)));
	}
	function key40(key128){
		var salt = new Uint8Array([0xd1,0x26,0x9e]);
		var mac40 = new Uint8Array(8);
		mac40.set(salt,0);
		mac40.set(key128.mac.subarray(3,8),3);
		
		var dec40 = new Uint8Array(8);
		dec40.set(salt,0);
		dec40.set(key128.decrypt.subarray(3,8),3);
		
		var enc40 = new Uint8Array(8);
		enc40.set(salt,0);
		enc40.set(key128.encrypt.subarray(3,8),3);
		
		return {
			mac:mac40,
			encrypt:enc40,
			decrypt:dec40
		};
		
	}
	function key56(key128){
		var salt = new Uint8Array([0xd1]);
		var mac56 = new Uint8Array(8);
		mac56.set(salt,0);
		mac56.set(key128.mac.subarray(1,8),1);
		
		var dec56 = new Uint8Array(8);
		dec56.set(salt,0);
		dec56.set(key128.decrypt.subarray(1,8),1);
		
		var enc56 = new Uint8Array(8);
		enc56.set(salt,0);
		enc56.set(key128.encrypt.subarray(1,8),1);
		
		return {
			mac:mac56,
			encrypt:enc56,
			decrypt:dec56
		};
	}
	function generateSessionKeys(o){
		cr = o.clientRandom;
		sr = o.serverRandom; 
		
		//test serverRandom -- https://msdn.microsoft.com/en-us/library/cc240919.aspx
		new Uint8Array([
			0x10, 0x11, 0x77, 0x20, 0x30, 0x61, 0x0a, 0x12, 
			0xe4, 0x34, 0xa1, 0x1e, 0xf2, 0xc3, 0x9f, 0x31, 
			0x7d, 0xa4, 0x5f, 0x01, 0x89, 0x34, 0x96, 0xe0,
			0xff, 0x11, 0x08, 0x69, 0x7f, 0x1a, 0xc3, 0xd2
		]);
		//end serverRandom test
		
		var pms = preMasterSecret(cr,sr);
		var skb = sessionKeyBlob(pms);
		
		var key128 = {
			mac:skb.subarray(0,16),
			decrypt:finalHash(skb.subarray(16,32)),
			encrypt:finalHash(skb.subarray(32,48))
		};
		switch(o.encryptionMethod){
			case 0x00000001://40 bit encryption
				return key40(key128);
				break;
			case 0x00000002://128 bit encryption
				return key128;
				break;
			case 0x00000008://56 bit encryption
				return key56(key128);
				break;
			default:
				throw new Error('Invalid encrytion method');
		}
	}
	
	function SHAComponent(mac,data){
		
		var ab = new Uint8Array(mac.length +  44 + data.length);
		ab.set(mac,0);
		ab.set(new Uint8Array([
			0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,
			0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,
			0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,
			0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,
			0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36
		]),mac.length);
		ab.set(new Uint8Array(new Uint32Array([data.length])),mac.length+40);
		ab.set(data,mac.length+44);
		return stringToU8(sha1(u8toString(ab)));
		
	}
	function MACSignature(mac,shacomponent){
		
		var ab = new Uint8Array(mac.length + 44 + shacomponent.length);
		ab.set(mac,0);
		ab.set(new Uint8Array([
			0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,
			0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,
			0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,
			0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,
			0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,
			0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,0x5C,0x5C
		]),mac.length);
		ab.set(shacomponent,mac.length + 44);
		return (stringToU8(md5(u8toString(ab)))).subarray(0,8);
		
	}
	
/*
 * RC4 symmetric cipher encryption/decryption
 *
 * @license Public Domain
 * @param string key - secret key for encryption/decryption
 * @param string str - string to be encrypted/decrypted
 * @return string
 */
function rc4(key, str) {
	var s = [], j = 0, x, res = '';
	for (var i = 0; i < 256; i++) {
		s[i] = i;
	}
	for (i = 0; i < 256; i++) {
		j = (j + s[i] + key.charCodeAt(i % key.length)) % 256;
		x = s[i];
		s[i] = s[j];
		s[j] = x;
	}
	i = 0;
	j = 0;
	for (var y = 0; y < str.length; y++) {
		i = (i + 1) % 256;
		j = (j + s[i]) % 256;
		x = s[i];
		s[i] = s[j];
		s[j] = x;
		res += String.fromCharCode(str.charCodeAt(y) ^ s[(s[i] + s[j]) % 256]);
	}
	return res;
}
	
function sign(mac,data){
        //throw new Error('Signing not implemented');
        return MACSignature(mac,SHAComponent(mac,data));
}
function encrypt(key,data){
        //throw new Error('Encrypt not implemented');
        return stringToU8(rc4(u8toString(key),u8toString(data)));
}
module.exports = {
    generateSessionKeys:generateSessionKeys,
    sign:sign,
    encrypt:encrypt
}