"use strict";

const Security = require("./util");

function writeSecurityHeaderBasicAndData(flags,data){
    if(!flags){return data}//optional security header
    var ab = new ArrayBuffer(data.length+4);
    var dv = new DataView(ab);
    dv.setUint16(0,flags,true);
    dv.setUint16(2,0);
    var u8 = new Uint8Array(ab);
    u8.set(data,4);
    return u8;
}
function writeSecurityHeaderNonFIPSAndData(flags,mac,encrypt,data){
    var sig = Security.sign(mac,data);
    var enc = Security.encrypt(encrypt,data);
    var ab = new Uint8Array(sig.length + enc.length);
    ab.set(sig,0);
    ab.set(enc,sig.length);
    return writeSecurityHeaderBasicAndData(flags,ab);
}
function writeSecurityHeaderAndData(flags,encryptionLevel,sessionKeys,data){
    switch(encryptionLevel){
        case 0x00000000://none
            return writeSecurityHeaderBasicAndData(flags,data);
            break;
        case 0x00000001://low
        case 0x00000002://client Compatible
        case 0x00000003://high
            flags = flags | 0x0008; //encrypt the packet
            return writeSecurityHeaderNonFIPSAndData(flags,sessionKeys.mac,sessionKeys.encrypt,data);
            break;
        case 0x00000004://FIPS
            throw new Error('FIPS Security not yet implemented');
        default:
            throw new Error('Invalid Security Level');
    }
}
module.exports = {
    writeSecurityHeaderAndData:writeSecurityHeaderAndData
};
