'use strict';

var DataStream          = require('DataStream');
var mcs                 = require('xrdp/core/mcs');
var tpkt                = require('xrdp/core/tpkt');
var x224                = require('xrdp/core/x224');

var securityHeader      = require('xrdp/security/header/writer');

const dataPdu             = require('./data');
const demandActive = require("./demand-active/reader");
const deactivateAll = require("./deactivate-all/reader");



	function uncompress(type,data){
		console.log('uncompress',data);
		return data;//todo -- finish uncompression
	}

	function writeShareControlHeaderAndData(ds,pduType,pduSource,u8array){
		ds.writeUint16(u8array.byteLength + 6);
		ds.writeUint16( pduType << 8 | 0x0001 << 12,DataStream.BIG_ENDIAN);
		ds.writeUint16(pduSource);
		ds.writeUint8Array(u8array);
		return 1;
	}
	function writeShareDataHeaderAndData(ds,channelId,shareId,streamId,pduType2,compressedType,data){
		var d =new DataStream();
		var compressedData;
		d.writeUint32(shareId);
		d.writeUint8(0x00);
		d.writeUint8(streamId);
		d.writeUint16(data.length);
		d.writeUint8(pduType2);
		d.writeUint8(compressedType);
		compressedData = data;//Todo add data compressor
		d.writeUint16(compressedData.length);
		d.writeUint8Array(data);
		writeShareControlHeaderAndData(ds,0x7,channelId,new Uint8Array(d.buffer));
		return 1;
	}

	function decodeShareControlHeader(dv){
		var o ={};
		if(dv.byteLength < 2 || dv.getUint16(0,true) > dv.byteLength){ return {}; }

		o.pduType = (dv.getUint16(2) & 0x0F00) >>8;
		if([0x1,0x3,0x6,0x7,0xA].indexOf(o.pduType) === -1){
			return undefined;
		}
		if( (dv.getUint16(2) & 0xF000) >> 12 !== 1){
			console.log('Invalid PDU version');
			return undefined;
		}
		o.pduSource = dv.getUint16(4,true);
		o.shareData = new DataView(dv.buffer,dv.byteOffset +6,dv.getUint16(0,true) - 6 );
		return o;
	}
	function decodeShareDataHeader(dv){
		var o ={};
		o.shareId = dv.getUint32(0,true);
		o.streamId = dv.getUint8(5);
		o.pduType2 = dv.getUint8(8);

		if(dv.getUint8(9) !== 0){
			if(dv.byteLength !== dv.getUint16(10,true) - 12){
				console.log('Invalid Compressed Length');
				return undefined;
			}
			o.data = uncompress(dv.getUint8(9),new DataView(dv.buffer,dv.byteOffset + 12,dv.getUint16(10,true)));
		}else{
			o.data = new DataView(dv.buffer,dv.byteOffset + 12,dv.getUint16(6,true));
		}
		if(o.data.byteLength !== dv.getUint16(6,true)){
			console.log('Uncompressed Length Invalid');
			return undefined;
		}
		return o;
	}

	function writeSecurityHeaderAndData(ds,encryptionMethod,flags,data){

		return 1;
	}




	function slowPathInputEvent(type,buf,encryptionLevel,sessionKeys){
		if([0x0000,0x0002,0x0004,0x0005,0x8001,0x8002].indexOf(type) === -1){
			throw "Invalid Input Event Type";
		}
		var ibuf = new ArrayBuffer(buf.byteLength + 6);
		var dv = new DataView(ibuf);
		dv.setUint32(0,Date.now(),true);
		dv.setUint16(4,type,true);
		(new Uint8Array(ibuf,6)).set(new Uint8Array(buf));
		return securityHeader.writeSecurityHeaderAndData(0x0000,encryptionLevel,sessionKeys,new Uint8Array(ibuf)).buffer
	}

	function parseHeader(dv){
		var data = tpkt.parsePacket(dv);
		if(typeof data === 'undefined'){ return undefined }
		if(typeof data.payload === 'undefined'){return {};}
		data = x224.parseDataPDU(data.payload);
		if(typeof data === 'undefined'){ return undefined;}
		if(typeof data.data === 'undefined'){return {};}
		data = mcs.parseSDin(data.data);
		if(typeof data === 'undefined'){ return undefined;}
		if(typeof data.userData === 'undefined'){return {};}
		return data;
	}
	function decodePduType(dv){
		var o = {},pos;
		if(dv.byteLength > 127){
			pos = 15;
		}else{
			pos = 14;
		}
		//TODO CHECK SECURITY HEADER
		o = decodeShareControlHeader(dv);
		switch(o.pduType){
			case 0x01: //demandActive
				if(! demandActive.process(new DataView(dv.buffer,dv.byteOffset + 6),o)){
					console.log('Error Decoding Demand Active');
					return undefined;
				}
				break;
			case 0x06: //deactivateAll
				if(! deactivateAll.process(new DataView(dv.buffer,dv.byteOffset +6),o)){
					console.log('Error Decoding deactivateAll');
					return undefined;
				}
				break;
			case 0x07: //data
				var data = decodeShareDataHeader(o.shareData);
				if(!data){return undefined;}
				if(!data.data){return {};}
				if(!dataPdu.decodeDataPdu(data.pduType2,data.data)){
					return undefined;
				}
				o = Object.assign(o,data);
				break;
			case 0x0A: //server redir pkt
				break;
			case 0x03: //confirmActive
				console.log('server sent a confirm active pdu... bailing');
			default:
				console.log('Invalid PDU Type');
				return undefined;
		}
		return o;
	}
	function frameAcknowledge(idx){

		dv.setUint32(0,idx,true);
		return buf;
	}

module.exports = {
		userChannelId:0x00,
		ioChannelId:0x00,
		shareId:0x00,
		compressedType:0x00,
		parseHeader:parseHeader,
		decodePduType:decodePduType,
		init:function(userChannelId,ioChannelId,shareId,compressionType,encryptionLevel,sessionKeys){
			this.userChannelId = userChannelId;
			this.ioChannelId = ioChannelId;
			this.shareId = shareId;
			this.compressedType = compressionType;
			this.encryptionLevel = encryptionLevel;
			this.sessionKeys = sessionKeys
		},
		confirmActivePdu:function(sourceDescriptor,capsetCount,u8CapsetArray){
			var ds = new DataStream();
			var data = new DataStream();
			data.writeUint32(this.shareId);
			data.writeUint16(0x03EA);//originatorID
			data.writeUint16(sourceDescriptor.length + 1);
			data.writeUint16(u8CapsetArray.length + 4);
			data.writeCString(sourceDescriptor);
			data.writeUint16(capsetCount);
			data.writeUint16(0x0000); //pad octets
			data.writeUint8Array(u8CapsetArray);
			writeShareControlHeaderAndData(ds,0x3,this.userChannelId,new Uint8Array(data.buffer));
			return tpkt.writePacket(x224.writeDataPDU(mcs.writeSDrqHeader(this.userChannelId,this.ioChannelId,0x01,0x03,
				securityHeader.writeSecurityHeaderAndData(0x0000,this.encryptionLevel,this.sessionKeys,new Uint8Array(ds.buffer))
			)));
		},
		dataPdu:function(pduType2,streamId,compressedType,u8array){
			var ds = new DataStream();
			var data = new DataStream();
			var compressedData;
			data.writeUint32(this.shareId);
			data.writeUint8(0);
			data.writeUint8(streamId);
			data.writeUint16(u8array.length);
			data.writeUint8(pduType2);
			data.writeUint8(compressedType);
			compressedData = u8array; // add compressor
			data.writeUint16(compressedData.length);
			data.writeUint8Array(compressedData);

			writeShareControlHeaderAndData(ds,0x07,this.userChannelId,new Uint8Array(data.buffer));
			return tpkt.writePacket(x224.writeDataPDU(mcs.writeSDrqHeader(this.userChannelId,this.ioChannelId,0x01,0x03,
				securityHeader.writeSecurityHeaderAndData(0x0000,this.encryptionLevel,this.sessionKeys,new Uint8Array(ds.buffer))
			)));
		},
		inputEventPdu:function(events){
			var ds = new DataStream();
			ds.writeUint16(events.length);
			ds.writeUint16(0x0000);
			while(events.length){ds.writeUint8Array(new Uint8Array(events.shift()));}
			return tpkt.writePacket(x224.writeDataPDU(mcs.writeSDrqHeader(this.userChannelId,this.ioChannelId,0x01,0x03,new Uint8Array(ds.buffer))));
		},
		inputEventSync:function(toggleFlags){
			var buf = new ArrayBuffer(6);
			var dv = new DataView(buf);
			dv.setUint32(2,toggleFlags,true);
			return slowPathInputEvent(0x0000,buf,this.encryptionLevel,this.sessionKeys);
		},
		inputUnusedEvent:function(){
			return slowPathInputEvent(0x0002,new ArrayBuffer(6));
		},
		inputEventScancode:function(keyboardFlags,keyCode){
			var buf = new ArrayBuffer(6);
			var dv = new DataView(buf);
			dv.setUint16(0,keyboardFlags,true);
			dv.setUint16(2,keyCode,true);
			return slowPathInputEvent(0x0004,buf,this.encryptionLevel,this.sessionKeys);
		},
		inputEventUnicode:function(keyboardFlags,unicodeCode){
			var buf = new ArrayBuffer(6);
			var dv = new DataView(buf);
			dv.setUint16(0,keyboardFlags,true);
			dv.setUint16(2,unicodeCode,true);
			return slowPathInputEvent(0x0005,buf,this.encryptionLevel,this.sessionKeys);
		},
		inputEventMouse:function(pointerFlags,pos){
			var buf = new ArrayBuffer(6);
			var dv = new DataView(buf);
			dv.setUint16(0,pointerFlags,true);
			dv.setUint16(2,pos[0],true);
			dv.setUint16(4,pos[1],true);
			return slowPathInputEvent(0x8001,buf,this.encryptionLevel,this.sessionKeys);
		},
		inputEventMouseX:function(pointerFlags,pos){
			var buf = new ArrayBuffer(6);
			var dv = new DataView(buf);
			dv.setUint16(0,pointerFlags,true);
			dv.setUint16(2,pos[0],true);
			dv.setUint16(4,pos[1],true);
			return slowPathInputEvent(0x8002,buf,this.encryptionLevel,this.sessionKeys);
		},
	    suppressOutput:function(){
			var buf = new ArrayBuffer(4);
			return this.dataPdu(0x23,0x02,0x00,new Uint8Array(buf));
		},
	    unsuppressOutput:function(rect){
			var buf = new ArrayBuffer(12);
			var dv = new DataView(buf);
			dv.setUint8(0,0x01);
			dv.setUint16(4,rect[0],true);
			dv.setUint16(6,rect[1],true);
			dv.setUint16(8,rect[2],true);
			dv.setUint16(10,rect[3],true);
			return this.dataPdu(0x23,0x02,0x00,new Uint8Array(buf));
		},
		frameAcknowledge:function(id){
			var buf = new ArrayBuffer(4);
			var dv = new DataView(buf);
			dv.setUint32(0,id,true);
			return this.dataPdu(0x38,0x02,0x00,new Uint8Array(buf));
		}
	}
