"use strict";

var c = require('xrdp/constants');
var conSeq = require('xrdp/connection/init');
var Connected = require('xrdp/connected');
var polyfill = require('xrdp/polyfill');
var capabilitySettings  = require('xrdp/capability-settings');
var Screen  = require('xrdp/canvas/screen');
var Offscreen = require('xrdp/cache/offscreen-bitmap');

	function ceilPow2( aSize ){
		return Math.pow( 2, Math.ceil( Math.log( aSize ) / Math.log( 2 ) ) );
	}
	function Client(fastx){
		var socketMessage = s;
		console.log('FastX Object',fastx);
		function debug(debugTrue){
			if(!debugTrue){return;}


			socketMessage = function(e){;
				console.log('SERVER',Array.from(new Uint8Array(e.data)).map(function(c){
					return (c.toString(16).length === 1)?("0" + c.toString(16)):c.toString(16);
				}).join(' ') );
				console.log(' ');
				s(e);
			}

			WebSocket.prototype.oldSend = WebSocket.prototype.send;
			WebSocket.prototype.send = function(data){;
				console.log('CLIENT',Array.from(new Uint8Array(data)).map(function(c){
					return (c.toString(16).length === 1)?("0" + c.toString(16)):c.toString(16);
				}).join(' ') );
				console.log(' ');
				this.oldSend(data);
			}
		}
		debug(fastx.debugXrdp);

		if(!fastx.ws){
			fastx.closed({
				reason:'Cannot create a websocket.  No url defined'
			});
			return;
		}
		if(fastx.Password){
			fastx.password = fastx.Password;
		}
		if(!fastx.password && !fastx.windows){
			fastx.closed({
				reason:'Password is required'
			});
			return;
		}
		if(fastx.RdpPort){//fix for /share? -connection method
			fastx.port = fastx.RdpPort;
		}else{
			fastx.RdpPort = fastx.port;
		}

		var w,h;
		if(fastx.geometry && fastx.geometry.toLowerCase() != '0x0' ){
			var g = fastx.geometry.split(/x/i);
			w = parseInt(g[0]);
			h = parseInt(g[1]);
		}else{
			w= 1024;
			h= 768;
		}

		console.log('Starting Xrdp');
		var socket;
		var buffer = new ArrayBuffer(0);
		var Client = {
			state:'connecting',
			settings:{
				WebGL:fastx.WebGL,
				windows:fastx.windows,
				suppressOutput:false,
				workerCnt:0,
				messageChannelData:true,

				userName:fastx.username || 'fastx',
				password:fastx.password,
				securityProtocol:(fastx.windows)?0:0x00000001 ,//rdp on windows, TLS security on linux,
				encryptionMethods: 0x00000001,// | 0x00000002 | 0x00000008,//40,128,56 bit security
				width:w,
				height:h,
				supportedColorDepths:0x0001 | 0x0002 | 0x0004 | 0x0008,
				earlyCapabilitiesFlags: 0x0001 | 0x0004 | 0x0020 | 0x0080,
				keyboard:{
					layout:fastx.keyboardLayout || 0x00000409,
					type: fastx.keyboardType || 0x00000004,
					subType:0x00,
					functionKeyNum:12
				},
				channels:['cliprdr','sndesk','snshar2','sncfg'],
				codecIds:{
					'nsCodec':0x01,
					'rfx':0x03,
					'rfxImage':0x04
				}
			},
			store:{
				receivedServerData:{},
				staticVirtualChannelIds:[],
				ioChannelId:null,
				messageChannelId:null,
				userChannelId:null,
				serverChannelId:null,
				serverCapabilities:null,
				shareId:null,
				automaticReconnectionCookie:null,
				serverLicensingEncryptionAbility:null,
				pointerImageCache:[],
				sessionKeys:{
					mac:null,
					encrypt:null,
					decrypt:null},
				//bitmapCaches:[],
				//persistentBitmapCaches:[],
				//persistentBitmapKeys:[],
				connectionStartTime:null,
				networkCharacteristicsByteCount:null,
				networkCharacteristicsSequenceNumber:null

			}
		}

		capabilitySettings.init(Client.settings);
		Offscreen.primary(Offscreen.create(Client.settings.width,Client.settings.height).canvas);
		Screen.canvas = Offscreen.primary().canvas;
		if(fastx.windowMode === 'rootless'){
			Client.settings.channels.push('rail');
			Client.settings.channels.push('snrail');
			Screen.canvas.style.display = 'none';
		}
		var connected = new Connected(fastx);



		function s(e){
			var bytesRead =0,stream,cat,dv;
			if(!e.data){return;}
			stream = new ArrayBuffer(buffer.byteLength + e.data.byteLength);
			cat = new Uint8Array(stream);
			cat.set(new Uint8Array(buffer));
			cat.set(new Uint8Array(e.data),buffer.byteLength);
			buffer = stream;
		//	if(socket.readyState !== 1){return;}
			do{
				dv = new DataView(buffer);
				switch(Client.state){
					case 'connecting':
						try{
							bytesRead = conSeq.processMessage(socket,Client,dv);
						}catch(e){
							console.log(e);
							socket.close(1000,e);
						}
						break;
					case 'connected': //connection finalization and dataPDUs
						try{
							bytesRead = connected.processMessage(socket,Client,dv);
						}catch(e){
							console.log(e);
							if(e.message){
								e.reason = e.message;
							}
							socket.close(1000,e);
						}
						break;
					default:
						bytesRead = 0;
						console.log(Client.state);
				}
				buffer = buffer.slice(bytesRead);
			}while(bytesRead && dv.byteOffset + bytesRead < dv.byteLength);
		}

		function allWorkersLoaded(e){
			if(socket){return;}
			console.log('Connecting to websocket');
			socket = new WebSocket(fastx.ws);
			socket.binaryType = 'arraybuffer';
			socket.onopen = function(){
				console.log('Websocket connection opened');
				parent.postMessage(Object.assign({type:'websocket-opened'}),'*');
				conSeq.init(socket,Client);

			};
			socket.onmessage = socketMessage;
			socket.onclose = function(e){
				console.log('Websocket Closed');
				console.log(e);
				var o = {};
				o.code = e.code;
				if(e.reason){
					o.reason = e.reason?e.reason:"Your connection has closed";
				}
				if(e.code === 1006){
					o.reason = e.reason?e.reason:"Your connection closed abnormally";
				}
				parent.postMessage(Object.assign(o,{type:'websocket-closed'}),'*');
				Screen.canvas.style.opacity = 0.2;
				document.getElementById('xrdp-menu').style.display = 'none';
			}
			//document.removeEventListener('snWorkersLoaded',allWorkersLoaded);
		}

		allWorkersLoaded();



		return;
	}
module.exports =  Client;
