'use strict';

import angular from 'angular';

import "./rail-taskbar";
import "./rail-window";

export default angular.module('fastx.RailDesktop',['fastx.RailTaskbar','fastx.RailWindow'])
.factory('RailDesktop',['$window','$rootScope','RailWindows','RailTaskbar','Wm',function($window,$rootScope,RailWindows,Taskbar,Wm){
	var f = {};
	f.init = function(canvas,cb){
		var active,x1,y1,zorder =[],nextAnim;
		var rootCanvas;
		rootCanvas = canvas;
		cb.sendResizeRequest(window.screen.width,window.screen.height);
		document.dispatchEvent(new CustomEvent('snResize',{
			detail:{
				width:window.screen.width,
				height:window.screen.height
			}
		}));

		angular.element('.xrdp-container').css('display','none');
		angular.element('body.xrdp').addClass('rail');

		var desktop = document.createElement('div');
		desktop.id = 'rail-desktop';
		desktop.className ='rail-desktop';
		desktop.style.width = window.innerWidth + 'px';
		desktop.style.height = (window.innerHeight) + 'px';
		document.querySelector('#appCtrl').appendChild(desktop);
		document.querySelector('#appCtrl').appendChild(Taskbar.init());
		document.addEventListener('snRailActivate',function(e){
			if(!RailWindows[e.detail.id]){return;}
			if(active && active.id === e.detail.id && active.active){
				return;//ignore when its already active
			}
			active = RailWindows[e.detail.id]
			if(active.style.popup){return;}
			active.activate();
			Taskbar.update(active);
		});
		document.addEventListener('snRailActivateNext',function(e){
			var z = zorder;
			var i;
			while(z.length){
				i = z.pop();
				if(i === e.detail.id){continue;}
				if(RailWindows[i] && !RailWindows[i].hidden){
					document.dispatchEvent(new CustomEvent('snRailActivate',{detail:{id:i}}));
					return;
				}
			}
			angular.element('.rail-taskbar-icon').removeClass('active');
		});
		document.addEventListener('snRailUpdateWindow',function(e){
			if(e.detail.wnd.windowHeight === 1 && e.detail.wnd.windowWidth === 1){return;}
			if(!Object.keys(e.detail.wnd.style).length){return;}
			if(!RailWindows[e.detail.wnd.windowId]){
				RailWindows[e.detail.wnd.windowId] = new Wm(e.detail.wnd,rootCanvas,desktop);
				Taskbar.addIcon(RailWindows[e.detail.wnd.windowId]);
			}else{
				RailWindows[e.detail.wnd.windowId].update(e.detail.wnd);
				Taskbar.update(RailWindows[e.detail.wnd.windowId]);
			}
		});
		document.addEventListener('snRailUpdateWindows',function(e){
			var i;
			for(i in RailWindows){
				RailWindows[i].updateCanvas();
			}
		});
		document.addEventListener('snRailDeleteWindow',function(e){
			if(RailWindows[e.detail.id]){
				RailWindows[e.detail.id].remove();
				Taskbar.deleteIcon(e.detail.id);
				delete RailWindows[e.detail.id];
			}
		});
		document.addEventListener('snRailZOrder',function(e){
			var arr = e.detail.ids;
			for(var i=0;i<arr.length;i++){
				if(RailWindows[arr[i]]){
					RailWindows[arr[i]].container.style.zIndex = i;
				}
			}
			zorder = arr;
			document.dispatchEvent(new CustomEvent('snRailActivate',{detail:{id:arr.pop()}}));
		});
		document.addEventListener('snRailMoveWindow',function(e){
			active = RailWindows[e.detail.id];
			active.moving = true;
		});
		document.addEventListener('snRailResizeWindow',function(e){
			active = RailWindows[e.detail.id];
			active.resizing = e.detail.resize;
			if(!active.active){
				active.activate();
			}
		});
		desktop.addEventListener('contextmenu',function(e){
			e.preventDefault();
			e.stopPropagation();
		},true);
		desktop.addEventListener('dblclick',function(e){
			e.preventDefault();
		},true);

		desktop.addEventListener('wheel',function(e){
			cb.mouseWheelEvent(e);
		});
		desktop.addEventListener('mousedown',function(e){
			e.preventDefault();
			$rootScope.cb.sendModifiersDown(e);
			$rootScope.cb.sendMouseDown(e.which,[e.clientX,e.clientY]);
			x1 = e.clientX;
			y1 = e.clientY;
			if(angular.element(e.target).hasClass('rail-desktop')){//end mouse
				e.stopImmediatePropagation();
			}
		});

		desktop.addEventListener('mousemove',function(e){
			if(!$rootScope.cb){return;}
			if(active && (active.moving || active.resizing)){return;}
			$rootScope.cb.sendMouseMove([e.clientX,e.clientY]);
		});
		desktop.addEventListener('mouseup',function(e){
			$rootScope.cb.sendMouseUp(e.which,[e.clientX,e.clientY]);
		});


		desktop.addEventListener('mousemove',function(e){
			e.preventDefault();
			if(active){
				if(active.o.showState !== 5){
					return;
				}
				if(active.moving){
					active.move(active.x + e.clientX - x1,active.y  + e.clientY - y1);
					x1 = e.clientX;
					y1 = e.clientY;
				}else if(active.resizing){active.resize(e);}
			}
		});
		desktop.addEventListener('mouseup',function(e){
			e.preventDefault();
			e.stopPropagation();
			if(active){
				delete active.resizing;
				active.moving = false;
				active.sendMoveRequest();
			}
		});
	}
	return f;
}]);
