'use strict';
var titleHeight = 24,cornerSize = 5;

import angular from 'angular';

export default angular.module('fastx.RailWindow',[])
.factory('Wm',['$rootScope','WindowResize','WindowTitle', function($rootScope,WindowResize,WindowTitle){
	function initCanvas(wnd){
		var canvas = document.createElement('canvas');
		canvas.style.position = 'absolute';
		canvas.style.top = (titleHeight) + 'px';
		//canvas.style.left =  '0px';
		/*
		canvas.addEventListener('mousedown',function(e){
			$rootScope.cb.sendMouseDown(e.which,[e.clientX,e.clientY]);
			$rootScope.cb.rail.activate(id);
		});
		canvas.addEventListener('mousemove',function(e){
			$rootScope.cb.sendMouseMove([e.clientX,e.clientY]);
		});
		canvas.addEventListener('mouseup',function(e){

			$rootScope.cb.sendMouseUp(e.which,[e.clientX,e.clientY]);
		});
		*/
		return canvas;
	}
	function initContainer(wnd){
		var c = document.createElement('div');
		c.className = 'rail-window';
		c.addEventListener('mousedown',function(e){
			document.dispatchEvent(new CustomEvent('snRailActivate',{
					detail:{
						id:wnd.id
					}
				})
			);
		});
		return c;
	}

	function Wnd(o,canvas,parent){
		this.rootCanvas = canvas;
		this.id = o.windowId;
		this.container = initContainer(this);
		this.canvas = initCanvas(this);
		this.container.appendChild(this.canvas);
		parent.appendChild(this.container);
		//this.container.title = o.windowId;
		this.update(o);

		var that = this;
		Object.defineProperties(this,{
			'hidden':{
				get:function(){
					return angular.element(that.container).css('display') === 'none';
				}
			},
			'active':{
				get:function(){
					return  !that.hidden && angular.element(that.container).hasClass('active');
				}
			},
			'moving':{
				get:function(){
					return angular.element(that.container).hasClass('moving')
				},
				set:function(m){
					angular.element(that.container).toggleClass('moving',m);
				}
			},
			'title':{
				get:function(){
					var t = that.container.querySelector('.window-title');
					return t?t.innerHTML:'';
				}
			}
		});

	}
	Wnd.prototype.remove = function(){
		delete this;
		angular.element(this.container).remove();;
	}
	Wnd.prototype.updateCanvas = function(){
		if(this.moving){return;}//dont update when moving windows around.  It looks weird
		var ctx = this.canvas.getContext('2d');
		this.canvas.width = this.w;
		this.canvas.height =this.h;

		var rect = this.canvas.getBoundingClientRect();
		ctx.drawImage(this.rootCanvas,rect.left,rect.top,
			this.w,this.h,0,0,this.w,this.h);
	}
	Wnd.prototype.update = function(o){
		if(o.style){
			this.style = o.style;
		}
		if(o.ownerWindowId){
			this.parent = o.ownerWindowId;
		}
		this.o = o;
		this.x = o.windowOffsetX;
		this.y = o.windowOffsetY;
		this.w = o.windowWidth;
		this.h = o.windowHeight;
		this.r = this.x + this.w;
		this.b = this.y + this.h;

		if(o.windowOffsetY < titleHeight){
			this.rect(this.x,titleHeight,this.r,titleHeight + this.h);
			this.sendResizeRequest();
		}

		switch(o.showState){
			case 0x00://do not show
				break;
			case 0x02://minimized
				this.minimize();
				break;
			case 0x03://maximized

				break;
			case 0x05://show in current size and position
				break;
			default:
				break;
		}

		if(o.icon){
			//update Icon
			console.log(o);
			Icon.process(o.icon);
			return;
		}
		this.container.style.width = (o.windowWidth) + 'px';
		this.container.style.height= (o.windowHeight + titleHeight)  + 'px';
		this.container.style.left =  (o.windowOffsetX ) + 'px';
		this.container.style.top  =  (o.windowOffsetY - titleHeight ) + 'px';

		if(o.windowOffsetX < 0){
			this.move(0,o.windowOffsetY - titleHeight);
		}
		if(o.windowOffsetY < titleHeight){
			this.move(o.windowOffsetX,titleHeight);
		}
		WindowTitle.update(this,o);
		WindowResize.update(this,o);
	}

	Wnd.prototype.move = function(x,y){
		if(x <= 0){x=0;}
		if(y <= titleHeight){y=titleHeight}

		this.x = x;
		this.y = y;
		this.container.style.left = (this.x) + 'px';
		this.container.style.top  =  (this.y - titleHeight) + 'px';

	}
	Wnd.prototype.sendMoveRequest = function(){
		$rootScope.cb.rail.windowMove(this.id,[this.x,this.y,this.x+ this.w,this.y + this.h]);
	}
	Wnd.prototype.sendResizeRequest = function(){
		$rootScope.cb.rail.windowResize(this.id,[this.x,this.y,this.r,this.b]);
	}
	Wnd.prototype.rect = function(left,top,right,bottom){
		this.old = {
			x:this.x,
			y:this.y,
			r:this.r,
			b:this.b
		};
		if(right - left < 60){
			right = left +60;
		}
		if(bottom - top < 24){
			bottom = top +24;
		}
		this.x = left;
		this.y = top;
		this.r = right;
		this.b = bottom;
		this.w = right - left;
		this.h = bottom - top;
		this.container.style.width =( this.w ) + 'px';
		this.container.style.height = (this.h +  titleHeight ) + 'px';
		this.container.style.left = (this.x) + 'px';
		this.container.style.top  =  (this.y - titleHeight) + 'px';

	}

	Wnd.prototype.minimize = function(){
		angular.element(this.container).removeClass('active');
		$rootScope.cb.rail.deactivate(this.id);
		$rootScope.cb.rail.minimizeWindow(this.id);
		this.container.style.display = 'none';
		document.dispatchEvent(new CustomEvent('snRailActivateNext',{detail:{id:this.id}}));
	}
	Wnd.prototype.show = function(){
		this.container.style.display = 'block';
		$rootScope.cb.rail.restoreWindow(this.id);
		$rootScope.cb.rail.activate(this.id);
		//	$rootScope.cb.snrail.raise(this.id);
	}
	Wnd.prototype.activate = function(send){
		if(this.parent){return;}
		angular.element('.active').removeClass('active');
		angular.element(this.container).addClass('active');
		//$rootScope.cb.snrail.raise(this.id);
		$rootScope.cb.rail.activate(this.id);
	}
	Wnd.prototype.resizeN = function(y){
		if(y < this.y + this.h){
			this.rect(this.x,y,this.x+ this.w,this.y + this.h );
		}
	}
	Wnd.prototype.resizeS = function(y){
		if(y > this.y){
			this.rect(this.x,this.y,this.r,y);
		}
	}
	Wnd.prototype.resizeE = function(x){
		if(x > this.x){
			this.rect(this.x,this.y,x,this.b);
		}
	}
	Wnd.prototype.resizeW = function(x){
		if(x < this.x + this.w){
			this.rect(x,this.y,this.r ,this.b);
		}
	}
	Wnd.prototype.resize = function(e){
		switch(this.resizing){
			case 'n':
				this.resizeN(e.clientY);
				break;
			case 's':
				this.resizeS(e.clientY);
				break;
			case 'e':
				this.resizeE(e.clientX);
				break;
			case 'w':
				this.resizeW(e.clientX);
				break;
			case 'ne':
				this.resizeN(e.clientY);
				this.resizeE(e.clientX);
				break;
			case 'se':
				this.resizeS(e.clientY);
				this.resizeE(e.clientX);
				break;
			case 'nw':
				this.resizeN(e.clientY);
				this.resizeW(e.clientX);
				break;
			case 'sw':
				this.resizeS(e.clientY);
				this.resizeW(e.clientX);
				break;
		}
	}

	return Wnd;
}])
.factory('WindowTitle',['$rootScope',function($rootScope){

	function initCloseButton(wnd){
		var i = document.createElement('i');
		i.className = 'icon ion-android-cancel rail-close-button rail-title-button';
		i.style.cursor = 'pointer';
		i.addEventListener('click',function(e){
			$rootScope.cb.rail.closeWindow(wnd.id);
			wnd.container.remove();
		});
		return i;
	}
	function initMinButton(wnd){
		var i = document.createElement('i');
		i.className = 'icon ion-android-remove-circle minimize-button rail-title-button';
		i.style.cursor = 'pointer';
		i.addEventListener('click',function(e){
			wnd.minimize();
		});
		return i
	}

	function initTitlebar(wnd){
		var t = document.createElement('div');
		t.className = 'rail-titlebar';
		t.style.height = titleHeight + 'px';
		t.addEventListener('mousedown',function(e){
			if(!wnd.active){
				wnd.activate(true);
				return;
			}
			document.dispatchEvent(new CustomEvent('snRailMoveWindow',{
					detail:{
						id:wnd.id
					}
				})
			);
		});
		return t;
	}
	function initTitle(wnd){
		var i = document.createElement('span');
		i.className ='window-title';
		i.innerHTML = wnd.title;
		return i;
	}
	function titleBar(wnd,o){
		function max(){
			if(!wnd.active){
				wnd.activate();
				return;
			}
			if(!wnd.maximized){
				wnd.rect(0,titleHeight,window.innerWidth,window.innerHeight - titleHeight);
				wnd.sendResizeRequest();
				$rootScope.cb.rail.maximizeWindow(wnd.id);
				wnd.maximized = true;
			}else {
				wnd.rect(wnd.old.x,wnd.old.y,wnd.old.r,wnd.old.b);
				wnd.sendResizeRequest();
				$rootScope.cb.rail.restoreWindow(wnd.id);
				wnd.maximized = false;
			}
		}
		function initMaxButton(wnd){
			var i = document.createElement('i');
			i.className = 'icon ion-android-add-circle maximize-button rail-title-button';
			i.style.cursor = 'pointer';
			i.addEventListener('click',max);
			return i;
		}

		var t = initTitlebar(wnd);
		t.addEventListener('dblclick',max);
		t.appendChild(initCloseButton(wnd));
		if(o.style.minimizeBox){
			t.appendChild(initMinButton(wnd));
		}
		if(o.style.maximizeBox){
			t.appendChild(initMaxButton(wnd));
		}
		t.appendChild(initTitle(wnd));
		return t;
	}

	function update(wnd,o){
		if(o.style.caption){
			var t = titleBar(wnd,o);
			var tb  = wnd.container.querySelector('.window-title');
			if(tb){
				tb = t;
			}else{
				wnd.container.appendChild(t);
			}
		}
		updateTitle(wnd,o);
	}

	function updateTitle(wnd,o){
		var tbar = wnd.container.querySelector('.window-title')
		if(tbar && o.title){
			wnd.container.querySelector('.window-title').innerHTML = o.title;
		}
	}

	return {
		update:update
	};
}])
.factory('WindowResize',[function(){
	function corner(id,cursor,w,h,resize){
		var c = document.createElement('div');
		c.style.cursor =  cursor;
		c.style.position = 'absolute';
		c.style.width = w + 'px';
		c.style.height= h + 'px';
		c.className = 'rail-resize';
		//c.style.background ='#00AA00';
		c.style.visibility = 'invisible';
		c.addEventListener('mousedown',function(e){
			document.dispatchEvent(new CustomEvent('snRailResizeWindow',{
					detail:{
						id:id,
						resize:resize
					}
				})
			);
		})
		return c;
	}

	function nw(wnd){
		var c = corner(wnd.id,'nwse-resize',cornerSize,titleHeight,'nw');
		c.style.top = '0px';
		c.style.left = '0px';
		c.style.zIndex = 2;
		return c;
	}
	function ne(wnd){
		var c = corner(wnd.id,'nesw-resize',cornerSize,titleHeight,'ne');
		c.style.top = '0px';
		c.style.right = '0px';
		c.style.zIndex = 2;
		return c;
	}
	function sw(wnd){
		var c = corner(wnd.id,'nesw-resize',cornerSize,cornerSize,'sw');
		c.style.bottom = '0px';
		c.style.left = '0px';
		c.style.zIndex = 2;
		return c;
	}
	function se(wnd){
		var c = corner(wnd.id,'nwse-resize',cornerSize,cornerSize,'se');
		c.style.bottom = '0px';
		c.style.right = '0px';
		c.style.zIndex = 2;
		return c;
	}
	function hbar(wnd,cursor,w,h,resize){
		var c = document.createElement('div');
		c.style.cursor =  cursor;
		c.style.position = 'absolute';
		c.style.height = w + 'px';
		c.style.width= '100%';
		c.className = 'rail-resize';
		c.style.visibility = 'invisible';
		c.addEventListener('mousedown',function(e){
			document.dispatchEvent(new CustomEvent('snRailResizeWindow',{
					detail:{
						id:wnd.id,
						resize:resize
					}
				})
			);
		});
		c.style.zIndex = 1;
		return c;
	}
	function vbar(wnd,cursor,w,h,resize){
		var c = document.createElement('div');
		c.style.cursor =  cursor;
		c.style.position = 'absolute';
		c.style.width = w + 'px';
		c.style.height= '100%';
		c.className = 'rail-resize';
		c.style.visibility = 'invisible';
		c.addEventListener('mousedown',function(e){
			document.dispatchEvent(new CustomEvent('snRailResizeWindow',{
					detail:{
						id:wnd.id,
						resize:resize
					}
				})
			);
		});
		c.style.zIndex = 1;
		return c;
	}
	function n(wnd){
		var c = hbar(wnd,'ns-resize',cornerSize,cornerSize,'n');
		c.style.top = '0px';
		return c;
	}
	function s(wnd){
		var c = hbar(wnd,'ns-resize',cornerSize,cornerSize,'s');
		c.style.bottom = '0px';
		c.style.borderBottom = '1px solid black';
		return c;
	}
	function e(wnd){
		var c = vbar(wnd,'ew-resize',cornerSize,titleHeight - cornerSize,'e');
		c.style.right = '0px';
		c.style.borderRight = '1px solid black';
		return c;
	}
	function w(wnd){
		var c = vbar(wnd,'ew-resize',cornerSize,titleHeight - cornerSize,'w');
		c.style.left = '0px';
		c.style.borderLeft = '1px solid black';
		return c;
	}

	function initResize(wnd,o){
		if(o.style.border){
			wnd.container.appendChild(n(wnd));
			wnd.container.appendChild(s(wnd));
			wnd.container.appendChild(e(wnd));
			wnd.container.appendChild(w(wnd));
			wnd.container.appendChild(ne(wnd));
			wnd.container.appendChild(nw(wnd));
			wnd.container.appendChild(se(wnd));
			wnd.container.appendChild(sw(wnd));
			wnd.container.style.overflow = 'hidden';
			wnd.container.style.borderTop = 'none';
		}else{
			wnd.container.style.border = 'none';
		}
	}
	return {
		update:initResize
	};
}]);
