'use strict'

import angular from 'angular';

import "./rail-desktop";
import "./rail-window";


export default angular.module('fastx.WindowManager',['fastx.RailWindow','fastx.RailDesktop'])

.factory('RailWindows',[function(){
	var windows = {};
	return windows;

}])
.factory('Rail',['$rootScope','$window','WindowOrder','RailDesktop',function($rootScope,$window,WindowOrder,Desktop){
	//RAIL stuff

	function init(canvas,cb){
		Desktop.init(canvas,cb);
		canvas.addEventListener('snWindowOrder',function(e){
			WindowOrder.process(e.detail);
		});
		cb.sendResizeRequest($window.screen.availWidth,$window.screen.availHeight);

	}
	return {
		init:init
	}
}])
.factory('WindowOrder',['Wm','RailDesktop','$rootScope',function(Wm,Desktop,$rootScope){
	function processWindow(o){
		if(o.deleted){
			document.dispatchEvent(new CustomEvent('snRailDeleteWindow',{detail:{id:o.windowId}}));
		}else if(o.icon){

		}else{
			document.dispatchEvent(new CustomEvent('snRailUpdateWindow',{detail:{wnd:o}}));
		}
	}
	function processDesktop(o){
		if(o.zOrderWindowIds){
			document.dispatchEvent(new CustomEvent('snRailZOrder',{detail:{ids:o.zOrderWindowIds}}));
		}
		if(o.activeWindowId){
			//document.dispatchEvent(new CustomEvent('snRailActivate',{detail:{id:o.activeWindowId}}));
		}
	}
	function processNotify(o){
		console.log('notify',o);
	}

	function process(o){
		//console.log(o);
		switch(o.type){
			case 'window':
				processWindow(o);
				break;
			case 'desktop':
				processDesktop(o);
				break;
			case 'notify':
				processNotify(o);
				break;
			default:
				console.log('Unknown windoworder',o);
		}
	}

	return {
		process:process,
	}
}])

.factory('Icon',[function(){
	var cache = {};

	var createIconData = function(o){
		var imdata = new Uint8ClampedArray(4*o.width*o.height);
		var h=0,w=0;
		for (h=0;h<o.height;h++){
			for(w=0;w < 4*o.width;w=w+4){
				imdata[4*h*o.width + w] 	= o.bitsColor[(4*(o.height - h - 1)*o.width + w)];
				imdata[4*h*o.width + w + 1] = o.bitsColor[(4*(o.height - h - 1)*o.width + w + 1)];
				imdata[4*h*o.width + w + 2] = o.bitsColor[(4*(o.height - h - 1)*o.width + w + 2)];
				imdata[4*h*o.width + w + 3] = o.bitsColor[(4*(o.height - h - 1)*o.width + w + 3)];;
			}
		}
		return imdata;
	}
	var createIcon = function(o){
		var canvas = document.createElement('canvas');
		canvas.height = o.height;
		canvas.width = o.width;
		var ctx = canvas.getContext('2d');
		var imdata = ctx.createImageData(o.width,o.height);
		imdata.data.set(createIconData(o));
		ctx.putImageData(imdata,0,0);
		ctx.drawImage(canvas,0,0,o.width,o.height);
		return canvas;
	}

	function process(o){
		var icon = createIcon(o);
		icon.style.position = 'fixed';
		icon.style.top = '0px';
		icon.style.left = '1px';
		document.body.appendChild(icon);

	}
	return {
		process:process
	}
}]);
