var express = require('express');
var path = require('path');
var favicon = require('serve-favicon');
var logger = require('morgan');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');
var lessMiddleware = require('less-middleware');
var os = require('os');
var app = express();
var ECT = require('ect');
var mkdirp = require('mkdirp');

var Theme 		= require('./common/lib/config/theme');
var routes 		= require('./gw/routes/index');
var api 		= require('./gw/routes/api');
var system 		= require('./gw/routes/system');
var server 		= require('./gw/routes/server');
var local 		= require('./local/routes/local');
var Auth 		= require('./gw/lib/auth/auth');


var ectRenderer = ECT({ watch: true, root: __dirname + '/views', ext : '.ect' });
app.set('view engine', 'ect');
app.engine('ect', ectRenderer.render);

delete process.env.SHELL;

// view engine setup
app.set('views', path.join(__dirname, 'views'));
//app.set('view engine', 'pug');
app.set('x-powered-by',false);

app.use(favicon(path.resolve(global.FXROOTDIR,'www/client/img/fastx_32px.ico')));



if(process.env.FXLOG){
	app.use(logger('dev'));
}
try{
	mkdirp.sync(path.resolve(global.FXROOTDIR,'var','config'),{
		mode:'0755'
	});

	mkdirp.sync(path.resolve(global.FXROOTDIR,'var','license'),{
		mode:'0755'
	});

	mkdirp.sync(path.resolve(global.FXROOTDIR,'var','uploads'),{
		mode:'0755'
	});

	mkdirp.sync(path.resolve(global.FXROOTDIR,'var','tmp'),{
		mode:'0755'
	});
}catch(e){
	console.log("There was an error attempting to create the required directories for fastx");
	console.log(e.message);
	console.log("The web server will now exit");
	process.exit(1);
}
app.use(bodyParser.json({limit:'5mb'}));
app.use(bodyParser.urlencoded({extended:true}));
app.use(cookieParser());
app.use(lessMiddleware(path.resolve(global.FXROOTDIR, 'www/client'),{
	preprocess:{
		less: function(src,req){
			var theme = Theme.data;
			var str ='';
			for (var key in theme){
					str += '@' + key + ':' + theme[key] + ';';
			}
			return str + src;
		}
	},
	//debug:true,
	dest:path.resolve(global.FXROOTDIR,'var','css')
}));

app.use(express.static(path.resolve(global.FXROOTDIR,'var','css')));
app.use(express.static(path.resolve(global.FXROOTDIR, 'www/client')));
app.use('/', routes);
app.use('/api', api);
app.use('/system',system);

Auth(app);
//app.use('/gw', gw);
//app.use('/broadcast', broadcast);
app.use('/local', local);



/// catch 404 and forward to error handler
app.use(function(req, res, next) {
    res.status(404);
	res.render('error/404');
});

/// error handlers

// development error handler
// will print stacktrace
if (process.env.FXDEBUG) {
	console.log('Debug Mode');
    app.use(function(err, req, res, next) {
        res.status(err.status || 500);
        res.render('error/error', {
            message: err.message,
            error: err
        });
    });
}

// production error handler
// no stacktraces leaked to user
app.use('/system',function(err, req, res, next){
	var msg ='';
	switch (res.statusCode){
		case 404:
			msg = 'Not Found';
			break;
		default:
			console.log(err.stack);
			msg = 'Internal Server Error';
	}

	res.json({
		error:{
			code:res.statusCode || 500,
			message:msg
		}
	});
});
app.use(['/api/*'],api.errorHandler);
//app.use(['/gw/server/:serverPool/:serverId/:method/*'],gw.errorHandler);
//app.use(['/broadcast/*'],broadcast.errorHandler);




app.use(function(err, req, res, next) {
	if(res.statusCode === 404){
		res.render('error/404');
		return;
	}else{
		res.status(500);
		res.render('error/500', {
			message: err.message,
			error: err,
		});
	}
});
module.exports = app;
