var path = require('path');
var fs = require('fs');

var Cache = require('./config-cache');

function certFileLocation(file,o){
	var paths = [
		path.resolve(global.FXROOTDIR,'var','certs',file),
		path.resolve(global.FXROOTDIR,'www',file),// v2.3 location
		path.resolve(global.FXROOTDIR,'www','server',file),
		path.resolve(global.FXROOTDIR,'www','server','certs',file),
		path.resolve(global.FXROOTDIR,file)
	];
	var p;
	while(p = paths.shift()){
		try{
            console.log('Trying',p);
			return fs.readFileSync(p,o);
		}catch(e){
                    if(e.code!=='ENOENT'){
                        console.log(e.message);
                    }
                    if(!paths.length){
                        console.log('Could not get a valid certificate. Exiting...');
                        process.exit(1);
                    }
		}
	}
}


var www = new Cache('www.json',{
		defaults:{port:3000},
		mode:'0644'
	});

www.hasHttpsOptions = function(d){
	return ( (d.cert_file && d.key_file) || d.pfx_file);
}

www.parseHttpsOptions = function(d){
	var o = d.https || {};
		try{
			if(d.cert_file){
				o.cert = certFileLocation(d.cert_file);
			}
			if(d.key_file){
				o.key = certFileLocation(d.key_file);
			}
			if(d.ca_file){
				//o.ca = [fs.readFileSync(this.config.ca_file)];
				o.ca = [];
				var lines = certFileLocation(d.ca_file,{encoding:'utf8'});
				lines = lines.split('\n');
				var cert = [];
				for(var i = 0; i < lines.length;i++){
					cert.push(lines[i]);
					if(lines[i].match(/-END CERTIFICATE-/)){
						o.ca.push(cert.join('\n'));
						cert = [];
					}
				}
			}
			if(d.pfx_file){
				o.pfx = certFileLocation(d.pfx_file);
			}
			if(d.secureProtocol){
				o.secureProtocol = d.secureProtocol;
			}
			if(d.ciphers){
				o.ciphers = d.ciphers;
			}
		}catch(e){
			console.log(e.message);
			console.log('Exiting...\n');
			process.exit(1);
		}
		return o;
}
module.exports = www;
