var path 	= require('path');
var fs 		= require('fs');

var command = require('../config/command');
var www 	= require('../config/www');
var settings = require('../config/settings');
var network = require('../config/network');
var admin 	= require('../config/admin');

var PATH 	= process.env.FX_CONFIG_PATH || path.resolve(global.FXROOTDIR,'/config/');

function convertSettings(settings,www){
	if(www.data.config){
		if(www.data.port || www.data.httpsport){
			return;//already converted
		}
		try{
			fs.writeFileSync(path.resolve(PATH,'settings.json.bak'),JSON.stringify(settings.data),{mode:parseInt(settings.o.mode || '0600',8)});
			var data = settings.data;
			data.sshport = www.data.config.sshport;
			data.keepAliveInterval = www.data.config.keepAliveInterval;
			data.recordLoginAttempts =  www.data.config.recordLoginAttempts;
			settings.data = data;
			console.log('Migrated options in www.json to settings.json');
		}catch(e){
			if(e.code !== 'ENOENT'){
				console.log('Error Migrating settings.json');
				console.log(e.message);
			}
		}
	}
}
function convertWWW(www){
	try{
		var twww = JSON.parse(fs.readFileSync(path.resolve(PATH,'www.json')));
		if(twww.www && twww.www.config){
			if(twww.port || twww.httpsport){
				return;//already converted
			}
			try{
				fs.writeFileSync(path.resolve(PATH,'www.json.bak'),JSON.stringify(twww),{mode:parseInt(www.o.mode || '0644',8)});
				var data = twww.www.config;
				data.www ={config:JSON.parse(JSON.stringify(twww.www.config))}; //backwards compat with 2.1 monitors 
				delete data.sshport;
				delete data.keepAliveInterval;
				delete data.recordLoginAttempts;
				www.data = data;
				console.log('Converted www.json to new format');
			}catch(e){
				if(e.code !== 'ENOENT'){
					console.log('Error Converting www.json');
					console.log(e.message);
				}else{
					//create a default www if it doesnt exist
					fs.writeFileSync(path.resolve(PATH,'www.json'),JSON.stringify({port:3000}),{mode:parseInt('0644',8)});
				}
			}
		}
	}catch(e){
		
	}
	
}

function convertNetwork(network){
	try{
		fs.readFileSync(path.resolve(PATH,'network.json'));
	}catch(e){
		convertSessionServer(network);
		convertGatewayServer(network);
	}
}
function convertSessionServer(network){
	var data = network.data;
	try{
		var ss = JSON.parse(fs.readFileSync(path.resolve(PATH,'session-server.json'),{encoding:'utf8'}));
		if(ss.serverKey){
			if(!data.keys){
				data.keys = {};
			}
			data.keys[ss.serverKey] = {
				urls:[{
					url:ss.gatewayUrl
				}]
			};
		}
		fs.renameSync(
			path.resolve(PATH,'session-server.json'),
			path.resolve(PATH,'session-server.json.bak')
		);
		console.log('Migrated options in session-server.json to network.json');
	}catch(e){
		if(e.code !== 'ENOENT'){
			console.log('Error Migrating session-server.json');
			console.log(e.message);
		}
	}
	network.data = data;
}
function convertGatewayServer(network){
	var data = network.data;
	try{
		var gw = JSON.parse(
			fs.readFileSync(
				path.resolve(PATH,'gateway-server.json'),{encoding:'utf8'}
			));
		
		if(!data.keys){
			data.keys = {};
		}
		if(gw.loadBalance){
			data.loadBalance = gw.loadBalance;
		}
		if(gw.rejectUnauthorized){
			data.rejectUnauthorized = gw.rejectUnauthorized;
		}
		if(gw.sessionServerCN){
			data.serverCN = gw.sessionServerCN;
		}
		var pool;
		while(pool = gw.serverPools.shift()){
			if(pool.serverKey){
				if(!data.keys[pool.serverKey]){
					data.keys[pool.serverKey] = {
						urls:[]
					};
				}
				data.keys[pool.serverKey].allowIncomingConnections = true;
			}
		}
		//only convert broadcast hosts. RRDNS must be converted manually (add it to the broadcast host list)
		if(gw.broadcastKey){
			if(!data.keys[gw.broadcastKey]){
				data.keys[gw.broadcastKey] = {
					urls:[]
				};
			}
			data.keys[gw.broadcastKey].allowIncomingConnections = true;
			if(gw.broadcastHosts){
				var host;
				while(host = gw.broadcastHosts.shift()){
					data.keys[gw.broadcastKey].urls.push(
						{url:[(www.data.httpsport?'https://':'http://'),host,':',(www.data.httpsport?www.data.httpsport:www.data.port)].join('')}
					);
				}
			}
		}
		fs.renameSync(
			path.resolve(PATH,'gateway-server.json'),
			path.resolve(PATH,'gateway-server.json.bak')
		);
		console.log('Migrated options in gateway-server.json to network.json');
	}catch(e){
		if(e.code !== 'ENOENT'){
			console.log('Error Migrating gateway-server.json');
			console.log(e.message);
		}
	}
	network.data = data;
}

/*** Conversion for command ***/
function convertCommand(command){
	try{
		fs.readFileSync(path.resolve(PATH,'command.json'));
	}catch(e){
		convertAllow(command);
		convertDeny(command);
	}
	
}
function convertAllow(){
	try{
		var allow = fs.readFileSync(path.resolve(PATH,'command.allow'),{encoding:'utf8'}).split('\n');
		var a,ar = [];
		var data = command.data;
		if(allow.length){
			while(allow.length ){
				a = allow.shift();
				if(!a.length || a.match(/^\s*$/)){//ignore blanks
					continue;
				}else if(a[0] === '#'){ //ignore comments
					continue;
				}else if(a === '<SUGGESTIONS>'){
					data.allowSuggestions = true;
				}else{
					if(a[0] !== '^'){
						a = '^' + a;
					}
					if(a.slice(-1) !== '$'){
						a = a + '$';
					}
					ar.push(a);
				}
			}
			data.allow = ar;
			fs.renameSync(
				path.resolve(PATH,'command.allow'),
				path.resolve(PATH,'command.allow.bak')
			);
			command.data = data;
		}
		console.log('Migrated options in command.allow to command.json');
	}catch(e){
		if(e.code !== 'ENOENT'){
				console.log('Error Migrating command.allow');
				console.log(e.message);
			}
	}
}
function convertDeny(command){
	try{
		var deny = fs.readFileSync(path.resolve(PATH,'command.deny'),{encoding:'utf8'}).split('\n');
		var d,de = [];
		if(deny.length){
			while(deny.length){
				d = deny.shift()
				if(!d.length || d.match(/^\s*$/)){//ignore blanks
					continue;
				}else if(d[0] === '#'){
					continue;
				}
				
				if(d[0] !== '^'){
					d = '^' + d;
				}
				if(d.slice(-1) !== '$'){
					d = d + '$';
				}
				de.push(d);
			}
			var data = command.data;
			data.deny = de;
			fs.renameSync(
				path.resolve(PATH,'command.deny'),
				path.resolve(PATH,'command.deny.bak')
			);
			command.data  = data;
		}
		console.log('Migrated options in command.deny to command.json');
	}catch(e){
		if(e.code !== 'ENOENT'){
			console.log('Error Migrating command.deny');
			console.log(e.message);
		}
	}
}
function convertAdmins(){
	if(admin.data.admins){
		var data = admin.data;
		data.users = JSON.parse(JSON.stringify(admin.data.admins));
		delete data.admins;
		admin.data = data;
	}
}

function migrate(){
	convertSettings(settings,www);
	convertWWW(www);
	convertCommand(command);
	convertNetwork(network);
	convertAdmins();
}
module.exports = migrate;





