var os = require('os');

var Monitor 	= require('./monitor/session-queue').SessionQueue;
var localPool 	= require('./local-pool'); 
var www 		= require('./config/www');
var Settings 	= require('./config/settings');
var Network 	= require('./config/network');
var metrics 	= require('./metrics');
const systemLog = require('./monitor/system-log');
var bc 			= _require('./gw/lib/broadcast').Broadcast;


function cpuAverage(){
	var cpus = os.cpus();
	var tot =0,idle =0;
	for(var cpu =0; cpu < cpus.length; cpu++){
		for(var i in cpus[cpu].times){
			tot+= cpus[cpu].times[i];
		}
		idle += cpus[cpu].times['idle'];
	}
	return {
			idle:idle/cpus.length,
			total:tot/cpus.length
	}
}

function percentageCPU(start){
	var tick  = cpuAverage();
	var diff = {
		idle:tick.idle - start.idle,
		total:tick.total - start.total
	}
	start = tick;//get new start
	return 100 - (100 * diff.idle / diff.total);
}
var start = cpuAverage();//update average

Monitor.prototype.update = function(o){
	if(!o.serverId){
		return;
	}
	this.updateServer(o);
}

Monitor.prototype.status = function(){
	var o = {};
	o.users =  this.getUserTokensByServerId(localPool.serverId);
	o.sessions = this.getSessionIdsByServerId(localPool.serverId);
	o.server =  {
		metrics:metrics(),
		hostname:os.hostname(),
		uptime:os.uptime(),
		loadavg:os.loadavg(),
		totalmem:os.totalmem(),
		freemem:os.freemem(),
		cpu:percentageCPU(start),
		sessions:o.sessions.length,
		loggedInUsers:o.users.length,
		url:[(www.data.httpsport)?'https://':'http://',Settings.data.hostname || os.hostname(),':',www.data.httpsport || www.data.port].join('')
		
		
	};
	return o;
}

Monitor.prototype.getLocalSessions = function(){
	return this.getSessionsByServerId(localPool.serverId);
}
Monitor.prototype.getLocalUsers = function(){
	return this.getUsersByServerId(localPool.serverId);
}
Monitor.prototype.updateLocalSession = function(data){
	return this.updateSession({
		serverId:localPool.serverId,
		serverPool:localPool.serverKey,
		id:data.id,
		data:data
	});
}
var m = new Monitor();
//register the local server

m.broadcastSessions = function(){
	bc.broadcastSessions({data:m.getLocalSessions()});
}
m.broadcastUsers = function(){
	var u = m.getLocalUsers();
	for(var i =0; i<u.length; i++){
		delete u[i].expires; //prevent circular reference
	}
	bc.broadcastUsers({data:u});
}
m.broadcastServer = function(){
	bc.broadcastServer({data:m.status()});
};
(function update(){
	//if(Network.loginEnabled){
		m.updateServer({ 
		serverId:localPool.serverId,
		serverPool:localPool.serverKey,
		data:m.status()
		}); 
		m.broadcastServer();
		m.broadcastSessions();
		m.broadcastUsers();
	//}
	setTimeout(update,30000);
})();
m.getServerUrl = function(serverId){
	return bc.getServerUrl(serverId);
}
 m.refreshServers = function(){
	 bc.refreshServers();
}
m.sendResponse = function(msgId,body,serverId){
	if(!serverId || (serverId === localPool.serverId)){
		this.sendLocalResponse(msgId,body);
	}else{
		bc.sendResponse(msgId,body,serverId);
	}
}
m.getUserData = function(token,serverId,cb){
	var u = m.getUserByToken(token);
	if(u){
		cb(u);
	}else{
		bc.getUserData(token,serverId);
		localPool.ee.once('snUserData:'+token,function(u){
			cb(u.data);
		});
	}
}
m.getSystemLog = systemLog;
m.getBroadcastServers = function(){
	var s,serv = [];
	for(s in bc.servers){
		serv.push({
			url:bc.servers[s].url,
			serverId:bc.servers[s].serverId
		});
	}
	return serv;
}
module.exports = m;
