var ServerPool 	= require('./pool/server-pool').ServerPool;
var Response 	= require('./response').Response;

var ERROR		= _require('common/lib/error-responses');
var Network 	= _require('common/lib/config/network');
var localPool 	= _require('common/lib/local-pool');
var checkSessionServerIdentity = _require('gw/lib/check-session-server-identity');

var Server = new ServerPool();
var response = new Response();

function ServerQueue(){}

Object.defineProperty(ServerQueue.prototype,"Server",{
	get:function(){
		return Server;
	}
});
ServerQueue.prototype.delegateMessages = Server.delegateMessages;


ServerQueue.prototype.updateServer = function(o){
	var server = Server.get(o.serverId);
	Server.update({
		id:o.serverId,
		serverId:o.serverId,
		serverPool:o.serverPool,
		data:o.data});
	Server.expires(o.serverId,90000,function(){
		localPool.ee.emit('snServerDeleted',o.serverId);
		console.log('Server Expired',o.data.server.url,o.serverId);
	});
	if(!server){
		console.log('New Server',o.data.server.url,o.serverId);
	}
	server = Server.get(o.serverId);
	localPool.ee.emit('snServerUpdated',server);
	return server;
}

ServerQueue.prototype.localMessage = function(o,cb){
	var that = this;
	o.serverId = localPool.serverId;
	cb = cb || function(msg){
		that.sendResponse(o.msgId,msg,localPool.serverId);
	}
	localPool.ee.emit('snLocalMessage',{m:o,cb:cb,serverId:localPool.serverId});
}
ServerQueue.prototype.sendLocalResponse = function(msgId,body,serverId){
	response.send(msgId,body);
}
ServerQueue.prototype.queueLocalServer = function(o,cb){
	response.store(o);
	this.localMessage(o,cb);
	return localPool.serverId;
}
ServerQueue.prototype.isLocalServer = function(serverPool){
	return (serverPool === localPool.serverKey);
}
ServerQueue.prototype.isLocalServerId = function(serverId){
	return (serverId === localPool.serverId);
}

ServerQueue.prototype.queueServer = function(serverId,o,cb){
	if(!serverId){return;}
	var server = Server.get(serverId);
	if(!server){ return; }
	response.store(o,serverId);
	if(serverId === localPool.serverId){
		this.localMessage(o);
	}else{
		Server.queueMessage(serverId,o);
	}
	return true;
}

ServerQueue.prototype.isLoadBalanced = function(){
	if(!Network.data.loadBalance){return false;}
	return Network.data.loadBalance.metric.length;
}
ServerQueue.prototype.loadBalanceAtStart = function(){
	return Network.data.loadBalanceAtStart;
}
ServerQueue.prototype.queueLeastLoaded = function(o){
	var sort ='', desc;
	if(typeof Network.data.loadBalance === 'string'){
		sort = Network.data.loadBalance.toLowerCase();;
	}else if(typeof Network.data.loadBalance === 'object'){
		sort = Network.data.loadBalance.metric.toLowerCase();
		desc = Network.data.loadBalance.desc;
	}
	var serverId = Server.getLeastLoadedServer(sort,desc);
	if(!serverId || this.isLocalServerId(serverId)){
		if(Network.loginEnabled){
			this.queueLocalServer(o);
			return localPool.serverId;
		}else{
			return;
		}
	}else{
		this.queueServer(serverId,o);
	}
	return serverId;
}
ServerQueue.prototype.getServerIds = function(){
	return Object.keys(Server.pool);
}
ServerQueue.prototype.getServerNames = function(){
	var names = {};
	for(var i in Server.pool){
		if(!Network.loginEnabled && i === localPool.serverId){
			continue;
		}
		names[i] = Server.pool[i].data.server.url;
	}
	return names;
}
ServerQueue.prototype.getServer = function(id){
	return Server.get(id);
}
ServerQueue.prototype.deleteServer = function(id){
	delete Server.pool[id];
}

ServerQueue.prototype.getServers = function(){
	//TODO need to change the way you get Servers.
	var servers = Server.getServers();
	var s =[];
	for(var i in servers){
		servers[i].data.server.serverId = i;
		servers[i].data.server.id = i;

		if(!Network.loginEnabled && i === localPool.serverId){
			continue;
		}
		s.push(servers[i].data.server);
	}
	return s;
}
module.exports.ServerQueue = ServerQueue;
