var extend = require('extend');


var SessionPool 	= require('./pool/session-pool').SessionPool;
var SessionQueue 	= require('./user-queue').UserQueue;
var LocalMessageQueue = require('./local-message-queue');
var localPool = require('../local-pool');
var Session			= new SessionPool();


function getFilters(filters){
	if(typeof filters ==='string'){
		filters = [filters];
	}
	return (filters.map(function(f){
		return parseFilter(f);
	})).filter(function(f){
		return f;
	});
}
function parseFilter(f){
	var k,v,all;
	all = f.split('==');
	if(all.length != 2){
		return;
	}
	return {
		name:all[0],
		values:all[1].split(',')
	};
}
function evalFilter(session,filter){
	if(session.hasOwnProperty(filter.name)){
		return (filter.values.indexOf(session[filter.name].toString()) >= 0);
			
	}else if(session.params && session.params.hasOwnProperty(filter.name)){
		return (filter.values.indexOf(session.params[filter.name].toString()) >=0);
	}
}

function filterSessions(sessions,filter){
	if(!filter){return sessions;}
	var filters = getFilters(filter);
	return sessions.filter(function(session){
		return filters.every(function(f){
			return evalFilter(session,f);
		});
	});
	
	
}
/************ SESSION **************/
SessionQueue.prototype.queueSession = function(id,o,cb){
	var s = this.getSession(id);
	if(!s){return;}
	if(!this.queueServer(s.serverId,o)){
		return;
	}
	return s.serverId;
}
SessionQueue.prototype.getSessions = function(query){
	return Session.getSessions(query);
}
SessionQueue.prototype.getSessionPool = function(){
	return Session.pool;
}
SessionQueue.prototype.paginateSessions = function(sessions,query){
	query = query || {};
	sessions.sort(function(a,b){
		var a1,b1;
		if(a.hasOwnProperty(query.sortBy)){
			a1 = a[query.sortBy];
			b1 = b[query.sortBy];
		}else if(a.params && a.params.hasOwnProperty(query.sortBy)){
			a1 = a.params[query.sortBy];
			b1 = b.params[query.sortBy];
		}
		if(typeof a1 === 'string'){
			a1 = a1.toLowerCase();
			b1 = b1.toLowerCase();
		}
		if(a1 > b1){return 1;}
		if(a1 < b1){return -1;}
		return 0;
	});
	return filterSessions(query.sortReverse?sessions.reverse():sessions,query.filter).slice(
		parseInt(query.start,10) || 0,
		(parseInt(query.start,10) || 0) + (parseInt(query.limit,10) || sessions.length)
	);
}
SessionQueue.prototype.queueLocalMessage = function(m){
	if(!m.uri){
		m.uri = ['/local','message',m.msgId,m.serverId].join('/');
	}
	delete m.msgId;
	delete m.serverId;
	var id = m.id;
	delete m.id;
	LocalMessageQueue.queueMessage(id,m);
}
SessionQueue.prototype.storeLocalMessageResponse = function(id,res){
	LocalMessageQueue.store(id,res);
}
SessionQueue.prototype.setUserSessions = function(userId,serverId,sessions){
	var s = Session.getSessionsByUserId(userId,serverId);
	//add all the new sessions
	//then delete ones not in the list
	var ids = Object.keys(sessions);//get the ids
	var i;
	for(i in sessions){
		this.updateSession(sessions[i]);
	}
	for(i in s){
		if(ids.indexOf(s[i].data.id) < 0){
			this.deleteSession(s[i].data.id);//merge
		} 
	}
}
SessionQueue.prototype.getSessionsByServerId = function(serverId){
	return Session.getListByServerId(serverId);
}
SessionQueue.prototype.getSessionIdsByServerId = function(serverId){
	return Session.getIdsByServerId(serverId);
}
SessionQueue.prototype.getSession = function(id){
	return Session.getSession(id);
}
SessionQueue.prototype.deleteSession = function(id){
	LocalMessageQueue.cleanup(id);
	var s = Session.getSession(id);
	if(s){
		s.id = id;
		localPool.ee.emit('snSessionDeleted',s);
	}
	delete Session.pool[id];
}
SessionQueue.prototype.deleteServerSessions = function(serverId){
	for(var i in Session.pool){
		if(Session.pool[i].serverId === serverId){
			delete Session.pool[i];
		}
	}
}
SessionQueue.prototype.updateSession = function(data){
	if(data.data && data.data.id){ //put it in the correct format
		data.id = data.data.id;
	}
	var updated = Session.update(data);
	
	Session.expires(data.id,60000);//expire after a minute
	if(updated){
		//send updated call
		localPool.ee.emit('snSessionUpdated',data);
	}
	return updated;
}

SessionQueue.prototype.setServerSessions = function(sessions,serverId){
	//this wont work. need to only set the sessions from the serverId
	var s,i,found;
	var sids= this.getSessionIdsByServerId(serverId);
	while(s = sids.shift()){
		found = false;
		for(i=0;i<sessions.length;i++){//find current on in the new array
			if(sessions[i].data.id === s){
				found = true;
				this.updateSession( (sessions.splice(i,1))[0]);
				break;
			}
		}
		if(!found){//delete ones not in the new array
			this.deleteSession(s);
		}
	}
	for(i=0;i<sessions.length;i++){//add new ones
		this.updateSession(sessions[i]);
	}
}
SessionQueue.prototype.userOwnsSession = function(username,id){
	return Session.userOwnsSession(username,id);
}


module.exports.SessionQueue = SessionQueue;