var Q = require('q');
var path = require('path');
const execFile = require('child_process').execFile;
const exec = require('child_process').exec;

var UserPool 	= require('./pool/user-pool').UserPool;
var UserQueue 	= require('./server-queue').ServerQueue;
var virtualExec = require('./virtual-exec');

var Admin 		= _require('common/lib/config/admin');
var localPool 	= _require('common/lib/local-pool');
var createParamString = _require('common/lib/create-param-string');


var User = new UserPool();


function setExpires(token){
	return setTimeout(function(){delete User.pool[token];},90000);
}
UserQueue.prototype.queueUser = function(token,o,cb){
	var u = User.get(token);
	if(!u){return;}
	this.queueServer(u.serverId,o);
	return u.serverId;
}
UserQueue.prototype.getUserByToken = function(token){
	return User.get(token);
}
UserQueue.prototype.getUserTokensByServerId = function(serverId){
	return User.getIdsByServerId(serverId);
}
UserQueue.prototype.getUsersByServerId = function(serverId){
	return User.getListByServerId(serverId);
}
UserQueue.prototype.deleteServerUsers = function(serverId){
	for(var i in User.pool){
		if(User.pool[i].serverId === serverId){
			delete User.pool[i];
		}
	}
}
//Returns first user
UserQueue.prototype.getUserByLogin = function(login){
	for(var i in User.pool){
		if(User.pool[i].data.login === login){
			return User.pool[i];
		}
	}
}
UserQueue.prototype.getUsersByLogin = function(login){
	var u = [];
	for(var i in User.pool){
		if(User.pool[i].data.login === login){
			u.push(User.pool[i]);
		}
	}
	return u;
}
UserQueue.prototype.setServerUsers = function(users,serverId){
	var u,i,found,users;
	var tokens = this.getUserTokensByServerId(serverId);
	while(u = tokens.shift()){
		found = false;
		for(i=0;i<tokens.length;i++){//find current one in the new array
			if(users[i] && users[i].data.token === u){
				found = true;
				this.updateUser(users.splice(i,1)[0]);
				break;
			}
		}
		if(!found){//delete ones not in the new array
			this.deleteUser(u);
		}
	}
	for(i=0;i<users.length;i++){//add new ones
		this.updateUser(users[i]);
	}
}

UserQueue.prototype.createUser = function(o){
	User.pool[o.data.token] = {
		serverId:o.serverId,
		serverPool:o.serverPool,
		data:o.data,
	};
}
UserQueue.prototype.updateUser = function(o){
	User.pool[o.data.token] = {
		serverId:o.serverId,
		serverPool:o.serverPool,
		data:o.data
	};
	User.expires(o.data.token,60000);
}
UserQueue.prototype.loggedIn = function(token){
	if(User.pool[token]){
		User.pool[token].data.loggedIn = true;
		clearTimeout(User.pool[token].expires);
		User.pool[token].data.groups = [];
		if(['link','ssh'].indexOf(User.pool[token].data.provider) >= 0){
			this.getUserGroupsByLogin(User.pool[token].data.login).then(function(g){
				User.pool[token].data.groups = g;
			},function(e){
				console.log(e.stack);
			});
		}
	}
}
UserQueue.prototype.getUserGroupsByLogin = function(login){
	var q = Q.defer();
	exec('groups ' + login,function(error,stdout,stderr){
		if(error){
			q.reject(error);
			return;
		}
		var g = stdout.split(':');
		if(!g.length ||g.length < 2){return;}
		g[1] = g[1].replace(/^\s+/,'');
		g[1] = g[1].replace(/\s+$/,'');
		g = g[1].split(' ');
		var i,groups =[];
		while(g.length){
			i = g.shift();
			if(i.length){
				groups.push(i);
			}
		}
		q.resolve(groups);
	});
	return q.promise;
}
UserQueue.prototype.authenticated = function(token){
	if(User.pool[token]){
		User.pool[token].data.authenticated = true;
	}
}
UserQueue.prototype.setLink = function(token,ee){
	if(User.pool[token]){
		User.pool[token].link = ee;
	}
}
UserQueue.prototype.userExec = function(token,o,cb){
	if(User.pool[token]){
		if(User.pool[token].link){
			cb = cb || function(){};
			User.pool[token].link.send(o,cb);
			return true;
		}else if(User.pool[token].data.virtualUser){
			virtualExec(User.pool[token].data.login,o,cb);
		}
	}
}
UserQueue.prototype.virtualExec = function(login,o,cb){
	virtualExec(login,o,cb);
}
UserQueue.prototype.sudoExec = function(login,m,cb){
	execFile('/usr/bin/sudo',['-E','-H','-n','-u',login,m.command].concat(createParamString(m.data,'start')),function(error,stdout,stderr){
		if(error){
			cb({
				error:{
					code:500,
					message:error.message
				}});
		}else if(stderr){
			cb({
				error:{
					code:1500,
					message:stderr
				}});
		}else{
			try{
				cb(JSON.parse(stdout));
			}catch(e){
				console.log(e);
				cb({
					error:{
						code:500,
						message:stdout
					}
				});
			}
		}
	});
}
UserQueue.prototype.nodeExec = function(m,cb){
	execFile(m.command,m.args || createParamString(m.data),function(error,stdout,stderr){
		if(error){
			cb({
				error:{
					code:500,
					message:error.message
				}});
		}else if(stderr){
			cb({
				error:{
					code:500,
					message:stderr
				}});
		}else{
			if(m.raw){
				cb({result:stdout});
			}else{
				try{
					cb(JSON.parse(stdout));
				}catch(e){
					cb({
						error:{
							code:500,
							message:stdout
						}
					});
				}
			}
		}
	});
}
UserQueue.prototype.isLoggedIn = function(token){
	if(User.pool[token]){
		return User.pool[token].loggedIn;
	}
}
UserQueue.prototype.isAdminByUserData = function(u){
	if(!u){
		return;
	}else if(Admin.isAdmin(u.data.login)){
		return true;
	}else if(Admin.isAdminGroups(u.data.groups)){
		return true;
	}
}
UserQueue.prototype.isAdmin = function(token){
	return this.isAdminByUserData(User.get(token));

}
UserQueue.prototype.isAdminLoginName = function(login){
	return Admin.isAdmin(login);
}
UserQueue.prototype.keepAlive = function(token){
	return this.queueUser(token,{
		token:token,
		action:'keepalive',
		noResponse:true
	});
}
UserQueue.prototype.deleteUser = function(token){
	if(User.pool[token] && User.pool[token].data){
		localPool.ee.emit('snDeleteUser',{
			login:User.pool[token].data.login,
			token:token
		});
	}
	delete User.pool[token];
}
UserQueue.prototype.getUserPool= function(){
	return User.pool;
}
UserQueue.prototype.updateUsers = function(o){
	User.updateUsers(o);
}
UserQueue.prototype.getUser = function(id){
	var u = User.get(id);
	if(u){delete u.expires;}
	return u;
}
module.exports.UserQueue = UserQueue;
