var uuid = require('node-uuid');
var path = require('path');
var mkdirp = require('mkdirp');

var ERROR = require('../error-responses');
var Suggestion = require('../config/bookmark');
var History = require('../config/user-history');
var Bookmark = require('../config/user-bookmark');



function bookmarkFindOne(id,b){
	return b.findIndex(function(c,i,a){
		return id === c.id;
	});
}


function VirtualStore(login,cb){
	this.home = path.resolve(global.FXROOTDIR,'/var/virtual-store/users/',login);
	try{
		mkdirp.sync(this.home,{
				mode:'0755'
			});
		this.bookmarks = new Bookmark(this.home);
		this.history = new History(this.home);
	}catch(e){
		cb({error:{
			code:e.code,
			message:e.message
		}});
	}
}

VirtualStore.prototype.bookmarkList = function(r,cb){
	cb({result:{
		bookmarks:this.bookmarks.data,
		history:this.history.data,
		suggestions:Suggestion.data
	}});
}
VirtualStore.prototype.bookmarkEdit = function(r,cb){
	var b = this.bookmarks.data;
	if(!r.data.command){
		cb(ERROR.invalidParameters);
		return;
	}
	var i = bookmarkFindOne(r.data.id,b);
	if(i >=0){
		b[i] = r.data;
	}else{
		r.data.id = uuid.v4();
		b.push(r.data);
	}
	this.bookmarks.data = b;
	this.bookmarkList(r,cb);
}
VirtualStore.prototype.bookmarkDelete = function(r,cb){
	var b = this.bookmarks.data;
	b = b.filter(function(c,i,a){
		return !(c.id === r.data.id);
	});
	this.bookmarks.data = b;
	b = this.history.data;
	b = b.filter(function(c,i,a){
		return !(c.id === r.data.id);
	});
	this.history.data = b;
	this.bookmarkList(r,cb);
}
VirtualStore.prototype.addHistory = function(data){
	var history = this.history.data;
	history = history.filter(function(c,i,a){
		return !(data.command === c.command &&
				data.geometry === c.geometry &&
				data.profile === c.profile &&
				data.name === c.name
		);
	});
	data.id = uuid.v4();
	history.unshift(data);
	this.history.data = history;
}

function virtualExec(login,o,cb){
	cb = cb || function(){};
	
	var vs = new VirtualStore(login,cb);
	switch(o.action){
		case 'bookmark':
			var op = o.data.op;
			delete o.data.op;
			switch(op){
				case 'list':
					vs.bookmarkList(o,cb);
					break;
				case 'edit':
					vs.bookmarkEdit(o,cb);
					break;
				case 'delete':
					vs.bookmarkDelete(o,cb);
					break;
			}
			break;
		case 'history':
			vs.addHistory(o.data);
			break;
		default:
			cb(ERROR.commandNotFound);
	}
}
module.exports = virtualExec;