var extend = require('extend');
var fs = require('fs');
var path = require('path');
var uuid = require('node-uuid');

var virtualExecFile = require('./auth/virtual-exec-file');

var Admin 		= _require('common/lib/config/admin');
var AuthMethods = _require('common/lib/config/auth-methods');
var Bookmark 	= _require('common/lib/config/bookmark');
var Client 		= _require('common/lib/config/client');
var Command 	= _require('common/lib/config/command');
var Email 		= _require('common/lib/config/email');
var ERROR 		= _require('common/lib/error-responses');
var localPool 	= _require('common/lib/local-pool');
var Monitor 	= _require('common/lib/monitor');
var Motd 		= _require('common/lib/config/motd');
var Network 	= _require('common/lib/config/network');
var ProfileCache = _require('common/lib/config/profile');
var Session 	= _require('common/lib/config/session');
var SessionProfile = _require('common/lib/config/session-profile');
var Settings 	= _require('common/lib/config/settings');
var WWW 		= _require('common/lib/config/www');
var XrdpDefaults = _require('common/lib/config/fastx-defaults');

function Api(o){}

var cookieExpireTime = function(){return {expires:new Date(Date.now() + 3600000)};}

Api.prototype.authtoken = function(req,res,next,token){
	if(Monitor.getUserByToken(token)){
		res.cookie('authtoken',token,cookieExpireTime());
		next();
		return;
	}else{
		res.clearCookie('authtoken');
		res.json(ERROR.authenticationRequired).end();
	}
}
Api.prototype.loggedIn = function(req,res,next){
	var u;
	if(req.cookies.token){
		u = Monitor.getUserByToken(req.cookies.token);
		if(u && u.data.loggedIn){
			req.token = req.cookies.token;
		}
	}
	if(req.params.token){
		u = Monitor.getUserByToken(req.params.token);
		if(u && u.data.loggedIn){
			req.token = req.params.token;
		}
	}
	if(!req.token || !u || !u.data.loggedIn){
		res.clearCookie('username');
		res.clearCookie('host');
		res.clearCookie('token');
		res.json(ERROR.authenticationRequired).end();
		return;
	}else{
		res.cookie('username',u.data.login,cookieExpireTime());
		res.cookie('host',u.data.hostname,cookieExpireTime());
		res.cookie('token',req.cookies.token,cookieExpireTime());
		Monitor.queueUser(req.token,{
			token:req.token,
			action:'keepalive'}
		);
		next();
		return;
	}
}
Api.prototype.token = function(req,res,next){
	var u = Monitor.getUserByToken(req.cookies.token || req.params.token);

	if(u){
		req.token = req.cookies.token ||req.params.token;
		res.cookie('username',u.data.login,cookieExpireTime());
		res.cookie('token',req.cookies.token,cookieExpireTime());
		res.json({
			result:{
				token:req.token,
				username:u.data.login
			}
		});
	}else{
		res.json(ERROR.authenticationRequired).end();
	}
}
Api.prototype.keepalive = function(o){
	o.res.json({result:null});
}

Api.prototype.adminLogin = function(o){
	if(o.data.username){
		Monitor.getUserGroupsByLogin(o.data.username).then(function(g){
			if(Admin.isAdmin(o.data.username) || Admin.isAdminGroups(g) ){
				var token = uuid.v4();
				o.res.cookie('authtoken',token,cookieExpireTime());
				var mydata = extend(o,{
					action:'login',
					token:token
				});
				Monitor.queueLocalServer(mydata);
			}else{
				o.res.json(ERROR.permissionDenied);
				return;
			}
		},function(e){
			o.res.json(ERROR.permissionDenied);
			return;
		});
	}else{
		o.res.json(ERROR.permissionDenied);
		return;
	}
}

Api.prototype.loginEnv = function(o){
	o.res.json({result:AuthMethods.data.ssh});
}
Api.prototype.login = function(o){
	var res = o.res;
	o.cb = function(msg){
		if(msg.result && msg.result.stage === 'success'){
			res.clearCookie('authtoken');
			res.cookie('token',msg.result.token,cookieExpireTime());
			Monitor.loggedIn(msg.result.token);
		}
	}
	if(o.data.admin){
		this.adminLogin(o);
		return;
	}
	if(o.data.username){
		var token = uuid.v4();
		o.res.cookie('authtoken',token,cookieExpireTime());
		var mydata = extend(o,{
				action:'login',
				token:token
			});
		var serverId;

		if(Monitor.isLoadBalanced()){
			serverId = Monitor.queueLeastLoaded(mydata);
			if(!serverId){
				o.res.json(ERROR.loginDisabled);
				return;
			}
		}else if(o.data.serverId){
			if(o.data.serverId === localPool.serverId && !Network.loginEnabled){
				o.res.json(ERROR.loginDisabled);
				return;
			}
			if(Monitor.queueServer(o.data.serverId,mydata)){
				serverId = o.data.serverId;
			}else{
				o.res.json(ERROR.serverNotFound);
				return;
			}
		}else if(Network.loginEnabled){
			 serverId = Monitor.queueLocalServer(mydata);
		}

		if(!serverId){
			o.res.json(ERROR.serverNotFound);
		}
	}else{
		o.res.json(ERROR.usernameRequired);
	}
}
Api.prototype.password = function(o){
	var res = o.res;
	o.cb = function(msg){
		if(msg.result && msg.result.stage === 'success'){
			res.clearCookie('authtoken');
			res.cookie('token',msg.result.token,cookieExpireTime());
			Monitor.loggedIn(msg.result.token);
		}
	}
	if(!o.data.password){
		o.res.json(ERROR.usernameIncorrect);
		return;
	}
	if(!Monitor.queueUser(o.token,extend(o,{action:'password'}))){
		o.res.json(ERROR.userNotFound);
	}
}
Api.prototype.publicKey = function(o){
	var res = o.res;
	o.cb = function(msg){
		if(msg.result && msg.result.stage === 'success'){
			res.clearCookie('authtoken');
			res.cookie('token',msg.result.token,cookieExpireTime());
			Monitor.loggedIn(msg.result.token);
		}
	}
	if(!Monitor.queueUser(o.token,extend(o,{action:'publicKey'}))){
		o.res.json(ERROR.userNotFound);
	}
}
Api.prototype.publicKeySigned = function(o){
	var res = o.res;
	o.cb = function(msg){
		if(msg.result && msg.result.stage === 'success'){
			res.clearCookie('authtoken');
			res.cookie('token',msg.result.token,cookieExpireTime());
			Monitor.loggedIn(msg.result.token);
		}
	}
	if(!Monitor.queueUser(o.token,extend(o,{action:'publicKeySigned'}))){
		o.res.json(ERROR.userNotFound);
	}
}
Api.prototype.keyboardInteractive = function(o){
	var res = o.res;
	o.cb = function(msg){
		if(msg.result && msg.result.stage === 'success'){
			res.clearCookie('authtoken');
			res.cookie('token',msg.result.token,cookieExpireTime());
			Monitor.loggedIn(msg.result.token);
		}
	}
	if(!Monitor.queueUser(o.token,extend(o,{action:'keyboardInteractive'}))){
		o.res.json(ERROR.userNotFound);
	}
}

Api.prototype.logout = function(o){
	o.res.clearCookie('token');
	o.res.json({result:null});
	delete o.res;
	Monitor.queueUser(o.token,extend(o,{action:'logout'}));
	Monitor.deleteUser(o.token);

}
Api.prototype.version = function(o){
	var result = {};
	try{
		result.version = localPool.version;
	}catch(e){}
	result.schedule = Settings.data.schedule;
	result.serverId = localPool.serverId;
	result.httpPort = WWW.data.port;
	result.hostname = Settings.data.hostname;
	result.motd = Motd.data.beforeLogin;
	result.authMethods = true;
	if(o.req.app.HTTPSserver){
		result.httpsPort = o.req.app.HTTPSserver.address().port;
	}
	o.res.json({result:result});
}
Api.prototype.xrdpConfig = function(o){
	o.res.json({result:{
		defaults:XrdpDefaults.data.map(function(c,i,a){
			c.BandwidthThrottleAdjust =false;
			return c;
		})
	}});
}
Api.prototype.server = function(o){
	var result ={};
	if(!Monitor.isLoadBalanced()){
		result = Monitor.getServerNames();
	}
	o.res.json({result:result});
}
/*********** Config **********/
Api.prototype.config = function(o){
	o.res.json({
		result:extend({
			motd:Motd.data.motd,
			hideCommandField:Command.data.allow && !Command.data.allow.length
		},Client.data)
	});
}

/*********** Admin ***********/
Api.prototype.admin = function(o){
	o.res.json(Monitor.isAdmin(o.token)?{result:null}:ERROR.permissionDenied);
}
Api.prototype.adminSession = function(o){
	if(!Monitor.isAdmin(o.token)){
		o.res.json(ERROR.permissionDenied);
		return;
	}
	var s = Monitor.getSessions();
	o.res.json({result:Monitor.paginateSessions(s,o.query)});
}
Api.prototype.adminSessionCount = function(o){
	if(!Monitor.isAdmin(o.token)){
		o.res.json(ERROR.permissionDenied);
		return;
	}
	o.res.json({result:Monitor.getSessions().length});
}
Api.prototype.adminServer = function(o){
	if(Monitor.isAdmin(o.token)){
		var servers = Monitor.getServers();
		if(servers.findIndex(function(c,i,a){
			return c.serverId === localPool.serverId;
		}) < 0){
			servers.unshift( (Monitor.getServer(localPool.serverId)).data.server);
		}
		o.res.json({result:servers});
	}else{
		o.res.json(ERROR.permissionDenied);
	}
}
Api.prototype.userData = function(o){
	var u = Monitor.getUserByToken(o.token);
	if(u){
		o.res.json({result:u.data});
	}else{
		o.res.json(ERROR.userNotFound);
	}
}

/****** Queue the User *******/
Api.prototype.start = function(o){
	var u = Monitor.getUserByToken(o.token);
	if(u){
		if(!o.data.id){
			var id = (uuid.v4()).replace(/-/g,'');
			o.data.id = id;
		}
		var ou = extend(o,{
			action:'start',
			resData:{
				login:u.data.login,
			},
			cb:function(d){
				if(d.result && !Network.data.enableDirectMode){
					delete d.result.baseUrl;
				}
			}
		});
		if(u.data.virtualUser || (Monitor.isLoadBalanced() && Monitor.loadBalanceAtStart())){
			if(u.data.virtualUser){
				Monitor.virtualExec(u.data.login,{
					action:'history',
					data:ou.data
				});
			}
			ou.sudo = u.data.login;
			Monitor.queueLeastLoaded(ou);
		}else{
			Monitor.queueUser(o.token,ou);
		}
	}else{
		o.res.json(ERROR.userNotFound);
	}
}
Api.prototype.schedule = function(o){
	//if not schedule error
	var u = Monitor.getUserByToken(o.token);
	if(u){
		var p;
		if(o.data.profile){
			p = new ProfileCache(o.data.profile).data;
		}else{
			p = Session.data;
		}
		if(p.schedule.nodeExec){
			virtualExecFile({
				nodeExec:true,
				script:path.resolve(p.schedule.script),
				data:o.data,
				cb:function(error,stdout,stderr){
					if(error){
						o.res.json({
							error:{
								code:500,
								message:error.message
							}
						});
					}else{
						o.res.json({
							result:{
								stdout:stdout,
								stderr:stderr
							}
						});
					}
				}
			});
		}else if(u.data.virtualUser || (Monitor.isLoadBalanced()  && Monitor.loadBalanceAtStart())){
			o.sudo = u.data.login;
			Monitor.queueLeastLoaded(extend(o,{action:'schedule'}));
		}else{
			Monitor.queueUser(o.token,extend(o,{action:'schedule'}));
		}
	}else{
		o.res.json(ERROR.userNotFound);
	}
}
Api.prototype.bookmarkStart = function (o) {
	Monitor.queueUser(o.token,extend(o,{action:'shortcut'}));
}

Api.prototype.bookmarkList = function(o){
	if(!Monitor.queueUser(o.token,extend(o,{action:'bookmark',cb:function(d){
		if(d.result){
			d.result.suggestions = Bookmark.data;
		}
	}}))){
		o.res.json(ERROR.userNotFound);
	}
}
Api.prototype.bookmarkEdit = function(o){
	if(!Monitor.queueUser(o.token,extend(o,{action:'bookmark'}))){
		o.res.json(ERROR.userNotFound);
	}
}
Api.prototype.bookmarkDelete = function(o){
	if(!Monitor.queueUser(o.token,extend(o,{action:'bookmark'}))){
		o.res.json(ERROR.userNotFound);
	}
}
Api.prototype.log = function(o){
	if(!Monitor.queueSession(o.id,extend(o,{action:'log'}))){
		o.res.json(ERROR.userNotFound);
	}
}
Api.prototype.profileList = function(o){
	o.res.json({result:SessionProfile.userData});
}

/********** Begin Session Actions *********/

Api.prototype.list = function(o){
	var u = Monitor.getUserByToken(o.token);
	var s = Monitor.getSessions({login:u.data.login});
	o.res.json({result:Monitor.paginateSessions(s,o.query)});
}
Api.prototype.listCount = function(o){
	var u = Monitor.getUserByToken(o.token);
	o.res.json({result:Monitor.getSessions({login:u.data.login}).length});
}
function sessionAction(action,o,waitForResult){
	var s = Monitor.getSession(o.id);
	if(!s){
		o.res.json(ERROR.sessionNotFound);
		return;
	}

	if(!Monitor.queueSession(o.id,extend(o,{action:action}))){
		o.res.json(ERROR.serverNotFound);
		Monitor.deleteServerSessions(s.serverId);
		return;
	}

	if(!waitForResult){
		Monitor.sendResponse(o.msgId,{result:null},localPool.serverId);
	}
}

Api.prototype.screenshot = function(o){
	if(Client.data.screenshot){
		sessionAction('screenshot',o,true);
	}else{
		o.res.json(ERROR.screenshotDisabled);
	}
}

Api.prototype.shareEmail = function(o){
	var mailOpts = {
		subject:'Join My Shared Session',
		html:['<p>Please click this link to join my shared session</p>',
			'<p><a href="',o.data.url,'">',o.data.url,'</a></p>'
			].join('')
	}
	Email.send(o.data.emails,mailOpts,function(){
		o.res.json({result:null})
	},
	function(e){
		o.res.json(e);
	});
};
Api.prototype.shareList = function(o){sessionAction('shareList',o,true);}
Api.prototype.shareEdit = function(o){sessionAction('shareEdit',o,true);}
Api.prototype.shareDelete = function(o){sessionAction('shareDelete',o,true);}
Api.prototype.shareReset = function(o){sessionAction('shareReset',o,true);}
Api.prototype.sharePassword = function(o){sessionAction('share',o,true);}


Api.prototype.terminate = function(o){sessionAction('terminate',o,true);}

Api.prototype.connectShortcut = function (o) {
	if(!o.data.id){
		o.res.json(ERROR.invalidParameters).end();
		return;
	}
	var u = Monitor.getUserByToken(o.token);
	var s = Monitor.getSessions({login:u.data.login,shortcut:o.data.id});
	var session;
	if (!s.length) {
		this.bookmarkStart(o);
		return;
		//start
	} else if (s.length === 1) {
		session = s[0];
		//connect
	} else {
		//sort and choose
		let disconnectedSessions = s.filter(function (c) { return c.clients === 0;});
		//disconnected (0 clients) come before connected
		if ( disconnectedSessions.length ) {
			//if not connected, the latest (most recent) lastDisconnect time comes first
			if(disconnectedSessions.filter(function (c) { return c.lastDisconnect}).length) {
				session = disconnectedSessions.sort( function (a, b) {
					return (a.lastDisconnect < b.lastDisconnect)?-1:1;
				})[0];
			} else {
				//Otherwise, the oldest "started" comes first
				session = disconnectedSessions.sort( function (a, b) {
					return (a.started < b.started)?-1:1;
				})[0];
			}
		} else {
			//if connected, the oldest lastConnect time comes first
			session = s.filter(function (c) { return c.clients > 0;})
				.sort( function (a, b) { return (a.lastConnect < b.lastConnect)?-1:1;
				})[0];
		}
	}
	this.connect ({
		res:o.res,
		token:o.token,
		id:session.id
	});


}
Api.prototype.connect = function(o){
	o.cb = function(d){
		if(d.result && !Network.data.enableDirectMode){
			delete d.result.baseUrl;
		}
	}
	sessionAction('connect',o,true);}
Api.prototype.disconnect = function(o){sessionAction('disconnect',o, true);}
Api.prototype.params = function(o){sessionAction('params',o, true);}
Api.prototype.exec = function(o){sessionAction('exec',o,true);}

module.exports.Api = Api;
