
var dns = require('dns');
var os = require('os');
var Q = require('q');
var URL = require('url');
var WebSocket = require('ws');

var checkServerIdentity = require('./check-session-server-identity');

var Network = _require('common/lib/config/network');
var www 	= _require('common/lib/config/www');
var localPool = _require('common/lib/local-pool');



var failedConnections = [];
var successConnections = [];
//TODO URL PARSE THE HOST -- Convert to IP address, then send them all out

function getNetAddresses(){
	var nets = os.networkInterfaces();
	var addrs = [];
	var iface;
	for(iface in nets){//get network interfaces
		nets[iface].forEach(function(c,i,a){
			if(!c.internal){ //ignore internal
				addrs.push(c.address);
			}
		});
	}
	return addrs;
}
function resolveIpToHostname(addr){
	var q = Q.denodeify(dns.reverse);
	return q(addr);
}
function getIpAddresses(hostname){
	var q = Q.denodeify(dns.lookup);
	return q(hostname,{all:true});
}

function createUrl(url,ip){
	return [url.protocol,(url.slashes)?'//':'','[',ip,']:',((url.port)?url.port:((url.protocol === 'https:')?443:80))].join('');
}
function isLocalUrl(ip,port,localAddresses){
	return ( (localAddresses.indexOf(ip) >= 0) && ([parseInt(www.data.port), parseInt(www.data.httpsport)].indexOf(parseInt(port,10)) >= 0))
};


function Broadcast(){
	this.localAddresses = [];
	this.servers = {};
	this.serverIds = {};
	var that = this;
	setInterval(function(){
		that.refresh();
	},30000);//refresh the host list once a minute
	this.refresh();
}
Broadcast.prototype.refresh = function(){
	this.refreshServers();
	this.localAddresses = getNetAddresses();
}

//get the list of all the hostnames to send it to
Broadcast.prototype.getHostnames = function(){
	var addrs = getNetAddresses();
	var resolvedHost;
	var p =[];
	var that = this;
	//console.log('all addresses',addrs);
	for(var i=0;i<addrs.length;i++){
		p.push(resolveIpToHostname(addrs[i]).then(function(h){return h;}));
	}
	return Q.allSettled(p).then(function(a){
		//console.log('all hostnames',a);
		var h =  [];
		var i;
		var urls = Network.urlObjects;
		urls.forEach(function(c,i,a){
			h.push(c.hostname);
		});
		a.forEach(function(c,i,arr){
			if(c.state === 'fulfilled'){
				c.forEach(function(t){
					if(h.indexOf(t) <0){
						h.push(t);
					}
				});
			}
		});
		return h;
	});
}
Broadcast.prototype.refreshServers = function(){
	var that = this;
	var urls = Network.urlObjects;
	var p =[],ips=[],rrdns =[];
	for(var i=0; i<urls.length;i++){
		p.push(getIpAddresses(urls[i].url.hostname));
	}
	return Q.allSettled(p).then(function(a){
		var url,host,key,myUrl,addr;
		a.forEach(function(c,i){
			if(c.state === 'fulfilled'){
				if(typeof c.value[0] === 'string'){
					c.value = [{address:c.value[0],family:c.value[1]}]; //fix for old node servers
				}
				c.value.forEach(function(t){
					if(ips.indexOf(t.address) <0 && !isLocalUrl(t.address,urls[i].url.port,that.localAddresses)){
						url = createUrl(urls[i].url,t.address);
						host = urls[i].url.hostname;
						key = urls[i].key;
						if(!that.servers[url]){
							that.createConnection(url,key,host);
						}else{
							/* ping the websocket and remove the connection if it fails */
							try {
								that.servers[url].ws.ping (url);
							} catch (e) {
								removeConnections (url);
								console.log ('Ping failed', url, e.message);
								that.createConnection(url,key,host);
							}
						}
					}
				});
			}
		});

	});
}

function removeConnections (url) {
	var i;
	i = successConnections.indexOf(url);
	if(i >=0){
		successConnections.splice(i,1);
	}
	i = failedConnections.indexOf(url);
	if(i >=0){
		failedConnections.splice(i,1);
	}
}
function fail (url, e) {
	if(failedConnections.indexOf(url) < 0){
		console.log('Broadcast: Failed to connect to',url,e.message);
		failedConnections.push(url);
	}
}
function cleanup (url,that) {
	var i;
	i = successConnections.indexOf(url);
	if(i >=0){
		successConnections.splice(i,1);
	}

	if(that.servers[url] && that.servers[url].serverId){
		delete that.serverIds[that.servers[url].serverId];
	}
	delete that.servers[url];
}
Broadcast.prototype.createConnection = function(url,key,host){
	var that = this;
	var myUrl = [url,'server?serverPool=' +key +'&serverId=' +localPool.serverId].join('/');
	var ws = new WebSocket(myUrl,{
		checkServerIdentity:checkServerIdentity,
		rejectUnauthorized:Network.data.rejectUnauthorized
	});
	ws.on('open',function(){
			var i = successConnections.indexOf(url);
			if(i < 0){
				console.log('Broadcast: Connection established to',url);
				successConnections.push(url);
			}else{
			}
	});
	ws.on('message',function(msg){
		try{
			var m = JSON.parse(msg);
		}catch(e){
			console.log('Invalid JSON',msg);
			ws.close(1000,'Communication Error');
			return;
		}
		if(m.result){
			serverId = m.result.serverId;
		}
		if(m.result && !that.serverIds[m.result.serverId]){
			if(that.servers[url]){
				that.serverIds[m.result.serverId] = that.servers[url];
				that.servers[url].serverId = m.result.serverId;
			}
		}
		localPool.ee.emit('snServerMessage',m);
	});
	ws.on('close',function(e){
		cleanup(url, that);
	});
	ws.on('error',function(e){
		fail (url, e);
		cleanup(url, that);
	});
	that.servers[url] = {
		ws:ws,
		url:url
	}
}

Broadcast.prototype.sendResponse = function(msgId,data,serverId){
	if(this.serverIds[serverId] && this.serverIds[serverId].ws.readyState === WebSocket.OPEN){
		this.serverIds[serverId].ws.send(
			JSON.stringify({data:{
				msgId:msgId,
				serverId:localPool.serverId,
				serverPool:localPool.serverKey,
				data:data
			},
			method:'response'
		}));
	}else{
		console.log('ws,not opened',data,serverId);
	}
}
Broadcast.prototype.getUserData = function(token, serverId){
	if(this.serverIds[serverId] && this.serverIds[serverId].ws.readyState === WebSocket.OPEN){
		this.serverIds[serverId].ws.send(
			JSON.stringify({data:{
				token:token,
				serverId:localPool.serverId,
				serverPool:localPool.serverKey
			},
			method:'get-user-data'
		}));
	}else{
		console.log('ws,not opened',serverId);
	}
}
Broadcast.prototype.broadcastData = function(method,data){
	for(var i in this.servers){
		if(this.servers[i].ws.readyState === WebSocket.OPEN){
			this.servers[i].ws.send(JSON.stringify({
				method:method,
				data:data
			}));
		}else{
			delete this.servers[i];
		}
	}
}
Broadcast.prototype.getServerUrl = function(serverId){
	return this.serverIds[serverId].url;
}
Broadcast.prototype.broadcastServer = function(server){
	this.broadcastData('server',server.data);
}
Broadcast.prototype.broadcastSessions = function(sessions){
	this.broadcastData('sessions',sessions.data);
}
Broadcast.prototype.broadcastUsers = function(users){
	this.broadcastData('users',users.data);
}
Broadcast.prototype.resetConnetionData = function(){
	successConnections = [];
	failedConnections = [];
	this.servers = {};
	this.serverIds = {};
	this.refresh();
}
//start broadcasting
var b = new Broadcast();

localPool.ee.on('snNetworkDataChanged',function(){
	b.resetConnetionData();
});
module.exports.Broadcast = b;
