var Admin 		= _require('common/lib/config/admin');
var AuthMethods	= _require('common/lib/config/auth-methods');
var Bookmark 	= _require('common/lib/config/bookmark');
var Client 		= _require('common/lib/config/client');
var Command 	= _require('common/lib/config/command');
var Email		= _require('common/lib/config/email');
var ERROR 		= _require('common/lib/error-responses');
var FastXDefaults = _require('common/lib/config/fastx-defaults');
var Ldap		= _require('common/lib/config/ldap');
var localPool 	= _require('common/lib/local-pool');
var Monitor 	= _require('common/lib/monitor');
var Motd		= _require('common/lib/config/motd');
var Network 	= _require('common/lib/config/network');
var Session 	= _require('common/lib/config/session');
var SessionProfile = _require('common/lib/config/session-profile');
var Settings 	= _require('common/lib/config/settings');
var Theme   	= _require('common/lib/config/theme');
var www 		= _require('common/lib/config/www');

function System(){}


function tryCatch(fun,o){
	try{
		fun(o);
	}catch(e){
		console.log(e.stack);
		o.res.json({error:{
			code:500,
			message:e.message
		}});
	}
}

System.prototype.getSessionProfiles = function(o){
	tryCatch(function(o){
		o.res.json({result:SessionProfile.data});
	},o);
}
System.prototype.setSessionProfiles = function(o){
	tryCatch(function(o){
		//SessionProfiles.data = o.data;
		o.res.json({result:SessionProfile.data});
		return;
	},o);
}
System.prototype.setDefaultSessionProfile = function(o){
	tryCatch(function(o){
		Session.data = o.data;
		o.res.json({result:Session.data});
		return;
	},o);
}
System.prototype.getDefaultSessionProfile = function(o){
	tryCatch(function(o){
		o.res.json({result:Session.data});
	},o);
}

System.prototype.sessionProfileEdit= function(o){
	tryCatch(function(o){
		if(!o.data.hasOwnProperty('name')){
			throw new Error(ERROR.invalidParameters.error.message);
		}
		if(o.data.id && o.data.id.length){
			var id = o.data.id + '.json';
			SessionProfile.remove(id);
		}
		var name = o.data.name + '.json';
		if(o.data.command && Object.keys(o.data.command).length === 0){
			delete o.data.command;
		}
		SessionProfile.edit(name,o.data);
		o.res.json({result:SessionProfile.data});
	},o);
}
System.prototype.sessionProfileDelete = function(o){
	tryCatch(function(o){
		SessionProfile.remove(o.data.id + '.json');
		o.res.json({result:SessionProfile.data});
	},o);
}



System.prototype.getAdminUser = function(o){
	tryCatch(function(o){
		o.res.json({result:Admin.data});
	},o);
}
System.prototype.setAdminUser = function(o){
	tryCatch(function(o){
		var u = Monitor.getUserByToken(o.token);
		if((o.data.users &&  o.data.users.indexOf(u.data.login) >= 0) || u.data.groups.some(function(i){
			if(o.data.groups && o.data.groups.length){
				return o.data.groups.indexOf(i) >= 0;
			}
		}) ){
			Admin.data = o.data;
			o.res.json({result:Admin.data});
			return;
		}
		throw new Error('You cannot remove yourself from the admins');
	},o);
}
System.prototype.adminUserAdd = function(o){
	tryCatch(function(o){
		Admin.add(o.data);
		o.res.json({result:Admin.data});
	},o);
}
System.prototype.adminUserDelete = function(o){
	tryCatch(function(o){
		if((Monitor.getUserByToken(o.token)).data.login !== o.data.login){
			Admin.remove(o.data);
			o.res.json({result:Admin.data});
		}else{
			throw new Error('Deleting yourself will cause a rift in the spacetime continuum');
		}
	},o);
}
System.prototype.setNetworkConfig = function(o){
	tryCatch(function(o){
		Network.data = o.data;
		localPool.ee.emit('snNetworkDataChanged',{});
		if(!Network.data.allowIncomingConnections){
			localPool.ee.emit('snIncomingConnectionsClosed',{});
		}
		try{
			Monitor.refreshServers();
		}catch(e){
			
		}
		o.res.json({result:Network.data});
	},o);
}
System.prototype.getNetworkConfig = function(o){
	tryCatch(function(o){
		o.res.json({result:Network.data});
	},o);
}
System.prototype.getCurrentNetworkConfig = System.prototype.getNetworkConfig;
System.prototype.deleteNetworkConfig = function(o){
	tryCatch(function(o){
		Network.unlink();
		o.res.json({result:null});
	},o);
}


System.prototype.getTheme = function(o){
	tryCatch(function(o){
		o.res.json({result:Theme.data});
	},o);
}
System.prototype.setTheme = function(o){
	tryCatch(function(o){
		if(!o.data.logo){
			delete o.data.logo;
		}
		Theme.data = o.data;
		o.res.json({result:Theme.data});
	},o);
}
System.prototype.getFastXDefaults = function(o){
	tryCatch(function(o){
		o.res.json({result:FastXDefaults.data});
	},o);
}
System.prototype.setFastXDefaults= function(o){
	tryCatch(function(o){
		FastXDefaults.data = o.data;
		o.res.json({result:FastXDefaults.data});
	},o);
}
System.prototype.getSession = function(o){
	tryCatch(function(o){
		o.res.json({result:{session:Session.data,client:Client.data}});
	},o);
}
System.prototype.setSession = function(o){
	tryCatch(function(o){
		Session.data = o.data.session;
		Client.data  = o.data.client;
		o.res.json({result:{session:Session.data,client:Client.data}});
	},o);
}

System.prototype.setCommand = function(o){
	tryCatch(function(o){
		Command.data = o.data;
		o.res.json({result:Command.data});
	},o);
}
System.prototype.getCommand = function(o){
	tryCatch(function(o){
		o.res.json({result:Command.data});
	},o);
}
System.prototype.listBookmark = function(o){
	tryCatch(function(o){
		o.res.json({result:Bookmark.data});
	},o);
}
System.prototype.setBookmarks = function(o){
	tryCatch(function(o){
		Bookmark.data = o.data;
		o.res.json({result:Bookmark.data});
	},o);
}
System.prototype.editBookmark = function(o){
	tryCatch(function(o){
		Bookmark.edit(o.data);
		o.res.json({result:Bookmark.data});
	},o);
}
System.prototype.deleteBookmark = function(o){
	tryCatch(function(o){
		Bookmark.remove(o.data.id);
		o.res.json({result:Bookmark.data});
	},o);
}
System.prototype.setSettings = function(o){
	tryCatch(function(o){
		Settings.data = o.data.settings;
		Client.data = o.data.client;
		o.res.json({result:{
			settings:Settings.data,
			client:Client.data
		}});
	},o);
}
System.prototype.getSettings = function(o){
	tryCatch(function(o){
		o.res.json({result:{
			settings:Settings.data,
			client:Client.data
		}});
	},o);
}
System.prototype.setWWW = function(o){
	tryCatch(function(o){
		www.data = o.data;
		o.res.json({result:www.data});
	},o);
}
System.prototype.getWWW= function(o){
	tryCatch(function(o){
		o.res.json({result:www.data});
	},o);
}
System.prototype.getEmail = function(o){
	tryCatch(function(o){
		o.res.json({result:Email.data});
	},o);
}
System.prototype.setEmail = function(o){
	tryCatch(function(o){
		Email.data = o.data;
		o.res.json({result:Email.data});
	},o);
}
System.prototype.getMotd = function(o){
	tryCatch(function(o){
		o.res.json({result:Motd.data});
	},o);
}
System.prototype.setMotd = function(o){
	tryCatch(function(o){
		Motd.data = o.data;
		o.res.json({result:Motd.data});
	},o);
}
System.prototype.getLdap = function(o){
	tryCatch(function(o){
		o.res.json({result:Ldap.data});
	},o);
}
System.prototype.setLdap = function(o){
	tryCatch(function(o){
		Ldap.data = o.data;
		o.res.json({result:Ldap.data});
	},o);
}
System.prototype.getAuthMethods = function(o){
	tryCatch(function(o){
		o.res.json({result:AuthMethods.data});
	},o);
}
System.prototype.setAuthMethods = function(o){
	tryCatch(function(o){
		AuthMethods.data = o.data;
		AuthMethods.refresh();
		o.res.json({result:AuthMethods.data});
	},o);
}
module.exports.System = new System();