var URL = require('url');
var WebSocket = require('ws');
var WebSocketServer = require('ws').Server;

var localPool = _require('common/lib/local-pool');
var ERROR 	= _require('common/lib/error-responses');
var Monitor = _require('common/lib/monitor');
var Network = _require('common/lib/config/network');

function cleanup(e){
	Monitor.deleteServer(e);
	Monitor.deleteServerSessions(e);
	Monitor.deleteServerUsers(e);
	var msgs = Monitor.delegateMessages(e);
	var msg;
	while(msg = msgs.shift()){
		if(msg.msgId){
			Monitor.sendResponse(msg.msgId,ERROR.cannotQueueServer,localPool.serverId);
		}
	}
}


function Server(o,servers){
	var wsserver = new WebSocketServer(o);

	localPool.ee.on('snServerMessageQueued',function(e){
		if(servers[e] && servers[e].ws && (servers[e].ws.readyState === WebSocket.OPEN)){
			servers[e].ws.send(JSON.stringify({
				result:{
					serverId:localPool.serverId,
					messages:Monitor.delegateMessages(e)
				}
			}));
		}else{
			//console.log('Error Queuing Server', e);
			cleanup(e);
			delete servers[e];
		}
	});
	localPool.ee.on('snIncomingConnectionsClosed',function(e){
		for(var i in servers){
			Monitor.deleteServer(i);
			if(servers[i].ws) {
				servers[i].ws.close(1000,'Incoming Connections Closed');
			}
			delete servers[i];
		}
	});

	wsserver.on('connection',function(ws){
		var u = URL.parse(ws.upgradeReq.url);
		var s = (u.search.replace('?','')).split('&');
		var params ={},p;
		for(var i =0; i< s.length;i++){
			p = s[i].split('=');
			params[p[0]] = p[1];
		}

		if(!params['serverPool'] || ((Object.keys(Network.data.keys)).indexOf(params['serverPool']) < 0)){
			ws.close(1000,'Permission Denied');
			return;
		}
		if(!Network.data.keys[ params['serverPool'] ].allowIncomingConnections){
			ws.close(1000,'Permission Denied');
			return;
		}

		if(!params['serverId']){
			ws.close(1000,'ServerId not found');
			return;
		}
		ws.on('message',function(msg){
			try{
				var m = JSON.parse(msg);
				if(!m){
					console.log('Invalid message',msg);
					ws.close(1000,'Invalid message sent');
					return;
				}
				process(params['serverId'],params['serverPool'],m);
			}catch(e){
				console.log('Invalid JSON',msg);
				ws.close(1000,'Failed to parse JSON');
			}
		});
		ws.on('close',function(e){});
		ws.on('error',function(e){
			console.log('Error event',e);
		});

		servers[params['serverId']] = {
			ws:ws
		};

		localPool.ee.emit('snServerMessageQueued',params['serverId']);
	});
	return wsserver;
}



function process(serverId,serverPool,m){
	switch(m.method){
		case 'get-user-data':
			Monitor.queueServer(serverId,{
				token:m.data.token,
				action:'user-data',
				data:Monitor.getUser(m.data.token)
			});
			localPool.ee.emit('snServerMessageQueued',serverId);
			break;
		case 'response':
			Monitor.sendResponse(m.data.msgId,m.data.data,localPool.serverId);
			break;
		case 'server':
			if(localPool.serverId !== serverId){
				Monitor.updateServer({
					serverId:serverId,
					serverPool:serverPool,
					data:m.data});
			}
			break;
		case 'users':
			if(localPool.serverId !== serverId){
				Monitor.setServerUsers(m.data,serverId);
			}
			break;
		case 'sessions':
			if(localPool.serverId !== serverId){
				Monitor.setServerSessions(m.data,serverId);
			}
			break;
		default:
			console.log('Unknown Method',m.method);
			break;
	}
}
module.exports.Server = Server;
