var bufferEqual = require('buffer-equal');
var net = require('net');
var WebSocket = require('ws');
var EventEmitter = require('events').EventEmitter;

function LOG(data,cl){
	if(!process.env.LOG_ALL_MESSAGES){return;}
	console.log(cl);
	var d = (data.toString('hex')
		.match(/.{1,32}/g))
		.forEach(function(c,i,a){
			console.log( (c.match(/.{1,2}/g)).join(' ') );
	});
	console.log('');
}
exports.ee = new EventEmitter();



exports.direct = function(ws,port){
	ws.binaryType = 'fragments';

	var tcp,isRdp;
	if(isNaN(port)){
		ws.close(1000,"Invalid Port Number");
		return;
	}
	if(port <= 1024){
		ws.close(1000,"Cannot connect to a Reserved Port");
		return;
	}

	try{
		tcp = net.connect({port:port});
	}catch(e){
		console.log(e.message);
		console.log(e.stack);
		ws.close(1000,e.message);
		return;
	}
	tcp.on('open',function(){

	});
	tcp.on('data',function(data){
		if(!isRdp){
			//connection confirm pdu
			var b1 = new Buffer([0x03, 0x00, 0x00, 0x13, 0x0e, 0xd0, 0x00, 0x00, 0x00, 0x00, 0x00,  0x02, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00]);
			var b2 = new Buffer([0x03, 0x00, 0x00, 0x0b, 0x06, 0xd0, 0x00, 0x00, 0x00, 0x00, 0x00]);
			if(false && !(bufferEqual(b1,data) || bufferEqual(b2,data)) ){
				ws.close(1000,'Invalid RDP Hello response');
				return;
			}
			isRdp = true;
		}
		try{
			//LOG(data,'Server');
			if(ws.bufferedAmount > 1000000){
				tcp.pause();
				ws.send(data,function(){
					tcp.resume();
				});
			}else{
				ws.send(data);
			}
		}catch(e){
			ws.close();
			tcp.destroy();
		}

	});
	tcp.on('error',function(e){
		//console.log('Error in websocket to TCP bridge');
		//console.log(e.message)
		ws.close();
		tcp.destroy();
	});
	tcp.on('end',function(){
		ws.close();
	});
	tcp.on('close',function(had_error){
		if(had_error){
			console.log('TCP Socket closed with error',had_error);
		}
		ws.close();
		tcp.end();
	});
	ws.on('error',function(e){
		console.log('Websocket error',e);
	});
	ws.on('close',function(code,reason){
		if(code !== 1000){
			console.log('Websocket closed',code,reason);
		}
		tcp.end();
	});
	ws.on('unexpected-response',function(req,res){
		console.log('Websocket unexpected response',res.status);
	});
	ws.on('message',function(frags){
		//LOG(msg,'Client');
		frags.forEach((c) => {
			if(!isRdp){
				var tmp = new Buffer(11);
				c.copy(tmp,0,0,3);//tpkt
				tmp[3] = 0;
				tmp[4] = 0;
				c.copy(tmp,5,5,11);
				var sig = Buffer.concat([
					new Buffer([0x03, 0x00, 0x00]),//tpkt
					new Buffer([0x00, 0x00]),//sizes
					new Buffer([0xe0, 0x00, 0x00, 0x00, 0x00, 0x00]) //ccr
					],11);
				/*
				console.log(msg);
				console.log(sig);
				console.log(tmp);
				*/
				if(!bufferEqual(tmp,sig) ) {
					ws.close(1000,'Invalid RDP Hello message');
					return;
				}
			}
			tcp.write(c);
		});
	});
}
exports.bridge = {}
exports.bridge.server = function(socket){
	socket.rdp.on('message',function(data,flags){
		if(socket.browser.readyState === WebSocket.OPEN){
			socket.browser.send(data);
		}else{
			socket.rdp.close();
			delete socket;
		}
	});
	socket.rdp.on('error',function(e){
		console.log(e.stack)
	});
	socket.rdp.on('close',function(code,message){
		if(socket && socket.browser && socket.browser.readyState === WebSocket.OPEN){
			socket.browser.close();
		}
		delete socket;
	});
}

exports.bridge.browser = function(socket){
	socket.browser.on('message',function(data,flags){
		if(socket && socket.rdp && socket.rdp.readyState === WebSocket.OPEN){
			socket.rdp.send(data);
		}else{
			socket.browser.close();
		}
	});
	socket.browser.on('error',function(error){
		console.log(error.message);
		console.log(error.stack);
	});
	socket.browser.on('close',function(code,message){
		if(socket && socket.rdp && socket.rdp.readyState === WebSocket.OPEN){
			socket.rdp.close();
		}
		delete socket;
	});
}
