var express = require('express');
var extend = require('extend');

var Auth = require('../lib/auth/auth');
var recordLoginAttempts = require('../lib/record-login-attempts');
var API = require('../lib/api').Api;

var router = express.Router();
var Api = new API();

process.chdir(__dirname + '/../');
router.all('*',function(req,res,next){
	if(['127.0.0.1','::ffff:127.0.0.1','::1'].indexOf(req.ip) >= 0){
		next();
	}else{
		next();
	}
});

router.errorHandler = function(err, req, res, next){
	var msg ='',code;
	console.log(err.stack);
	switch (res.statusCode){
		case 200:
			code = 500;
			msg = err.message;
			break;
		case 404:
			msg = 'Not Found';
			break;
		default:
			msg = 'Internal Server Error';
	}
	res.json({
		error:{
			code:code || res.statusCode || 500,
			message:msg
		}
	});
};

router.route('/')
.get(function(req,res){
	Api.version({
		req:req,
		res:res
	});
})
.post(function(req,res){
	Api.version({
		req:req,
		res:res
	});
});
router.route('/token')
.get(Api.token)
.post(Api.token);
router.route('/server')
.get(Api.server)
.post(Api.server);
router.route('/xrdp/config')
.get(function(req,res){
	Api.xrdpConfig({
		res:res
	});
})
.post(function(req,res){
	Api.xrdpConfig({
		res:res
	});
});

router.route('/login/env').all(function(req,res){
	Api.loginEnv({res:res});
});
router.route('/login')
.post(function(req,res){
	recordLoginAttempts(req.ip,req.body);
	Api.login({
		res:res,
		data:req.body
	});
});

router.route('/connect/share')
.post(function(req,res){
	Api.sharePassword({
		res:res,
		id:req.body.id,
		data:req.body
	});
});

router.param('authtoken',Api.authtoken);

/***** AUTHENTICATION API *****/
router.route('/login/keyboard-interactive/:authtoken')
.post(function(req,res){
	Api.keyboardInteractive({
		res:res,
		token:req.params.authtoken,
		data:req.body
	});
});
router.route('/login/password/:authtoken')
.post(function(req,res){
	Api.password({
		res:res,
		token:req.params.authtoken,
		data:req.body
	});
});
router.route('/login/public-key/:authtoken')
.post(function(req,res){
	Api.publicKey({
		res:res,
		token:req.params.authtoken,
		data:req.body
	});
});
router.route('/login/public-key/signed/:authtoken')
.post(function(req,res){
	Api.publicKeySigned({
		res:res,
		token:req.params.authtoken,
		data:req.body
	});
});

/*************** Things Must Be logged in from here *************/
router.route('/user/profile/:token?')
.get(Api.loggedIn,function(req,res){
	Api.userData({res:res,token:req.token});
})
.post(function(req,res){
	Api.userData({res:res,token:req.token});
});
router.route('/logout/:token?')
.post(Api.loggedIn,function(req,res){
	res.clearCookie('token');
	res.clearCookie('host');
	res.clearCookie('username');
	Api.logout({
		res:res,
		token:req.token
	});
});
router.route('/keepalive/:token?')
.post(Api.loggedIn,Api.token);


/***** SESSION LIST *****/
router.route('/list/:token?')
.get(Api.loggedIn,function(req,res){
	Api.list({
		res:res,
		token:req.token,
		query:req.query || {}
	});
})
.post(Api.loggedIn,function(req,res){
	Api.list({
		res:res,
		token:req.token,
		query:req.body || {}
	});
});
router.route('/list/count/:token?')
.get(Api.loggedIn,function(req,res){
	Api.listCount({
		res:res,
		token:req.token
	});
})
.post(Api.loggedIn,function(req,res){
	Api.listCount({
		res:res,
		token:req.token
	});
});
/***** SESSION START *****/
router.route('/start/bookmark/:token?')
.post(Api.loggedIn,function(req,res){
	Api.bookmarkStart({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/start/:token?')
.post(Api.loggedIn,function(req,res){
	Api.start({
		res:res,
		token:req.token,
		data:req.body
	},[(req.secure)?'wss://':'ws://',req.headers.host,'/connect'].join(''));
});
router.route('/schedule/:token?')
.post(Api.loggedIn,function(req,res){
	Api.schedule({
		res:res,
		token:req.token,
		data:req.body
	});
});
/***** SESSION ACTIONS API *****/
router.route('/screenshot/:token?')
.post(Api.loggedIn,function(req,res){
	Api.screenshot({
		res:res,
		token:req.token,
		id:req.body.id,
		data:req.body
	});
});
router.route('/connect/shortcut/:token?')
.post(Api.loggedIn,function(req,res){
	Api.connectShortcut({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/connect/:token?')
.post(Api.loggedIn,function(req,res){
	Api.connect({
		res:res,
		token:req.token,
		id:req.body.id
	});
});
router.route('/terminate/:token?')
.post(Api.loggedIn,function(req,res){
	Api.terminate({
		res:res,
		token:req.token,
		id:req.body.id
	});
});
router.route('/disconnect/:token?')
.post(Api.loggedIn,function(req,res){
	Api.disconnect({
		res:res,
		token:req.token,
		id:req.body.id
	});
});
router.route('/params/:token?')
.post(Api.loggedIn,function(req,res){
	Api.params({
		res:res,
		token:req.token,
		id:req.body.id,
		data:req.body.params
	});
});
router.route('/exec/:token?')
.post(Api.loggedIn,function(req,res){
	Api.exec({
		res:res,
		token:req.token,
		id:req.body.id,
		data:req.body
	});
});
router.route('/log/:token?')
.post(Api.loggedIn,function(req,res){
	Api.log({
		res:res,
		token:req.token,
		id:req.body.id,
		data:req.body
	});
});
/***** BOOKMARK API *****/

router.route('/bookmark/edit/:token?')
.post(Api.loggedIn,function(req,res){
	Api.bookmarkEdit({
		res:res,
		token:req.token,
		data:extend({op:'edit'},req.body)
	});
});
router.route('/bookmark/delete/:token?')
.post(Api.loggedIn,function(req,res){
	Api.bookmarkDelete({
		res:res,
		token:req.token,
		data:extend({op:'delete'},req.body)
	});
});
router.route('/bookmark/:token?')
.get(Api.loggedIn,function(req,res){
	Api.bookmarkList({
		res:res,
		token:req.token,
		data:{op:'list'}
	});
})
.post(Api.loggedIn,function(req,res){
	Api.bookmarkList({
		res:res,
		token:req.token,
		data:{op:'list'}
	});
});
/******** SHARE API *********/


router.route('/share/edit/:token?')
.post(Api.loggedIn,function(req,res){
	Api.shareEdit({
		res:res,
		token:req.token,
		id:req.body.id,
		data:extend({op:'edit'},req.body)
	});
});

router.route('/share/delete/:token?')
.post(Api.loggedIn,function(req,res){
	Api.shareDelete({
		res:res,
		token:req.token,
		id:req.body.id,
		data:extend({op:'delete'},req.body)
	});
});
router.route('/share/reset/:token?')
.post(Api.loggedIn,function(req,res){
	Api.shareReset({
		res:res,
		token:req.token,
		id:req.body.id,
		data:extend({op:'reset'},req.body)
	});
});
router.route('/share/email/:token?')
.post(Api.loggedIn,function(req,res){
	Api.shareEmail({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/share/list/:token?')
.get(Api.loggedIn,function(req,res){
	Api.shareList({
		res:res,
		token:req.token,
		id:req.query.id,
		data:req.query
	});
})
.post(Api.loggedIn,function(req,res){
	Api.shareList({
		res:res,
		token:req.token,
		id:req.body.id,
		data:req.body
	});
});
router.route('/share/:token?')
.get(Api.loggedIn,function(req,res){
	Api.shareList({
		res:res,
		token:req.token,
		id:req.query.id,
		data:req.query
	});
})
.post(Api.loggedIn,function(req,res){
	Api.shareList({
		res:res,
		token:req.token,
		id:req.body.id,
		data:req.body
	});
});
/****** PROFILE API ****/
router.route('/session/profile/:token?')
.get(Api.loggedIn,function(req,res){
	Api.profileList({
		res:res,
		token:req.token
	});
})
.post(Api.loggedIn,function(req,res){
	Api.profileList({
		res:res,
		token:req.token
	});
});

/****** CONFIG API ****/
router.route('/config/:token?')
.get(Api.loggedIn,function(req,res){
	Api.config({
		res:res,
		token:req.token
	});
})
.post(Api.loggedIn,function(req,res){
	Api.config({
		res:res,
		token:req.token
	});
});

/****** ADMIN API ***console.log(req.cookies);****/


router.route('/admin/session/count/:token?')
.get(Api.loggedIn,function(req,res){
	Api.adminSessionCount({
		res:res,
		token:req.token,
		query:req.query || {}
	});
})
.post(Api.loggedIn,function(req,res){
	Api.adminSessionCount({
		res:res,
		token:req.token,
		query:req.body || {}
	});
});
router.route('/admin/session/:token?')
.get(Api.loggedIn,function(req,res){
	Api.adminSession({
		res:res,
		token:req.token,
		query:req.query || {}
	});
})
.post(Api.loggedIn,function(req,res){
	Api.adminSession({
		res:res,
		token:req.token,
		query:req.body || {}
	});
});
router.route('/admin/server/:token?')
.get(Api.loggedIn,function(req,res){
	Api.adminServer({
		res:res,
		token:req.token
	});
})
.post(Api.loggedIn,function(req,res){
	Api.adminServer({
		res:res,
		token:req.token
	});
});
router.route('/admin/:token?')
.get(Api.loggedIn,function(req,res){
	Api.admin({
		res:res,
		token:req.token
	});
})
.post(Api.loggedIn,function(req,res){
	Api.admin({
		res:res,
		token:req.token
	});
});
module.exports = router;
