var express = require('express');
var path = require('path');
var passport = require('passport');
var URL = require('url');

var VirtualUser = require('../lib/virtual-user');

var AuthMethods = _require('common/lib/config/auth-methods');
var ERROR 		= _require('common/lib/error-responses');
var Monitor 	= _require('common/lib/monitor');
var Motd 		= _require('common/lib/config/motd');
var Settings 	= _require('common/lib/config/settings');

var router = express.Router();
var cookieExpireTime = function(){return {expires:new Date(Date.now() + 3600000)};}


function secure(req,res,next){
	if(['127.0.0.1','::ffff:127.0.0.1','::1'].indexOf(req.ip) >= 0){
		next();
		return;
	}else if(!req.secure && req.app.HTTPSserver){
		var port = req.app.HTTPSserver.address().port;
		var host =  req.headers.host.split(':')[0];
		res.redirect('https://' + host + ((port !==443)?':'+port:'') + req.originalUrl);
		return;
	}else{
		next();
	}
}
function loggedIn(req,res,next){
	if(req.cookies['token']){
		var u = Monitor.getUserByToken(req.cookies['token'])
		if(u){
			if(u.data.loggedIn){
				res.set('X-FastX-Token',req.cookies['token']);
				res.clearCookie('authtoken');
				res.cookie('token',u.data.token,cookieExpireTime());
				var url =   req.query['redirectUrl'] || req.cookies['redirectUrl'] || '/';
				res.clearCookie('redirectUrl');
				res.redirect(url);
				return;
			}else{
				res.cookie('authtoken',u.data.token,cookieExpireTime());
				res.clearCookie('token');
			}
		}else{
			res.clearCookie('token');
		}
	}
	next();
}


function authenticate(authType,req,res,next){
	passport.authenticate(authType, function(err,user,info){
		if (err) {
			console.log(err.stack);
			return;
		}if (!user) {
			console.log('no user');
			return;
		}
		var u = new VirtualUser(user);
		res.json({result:{
			stage:"success",
			token:u.data.token,
			username:u.data.login,
			host:Settings.data.hostname
		}});
	})(req,res,next);
}
function authenticateOkta(req,res,next){
	passport.authenticate('saml', function(err,user,info){
		if (err) {
			res.clearCookie('token');
			res.status(401).render('auth-error',{
				error:err
			});
			return;
		}if (!user) {
			res.clearCookie('token');
			res.status(401).render('auth-error',{
				error:{
					message:'User not found'
				}
			});
			return;
		}

		var u = new VirtualUser({
			login:user.nameID,
			provider:'okta',
			id:user.login || user.nameID,
			email:user.email,
			firstName:user.firstName,
			lastName:user.lastName
		});
		res.cookie('token',u.data.token,cookieExpireTime());
		var url = req.cookies['redirectUrl'];
		res.clearCookie('redirectUrl');
		res.set('X-FastX-Token',u.data.token);
		res.redirect(url || '/');
	})(req,res,next);
}

router.use(secure);
router.use(loggedIn);
router.route('/')
.get(function(req,res){
	var redirect = 'ssh';
	switch(AuthMethods.data.defaultPage){
		case 'okta':
			if(AuthMethods.data.okta.entryPoint){
				redirect = 'okta';
			}

		default:
			break;
	}
	var q = URL.parse(req.originalUrl).search;
	res.redirect('/auth/' + redirect + (q?q:''));
});

router.route('/ssh')
.get(function(req,res){
	if(req.cookies['authtoken']){
		var u = Monitor.getUserByToken(req.cookies['authtoken']);
		if(u){
			if(u.data.loggedIn){
				res.set('X-FastX-Token',u.data.token);
				res.clearCookie('authtoken');
				res.cookie('token',u.data.token,cookieExpireTime());
				var url = req.cookies['redirectUrl'] || '/';
				res.clearCookie('redirectUrl');
				res.redirect(url);
				return;
			}else{
				res.cookie('authtoken',u.data.token,cookieExpireTime());
			}
		}else{
			res.clearCookie('authtoken');
		}
	}
	if(req.query['redirectUrl']){
		res.cookie('redirectUrl',req.query['redirectUrl'],cookieExpireTime());
	}
	res.sendFile(path.resolve(global.FXROOTDIR,'www/client/js/ssh/index.html'));
});
router.route('/okta')
.get(function(req,res,next){
	var q = req.query;
	for(var i in q){
		res.cookie(i,q[i],cookieExpireTime());
	}
	next();
},passport.authenticate('saml'))
.post(authenticateOkta);

router.route('/:authType/callback')
.get(function(req,res,next){
	authenticate(req.params.authType,req,res,next);
});
module.exports = router;
