var express = require('express');
var extend = require('extend');
var fs = require('fs');
var path = require('path');
var targz = require('targz');
var URL = require('url');

var auth = require('./route-util').auth;
var secure = require('./route-util').secure;
var cookieExpireTime = require('./route-util').cookieExpireTime;

var AuthMethods 	= _require('common/lib/config/auth-methods');
var FastXDefaults 	= _require('common/lib/config/fastx-defaults');
var Client = _require ('common/lib/config/client');

var localPool 		= _require('common/lib/local-pool');
var Monitor 		= _require('common/lib/monitor');
var Motd 			= _require('common/lib/config/motd');




var router = express.Router();
router.route('/js/xrdp/fastx-defaults.js')
.get(function(req,res){
	res.send(['window.FastXDefaults =',JSON.stringify(FastXDefaults.data),';'].join(''));
});

/* GET home page. */
router.route(['/'])
.get(secure,auth,function(req, res){
	res.render('fastx',{
		user:req.user.data,
		isAdmin:req.isAdmin,
		title:req.user.data.login + ' -- Sessions',
		scripts:'scripts/fastx.ect',
		baseUrl:'./',
		version:localPool.version
	});
	//res.sendFile(path.resolve(__dirname + '/../../../client/js/fastx/index.html'));
});
router.route('/motd')
.get(secure,auth,function(req,res){
	res.render('motd',{motd:Motd.data.motd || '<span style="color:black">You are logged in.</span>'});
});
router.route(['/admin/sessions'])
.get(secure,auth,function(req, res){
	if(!req.isAdmin){
		res.status(403);
		res.render('error/403');
		return;
	}
	res.render('admin/sessions',{
		user:req.user.data,
		isAdmin:req.isAdmin,
		title:'Admin - Sessions',
		scripts:'scripts/admin-sessions.ect',
		baseUrl:'../',
		version:localPool.version
	});
});
router.route(['/admin/servers'])
.get(secure,auth,function(req, res){
	if(!req.isAdmin){
		res.status(403);
		res.render('error/403');
		return;
	}
	res.render('admin/servers',{
		user:req.user.data,
		isAdmin:req.isAdmin,
		title:'Admin - Servers',
		scripts:'scripts/admin-servers.ect',
		baseUrl:'../',
		version:localPool.version
	});
});

router.route('/download/system/config')
.get(secure,auth,function(req,res){
	var dest = path.resolve(global.FXROOTDIR,'var','tmp',(new Date()).toString() + 'fastx-config.tar.gz');
	targz.compress({
		src:path.resolve(global.FXROOTDIR,'var','config'),
		dest:dest
	},function(e){
		if(e){
			res.render('error',{
				user:req.user.data,
				isAdmin:req.isAdmin,
				title:e.message,
				error:e,
				scripts:'',
				baseUrl:'../..',
				version:localPool.version
			});
		}else{
			res.download(dest,'config.tar.gz',function(e){
				if(e){
					console.log(e.stack);
				}
				fs.unlinkSync(dest);
			});
		}
	});
});

router.route(['/system'])
.get(secure,auth,function(req, res){
	if(!req.isAdmin){
		res.status(403);
		res.render('error/403');
		return;
	}
	res.render('system',{
		user:req.user.data,
		isAdmin:req.isAdmin,
		title:'System',
		scripts:'scripts/system.ect',
		baseUrl:'./',
		version:localPool.version
	});
});
router.route('/windows')
.get(secure,auth,function(req,res){
	res.sendFile(path.resolve(global.FXROOTDIR, 'www/client/js/xrdp/index.html'));
});
router.route(['/xrdp'])
.get(secure,function(req,res){
	res.set('Access-Control-Allow-Origin','*');
	res.sendFile(path.resolve(global.FXROOTDIR,'www/client/js/xrdp/index.html'));
});
router.route(['/xrdp/config'])
.get(secure,function(req,res){
	res.json({
		result: {
			keyboard: {
				layout: Client.data.keyboardLayout,
				type: Client.data.keyboardType
			}
		}
	});
});
router.route(['/connect'])
.get(secure,function(req,res){
	res.sendFile(path.resolve(global.FXROOTDIR, 'www/client/js/connect/index.html'));
});
router.route(['/shortcut'])
.get(secure,function(req,res){
	res.sendFile(path.resolve(global.FXROOTDIR, 'www/client/js/shortcut/index.html'));
});
router.route(['/share'])
.get(function(req,res){
	res.sendFile(path.resolve(global.FXROOTDIR,'www/client/js/share/index.html'));
});
router.route(['/examples/*'])
.get(auth,function(req,res){
	res.sendFile(path.resolve(global.FXROOTDIR,'www/client/examples/connect.html'));
});

module.exports = router;
