var express = require('express');
var exec = require('child_process').exec;

var fs = require('fs');
var multer = require('multer');

var path = require('path');

var System = require('../lib/system').System;

var ERROR 	= _require('common/lib/error-responses');
var Monitor = _require('common/lib/monitor');

var upload = multer({
	dest:path.resolve(global.FXROOTDIR,'var','uploads')
});

var router = express.Router();
router.all('*',function(req,res,next){
	if(!req.secure && req.app.HTTPSserver){
		var port = req.app.HTTPSserver.address().port;
		var host =  req.headers.host.split(':')[0];
		res.redirect(301,'https://' + host + ((port !==443)?':'+port:'') + req.url);
		return;
	}
	next();
});
var cookieExpireTime = function(){return {expires:new Date(Date.now() + 3600000)};}
 function loggedIn(req,res,next){
	var u;
	if(req.cookies.token){
		u = Monitor.getUserByToken(req.cookies.token);
		if(u && u.data.loggedIn){
			req.token = req.cookies.token;
		}
	}
	if(req.params.token){
		u = Monitor.getUserByToken(req.params.token);
		if(u && u.data.loggedIn){
			req.token = req.params.token;
		}
	}
	if(!req.token || !u || !u.data.loggedIn){
		res.clearCookie('username');
		res.clearCookie('host');
		res.clearCookie('token');
		res.json(ERROR.authenticationRequired).end();
		return;
	}else{
		if(!Monitor.isAdminByUserData(u)){
			res.json(ERROR.permissionDenied).end();
			return;
		}
		res.cookie('username',u.data.login,cookieExpireTime());
		res.cookie('host',u.data.hostname,cookieExpireTime());
		res.cookie('token',req.cookies.token,cookieExpireTime());
		Monitor.queueUser(req.token,{
			token:req.token,
			action:'keepalive'}
		);
		next();
		return;
	}
}
function getUploads(req,res){
	var p = path.resolve(global.FXROOTDIR,'var','uploads');
	if(req.params.filename){
		res.download(path.resolve(p,req.params.filename),req.params.filename);
		return;
	}else{
		fs.readdir(p,function(err,files){
			if(err){
				res.json({error:err});
			}else{
				files = files.map(function(c){
					try{
						var stats = (fs.statSync(path.resolve(p,c)));
						return {
							name:c,
							size:stats.size,
							atime:stats.atime,
							ctime:stats.ctime,
							mtime:stats.mtime
						};
					}catch(e){
						return {
							name:c
						};
					}
				});
				res.json({result:files});
			}
		});
	}
}
router.route('/uploads/:filename?')
.delete(loggedIn,function(req,res,next){
	var p = path.resolve(global.FXROOTDIR,'var','uploads');
	if(req.params.filename){
		try{
			fs.unlinkSync(path.resolve(p,req.params.filename));
			delete req.params.filename;
			next();
		}catch(e){
			console.log(e);
			res.json({error:e});
		}
	}

},getUploads)
.get(loggedIn,getUploads)
.post(loggedIn,upload.single('file'),function(req,res){
	var p =path.resolve(global.FXROOTDIR,'var','uploads');
	fs.readdir(path.resolve(global.FXROOTDIR,'var','uploads'),function(err,files){
		if(err){
			res.json({error:err});
		}else{
			var name = req.file.originalname;
			var extname = path.extname(name);
			var basename = path.basename(name,extname);
			var n = 1;
			while(files.indexOf(name) >=0){
				name = [basename,' ','(',n,')',extname].join('');
				n++;
			}
			fs.rename(path.resolve(p,req.file.filename),path.resolve(p,name),function(e){
				res.redirect('/system#!?nav=upload');
			});
		}
	});
});
router.route('/upgrade/:token?')
.get(loggedIn,function(req,res){
	Monitor.nodeExec({
		command:path.resolve(global.FXROOTDIR,'api','license'),
	},function(d){
		if(d.result){
			res.json({result:{
				product:"fastx2",
				version:fs.readFileSync(path.resolve(global.FXROOTDIR,'version'),{
					encoding:'utf-8'
				}),
				arch:"x86_64",
				license_version:d.max_version
			}});
		}else{
			res.json(d);
		}
	});
});

router.route('/session/profile/edit/:token?')
.post(loggedIn,function(req,res){
	System.sessionProfileEdit({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/session/profile/delete/:token?')
.post(loggedIn,function(req,res){
	System.sessionProfileDelete({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/session/profile/default/:token?')
.get(loggedIn,function(req,res){
	System.getDefaultSessionProfile({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setDefaultSessionProfile({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/session/profile/:token?')
.get(loggedIn,function(req,res){
	System.getSessionProfiles({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setSessionProfiles({
		res:res,
		token:req.token,
		data:req.body
	});
});

router.route('/admin/add/:token?')
.post(loggedIn,function(req,res){
	System.adminUserAdd({
		res:res,
		token:re.token,
		data:req.body
	});
});
router.route('/admin/delete/:token?')
.post(loggedIn,function(req,res){
	System.adminUserDelete({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/admin/:token?')
.get(loggedIn,function(req,res){
	System.getAdminUser({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setAdminUser({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/network/current/:token?')
.get(loggedIn,function(req,res){
	System.getCurrentNetworkConfig({
		res:res,
		token:req.token
	});
})
router.route('/network/revert/:token?')
.post(loggedIn,function(req,res){
	System.revertToCurrentNetworkConfig({
		res:res,
		token:req.token
	});
});

router.route('/network/delete/:token?')
.post(loggedIn,function(req,res){
	System.deleteNetworkConfig({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/network/:token?')
.get(loggedIn,function(req,res){
	System.getNetworkConfig({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setNetworkConfig({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/theme/:token?')
.get(loggedIn,function(req,res){
	System.getTheme({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setTheme({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/fastx-defaults/:token?')
.get(loggedIn,function(req,res){
	System.getFastXDefaults({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setFastXDefaults({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/session/:token?')
.get(loggedIn,function(req,res){
	System.getSession({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setSession({
		res:res,
		token:req.token,
		data:req.body
	});
});

router.route('/bookmark/edit/:token?')
.post(loggedIn,function(req,res){
	System.editBookmark({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/bookmark/delete/:token?')
.post(loggedIn,function(req,res){
	System.deleteBookmark({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/bookmark/:token?')
.get(loggedIn,function(req,res){
	System.listBookmark({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setBookmarks({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/command/:token?')
.get(loggedIn,function(req,res){
	System.getCommand({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setCommand({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/settings/:token?')
.get(loggedIn,function(req,res){
	System.getSettings({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setSettings({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/email/:token?')
.get(loggedIn,function(req,res){
	System.getEmail({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setEmail({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/motd/:token?')
.get(loggedIn,function(req,res){
	System.getMotd({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setMotd({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/ldap/:token?')
.get(loggedIn,function(req,res){
	System.getLdap({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setLdap({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/auth-methods/:token?')
.get(loggedIn,function(req,res){
	System.getAuthMethods({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setAuthMethods({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/www/:token?')
.get(loggedIn,function(req,res){
	System.getWWW({
		res:res,
		token:req.token
	});
})
.post(loggedIn,function(req,res){
	System.setWWW({
		res:res,
		token:req.token,
		data:req.body
	});
});
router.route('/log/:token?')
.get(loggedIn,function(req,res){
	var o ={};
	Monitor.getSystemLog(20000).then(function(data){
		o.content = data.result.text;
		o.filename = data.result.file;
		res.json({result:o});
	},function(e){
		res.json(e);
	});
});
function getLicenseFiles(req,res){
	var p = path.resolve(global.FXROOTDIR,'var','license');
	if(req.params.filename){
		res.json({
			result:{
				filename:req.params.filename,
				content:fs.readFileSync(path.resolve(p,req.params.filename),{encoding:'utf-8'})
			}
		});
		return;
	}else{
		fs.readdir(p,function(err,files){
			if(err){
				res.json({error:err});
			}else{
				files = files.filter(function(c){
					return c.match(/\.lic$/); //exclude non lic files
				}).map(function(c){
					try{
						var stats = (fs.statSync(path.resolve(p,c)));
						return {
							name:c,
							size:stats.size,
							mtime:stats.mtime,
							content:fs.readFileSync(path.resolve(global.FXROOTDIR,'var','license',c),{encoding:'utf-8'})
						};
					}catch(e){
						console.log('Error reading license file in system/license',c,e.message);
					}
				}).filter(function(c){
					return c;
				});
				res.json({result:files});
			}
		});
	}
}
router.route('/license/reread')
.get(loggedIn,function(req,res){
	Monitor.nodeExec({
		command:path.resolve(global.FXROOTDIR,'rlm','rlmutil'),
		args:['rlmreread', '-c', path.resolve (global.FXROOTDIR,'var','license')],
		raw:true
	},function(d){
		console.log ('Rereading license files');
		if(d.result){
			console.log (d.result);
			res.redirect('/system/license');
		}else{
			res.json(d);
		}
	});
});
router.route('/license/file/:filename?')
.delete(loggedIn,function(req,res,next){
	var p = path.resolve(global.FXROOTDIR,'var','license');
	if(req.params.filename){
		try{
			fs.unlinkSync(path.resolve(p,req.params.filename));
			delete req.params.filename;
			next();
		}catch(e){
			res.json({error:e}).end();
		}
	}

},getLicenseFiles)
.get(loggedIn,getLicenseFiles)
.post(loggedIn,function(req,res){
	var fname = req.params.filename;
	delete req.params.filename;
	var p =path.resolve(global.FXROOTDIR,'var','license');
	function wf(name){
		fs.writeFile(path.resolve(global.FXROOTDIR,'var','license',name),req.body.content,function(err){
				if(err){
					res.json({error:err});
				}else{
					getLicenseFiles(req,res);
				}
			});
	}
	if(fname){
		fs.readdir(path.resolve(global.FXROOTDIR,'var','license'),function(err,files){
			if(err){
				res.json({error:err});
			}else{
				var extname = path.extname(fname);
				var basename = path.basename(fname,extname);

				name = fname;
				var n = 1;
				if(req.body.nooverride){
					if(files.indexOf(name) >=0){
						//if name exists ignore
						getLicenseFiles(req,res);
					}else{
						wf(name);
					}
					return;
				}
				while(files.indexOf(name) >=0){
					name = [basename,' ','(',n,')',extname].join('');
					n++;
				}
				wf(name);
			}
		});
	}else{
		getLicenseFiles(req,res);
	}
});
router.route('/license/:token?')
.get(loggedIn,function(req,res){
	Monitor.nodeExec({
		command:path.resolve(global.FXROOTDIR,'api','license'),
	},
	function(d){
		res.json(d);
	});
});
module.exports = router;
