var extend = require('extend');
var os = require('os');
var path = require('path');

var Bridge 		= require('../bridge').Bridge;
var Sessions 	= require('../sessions').Sessions;
var SessionAction = require('./user-action').UserAction;

var Client 		= _require('common/lib/config/client');
var ERROR 		= _require('common/lib/error-responses');
var localPool 	= _require('common/lib/local-pool');
var Monitor 	= _require('common/lib/monitor');
var Session 	= _require('common/lib/config/session');
var Settings 	= _require('common/lib/config/settings');

function cleanOnError(m,msg){
	if(msg.error){
		switch(msg.error.code){
		case -32009:
		case 401:
			Sessions.cleanup(m.id);
				break;
			default:
				break;
		}
	}
}
SessionAction.prototype.updateSession = function(id,data,override){
	return Sessions.update(id,data,override);
}
SessionAction.prototype.deleteSession = function(id){
	Monitor.deleteSession(id);
}
SessionAction.prototype.getSessionMessages = function(id){
	return Sessions.getMessages(id);
}

SessionAction.prototype.list = function(m,cb){
	var that = this;
	var u = Monitor.getUserByToken(m.token);
	if(u){
		var login = u.data.login;
	}else{
		Monitor.sendResponse(m.msgId,ERROR.authenticationRequired,m.serverId);
		return;
	}
	var ids = [];

	myCb = function(msg){
		if(msg.result){
			var i,ses = [];
			for(i=0; i< msg.result.length; i++){
				if(msg.result[i].terminated){continue;}//filter out terminated sessions
				msg.result[i].login = login;
				msg.result[i].hostname = Settings.data.hostname;
				Monitor.updateLocalSession(msg.result[i]);
				ids.push(msg.result[i].id);
				ses.push(msg.result[i]);
			}

			//merge the user sessions
			var localSessions = Monitor.getLocalSessions();
			for(var s in localSessions){
				if( (localSessions[s].data.login === login) && (ids.indexOf(localSessions[s].data.id) < 0)){
					Monitor.deleteSession(localSessions[s].data.id);
				}
			}
			cb({result:Monitor.paginateSessions(ses,m.query)});
			Monitor.broadcastSessions();
			return;
		}else{
			cb(msg);
		}

	}
	this.localExec(m,myCb);
}
SessionAction.prototype.listCount = function(m,cb){
	m.action = 'list';
	this.list(m,function(r){
		if(r.result){
			r.result = r.result.length;
		}
		cb(r);
	});
}
SessionAction.prototype.loggedInExec = function(m,cb){
	var u = Monitor.getUserByToken(m.token);
	if(u){
		var login = u.data.login;
	}else{
		Monitor.sendResponse(m.msgId,ERROR.authenticationRequired,m.serverId);
		return;
	}
	if(login && s && login === s.data.login){
		this.localExec(m,cb);//if user owns session exec it
		return;
	}
	this.queueSessionExec(m,cb);
}
SessionAction.prototype.queueSessionExec = function(m,cb){
	var s = Monitor.getSession(m.id);
	if(!s){
		Monitor.sendResponse(m.msgId,ERROR.notFound,m.serverId);
		return;
	}
	var that = this;
	Monitor.getUserData(m.token,m.serverId,function(u){
		if(!u){
			Monitor.sendResponse(m.msgId,ERROR.permissionDenied,m.serverId);
			return;
		}
		if(s.data.login === u.data.login){
			if(s.serverId === u.serverId && !u.data.virtualUser){
				that.localExec(m,cb);//if user owns session exec it
			}else{
				Monitor.queueLocalMessage(m);//let xrdp exec it
			}
		}else if(Monitor.isAdminByUserData(u)){
			Monitor.queueLocalMessage(m);//let xrdp exec it
		}else{
			Monitor.sendResponse(m.msgId,ERROR.sessionAdminPermissionDenied,m.serverId);
		}
	});
}

SessionAction.prototype.sessionExec = function(m,cb){
	var that = this;
	cb = cb || function cb(msg){
		if(m.msgId){
			Monitor.sendResponse(m.msgId,msg,m.serverId);
		}
	}
	this.queueSessionExec(m,cb);
}

SessionAction.prototype.start = function(m,cb){
	if(Client.data.disableStart){
		Monitor.sendResponse(m.msgId,ERROR.disableSessionStart,m.serverId);
		return;
	}
	if(Settings.data.maxSessions &&  Monitor.getLocalSessions().length > Settings.data.maxSessions){
		Monitor.sendResponse(m.msgId,ERROR.maxSessions,m.serverId);
		return;
	}
	var that = this;
	myCb = function(msg){
		if(msg.result){
			msg.result.baseUrl = Settings.baseUrl;
			delete msg.result.sessionKey;
			var u = Monitor.getUserByToken(m.token) || { data:{} };
			Monitor.updateLocalSession({
				id:msg.result.id,
				command:m.data.command,
				clients:0,
				geometry:m.data.geometry,
				login:u.data.login,
				hostname:os.hostname(),
				params:{
					name:m.data.name,
					email:u.data.email
				}
			}); //update minimum amount of data for session launching
			Monitor.broadcastSessions();
		}

		Monitor.sendResponse(m.msgId,msg,m.serverId);
	}

	//set alternate configs;
	if(process.env.FX_CONFIG_PATH){
		m.data['config_path'] = process.env.FX_CONFIG_PATH;
	}
	if(m.sudo){
		this.sudoExec(m.sudo,m,myCb);
	}else{
		this.localExec(m,myCb);
	}
}
SessionAction.prototype.schedule = function(m,cb){
	//set alternate configs;
	if(Client.data.disableStart){
		Monitor.sendResponse(m.msgId,ERROR.disableSessionStart,m.serverId);
		return;
	}
	if(Settings.data.maxSessions &&  Monitor.getLocalSessions().length > Settings.data.maxSessions){
		Monitor.sendResponse(m.msgId,ERROR.maxSessions,m.serverId);
		return;
	}
	if(process.env.FX_CONFIG_PATH){
		m.data['config_path'] = process.env.FX_CONFIG_PATH;
	}
	var myCb = function(msg){
		Monitor.sendResponse(m.msgId,msg,m.serverId);
	}

	this.localExec(m,myCb);
}
SessionAction.prototype.connect = function(m,cb){
	var that = this;
	m.uri = '/local/connect/' + m.msgId + '/' + (m.serverId || localPool.serverId);
	var myCb = function(msg){
		if(msg.result ){
			msg.result.baseUrl = Settings.baseUrl;
		}
		Monitor.sendResponse(m.msgId,msg,m.serverId);
	}
	this.sessionExec(m,myCb);
}
SessionAction.prototype.localShare = function(m,cb){
	Monitor.sendResponse(m.msgId,{result:null},m.serverId);
}
SessionAction.prototype.terminate = function(m,cb){
	myCb = function(msg){
		cleanOnError(m,msg);
		cb(msg);
	}
	this.sessionExec(m,myCb);
}
SessionAction.prototype.disconnect = function(m,cb){
	this.sessionExec(m);
}
SessionAction.prototype.exec = function(m,cb){
	this.sessionExec(m);
}
SessionAction.prototype.log = function(m,cb){
	this.sessionExec(m);
}
SessionAction.prototype.params = function(m,cb){
	this.sessionExec(m);
}
SessionAction.prototype.screenshot = function(m,cb){
	this.sessionExec(m);
}
SessionAction.prototype.shareEdit = function(m,cb){
	m.action = 'share';
	m.data.op ='edit';
	if(Array.isArray(m.data.perms)){
		m.data.perms= m.data.perms.join(',');
	}
	this.sessionExec(m);
}
SessionAction.prototype.shareDelete = function(m,cb){
	m.action = 'share';
	m.data.op ='delete';
	this.sessionExec(m);
}
SessionAction.prototype.shareList = function(m,cb){
	m.action = 'share';
	m.data.op ='list';
	this.sessionExec(m);
}
SessionAction.prototype.shareReset = function(m,cb){
	m.action = 'share';
	m.data.op ='reset';
	this.sessionExec(m);
}
SessionAction.prototype.share = function(m,cb){
	if(Array.isArray(m.data.perms)) {
		m.data.perms = m.data.perms.join('');
	}
	cb = cb || function cb(msg){
		if(m.msgId){
			Monitor.sendResponse(m.msgId,msg,m.serverId);
		}
	}
	Monitor.queueLocalMessage(m);//let xrdp exec it
}
module.exports.SessionAction = SessionAction;
