var crypto = require('crypto');
var EventEmitter = require('events').EventEmitter;
var fs = require('fs');
var net = require('net');
var Q = require('q');
var uuid = require('node-uuid');

var Netstring = require('./netstring');
var userid = require('./userid');

var createParamString = _require('common/lib/create-param-string');
var LdapClient = _require('common/lib/ldap/client');
var localPool 	= _require('common/lib/local-pool');
var Monitor	 	= _require('common/lib/monitor');

function sanitize(str){
	if(typeof str === 'string'){
		return "'" + str.replace(/\'/g,"\'\\\'\'") + "'";
	}else{
		return str;
	}
}



function hashToken(token){
	const hash = crypto.createHash('sha256');
	hash.update(token);
	return hash.digest('hex');
}

function socketConnect(file,onConnect,token){
	var buf = new Buffer(0);
	var sock = net.createConnection(file,onConnect);
	var msgs = {};
	var timeout;

	setTimeout(function(){//1 minute to complete login
		var u = Monitor.getUserByToken(token);
		if(!u || !u.data.loggedIn){
			close();
		}
	},60000);
	sock.on('data',function(b){
		buf = Buffer.concat([buf,b],buf.length + b.length);
		try{
			var str;
			while (buf.length > 0 && (str = Netstring.decode(buf))) {
				//wait for ready message
				buf = buf.slice(str.next);
				var o = JSON.parse(str.data);
				if( (o.msgId && typeof msgs[o.msgId] === 'function')) {
					var id = o.msgId;
					delete o.msgId;
					msgs[id](o);
					delete msgs[id];
				}else{
					var u = Monitor.getUserByToken(token);
					if(!u || !u.data.loggedIn){
						send({
							msgId:uuid.v4(),
							token:token
						},function(data){
							Monitor.loggedIn(token);
							Monitor.broadcastUsers();
						});
					}
				}
			}
		}catch(e){
			console.log(e);
			console.log(e.stack);
			close();
		}
	});

	function send(r,cb){
		cb = cb || function(){};
		try{
			if(r.msgId){
				msgs[r.msgId] = cb;
			}
			if(r.data){
				r.args = createParamString(r.data,r.action);
				delete r.data;
			}
			sock.write(Netstring.encode(JSON.stringify(r)),function(){
				//console.log('sent',r);
			});
		}catch(e){
			console.log(e);
			console.log(e.stack);
			close();
		}
	};
	function close(cb){
		cb = cb || function(){};
		if(sock){
			sock.end();
			sock.destroy();
		}
		Monitor.deleteUser(token);
		cb();
	}
	function keepalive(ms,cb){
		cb = cb || function(){};
		clearTimeout(timeout);
		timeout = setTimeout(function(){
			close();
		},ms || 600000);
		cb();
	}
	return {
		send:send,
		keepalive:keepalive,
		close:close
	};
}

function token(file,token,myLogin){

		var q = Q.defer();
		userid.username(file,myLogin,function(err,login){
			try{
				if(err){
					throw new Error(err);
					return
				}
				var ldc = new LdapClient(function(e){
					Monitor.createUser({
						id:token,
						serverId:localPool.serverId,
						serverPool:localPool.serverKey,
						data:{
							token:token,
							login:login,
							provider:'link',
							firstName:'',
							lastName:'',
							email:''
						}
					});
				});
				ldc.search(['(uid=',login,')'].join(''),function(o){
					o = o || {};
					Monitor.createUser({
						id:token,
						serverId:localPool.serverId,
						serverPool:localPool.serverKey,
						data:{
							token:token,
							login:login,
							provider:'link',
							firstName:o.givenName,
							lastName:o.sn,
							email:o.mail
						}
					});
					Monitor.authenticated(token);
					Monitor.setLink(token,socketConnect(file,function(){q.resolve(null);},token));
				});
			}catch(e){
				console.log('Error in link', e);
				console.log(e.stack);
				q.reject(e);
			}
		});
	return q.promise;
}
function webtoken(file,webtoken,myLogin){
	var q = Q.defer();
	userid.username(file,myLogin,function(err,login){
		try{
			if(err){
				throw new Error(err);
				return;
			}
			var users = Monitor.getUsersByLogin(login);
			var u;
			while(u = users.shift()){
				if(u.data.authenticated && !u.data.loggedIn){
					if(webtoken === hashToken(u.data.token)){
						Monitor.setLink(u.data.token,socketConnect(file,function(){q.resolve(u.data.token);},u.data.token));
						return q.promise;
					}
				}
			}
			throw new Error('webtoken hash not found');
		}catch(e){
			console.log('Error in link', e);
			console.log(e.stack);
			q.reject(e);
		}
	});
	return q.promise;
}
function link(o){
	var q = Q.defer();
	if(o.webtoken){
		webtoken(o.socket,o.webtoken,o.login).then(function(token){
			q.resolve({token:token})
		},q.reject);
	}else if(o.token){
		token(o.socket,o.token,o.login).then(function(){
			q.resolve({token:null});
		},q.reject);
	}
	return q.promise;
}
module.exports = {
	link:link,
	hashToken:hashToken
}
