var extend = require('extend');
var SSH = require('fastx-ssh2');

var link = require('../link');
var SSHExec = require('./ssh-exec').SSHExec;

var AuthMethods = _require('common/lib/config/auth-methods');
var LdapClient 	= _require('common/lib/ldap/client');
var localPool 	= _require('common/lib/local-pool');
var settings 	= _require('common/lib/config/settings');
var Monitor 	= _require('common/lib/monitor');

function Connection(m){
	this.ssh = new SSH();
	this.msgId;
	this.cb = function(){};
	this.token = m.token;
	this.login;
	this.destroy = m.destroy || function(){};
	
	var self = this;
	this.ssh.on('ready',function(){
		onReady(m.token,self.msgId,self);
	});
	this.ssh.on('keyboard-interactive',function(name,instructions,lang,prompts,cb){
		self.cb = cb;
		self.prompts = [];
		var p;
		while(p = prompts.shift()){
			self.prompts.push(p.prompt);
		}
		self.answers = [];
		onKeyboardInteractive(self,m.token);
	});
	this.ssh.on('password',function(cb){
		self.cb = cb;
		Monitor.sendResponse(self.msgId,{result:{
			stage:'password',
			token:m.token
		}},self.serverId);
	});
	this.ssh.on('public-key',function(cb){
		self.cb = cb;
		Monitor.sendResponse(self.msgId,{result:{
			stage:'public-key',
			token:m.token
		}},self.serverId);
	});
	this.ssh.on('sign',function(buf,cb){
		self.cb = cb;
		Monitor.sendResponse(self.msgId,{result:{
			stage:'sign',
			token:m.token,
			signData:buf.toString('base64')
		}},self.serverId);
	});
	this.ssh.on('error',function(e){
		onError(e,self.msgId,self.token,self.serverId);
		self.destroy();
	});
	this.ssh.on('end',function(){
		Monitor.sendResponse(self.msgId,{
			error:{
				code:500,
				message:'SSH connection shut down abnormally'
			}
		},self.serverId);
		self.destroy();
	});
}
Connection.prototype.connect = function(o){
	this.login = o.username;
	try{
		var ldc = new LdapClient(function(){
			Monitor.createUser({
				id:o.token,
				serverId:localPool.serverId,
				serverPool:localPool.serverKey,
				data:{
					token:o.token,
					login:o.username,
					provider:'ssh',
					firstName:'',
					lastName:'',
					email:'',
					hostname:settings.data.hostname
				}
			});
			Monitor.broadcastUsers();
		});
		ldc.search(['(uid=',o.username,')'].join(''),function(d){
			d = d || {};
			Monitor.createUser({
				id:o.token,
				serverId:localPool.serverId,
				serverPool:localPool.serverKey,
				data:{
					token:o.token,
					login:o.username,
					provider:'ssh',
					firstName:d.givenName,
					lastName:d.sn,
					email:d.mail,
					hostname:settings.data.hostname
				}
			});
			Monitor.broadcastUsers();
		});
		if(!o.basic){
			this.ssh.connect({
				port:settings.data.sshport,
				tryKeyboard:true,
				passwordPrompt:true,
				publicKeyPrompt:o.usePK,
				readyTimeout:30000,
				username:o.username,
				password:o.password,
				//debug:function(d){console.log(d);}
			});
		}else{
			this.basic = true;
			this.pw = o.password ||'';
			this.ssh.connect({
				port:settings.data.sshport,
				tryKeyboard:true,
				readyTimeout:30000,
				username:o.username,
				password:o.password,
				//debug:function(d){console.log(d);}
			});
		}
	}catch(e){
		console.log('connect--d',e);
	}	
}
Connection.prototype.setMessageId = function(id,serverId){
	this.msgId = id;
	this.serverId = serverId;
}

function onKeyboardInteractive(self,token){
	if(self.basic){
		self.cb([self.pw]);return;}//basic authentication
	
	var p = self.prompts.shift();//short circuit password
	var params = {result:{
			stage:'keyboard-interactive',
			token:token,
			params:{
				name:'',
				instructions:'',
				lang:'',
				prompts:self.prompts
			},
			prompt:p
		}};
	Monitor.sendResponse(self.msgId,params,self.serverId);
}

function onReady(token,msgId,self){
	Monitor.authenticated(token);
	var sshexec = new SSHExec(self.ssh);
	sshexec.exec('link',{'webtoken':link.hashToken(token)},function(o){
		Monitor.loggedIn(token);
		if(o.result){
			var params = {
				result:{
					stage:'success',
					token:token,
					host:settings.data.hostname
				}};
			Monitor.broadcastUsers();
			Monitor.sendResponse(msgId,params,self.serverId);
			
		}else{ 
			if(o.error){
				Monitor.sendResponse(msgId,o,self.serverId);
			}else{
				Monitor.sendResponse(msgId,{error:{message:'Invalid JSON during connection',code:500}},self.serverId);
			}
		}
	});
}
function onError(e,msgId,token,serverId){
	var params;
	if(e.level === 'client-timeout'){
		var u = Monitor.getUserByToken(token);
		if(u && u.data && u.data.loggedIn){
			return;
		}
	}
	Monitor.deleteUser(token);
	Monitor.broadcastUsers();
	if(e.level){
		switch(e.level){
			case 'authentication':
				params = {error:{
					code:-32016,
					message:e.message || 'Username or password incorrect'
				}};
				break;
			case 'connection-socket':
				params = {error:{
					code:-32014,
					message:'Could not connect to the SSH Server.  Are you sure it is running?'
				}};
				break;
			case 'client-authentication':
				params = {error:{
					code:-32016,
					message:e.message || 'Username or password incorrect'
				}};
				break;
			case 'protocol':
				params = {error:{
						code:e.code,
						message:e.message
					}
				};
				break;
			case 'client-timeout':
				params ={error:{
						code:500,
						message:'Client Authentication Timeout'
					}};
				break
			default:
				if(e.code === 'ECONNREFUSED'){
					params ={error:{
						code:500,
						message:'Connection refused when attempting ssh login.  Is SSH running?'
					}};
				}else{
					console.log('SSH General error',e);
					params = {error:{
					code:500,
					message:"Socket error when logging in: " + e.level
					}};
				}
		};
	}else if(e['client-ssh']){
		params = {error:{
					code:500,
					message:"SSH Disconnect error: " + e.description
				}};
	}else{
		console.log('ssh error',e);
	}
	Monitor.sendResponse(msgId,params,serverId);
}

module.exports.Connection = Connection;