var path = require('path');

function sanitize(str){
	if(typeof str === 'string'){
		return "'" + str.replace(/\'/g,"\'\\\'\'") + "'";
	}else{
		return str;
	}
}

function SSHExec(ssh){
	this.ssh = ssh;
}
SSHExec.prototype.runCommand = function(command,params,cb){
	var c = [command];
	var p = Object.keys(params || {});
	for(i=0;i<p.length;i++){
		c.push( ['--',p[i],'=',sanitize(params[p[i]])].join('')); 
	}
	this.ssh.exec(c.join(' '),function(err,stream){
		if(err){
			console.log('Error in ssh exec:',err);
			cb({error:{code:500,message:err}});
			return;
		}
		var ebuf=[],buf=[];
		stream.stderr
		.on('data',function(data){ebuf.push(data);})
		.on('end',function(){
			if(ebuf.length){
				var msg = Buffer.concat(ebuf).toString();
				cb({error:{code:500,message:msg}});
			}
		});
		stream
		.on('data',function(data){buf.push(data);})
		.on('end',function(){
			if(buf.length){
				var msg = Buffer.concat(buf).toString();
				cb({result:msg});
			}
		});
	});
}
SSHExec.prototype.exec = function(command,params,cb,raw){
	var fastx_api = path.resolve(__dirname , '../../../../../api',command);
	var i,c=[fastx_api];
	var prefix = '--';
	if(command === 'params' && params['id']){
		c.push(['--id','=',sanitize(params['id'])].join(''));
		delete params['id'];
		prefix='';
	}
	if(command === 'bookmark' && params['op']){
		c.push(['--op','=',sanitize(params['op'])].join(''));
		delete params['op'];
		prefix='';
	}
	var p = Object.keys(params || {});
	for(i=0;i<p.length;i++){
		c.push( [prefix,p[i],'=',sanitize(params[p[i]])].join('')); 
	}
	this.ssh.exec(c.join(' '),function(err,stream){
		if(err){
			console.log('Error in ssh exec:',err);
			cb({error:{code:500,message:err}});
			return;
		}
		var ebuf=[],buf=[];
		stream.stderr
		.on('data',function(data){ebuf.push(data);})
		.on('end',function(){
			if(ebuf.length){
				var msg = Buffer.concat(ebuf).toString();
				cb({error:{code:500,message:msg}});
			}
		});
		stream
		.on('data',function(data){buf.push(data);})
		.on('end',function(){
			if(buf.length){
				var msg = Buffer.concat(buf).toString();
				var j;
				if(!raw){
					msg.replace(/^[^{]*/,'');
					msg.replace(/\n$/,'');
					try{
						j = JSON.parse(msg);
					}catch(e){
						cb({error:{code:500,message:msg}});
						return ;
					}
					cb(j);
				}else{
					cb({result:msg});
				}
			}
		});
	});
}
module.exports.SSHExec = SSHExec;
