const net = require('net');
var Q = require('q');
var WebSocket = require('ws');

var localPool 	= _require('common/lib/local-pool');
var Monitor 	= _require('common/lib/monitor');
var www 		= _require('common/lib/config/www');

function bridge(o){
	var q = Q.defer();
	var s = Monitor.getSession(o.id);
	if(!s){
		q.reject(new Error('Session not found'));
		return q.promise;
	}
	if(s.serverId === localPool.serverId){
		q.resolve({port:o.port});
	}else{
		try{
			var server = net.createServer({pauseOnConnect:true},function(tcp){
				var ws = new WebSocket(['http://localhost:',www.data.port,'/connect?',
					(new Buffer(JSON.stringify({id:s.data.id,port:o.port}))).toString('base64')].join(''));
				ws.binaryType = 'arraybuffer';
				ws.on('open',function(){
					tcp.resume();
				});
				ws.on('message',function(msg){
					try {
						tcp.write(msg);
					} catch (e) {
						console.log (e);
					}
				});
				ws.on('close',function(code,message){
					tcp.end();
				});
				ws.on('error',function(){
					tcp.end();
				});

				tcp.on('open',function(){
					server.close(function(){});
				});
				tcp.on('data',function(data){
					try {
						ws.send(data,{binary:true},function(error){
							if(error){
								tcp.end();
							}
						});
					} catch (e) {
						console.log (e);
					}
				});
				tcp.on('end',function(){
					ws.close();
				});
				tcp.on('error',function(){
				});
			});
			server.listen(function(){
				q.resolve({port:server.address().port});
				setTimeout(function(){
					server.close();
				},15000);
			});
		}catch(e){
			console.log(e);
			console.log(e.stack);
		}
	}
	return q.promise;
}

module.exports = {
	bridge:bridge
}
