var express = require('express');
var os = require('os');

var Bridge 		= require('../lib/bridge').Bridge;
var link 		= require('../lib/link');
var localApi 	= require('../lib/local-api').LocalApi;
var tcpBridge 	= require('../lib/tcp-bridge');

var Monitor 	= _require('common/lib/monitor');
var Settings 	= _require('common/lib/config/settings');
var localPool 	= _require('common/lib/local-pool');
var Bookmark 	= _require('common/lib/config/bookmark');
var Command 	= _require('common/lib/config/command');
var Profile 	= _require('common/lib/config/profile');
var Session 	= _require('common/lib/config/session');


var router = express.Router();
router.all('*',function(req,res,next){
	if(['127.0.0.1','::ffff:127.0.0.1','::1'].indexOf(req.ip) >= 0){
		next();
	}else{
		res.status(403).end();
	}
});
router.route('/session/:sessionId')
	.post(function(req,res){
		req.body.id = req.params.sessionId;
		req.body.hostname = os.hostname();
		if( Monitor.updateLocalSession(req.body) ){
			Monitor.broadcastSessions();
		}
		Monitor.storeLocalMessageResponse(req.params.sessionId,res);
	})
	.delete(function(req,res){
		Monitor.deleteSession(req.params.sessionId);
		Monitor.broadcastSessions();
		res.send(null);
	});
router.route('/link')
	.post(function(req,res){
		link.link(req.body).then(function(j){
			res.json({result:j});
		},function(e){
			res.json({error:{message:e.message,code:500}});
		});
	});
router.route('/listen')
.post(function(req,res){
		tcpBridge.bridge(req.body).then(function(j){
			res.json({result:j});
		},function(e){
			res.json({error:{message:e.message,code:500}});
		});
	});
router.route('/connect/:msgId/:serverId')
	.post(function(req,res){
		res.send(null);
		if(req.body.result){
			req.body.result.baseUrl = Settings.baseUrl;
		}
		Monitor.sendResponse(req.params.msgId,req.body,req.params.serverId);
	});

router.route('/message/:msgId')
	.post(function(req,res){
		res.send(null);
		Monitor.sendResponse(req.params.msgId,req.body);
	});
router.route('/message/:msgId/:serverId')
	.post(function(req,res){
		res.send(null);
		Monitor.sendResponse(req.params.msgId,req.body,req.params.serverId);
	});
router.route('/config')
.post(function(req,res){
	var configs = req.body.configs || [];
	var config,c,o={};
	while(configs.length){
		config = configs.shift();
		c = (typeof config === 'string')?config:config.type;
		switch(c){
			case 'profile':
				if(typeof config === 'string' || !config.id){
					o.profile = Session.data;
				}
				if(config.id){
					o.profile = (new Profile(config.id)).data;
				}
				break;
			case 'command':
				o.command = Command.data;
				break;
			case 'bookmark':
				o.bookmark = Bookmark.data;
				break;
			case 'session':
				o.session = Session.data;
				break;
		}
	}
	res.json(o);
});
module.exports = router;
